/* valaprojectgenerator.vala
 *
 * Copyright (C) 2007-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 * 	Andrea Del Signore <sejerpz@tin.it>
 */

#include <gen-project/valaprojectgenerator.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>
#include <glib/gstdio.h>
#include <config.h>
#include <stdio.h>
#include <string.h>
#include <gobject/gvaluecollector.h>


#define VALA_TYPE_PROJECT_TYPE (vala_project_type_get_type ())

#define VALA_TYPE_PROJECT_LICENSE (vala_project_license_get_type ())

#define VALA_TYPE_PROJECT_GENERATOR (vala_project_generator_get_type ())
#define VALA_PROJECT_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_PROJECT_GENERATOR, ValaProjectGenerator))
#define VALA_PROJECT_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_PROJECT_GENERATOR, ValaProjectGeneratorClass))
#define VALA_IS_PROJECT_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_PROJECT_GENERATOR))
#define VALA_IS_PROJECT_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_PROJECT_GENERATOR))
#define VALA_PROJECT_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_PROJECT_GENERATOR, ValaProjectGeneratorClass))

typedef struct _ValaProjectGenerator ValaProjectGenerator;
typedef struct _ValaProjectGeneratorClass ValaProjectGeneratorClass;
typedef struct _ValaProjectGeneratorPrivate ValaProjectGeneratorPrivate;
typedef struct _ValaParamSpecProjectGenerator ValaParamSpecProjectGenerator;

typedef enum  {
	VALA_PROJECT_TYPE_CONSOLE_APPLICATION,
	VALA_PROJECT_TYPE_GTK_APPLICATION
} ValaProjectType;

typedef enum  {
	VALA_PROJECT_LICENSE_GPL2,
	VALA_PROJECT_LICENSE_GPL3,
	VALA_PROJECT_LICENSE_LGPL2,
	VALA_PROJECT_LICENSE_LGPL3
} ValaProjectLicense;

struct _ValaProjectGenerator {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaProjectGeneratorPrivate * priv;
};

struct _ValaProjectGeneratorClass {
	GTypeClass parent_class;
	void (*finalize) (ValaProjectGenerator *self);
};

struct _ValaParamSpecProjectGenerator {
	GParamSpec parent_instance;
};



static char* string_substring (const char* self, glong offset, glong len);
GType vala_project_type_get_type (void);
GType vala_project_license_get_type (void);
struct _ValaProjectGeneratorPrivate {
	GtkDialog* config_dialog;
	GtkFileChooserButton* project_folder_button;
	GtkComboBox* project_type_combobox;
	GtkComboBox* license_combobox;
	GtkEntry* name_entry;
	GtkEntry* email_entry;
	char* project_path;
	char* project_name;
	char* namespace_name;
	char* make_name;
	char* upper_case_make_name;
	char* real_name;
	char* email_address;
	ValaProjectType project_type;
	ValaProjectLicense project_license;
};

#define VALA_PROJECT_GENERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_PROJECT_GENERATOR, ValaProjectGeneratorPrivate))
enum  {
	VALA_PROJECT_GENERATOR_DUMMY_PROPERTY
};
static char* vala_project_generator_option_project_path = NULL;
static ValaProjectType vala_project_generator_option_project_type = VALA_PROJECT_TYPE_GTK_APPLICATION;
static ValaProjectLicense vala_project_generator_option_project_license = VALA_PROJECT_LICENSE_LGPL2;
static gboolean vala_project_generator_option_version = FALSE;
static char* vala_project_generator_option_author = NULL;
static char* vala_project_generator_option_email = NULL;
static char* vala_project_generator_option_project_name = NULL;
static char** vala_project_generator_option_files = NULL;
static gint vala_project_generator_option_files_length1 = 0;
static gint vala_project_generator_option_files_size = 0;
static gboolean vala_project_generator_option_parse_callback (const char* option_name, const char* value, void* data, GError** error);
static ValaProjectGenerator* vala_project_generator_construct (GType object_type);
static ValaProjectGenerator* vala_project_generator_new (void);
static GtkHBox* vala_project_generator_create_hbox (ValaProjectGenerator* self, const char* title, GtkSizeGroup* size_group);
static void vala_project_generator_build_dialog_ui (ValaProjectGenerator* self);
static GtkResponseType vala_project_generator_ask_parameters (ValaProjectGenerator* self);
static void vala_project_generator_create_project (ValaProjectGenerator* self);
static void vala_project_generator_write_autogen_sh (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_configure_ac (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_toplevel_makefile_am (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_src_makefile_am (ValaProjectGenerator* self, GError** error);
static char* vala_project_generator_generate_source_file_header (ValaProjectGenerator* self, const char* filename);
static void vala_project_generator_write_main_vala (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_mainwindow_vala (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_potfiles (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_linguas (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_authors (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_maintainers (ValaProjectGenerator* self, GError** error);
static char* vala_project_generator_get_automake_path (ValaProjectGenerator* self);
static gint vala_project_generator_main (char** args, int args_length1);
static GParamSpec* vala_param_spec_project_generator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
static gpointer vala_value_get_project_generator (const GValue* value);
static void vala_value_set_project_generator (GValue* value, gpointer v_object);
static gpointer vala_project_generator_parent_class = NULL;
static void vala_project_generator_finalize (ValaProjectGenerator* obj);
static GType vala_project_generator_get_type (void);
static gpointer vala_project_generator_ref (gpointer instance);
static void vala_project_generator_unref (gpointer instance);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const GOptionEntry VALA_PROJECT_GENERATOR_options[] = {{"projectdir", 'p', 0, G_OPTION_ARG_FILENAME, &vala_project_generator_option_project_path, "Project directory", "DIRECTORY"}, {"type", 't', 0, G_OPTION_ARG_CALLBACK, (void*) vala_project_generator_option_parse_callback, "Project TYPE: gtk+, console", "TYPE"}, {"license", 'l', 0, G_OPTION_ARG_CALLBACK, (void*) vala_project_generator_option_parse_callback, "License TYPE: gpl2, gpl3, lgpl2, lgpl3", "TYPE"}, {"version", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_project_generator_option_version, "Display version number", NULL}, {"author", 'a', 0, G_OPTION_ARG_STRING, &vala_project_generator_option_author, "Author name", "NAME"}, {"email", 'e', 0, G_OPTION_ARG_STRING, &vala_project_generator_option_email, "Author email address", "EMAIL"}, {"", (gchar) 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &vala_project_generator_option_files, "Project NAME", "NAME"}, {NULL}};


static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}



GType vala_project_type_get_type (void) {
	static GType vala_project_type_type_id = 0;
	if (G_UNLIKELY (vala_project_type_type_id == 0)) {
		static const GEnumValue values[] = {{VALA_PROJECT_TYPE_CONSOLE_APPLICATION, "VALA_PROJECT_TYPE_CONSOLE_APPLICATION", "console-application"}, {VALA_PROJECT_TYPE_GTK_APPLICATION, "VALA_PROJECT_TYPE_GTK_APPLICATION", "gtk-application"}, {0, NULL, NULL}};
		vala_project_type_type_id = g_enum_register_static ("ValaProjectType", values);
	}
	return vala_project_type_type_id;
}



GType vala_project_license_get_type (void) {
	static GType vala_project_license_type_id = 0;
	if (G_UNLIKELY (vala_project_license_type_id == 0)) {
		static const GEnumValue values[] = {{VALA_PROJECT_LICENSE_GPL2, "VALA_PROJECT_LICENSE_GPL2", "gpl2"}, {VALA_PROJECT_LICENSE_GPL3, "VALA_PROJECT_LICENSE_GPL3", "gpl3"}, {VALA_PROJECT_LICENSE_LGPL2, "VALA_PROJECT_LICENSE_LGPL2", "lgpl2"}, {VALA_PROJECT_LICENSE_LGPL3, "VALA_PROJECT_LICENSE_LGPL3", "lgpl3"}, {0, NULL, NULL}};
		vala_project_license_type_id = g_enum_register_static ("ValaProjectLicense", values);
	}
	return vala_project_license_type_id;
}


static gboolean vala_project_generator_option_parse_callback (const char* option_name, const char* value, void* data, GError** error) {
	GError * inner_error;
	gboolean _tmp0;
	g_return_val_if_fail (option_name != NULL, FALSE);
	g_return_val_if_fail (value != NULL, FALSE);
	inner_error = NULL;
	_tmp0 = FALSE;
	if (_vala_strcmp0 (option_name, "--type") == 0) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = _vala_strcmp0 (option_name, "-t") == 0;
	}
	if (_tmp0) {
		if (_vala_strcmp0 (value, "gtk+") == 0) {
			vala_project_generator_option_project_type = VALA_PROJECT_TYPE_GTK_APPLICATION;
		} else {
			if (_vala_strcmp0 (value, "console") == 0) {
				vala_project_generator_option_project_type = VALA_PROJECT_TYPE_CONSOLE_APPLICATION;
			} else {
				char* _tmp1;
				GError* _tmp2;
				_tmp1 = NULL;
				_tmp2 = NULL;
				inner_error = (_tmp2 = g_error_new_literal (G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, _tmp1 = g_strdup_printf (_ ("project of type %s is not supported"), value)), _tmp1 = (g_free (_tmp1), NULL), _tmp2);
				if (inner_error != NULL) {
					g_propagate_error (error, inner_error);
					return FALSE;
				}
			}
		}
	} else {
		gboolean _tmp3;
		_tmp3 = FALSE;
		if (_vala_strcmp0 (option_name, "--license") == 0) {
			_tmp3 = TRUE;
		} else {
			_tmp3 = _vala_strcmp0 (option_name, "-l") == 0;
		}
		if (_tmp3) {
			if (_vala_strcmp0 (value, "gpl2") == 0) {
				vala_project_generator_option_project_license = VALA_PROJECT_LICENSE_GPL2;
			} else {
				if (_vala_strcmp0 (value, "gpl3") == 0) {
					vala_project_generator_option_project_license = VALA_PROJECT_LICENSE_GPL3;
				} else {
					if (_vala_strcmp0 (value, "lgpl2") == 0) {
						vala_project_generator_option_project_license = VALA_PROJECT_LICENSE_LGPL2;
					} else {
						if (_vala_strcmp0 (value, "lgpl3") == 0) {
							vala_project_generator_option_project_license = VALA_PROJECT_LICENSE_LGPL3;
						} else {
							char* _tmp4;
							GError* _tmp5;
							_tmp4 = NULL;
							_tmp5 = NULL;
							inner_error = (_tmp5 = g_error_new_literal (G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, _tmp4 = g_strdup_printf (_ ("license of type %s is not available"), value)), _tmp4 = (g_free (_tmp4), NULL), _tmp5);
							if (inner_error != NULL) {
								g_propagate_error (error, inner_error);
								return FALSE;
							}
						}
					}
				}
			}
		} else {
			char* _tmp6;
			GError* _tmp7;
			_tmp6 = NULL;
			_tmp7 = NULL;
			inner_error = (_tmp7 = g_error_new_literal (G_OPTION_ERROR, G_OPTION_ERROR_UNKNOWN_OPTION, _tmp6 = g_strdup_printf (_ ("unknown option %s"), option_name)), _tmp6 = (g_free (_tmp6), NULL), _tmp7);
			if (inner_error != NULL) {
				g_propagate_error (error, inner_error);
				return FALSE;
			}
		}
	}
	return TRUE;
}


static ValaProjectGenerator* vala_project_generator_construct (GType object_type) {
	ValaProjectGenerator* self;
	char* _tmp1;
	const char* _tmp0;
	char* _tmp3;
	const char* _tmp2;
	self = (ValaProjectGenerator*) g_type_create_instance (object_type);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->project_path = (_tmp1 = (_tmp0 = vala_project_generator_option_project_path, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->priv->project_path = (g_free (self->priv->project_path), NULL), _tmp1);
	_tmp3 = NULL;
	_tmp2 = NULL;
	self->priv->project_name = (_tmp3 = (_tmp2 = vala_project_generator_option_project_name, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), self->priv->project_name = (g_free (self->priv->project_name), NULL), _tmp3);
	return self;
}


static ValaProjectGenerator* vala_project_generator_new (void) {
	return vala_project_generator_construct (VALA_TYPE_PROJECT_GENERATOR);
}


static GtkHBox* vala_project_generator_create_hbox (ValaProjectGenerator* self, const char* title, GtkSizeGroup* size_group) {
	GtkHBox* hbox;
	GtkVBox* _tmp0;
	GtkVBox* vbox;
	GtkLabel* label;
	GtkLabel* _tmp1;
	GtkHBox* _tmp2;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (title != NULL, NULL);
	g_return_val_if_fail (size_group != NULL, NULL);
	hbox = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 6));
	_tmp0 = NULL;
	vbox = (_tmp0 = self->priv->config_dialog->vbox, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) hbox, FALSE, FALSE, (guint) 0);
	gtk_widget_show ((GtkWidget*) hbox);
	label = g_object_ref_sink ((GtkLabel*) gtk_label_new ("    "));
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) label, FALSE, FALSE, (guint) 0);
	gtk_widget_show ((GtkWidget*) label);
	_tmp1 = NULL;
	label = (_tmp1 = g_object_ref_sink ((GtkLabel*) gtk_label_new (title)), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp1);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) label, FALSE, FALSE, (guint) 0);
	g_object_set ((GtkMisc*) label, "xalign", (float) 0, NULL);
	gtk_widget_show ((GtkWidget*) label);
	gtk_size_group_add_widget (size_group, (GtkWidget*) label);
	_tmp2 = NULL;
	return (_tmp2 = hbox, (vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL)), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp2);
}


static void vala_project_generator_build_dialog_ui (ValaProjectGenerator* self) {
	GtkDialog* _tmp0;
	GtkVBox* _tmp1;
	GtkVBox* vbox;
	GtkSizeGroup* size_group;
	GtkLabel* label;
	GtkHBox* hbox;
	GtkLabel* _tmp2;
	GtkHBox* _tmp7;
	GtkComboBox* _tmp8;
	GtkHBox* _tmp9;
	GtkComboBox* _tmp10;
	GtkLabel* _tmp11;
	GtkHBox* _tmp12;
	GtkEntry* _tmp13;
	GtkHBox* _tmp18;
	GtkEntry* _tmp19;
	GtkWidget* _tmp24;
	GtkWidget* ok_button;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	self->priv->config_dialog = (_tmp0 = g_object_ref_sink ((GtkDialog*) gtk_dialog_new ()), (self->priv->config_dialog == NULL) ? NULL : (self->priv->config_dialog = (g_object_unref (self->priv->config_dialog), NULL)), _tmp0);
	gtk_window_set_title ((GtkWindow*) self->priv->config_dialog, "Vala Project Generator");
	gtk_container_set_border_width ((GtkContainer*) self->priv->config_dialog, (guint) 12);
	gtk_dialog_set_has_separator (self->priv->config_dialog, FALSE);
	_tmp1 = NULL;
	vbox = (_tmp1 = self->priv->config_dialog->vbox, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	label = NULL;
	hbox = NULL;
	gtk_box_set_spacing ((GtkBox*) vbox, 6);
	_tmp2 = NULL;
	label = (_tmp2 = g_object_ref_sink ((GtkLabel*) gtk_label_new ("<b>Project</b>")), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp2);
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) label, FALSE, FALSE, (guint) 0);
	gtk_label_set_use_markup (label, TRUE);
	g_object_set ((GtkMisc*) label, "xalign", (float) 0, NULL);
	gtk_widget_show ((GtkWidget*) label);
	if (vala_project_generator_option_project_path == NULL) {
		GtkHBox* _tmp3;
		GtkFileChooserButton* _tmp4;
		_tmp3 = NULL;
		hbox = (_tmp3 = vala_project_generator_create_hbox (self, "Project folder:", size_group), (hbox == NULL) ? NULL : (hbox = (g_object_unref (hbox), NULL)), _tmp3);
		_tmp4 = NULL;
		self->priv->project_folder_button = (_tmp4 = g_object_ref_sink ((GtkFileChooserButton*) gtk_file_chooser_button_new ("Select a project folder", GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)), (self->priv->project_folder_button == NULL) ? NULL : (self->priv->project_folder_button = (g_object_unref (self->priv->project_folder_button), NULL)), _tmp4);
		gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->project_folder_button, TRUE, TRUE, (guint) 0);
		gtk_widget_show ((GtkWidget*) self->priv->project_folder_button);
	} else {
		char* _tmp6;
		const char* _tmp5;
		_tmp6 = NULL;
		_tmp5 = NULL;
		self->priv->project_path = (_tmp6 = (_tmp5 = vala_project_generator_option_project_path, (_tmp5 == NULL) ? NULL : g_strdup (_tmp5)), self->priv->project_path = (g_free (self->priv->project_path), NULL), _tmp6);
	}
	_tmp7 = NULL;
	hbox = (_tmp7 = vala_project_generator_create_hbox (self, "Project type:", size_group), (hbox == NULL) ? NULL : (hbox = (g_object_unref (hbox), NULL)), _tmp7);
	_tmp8 = NULL;
	self->priv->project_type_combobox = (_tmp8 = g_object_ref_sink ((GtkComboBox*) gtk_combo_box_new_text ()), (self->priv->project_type_combobox == NULL) ? NULL : (self->priv->project_type_combobox = (g_object_unref (self->priv->project_type_combobox), NULL)), _tmp8);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->project_type_combobox, TRUE, TRUE, (guint) 0);
	gtk_combo_box_append_text (self->priv->project_type_combobox, "Console Application");
	gtk_combo_box_append_text (self->priv->project_type_combobox, "GTK+ Application");
	gtk_combo_box_set_active (self->priv->project_type_combobox, (gint) vala_project_generator_option_project_type);
	gtk_widget_show ((GtkWidget*) self->priv->project_type_combobox);
	_tmp9 = NULL;
	hbox = (_tmp9 = vala_project_generator_create_hbox (self, "License:", size_group), (hbox == NULL) ? NULL : (hbox = (g_object_unref (hbox), NULL)), _tmp9);
	_tmp10 = NULL;
	self->priv->license_combobox = (_tmp10 = g_object_ref_sink ((GtkComboBox*) gtk_combo_box_new_text ()), (self->priv->license_combobox == NULL) ? NULL : (self->priv->license_combobox = (g_object_unref (self->priv->license_combobox), NULL)), _tmp10);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->license_combobox, TRUE, TRUE, (guint) 0);
	gtk_combo_box_append_text (self->priv->license_combobox, "GNU General Public License, version 2 or later");
	gtk_combo_box_append_text (self->priv->license_combobox, "GNU General Public License, version 3 or later");
	gtk_combo_box_append_text (self->priv->license_combobox, "GNU Lesser General Public License, version 2.1 or later");
	gtk_combo_box_append_text (self->priv->license_combobox, "GNU Lesser General Public License, version 3 or later");
	gtk_combo_box_set_active (self->priv->license_combobox, (gint) vala_project_generator_option_project_license);
	gtk_widget_show ((GtkWidget*) self->priv->license_combobox);
	_tmp11 = NULL;
	label = (_tmp11 = g_object_ref_sink ((GtkLabel*) gtk_label_new ("<b>Author</b>")), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp11);
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) label, FALSE, FALSE, (guint) 0);
	gtk_label_set_use_markup (label, TRUE);
	g_object_set ((GtkMisc*) label, "xalign", (float) 0, NULL);
	gtk_widget_show ((GtkWidget*) label);
	_tmp12 = NULL;
	hbox = (_tmp12 = vala_project_generator_create_hbox (self, "Name:", size_group), (hbox == NULL) ? NULL : (hbox = (g_object_unref (hbox), NULL)), _tmp12);
	_tmp13 = NULL;
	self->priv->name_entry = (_tmp13 = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (self->priv->name_entry == NULL) ? NULL : (self->priv->name_entry = (g_object_unref (self->priv->name_entry), NULL)), _tmp13);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->name_entry, TRUE, TRUE, (guint) 0);
	if (vala_project_generator_option_author != NULL) {
		char* _tmp15;
		const char* _tmp14;
		_tmp15 = NULL;
		_tmp14 = NULL;
		self->priv->real_name = (_tmp15 = (_tmp14 = vala_project_generator_option_author, (_tmp14 == NULL) ? NULL : g_strdup (_tmp14)), self->priv->real_name = (g_free (self->priv->real_name), NULL), _tmp15);
	} else {
		char* _tmp17;
		const char* _tmp16;
		_tmp17 = NULL;
		_tmp16 = NULL;
		self->priv->real_name = (_tmp17 = (_tmp16 = g_getenv ("REAL_NAME"), (_tmp16 == NULL) ? NULL : g_strdup (_tmp16)), self->priv->real_name = (g_free (self->priv->real_name), NULL), _tmp17);
	}
	if (self->priv->real_name != NULL) {
		gtk_entry_set_text (self->priv->name_entry, self->priv->real_name);
	}
	gtk_widget_show ((GtkWidget*) self->priv->name_entry);
	_tmp18 = NULL;
	hbox = (_tmp18 = vala_project_generator_create_hbox (self, "E-mail address:", size_group), (hbox == NULL) ? NULL : (hbox = (g_object_unref (hbox), NULL)), _tmp18);
	_tmp19 = NULL;
	self->priv->email_entry = (_tmp19 = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (self->priv->email_entry == NULL) ? NULL : (self->priv->email_entry = (g_object_unref (self->priv->email_entry), NULL)), _tmp19);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->email_entry, TRUE, TRUE, (guint) 0);
	if (vala_project_generator_option_email != NULL) {
		char* _tmp21;
		const char* _tmp20;
		_tmp21 = NULL;
		_tmp20 = NULL;
		self->priv->email_address = (_tmp21 = (_tmp20 = vala_project_generator_option_email, (_tmp20 == NULL) ? NULL : g_strdup (_tmp20)), self->priv->email_address = (g_free (self->priv->email_address), NULL), _tmp21);
	} else {
		char* _tmp23;
		const char* _tmp22;
		_tmp23 = NULL;
		_tmp22 = NULL;
		self->priv->email_address = (_tmp23 = (_tmp22 = g_getenv ("EMAIL_ADDRESS"), (_tmp22 == NULL) ? NULL : g_strdup (_tmp22)), self->priv->email_address = (g_free (self->priv->email_address), NULL), _tmp23);
	}
	if (self->priv->email_address != NULL) {
		gtk_entry_set_text (self->priv->email_entry, self->priv->email_address);
	}
	gtk_widget_show ((GtkWidget*) self->priv->email_entry);
	gtk_dialog_add_button (self->priv->config_dialog, GTK_STOCK_CANCEL, (gint) GTK_RESPONSE_CANCEL);
	_tmp24 = NULL;
	ok_button = (_tmp24 = gtk_dialog_add_button (self->priv->config_dialog, "Create Project", (gint) GTK_RESPONSE_OK), (_tmp24 == NULL) ? NULL : g_object_ref (_tmp24));
	gtk_widget_grab_default (ok_button);
	(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
	(size_group == NULL) ? NULL : (size_group = (g_object_unref (size_group), NULL));
	(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
	(hbox == NULL) ? NULL : (hbox = (g_object_unref (hbox), NULL));
	(ok_button == NULL) ? NULL : (ok_button = (g_object_unref (ok_button), NULL));
}


static GtkResponseType vala_project_generator_ask_parameters (ValaProjectGenerator* self) {
	GtkResponseType response;
	g_return_val_if_fail (self != NULL, 0);
	response = 0;
	vala_project_generator_build_dialog_ui (self);
	response = (GtkResponseType) gtk_dialog_run (self->priv->config_dialog);
	if (response == GTK_RESPONSE_OK) {
		char* _tmp8;
		const char* _tmp7;
		char* _tmp10;
		const char* _tmp9;
		if (vala_project_generator_option_project_path == NULL) {
			char* _tmp1;
			const char* _tmp0;
			_tmp1 = NULL;
			_tmp0 = NULL;
			self->priv->project_path = (_tmp1 = (_tmp0 = gtk_file_chooser_get_current_folder ((GtkFileChooser*) self->priv->project_folder_button), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->priv->project_path = (g_free (self->priv->project_path), NULL), _tmp1);
		} else {
			char* _tmp3;
			const char* _tmp2;
			_tmp3 = NULL;
			_tmp2 = NULL;
			self->priv->project_path = (_tmp3 = (_tmp2 = vala_project_generator_option_project_path, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), self->priv->project_path = (g_free (self->priv->project_path), NULL), _tmp3);
		}
		if (vala_project_generator_option_project_name == NULL) {
			char* _tmp4;
			_tmp4 = NULL;
			self->priv->project_name = (_tmp4 = g_path_get_basename (self->priv->project_path), self->priv->project_name = (g_free (self->priv->project_name), NULL), _tmp4);
		} else {
			char* _tmp6;
			const char* _tmp5;
			_tmp6 = NULL;
			_tmp5 = NULL;
			self->priv->project_name = (_tmp6 = (_tmp5 = vala_project_generator_option_project_name, (_tmp5 == NULL) ? NULL : g_strdup (_tmp5)), self->priv->project_name = (g_free (self->priv->project_name), NULL), _tmp6);
		}
		self->priv->project_type = (ValaProjectType) gtk_combo_box_get_active (self->priv->project_type_combobox);
		self->priv->project_license = (ValaProjectLicense) gtk_combo_box_get_active (self->priv->license_combobox);
		_tmp8 = NULL;
		_tmp7 = NULL;
		self->priv->real_name = (_tmp8 = (_tmp7 = gtk_entry_get_text (self->priv->name_entry), (_tmp7 == NULL) ? NULL : g_strdup (_tmp7)), self->priv->real_name = (g_free (self->priv->real_name), NULL), _tmp8);
		_tmp10 = NULL;
		_tmp9 = NULL;
		self->priv->email_address = (_tmp10 = (_tmp9 = gtk_entry_get_text (self->priv->email_entry), (_tmp9 == NULL) ? NULL : g_strdup (_tmp9)), self->priv->email_address = (g_free (self->priv->email_address), NULL), _tmp10);
	}
	return response;
}


static void vala_project_generator_create_project (ValaProjectGenerator* self) {
	GError * inner_error;
	GString* project_name_str;
	GString* make_name_str;
	GString* namespace_name_str;
	char* _tmp7;
	const char* _tmp6;
	char* _tmp11;
	char* _tmp10;
	char* _tmp9;
	char* _tmp8;
	char* _tmp13;
	const char* _tmp12;
	char* _tmp14;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	/* only use [a-zA-Z0-9-]* as projectname*/
	project_name_str = g_string_new ("");
	make_name_str = g_string_new ("");
	namespace_name_str = g_string_new ("");
	{
		gint i;
		i = 0;
		for (; i < g_utf8_strlen (self->priv->project_name, -1); i++) {
			gunichar c;
			gboolean _tmp0;
			gboolean _tmp1;
			gboolean _tmp2;
			c = g_utf8_get_char (g_utf8_offset_to_pointer (self->priv->project_name, i));
			_tmp0 = FALSE;
			_tmp1 = FALSE;
			_tmp2 = FALSE;
			if (c >= 'a') {
				_tmp2 = c <= 'z';
			} else {
				_tmp2 = FALSE;
			}
			if ((_tmp2)) {
				_tmp1 = TRUE;
			} else {
				gboolean _tmp3;
				_tmp3 = FALSE;
				if (c >= 'A') {
					_tmp3 = c <= 'Z';
				} else {
					_tmp3 = FALSE;
				}
				_tmp1 = (_tmp3);
			}
			if (_tmp1) {
				_tmp0 = TRUE;
			} else {
				gboolean _tmp4;
				_tmp4 = FALSE;
				if (c >= '0') {
					_tmp4 = c <= '9';
				} else {
					_tmp4 = FALSE;
				}
				_tmp0 = (_tmp4);
			}
			if (_tmp0) {
				g_string_append_unichar (project_name_str, c);
				g_string_append_unichar (make_name_str, c);
				g_string_append_unichar (namespace_name_str, c);
			} else {
				gboolean _tmp5;
				_tmp5 = FALSE;
				if (c == '-') {
					_tmp5 = TRUE;
				} else {
					_tmp5 = c == ' ';
				}
				if (_tmp5) {
					g_string_append_unichar (project_name_str, (gunichar) '-');
					g_string_append_unichar (make_name_str, (gunichar) '_');
				}
			}
		}
	}
	_tmp7 = NULL;
	_tmp6 = NULL;
	self->priv->project_name = (_tmp7 = (_tmp6 = project_name_str->str, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), self->priv->project_name = (g_free (self->priv->project_name), NULL), _tmp7);
	_tmp11 = NULL;
	_tmp10 = NULL;
	_tmp9 = NULL;
	_tmp8 = NULL;
	self->priv->namespace_name = (_tmp11 = g_strconcat (_tmp9 = g_utf8_strup (_tmp8 = string_substring (namespace_name_str->str, (glong) 0, (glong) 1), -1), _tmp10 = string_substring (namespace_name_str->str, (glong) 1, g_utf8_strlen (namespace_name_str->str, -1) - 1), NULL), self->priv->namespace_name = (g_free (self->priv->namespace_name), NULL), _tmp11);
	_tmp10 = (g_free (_tmp10), NULL);
	_tmp9 = (g_free (_tmp9), NULL);
	_tmp8 = (g_free (_tmp8), NULL);
	_tmp13 = NULL;
	_tmp12 = NULL;
	self->priv->make_name = (_tmp13 = (_tmp12 = make_name_str->str, (_tmp12 == NULL) ? NULL : g_strdup (_tmp12)), self->priv->make_name = (g_free (self->priv->make_name), NULL), _tmp13);
	_tmp14 = NULL;
	self->priv->upper_case_make_name = (_tmp14 = g_utf8_strup (self->priv->make_name, -1), self->priv->upper_case_make_name = (g_free (self->priv->upper_case_make_name), NULL), _tmp14);
	{
		char* _tmp15;
		char* _tmp16;
		char* _tmp17;
		char* _tmp18;
		char* _tmp19;
		char* _tmp20;
		char* s;
		char* automake_path;
		char* license_filename;
		gboolean _tmp33;
		vala_project_generator_write_autogen_sh (self, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		vala_project_generator_write_configure_ac (self, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		_tmp15 = NULL;
		g_mkdir (_tmp15 = g_strconcat (self->priv->project_path, "/src", NULL), 0777);
		_tmp15 = (g_free (_tmp15), NULL);
		_tmp16 = NULL;
		g_mkdir (_tmp16 = g_strconcat (self->priv->project_path, "/po", NULL), 0777);
		_tmp16 = (g_free (_tmp16), NULL);
		vala_project_generator_write_toplevel_makefile_am (self, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		vala_project_generator_write_src_makefile_am (self, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		if (self->priv->project_type == VALA_PROJECT_TYPE_CONSOLE_APPLICATION) {
			vala_project_generator_write_main_vala (self, &inner_error);
			if (inner_error != NULL) {
				if (inner_error->domain == G_FILE_ERROR) {
					goto __catch0_g_file_error;
				}
				goto __finally0;
			}
		} else {
			if (self->priv->project_type == VALA_PROJECT_TYPE_GTK_APPLICATION) {
				vala_project_generator_write_mainwindow_vala (self, &inner_error);
				if (inner_error != NULL) {
					if (inner_error->domain == G_FILE_ERROR) {
						goto __catch0_g_file_error;
					}
					goto __finally0;
				}
			}
		}
		vala_project_generator_write_authors (self, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		vala_project_generator_write_maintainers (self, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		vala_project_generator_write_linguas (self, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		vala_project_generator_write_potfiles (self, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		_tmp17 = NULL;
		g_file_set_contents (_tmp17 = g_strconcat (self->priv->project_path, "/NEWS", NULL), "", (glong) (-1), &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		_tmp17 = (g_free (_tmp17), NULL);
		_tmp18 = NULL;
		g_file_set_contents (_tmp18 = g_strconcat (self->priv->project_path, "/README", NULL), "", (glong) (-1), &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		_tmp18 = (g_free (_tmp18), NULL);
		_tmp19 = NULL;
		g_file_set_contents (_tmp19 = g_strconcat (self->priv->project_path, "/ChangeLog", NULL), "", (glong) (-1), &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		_tmp19 = (g_free (_tmp19), NULL);
		_tmp20 = NULL;
		g_file_set_contents (_tmp20 = g_strconcat (self->priv->project_path, "/po/ChangeLog", NULL), "", (glong) (-1), &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		_tmp20 = (g_free (_tmp20), NULL);
		s = NULL;
		automake_path = vala_project_generator_get_automake_path (self);
		if (automake_path != NULL) {
			char* install_filename;
			install_filename = g_strconcat (automake_path, "/INSTALL", NULL);
			if (g_file_test (install_filename, G_FILE_TEST_EXISTS)) {
				char* _tmp23;
				gboolean _tmp22;
				char* _tmp21;
				char* _tmp24;
				_tmp23 = NULL;
				_tmp21 = NULL;
				_tmp22 = g_file_get_contents (install_filename, &_tmp21, NULL, &inner_error);
				s = (_tmp23 = _tmp21, s = (g_free (s), NULL), _tmp23);
				_tmp22;
				if (inner_error != NULL) {
					install_filename = (g_free (install_filename), NULL);
					s = (g_free (s), NULL);
					automake_path = (g_free (automake_path), NULL);
					if (inner_error->domain == G_FILE_ERROR) {
						goto __catch0_g_file_error;
					}
					goto __finally0;
				}
				_tmp24 = NULL;
				g_file_set_contents (_tmp24 = g_strconcat (self->priv->project_path, "/INSTALL", NULL), s, (glong) (-1), &inner_error);
				if (inner_error != NULL) {
					install_filename = (g_free (install_filename), NULL);
					s = (g_free (s), NULL);
					automake_path = (g_free (automake_path), NULL);
					if (inner_error->domain == G_FILE_ERROR) {
						goto __catch0_g_file_error;
					}
					goto __finally0;
				}
				_tmp24 = (g_free (_tmp24), NULL);
			}
			install_filename = (g_free (install_filename), NULL);
		}
		license_filename = NULL;
		if (self->priv->project_license == VALA_PROJECT_LICENSE_GPL2) {
			char* _tmp25;
			_tmp25 = NULL;
			license_filename = (_tmp25 = g_strdup (PACKAGE_DATADIR "/licenses/gpl-2.0.txt"), license_filename = (g_free (license_filename), NULL), _tmp25);
			if (!g_file_test (license_filename, G_FILE_TEST_EXISTS)) {
				char* _tmp26;
				_tmp26 = NULL;
				license_filename = (_tmp26 = g_strdup ("/usr/share/common-licenses/GPL-2"), license_filename = (g_free (license_filename), NULL), _tmp26);
			}
		} else {
			if (self->priv->project_license == VALA_PROJECT_LICENSE_LGPL2) {
				char* _tmp27;
				_tmp27 = NULL;
				license_filename = (_tmp27 = g_strdup (PACKAGE_DATADIR "/licenses/lgpl-2.1.txt"), license_filename = (g_free (license_filename), NULL), _tmp27);
				if (!g_file_test (license_filename, G_FILE_TEST_EXISTS)) {
					char* _tmp28;
					_tmp28 = NULL;
					license_filename = (_tmp28 = g_strdup ("/usr/share/common-licenses/LGPL-2.1"), license_filename = (g_free (license_filename), NULL), _tmp28);
				}
			} else {
				if (self->priv->project_license == VALA_PROJECT_LICENSE_GPL3) {
					char* _tmp29;
					_tmp29 = NULL;
					license_filename = (_tmp29 = g_strdup (PACKAGE_DATADIR "/licenses/gpl-3.0.txt"), license_filename = (g_free (license_filename), NULL), _tmp29);
					if (!g_file_test (license_filename, G_FILE_TEST_EXISTS)) {
						char* _tmp30;
						_tmp30 = NULL;
						license_filename = (_tmp30 = g_strdup ("/usr/share/common-licenses/GPL-3"), license_filename = (g_free (license_filename), NULL), _tmp30);
					}
				} else {
					if (self->priv->project_license == VALA_PROJECT_LICENSE_LGPL3) {
						char* _tmp31;
						_tmp31 = NULL;
						license_filename = (_tmp31 = g_strdup (PACKAGE_DATADIR "/licenses/lgpl-3.0.txt"), license_filename = (g_free (license_filename), NULL), _tmp31);
						if (!g_file_test (license_filename, G_FILE_TEST_EXISTS)) {
							char* _tmp32;
							_tmp32 = NULL;
							license_filename = (_tmp32 = g_strdup ("/usr/share/common-licenses/LGPL-3"), license_filename = (g_free (license_filename), NULL), _tmp32);
						}
					}
				}
			}
		}
		_tmp33 = FALSE;
		if (license_filename != NULL) {
			_tmp33 = g_file_test (license_filename, G_FILE_TEST_EXISTS);
		} else {
			_tmp33 = FALSE;
		}
		if (_tmp33) {
			char* _tmp36;
			gboolean _tmp35;
			char* _tmp34;
			char* _tmp37;
			_tmp36 = NULL;
			_tmp34 = NULL;
			_tmp35 = g_file_get_contents (license_filename, &_tmp34, NULL, &inner_error);
			s = (_tmp36 = _tmp34, s = (g_free (s), NULL), _tmp36);
			_tmp35;
			if (inner_error != NULL) {
				s = (g_free (s), NULL);
				automake_path = (g_free (automake_path), NULL);
				license_filename = (g_free (license_filename), NULL);
				if (inner_error->domain == G_FILE_ERROR) {
					goto __catch0_g_file_error;
				}
				goto __finally0;
			}
			_tmp37 = NULL;
			g_file_set_contents (_tmp37 = g_strconcat (self->priv->project_path, "/COPYING", NULL), s, (glong) (-1), &inner_error);
			if (inner_error != NULL) {
				s = (g_free (s), NULL);
				automake_path = (g_free (automake_path), NULL);
				license_filename = (g_free (license_filename), NULL);
				if (inner_error->domain == G_FILE_ERROR) {
					goto __catch0_g_file_error;
				}
				goto __finally0;
			}
			_tmp37 = (g_free (_tmp37), NULL);
		}
		s = (g_free (s), NULL);
		automake_path = (g_free (automake_path), NULL);
		license_filename = (g_free (license_filename), NULL);
	}
	goto __finally0;
	__catch0_g_file_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_critical ("valaprojectgenerator.vala:318: Error while creating project: %s", e->message);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally0:
	if (inner_error != NULL) {
		(project_name_str == NULL) ? NULL : (project_name_str = (g_string_free (project_name_str, TRUE), NULL));
		(make_name_str == NULL) ? NULL : (make_name_str = (g_string_free (make_name_str, TRUE), NULL));
		(namespace_name_str == NULL) ? NULL : (namespace_name_str = (g_string_free (namespace_name_str, TRUE), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	(project_name_str == NULL) ? NULL : (project_name_str = (g_string_free (project_name_str, TRUE), NULL));
	(make_name_str == NULL) ? NULL : (make_name_str = (g_string_free (make_name_str, TRUE), NULL));
	(namespace_name_str == NULL) ? NULL : (namespace_name_str = (g_string_free (namespace_name_str, TRUE), NULL));
}


static void vala_project_generator_write_autogen_sh (ValaProjectGenerator* self, GError** error) {
	GError * inner_error;
	GString* s;
	char* _tmp0;
	char* _tmp1;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	s = g_string_new ("");
	g_string_append (s, "#!/bin/sh\n");
	g_string_append (s, "# Run this to generate all the initial makefiles, etc.\n\n");
	g_string_append (s, "srcdir=`dirname $0`\n");
	g_string_append (s, "test -z \"$srcdir\" && srcdir=.\n\n");
	g_string_append_printf (s, "PKG_NAME=\"%s\"\n\n", self->priv->project_name);
	g_string_append (s, ". gnome-autogen.sh\n");
	_tmp0 = NULL;
	g_file_set_contents (_tmp0 = g_strconcat (self->priv->project_path, "/autogen.sh", NULL), s->str, (glong) (-1), &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
		return;
	}
	_tmp0 = (g_free (_tmp0), NULL);
	_tmp1 = NULL;
	g_chmod (_tmp1 = g_strconcat (self->priv->project_path, "/autogen.sh", NULL), 0755);
	_tmp1 = (g_free (_tmp1), NULL);
	(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
}


static void vala_project_generator_write_configure_ac (ValaProjectGenerator* self, GError** error) {
	GError * inner_error;
	gboolean use_gtk;
	GString* s;
	char* _tmp0;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	use_gtk = (self->priv->project_type == VALA_PROJECT_TYPE_GTK_APPLICATION);
	s = g_string_new ("");
	g_string_append_printf (s, "AC_INIT([%s], [0.1.0], [%s], [%s])\n", self->priv->project_name, self->priv->email_address, self->priv->project_name);
	g_string_append (s, "AC_CONFIG_SRCDIR([Makefile.am])\n");
	g_string_append (s, "AC_CONFIG_HEADERS(config.h)\n");
	g_string_append (s, "AM_INIT_AUTOMAKE([dist-bzip2])\n");
	g_string_append (s, "AM_MAINTAINER_MODE\n\n");
	g_string_append (s, "AC_PROG_CC\n");
	g_string_append (s, "AM_PROG_CC_C_O\n");
	g_string_append (s, "AC_DISABLE_STATIC\n");
	g_string_append (s, "AC_PROG_LIBTOOL\n\n");
	g_string_append (s, "AC_PATH_PROG(VALAC, valac, valac)\n");
	g_string_append (s, "AC_SUBST(VALAC)\n\n");
	g_string_append (s, "AH_TEMPLATE([GETTEXT_PACKAGE], [Package name for gettext])\n");
	g_string_append_printf (s, "GETTEXT_PACKAGE=%s\n", self->priv->project_name);
	g_string_append (s, "AC_DEFINE_UNQUOTED(GETTEXT_PACKAGE, \"$GETTEXT_PACKAGE\")\n");
	g_string_append (s, "AC_SUBST(GETTEXT_PACKAGE)\n");
	g_string_append (s, "AM_GLIB_GNU_GETTEXT\n");
	g_string_append (s, "IT_PROG_INTLTOOL([0.35.0])\n\n");
	g_string_append (s, "AC_SUBST(CFLAGS)\n");
	g_string_append (s, "AC_SUBST(CPPFLAGS)\n");
	g_string_append (s, "AC_SUBST(LDFLAGS)\n\n");
	g_string_append (s, "GLIB_REQUIRED=2.12.0\n");
	if (use_gtk) {
		g_string_append (s, "GTK_REQUIRED=2.10.0\n");
	}
	g_string_append (s, "\n");
	g_string_append_printf (s, "PKG_CHECK_MODULES(%s, glib-2.0 >= $GLIB_REQUIRED gobject-2.0 >= $GLIB_REQUIRED", self->priv->upper_case_make_name);
	if (use_gtk) {
		g_string_append (s, " gtk+-2.0 >= $GTK_REQUIRED");
	}
	g_string_append (s, ")\n");
	g_string_append_printf (s, "AC_SUBST(%s_CFLAGS)\n", self->priv->upper_case_make_name);
	g_string_append_printf (s, "AC_SUBST(%s_LIBS)\n\n", self->priv->upper_case_make_name);
	g_string_append (s, "AC_CONFIG_FILES([Makefile\n");
	g_string_append (s, "\tsrc/Makefile\n");
	g_string_append (s, "\tpo/Makefile.in])\n\n");
	g_string_append (s, "AC_OUTPUT\n");
	_tmp0 = NULL;
	g_file_set_contents (_tmp0 = g_strconcat (self->priv->project_path, "/configure.ac", NULL), s->str, (glong) (-1), &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
		return;
	}
	_tmp0 = (g_free (_tmp0), NULL);
	(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
}


static void vala_project_generator_write_toplevel_makefile_am (ValaProjectGenerator* self, GError** error) {
	GError * inner_error;
	GString* s;
	char* _tmp0;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	s = g_string_new ("");
	g_string_append (s, "NULL = \n\n");
	g_string_append (s, "#Build in these directories:\n\n");
	g_string_append (s, "SUBDIRS = \\\n");
	g_string_append (s, "\tsrc \\\n");
	g_string_append (s, "\tpo \\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append_printf (s, "%sdocdir = ${prefix}/doc/%s\n", self->priv->project_name, self->priv->project_name);
	g_string_append_printf (s, "%sdoc_DATA = \\\n", self->priv->project_name);
	g_string_append (s, "\tChangeLog \\\n");
	g_string_append (s, "\tREADME \\\n");
	g_string_append (s, "\tCOPYING \\\n");
	g_string_append (s, "\tAUTHORS \\\n");
	g_string_append (s, "\tINSTALL \\\n");
	g_string_append (s, "\tNEWS\\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append (s, "EXTRA_DIST = \\\n");
	g_string_append_printf (s, "\t$(%sdoc_DATA) \\\n", self->priv->project_name);
	g_string_append (s, "\tintltool-extract.in \\\n");
	g_string_append (s, "\tintltool-merge.in \\\n");
	g_string_append (s, "\tintltool-update.in\\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append (s, "DISTCLEANFILES = \\\n");
	g_string_append (s, "\tintltool-extract \\\n");
	g_string_append (s, "\tintltool-merge \\\n");
	g_string_append (s, "\tintltool-update\n");
	g_string_append (s, "\tpo/.intltool-merge-cache \\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	_tmp0 = NULL;
	g_file_set_contents (_tmp0 = g_strconcat (self->priv->project_path, "/Makefile.am", NULL), s->str, (glong) (-1), &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
		return;
	}
	_tmp0 = (g_free (_tmp0), NULL);
	(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
}


static void vala_project_generator_write_src_makefile_am (ValaProjectGenerator* self, GError** error) {
	GError * inner_error;
	gboolean use_gtk;
	GString* s;
	char* _tmp0;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	use_gtk = (self->priv->project_type == VALA_PROJECT_TYPE_GTK_APPLICATION);
	s = g_string_new ("");
	g_string_append (s, "NULL = \n\n");
	g_string_append (s, "AM_CPPFLAGS = \\\n");
	g_string_append_printf (s, "\t$(%s_CFLAGS) \\\n", self->priv->upper_case_make_name);
	g_string_append (s, "\t-include $(CONFIG_HEADER) \\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append_printf (s, "BUILT_SOURCES = %s.vala.stamp\n\n", self->priv->project_name);
	g_string_append_printf (s, "bin_PROGRAMS = %s\n\n", self->priv->project_name);
	g_string_append_printf (s, "%s_VALASOURCES = \\\n", self->priv->make_name);
	if (use_gtk) {
		g_string_append (s, "\tmainwindow.vala \\\n");
	} else {
		g_string_append (s, "\tmain.vala \\\n");
	}
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append_printf (s, "%s_SOURCES = \\\n", self->priv->make_name);
	g_string_append_printf (s, "\t$(%s_VALASOURCES:.vala=.c) \\\n", self->priv->make_name);
	g_string_append_printf (s, "\t$(%s_VALASOURCES:.vala=.h) \\\n", self->priv->make_name);
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append_printf (s, "%s.vala.stamp: $(%s_VALASOURCES)\n", self->priv->project_name, self->priv->make_name);
	g_string_append (s, "\t$(VALAC) -C ");
	if (use_gtk) {
		g_string_append (s, "--pkg gtk+-2.0 ");
	}
	g_string_append (s, "--basedir $(top_srcdir)/src $^\n");
	g_string_append (s, "\ttouch $@\n\n");
	g_string_append_printf (s, "%s_LDADD = \\\n", self->priv->make_name);
	g_string_append_printf (s, "\t$(%s_LIBS) \\\n", self->priv->upper_case_make_name);
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append (s, "EXTRA_DIST = \\\n");
	g_string_append_printf (s, "\t$(%s_VALASOURCES) \\\n", self->priv->make_name);
	g_string_append_printf (s, "\t%s.vala.stamp \\\n", self->priv->project_name);
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append (s, "DISTCLEANFILES = \\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	_tmp0 = NULL;
	g_file_set_contents (_tmp0 = g_strconcat (self->priv->project_path, "/src/Makefile.am", NULL), s->str, (glong) (-1), &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
		return;
	}
	_tmp0 = (g_free (_tmp0), NULL);
	(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
}


static char* vala_project_generator_generate_source_file_header (ValaProjectGenerator* self, const char* filename) {
	GString* s;
	GTimeVal _tmp0 = {0};
	GTimeVal tv;
	GDate _tmp1 = {0};
	GDate d;
	char* license_name;
	char* license_version;
	char* program_type;
	const char* _tmp14;
	char* _tmp15;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (filename != NULL, NULL);
	s = g_string_new ("");
	tv = (g_get_current_time (&_tmp0), _tmp0);
	d = (memset (&_tmp1, 0, sizeof (GDate)), _tmp1);
	g_date_set_time_val (&d, &tv);
	g_string_append_printf (s, "/* %s\n", filename);
	g_string_append (s, " *\n");
	g_string_append_printf (s, " * Copyright (C) %d  %s\n", (gint) g_date_get_year (&d), self->priv->real_name);
	g_string_append (s, " *\n");
	license_name = g_strdup ("");
	license_version = NULL;
	program_type = NULL;
	switch (self->priv->project_license) {
		case VALA_PROJECT_LICENSE_GPL2:
		{
			char* _tmp2;
			char* _tmp3;
			char* _tmp4;
			_tmp2 = NULL;
			license_name = (_tmp2 = g_strdup ("GNU General Public License"), license_name = (g_free (license_name), NULL), _tmp2);
			_tmp3 = NULL;
			license_version = (_tmp3 = g_strdup ("2"), license_version = (g_free (license_version), NULL), _tmp3);
			_tmp4 = NULL;
			program_type = (_tmp4 = g_strdup ("program"), program_type = (g_free (program_type), NULL), _tmp4);
			break;
		}
		case VALA_PROJECT_LICENSE_GPL3:
		{
			char* _tmp5;
			char* _tmp6;
			char* _tmp7;
			_tmp5 = NULL;
			license_name = (_tmp5 = g_strdup ("GNU General Public License"), license_name = (g_free (license_name), NULL), _tmp5);
			_tmp6 = NULL;
			license_version = (_tmp6 = g_strdup ("3"), license_version = (g_free (license_version), NULL), _tmp6);
			_tmp7 = NULL;
			program_type = (_tmp7 = g_strdup ("program"), program_type = (g_free (program_type), NULL), _tmp7);
			break;
		}
		case VALA_PROJECT_LICENSE_LGPL2:
		{
			char* _tmp8;
			char* _tmp9;
			char* _tmp10;
			_tmp8 = NULL;
			license_name = (_tmp8 = g_strdup ("GNU Lesser General Public License"), license_name = (g_free (license_name), NULL), _tmp8);
			_tmp9 = NULL;
			license_version = (_tmp9 = g_strdup ("2.1"), license_version = (g_free (license_version), NULL), _tmp9);
			_tmp10 = NULL;
			program_type = (_tmp10 = g_strdup ("library"), program_type = (g_free (program_type), NULL), _tmp10);
			break;
		}
		case VALA_PROJECT_LICENSE_LGPL3:
		{
			char* _tmp11;
			char* _tmp12;
			char* _tmp13;
			_tmp11 = NULL;
			license_name = (_tmp11 = g_strdup ("GNU Lesser General Public License"), license_name = (g_free (license_name), NULL), _tmp11);
			_tmp12 = NULL;
			license_version = (_tmp12 = g_strdup ("3"), license_version = (g_free (license_version), NULL), _tmp12);
			_tmp13 = NULL;
			program_type = (_tmp13 = g_strdup ("library"), program_type = (g_free (program_type), NULL), _tmp13);
			break;
		}
	}
	g_string_append_printf (s, " * This %s is free software: you can redistribute it and/or modify\n", program_type);
	g_string_append_printf (s, " * it under the terms of the %s as published by\n", license_name);
	g_string_append_printf (s, " * the Free Software Foundation, either version %s of the License, or\n", license_version);
	g_string_append (s, " * (at your option) any later version.\n");
	g_string_append (s, " *\n");
	g_string_append_printf (s, " * This %s is distributed in the hope that it will be useful,\n", program_type);
	g_string_append (s, " * but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
	g_string_append (s, " * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
	g_string_append_printf (s, " * %s for more details.\n", license_name);
	g_string_append (s, " *\n");
	g_string_append_printf (s, " * You should have received a copy of the %s\n", license_name);
	g_string_append_printf (s, " * along with this %s.  If not, see <http://www.gnu.org/licenses/>.\n", program_type);
	g_string_append (s, " *\n");
	g_string_append (s, " * Author:\n");
	g_string_append_printf (s, " * \t%s <%s>\n", self->priv->real_name, self->priv->email_address);
	g_string_append (s, " */\n\n");
	_tmp14 = NULL;
	_tmp15 = NULL;
	return (_tmp15 = (_tmp14 = s->str, (_tmp14 == NULL) ? NULL : g_strdup (_tmp14)), (s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL)), license_name = (g_free (license_name), NULL), license_version = (g_free (license_version), NULL), program_type = (g_free (program_type), NULL), _tmp15);
}


static void vala_project_generator_write_main_vala (ValaProjectGenerator* self, GError** error) {
	GError * inner_error;
	GString* s;
	char* _tmp0;
	char* _tmp1;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	s = g_string_new ("");
	_tmp0 = NULL;
	g_string_append (s, _tmp0 = vala_project_generator_generate_source_file_header (self, "main.vala"));
	_tmp0 = (g_free (_tmp0), NULL);
	g_string_append (s, "using GLib;\n");
	g_string_append (s, "\n");
	g_string_append_printf (s, "public class %s.Main : Object {\n", self->priv->namespace_name);
	g_string_append (s, "\tpublic Main () {\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tpublic void run () {\n");
	g_string_append (s, "\t\tstdout.printf (\"Hello, world!\\n\");\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tstatic int main (string[] args) {\n");
	g_string_append (s, "\t\tvar main = new Main ();\n");
	g_string_append (s, "\t\tmain.run ();\n");
	g_string_append (s, "\t\treturn 0;\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "}\n");
	_tmp1 = NULL;
	g_file_set_contents (_tmp1 = g_strconcat (self->priv->project_path, "/src/main.vala", NULL), s->str, (glong) (-1), &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
		return;
	}
	_tmp1 = (g_free (_tmp1), NULL);
	(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
}


static void vala_project_generator_write_mainwindow_vala (ValaProjectGenerator* self, GError** error) {
	GError * inner_error;
	GString* s;
	char* _tmp0;
	char* _tmp1;
	char* _tmp2;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	s = g_string_new ("");
	_tmp0 = NULL;
	g_string_append (s, _tmp0 = vala_project_generator_generate_source_file_header (self, "mainwindow.vala"));
	_tmp0 = (g_free (_tmp0), NULL);
	g_string_append (s, "using GLib;\n");
	g_string_append (s, "using Gtk;\n");
	g_string_append (s, "\n");
	g_string_append_printf (s, "public class %s.MainWindow : Window {\n", self->priv->namespace_name);
	g_string_append (s, "\tprivate TextBuffer text_buffer;\n");
	g_string_append (s, "\tprivate string filename;\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tpublic MainWindow () {\n");
	_tmp1 = NULL;
	g_string_append_printf (s, "\t\ttitle = \"%s\";\n", _tmp1 = g_strescape (self->priv->project_name, ""));
	_tmp1 = (g_free (_tmp1), NULL);
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tconstruct {\n");
	g_string_append (s, "\t\tset_default_size (600, 400);\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tdestroy += Gtk.main_quit;\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar vbox = new VBox (false, 0);\n");
	g_string_append (s, "\t\tadd (vbox);\n");
	g_string_append (s, "\t\tvbox.show ();\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar toolbar = new Toolbar ();\n");
	g_string_append (s, "\t\tvbox.pack_start (toolbar, false, false, 0);\n");
	g_string_append (s, "\t\ttoolbar.show ();\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar button = new ToolButton.from_stock (Gtk.STOCK_SAVE);\n");
	g_string_append (s, "\t\ttoolbar.insert (button, -1);\n");
	g_string_append (s, "\t\tbutton.is_important = true;\n");
	g_string_append (s, "\t\tbutton.clicked += on_save_clicked;\n");
	g_string_append (s, "\t\tbutton.show ();\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar scrolled_window = new ScrolledWindow (null, null);\n");
	g_string_append (s, "\t\tvbox.pack_start (scrolled_window, true, true, 0);\n");
	g_string_append (s, "\t\tscrolled_window.hscrollbar_policy = PolicyType.AUTOMATIC;\n");
	g_string_append (s, "\t\tscrolled_window.vscrollbar_policy = PolicyType.AUTOMATIC;\n");
	g_string_append (s, "\t\tscrolled_window.show ();\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\ttext_buffer = new TextBuffer (null);\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar text_view = new TextView.with_buffer (text_buffer);\n");
	g_string_append (s, "\t\tscrolled_window.add (text_view);\n");
	g_string_append (s, "\t\ttext_view.show ();\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tpublic void run () {\n");
	g_string_append (s, "\t\tshow ();\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tGtk.main ();\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tprivate void on_save_clicked (ToolButton button) {\n");
	g_string_append (s, "\t\tif (filename == null) {\n");
	g_string_append (s, "\t\t\tvar dialog = new FileChooserDialog (_(\"Save File\"), this, FileChooserAction.SAVE,\n");
	g_string_append (s, "\t\t\t\tGtk.STOCK_CANCEL, ResponseType.CANCEL,\n");
	g_string_append (s, "\t\t\t\tGtk.STOCK_SAVE, ResponseType.ACCEPT);\n");
	g_string_append (s, "\t\t\tdialog.set_do_overwrite_confirmation (true);\n");
	g_string_append (s, "\t\t\tif (dialog.run () == ResponseType.ACCEPT) {\n");
	g_string_append (s, "\t\t\t\tfilename = dialog.get_filename ();\n");
	g_string_append (s, "\t\t\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\t\tdialog.destroy ();\n");
	g_string_append (s, "\t\t\tif (filename == null) {\n");
	g_string_append (s, "\t\t\t\treturn;\n");
	g_string_append (s, "\t\t\t}\n");
	g_string_append (s, "\t\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\ttry {\n");
	g_string_append (s, "\t\t\tTextIter start_iter, end_iter;\n");
	g_string_append (s, "\t\t\ttext_buffer.get_bounds (out start_iter, out end_iter);\n");
	g_string_append (s, "\t\t\tstring text = text_buffer.get_text (start_iter, end_iter, true);\n");
	g_string_append (s, "\t\t\tFileUtils.set_contents (filename, text, -1);\n");
	g_string_append (s, "\t\t} catch (FileError e) {\n");
	g_string_append (s, "\t\t\tcritical (\"Error while trying to save file: %s\", e.message);\n");
	g_string_append (s, "\t\t\tfilename = null;\n");
	g_string_append (s, "\t\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tstatic int main (string[] args) {\n");
	g_string_append (s, "\t\tGtk.init (ref args);\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar window = new MainWindow ();\n");
	g_string_append (s, "\t\twindow.run ();\n");
	g_string_append (s, "\t\treturn 0;\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "}\n");
	_tmp2 = NULL;
	g_file_set_contents (_tmp2 = g_strconcat (self->priv->project_path, "/src/mainwindow.vala", NULL), s->str, (glong) (-1), &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
		return;
	}
	_tmp2 = (g_free (_tmp2), NULL);
	(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
}


static void vala_project_generator_write_potfiles (ValaProjectGenerator* self, GError** error) {
	GError * inner_error;
	gboolean use_gtk;
	GString* s;
	char* _tmp0;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	use_gtk = (self->priv->project_type == VALA_PROJECT_TYPE_GTK_APPLICATION);
	s = g_string_new ("");
	g_string_append (s, "[encoding: UTF-8]\n");
	g_string_append (s, "# List of source files which contain translatable strings.\n");
	if (use_gtk) {
		g_string_append (s, "src/mainwindow.vala\n");
	} else {
		g_string_append (s, "src/main.vala\n");
	}
	_tmp0 = NULL;
	g_file_set_contents (_tmp0 = g_strconcat (self->priv->project_path, "/po/POTFILES.in", NULL), s->str, (glong) (-1), &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
		return;
	}
	_tmp0 = (g_free (_tmp0), NULL);
	if (use_gtk) {
		char* _tmp1;
		_tmp1 = NULL;
		g_file_set_contents (_tmp1 = g_strconcat (self->priv->project_path, "/po/POTFILES.skip", NULL), "src/mainwindow.c\n", (glong) (-1), &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
			return;
		}
		_tmp1 = (g_free (_tmp1), NULL);
	} else {
		char* _tmp2;
		_tmp2 = NULL;
		g_file_set_contents (_tmp2 = g_strconcat (self->priv->project_path, "/po/POTFILES.skip", NULL), "src/main.c\n", (glong) (-1), &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
			return;
		}
		_tmp2 = (g_free (_tmp2), NULL);
	}
	(s == NULL) ? NULL : (s = (g_string_free (s, TRUE), NULL));
}


static void vala_project_generator_write_linguas (ValaProjectGenerator* self, GError** error) {
	GError * inner_error;
	char* s;
	char* _tmp0;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	s = g_strdup ("# please keep this list sorted alphabetically\n#\n");
	_tmp0 = NULL;
	g_file_set_contents (_tmp0 = g_strconcat (self->priv->project_path, "/po/LINGUAS", NULL), s, (glong) (-1), &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		s = (g_free (s), NULL);
		return;
	}
	_tmp0 = (g_free (_tmp0), NULL);
	s = (g_free (s), NULL);
}


static void vala_project_generator_write_authors (ValaProjectGenerator* self, GError** error) {
	GError * inner_error;
	char* s;
	char* _tmp0;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	s = g_strdup_printf ("%s <%s>\n", self->priv->real_name, self->priv->email_address);
	_tmp0 = NULL;
	g_file_set_contents (_tmp0 = g_strconcat (self->priv->project_path, "/AUTHORS", NULL), s, (glong) (-1), &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		s = (g_free (s), NULL);
		return;
	}
	_tmp0 = (g_free (_tmp0), NULL);
	s = (g_free (s), NULL);
}


static void vala_project_generator_write_maintainers (ValaProjectGenerator* self, GError** error) {
	GError * inner_error;
	char* s;
	char* _tmp0;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	s = g_strdup_printf ("%s\nE-mail: %s\n", self->priv->real_name, self->priv->email_address);
	_tmp0 = NULL;
	g_file_set_contents (_tmp0 = g_strconcat (self->priv->project_path, "/MAINTAINERS", NULL), s, (glong) (-1), &inner_error);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		s = (g_free (s), NULL);
		return;
	}
	_tmp0 = (g_free (_tmp0), NULL);
	s = (g_free (s), NULL);
}


static char* vala_project_generator_get_automake_path (ValaProjectGenerator* self) {
	char** _tmp1;
	gint automake_paths_size;
	gint automake_paths_length1;
	char** _tmp0;
	char** automake_paths;
	char* _tmp4;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	automake_paths = (_tmp1 = (_tmp0 = g_new0 (char*, 3 + 1), _tmp0[0] = g_strdup ("/usr/share/automake"), _tmp0[1] = g_strdup ("/usr/share/automake-1.10"), _tmp0[2] = g_strdup ("/usr/share/automake-1.9"), _tmp0), automake_paths_length1 = 3, automake_paths_size = automake_paths_length1, _tmp1);
	{
		char** automake_path_collection;
		int automake_path_collection_length1;
		int automake_path_it;
		automake_path_collection = automake_paths;
		automake_path_collection_length1 = automake_paths_length1;
		for (automake_path_it = 0; automake_path_it < automake_paths_length1; automake_path_it = automake_path_it + 1) {
			const char* _tmp3;
			char* automake_path;
			_tmp3 = NULL;
			automake_path = (_tmp3 = automake_path_collection[automake_path_it], (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
			{
				if (g_file_test (automake_path, G_FILE_TEST_IS_DIR)) {
					char* _tmp2;
					_tmp2 = NULL;
					return (_tmp2 = automake_path, automake_paths = (_vala_array_free (automake_paths, automake_paths_length1, (GDestroyNotify) g_free), NULL), _tmp2);
				}
				automake_path = (g_free (automake_path), NULL);
			}
		}
	}
	_tmp4 = NULL;
	return (_tmp4 = NULL, automake_paths = (_vala_array_free (automake_paths, automake_paths_length1, (GDestroyNotify) g_free), NULL), _tmp4);
}


static gint vala_project_generator_main (char** args, int args_length1) {
	GError * inner_error;
	ValaProjectGenerator* generator;
	gboolean _tmp5;
	gint _tmp7;
	inner_error = NULL;
	gtk_init (&args_length1, &args);
	bindtextdomain (GETTEXT_PACKAGE, NULL);
	{
		GOptionContext* opt_context;
		opt_context = g_option_context_new ("- Vala Project Generator");
		g_option_context_set_help_enabled (opt_context, TRUE);
		g_option_context_add_main_entries (opt_context, VALA_PROJECT_GENERATOR_options, NULL);
		g_option_context_parse (opt_context, &args_length1, &args, &inner_error);
		if (inner_error != NULL) {
			(opt_context == NULL) ? NULL : (opt_context = (g_option_context_free (opt_context), NULL));
			if (inner_error->domain == G_OPTION_ERROR) {
				goto __catch1_g_option_error;
			}
			goto __finally1;
		}
		if (vala_project_generator_option_files != NULL) {
			char* _tmp1;
			const char* _tmp0;
			if (vala_project_generator_option_files[1] != NULL) {
				/*more then a project name*/
				inner_error = g_error_new_literal (G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, _ ("Just a single project name can be specified on the command line"));
				if (inner_error != NULL) {
					(opt_context == NULL) ? NULL : (opt_context = (g_option_context_free (opt_context), NULL));
					if (inner_error->domain == G_OPTION_ERROR) {
						goto __catch1_g_option_error;
					}
					goto __finally1;
				}
			}
			_tmp1 = NULL;
			_tmp0 = NULL;
			vala_project_generator_option_project_name = (_tmp1 = (_tmp0 = vala_project_generator_option_files[0], (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), vala_project_generator_option_project_name = (g_free (vala_project_generator_option_project_name), NULL), _tmp1);
			if (vala_project_generator_option_project_path == NULL) {
				char* _tmp2;
				/*default to current directory in not specified with -p*/
				_tmp2 = NULL;
				vala_project_generator_option_project_path = (_tmp2 = g_get_current_dir (), vala_project_generator_option_project_path = (g_free (vala_project_generator_option_project_path), NULL), _tmp2);
			}
		}
		(opt_context == NULL) ? NULL : (opt_context = (g_option_context_free (opt_context), NULL));
	}
	goto __finally1;
	__catch1_g_option_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			gint _tmp3;
			fprintf (stdout, "%s\n", e->message);
			fprintf (stdout, _ ("Run '%s --help' to see a full list of available command line options.\n"), args[0]);
			return (_tmp3 = 1, (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), _tmp3);
		}
	}
	__finally1:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return 0;
	}
	if (vala_project_generator_option_version) {
		fprintf (stdout, "vala-gen-project %s\n", PACKAGE_VERSION);
		return 0;
	}
	generator = vala_project_generator_new ();
	_tmp5 = FALSE;
	if (vala_project_generator_option_project_name != NULL) {
		_tmp5 = TRUE;
	} else {
		_tmp5 = vala_project_generator_ask_parameters (generator) == GTK_RESPONSE_OK;
	}
	if (_tmp5) {
		gint _tmp6;
		vala_project_generator_create_project (generator);
		return (_tmp6 = 0, (generator == NULL) ? NULL : (generator = (vala_project_generator_unref (generator), NULL)), _tmp6);
	}
	return (_tmp7 = 1, (generator == NULL) ? NULL : (generator = (vala_project_generator_unref (generator), NULL)), _tmp7);
}


int main (int argc, char ** argv) {
	g_type_init ();
	return vala_project_generator_main (argv, argc);
}


static void vala_value_project_generator_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vala_value_project_generator_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vala_project_generator_unref (value->data[0].v_pointer);
	}
}


static void vala_value_project_generator_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vala_project_generator_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vala_value_project_generator_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vala_value_project_generator_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValaProjectGenerator* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vala_project_generator_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vala_value_project_generator_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValaProjectGenerator** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vala_project_generator_ref (value->data[0].v_pointer);
	}
	return NULL;
}


static GParamSpec* vala_param_spec_project_generator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValaParamSpecProjectGenerator* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALA_TYPE_PROJECT_GENERATOR), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


static gpointer vala_value_get_project_generator (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_PROJECT_GENERATOR), NULL);
	return value->data[0].v_pointer;
}


static void vala_value_set_project_generator (GValue* value, gpointer v_object) {
	ValaProjectGenerator* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_PROJECT_GENERATOR));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALA_TYPE_PROJECT_GENERATOR));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vala_project_generator_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vala_project_generator_unref (old);
	}
}


static void vala_project_generator_class_init (ValaProjectGeneratorClass * klass) {
	vala_project_generator_parent_class = g_type_class_peek_parent (klass);
	VALA_PROJECT_GENERATOR_CLASS (klass)->finalize = vala_project_generator_finalize;
	g_type_class_add_private (klass, sizeof (ValaProjectGeneratorPrivate));
}


static void vala_project_generator_instance_init (ValaProjectGenerator * self) {
	self->priv = VALA_PROJECT_GENERATOR_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void vala_project_generator_finalize (ValaProjectGenerator* obj) {
	ValaProjectGenerator * self;
	self = VALA_PROJECT_GENERATOR (obj);
	(self->priv->config_dialog == NULL) ? NULL : (self->priv->config_dialog = (g_object_unref (self->priv->config_dialog), NULL));
	(self->priv->project_folder_button == NULL) ? NULL : (self->priv->project_folder_button = (g_object_unref (self->priv->project_folder_button), NULL));
	(self->priv->project_type_combobox == NULL) ? NULL : (self->priv->project_type_combobox = (g_object_unref (self->priv->project_type_combobox), NULL));
	(self->priv->license_combobox == NULL) ? NULL : (self->priv->license_combobox = (g_object_unref (self->priv->license_combobox), NULL));
	(self->priv->name_entry == NULL) ? NULL : (self->priv->name_entry = (g_object_unref (self->priv->name_entry), NULL));
	(self->priv->email_entry == NULL) ? NULL : (self->priv->email_entry = (g_object_unref (self->priv->email_entry), NULL));
	self->priv->project_path = (g_free (self->priv->project_path), NULL);
	self->priv->project_name = (g_free (self->priv->project_name), NULL);
	self->priv->namespace_name = (g_free (self->priv->namespace_name), NULL);
	self->priv->make_name = (g_free (self->priv->make_name), NULL);
	self->priv->upper_case_make_name = (g_free (self->priv->upper_case_make_name), NULL);
	self->priv->real_name = (g_free (self->priv->real_name), NULL);
	self->priv->email_address = (g_free (self->priv->email_address), NULL);
}


static GType vala_project_generator_get_type (void) {
	static GType vala_project_generator_type_id = 0;
	if (vala_project_generator_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vala_value_project_generator_init, vala_value_project_generator_free_value, vala_value_project_generator_copy_value, vala_value_project_generator_peek_pointer, "p", vala_value_project_generator_collect_value, "p", vala_value_project_generator_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValaProjectGeneratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_project_generator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaProjectGenerator), 0, (GInstanceInitFunc) vala_project_generator_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vala_project_generator_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValaProjectGenerator", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return vala_project_generator_type_id;
}


static gpointer vala_project_generator_ref (gpointer instance) {
	ValaProjectGenerator* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


static void vala_project_generator_unref (gpointer instance) {
	ValaProjectGenerator* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALA_PROJECT_GENERATOR_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




