/*
 *  vsccompletionvisitor.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#ifndef __VSCCOMPLETIONVISITOR_H__
#define __VSCCOMPLETIONVISITOR_H__

#include <glib.h>
#include <glib-object.h>
#include <vala/valacodevisitor.h>
#include <vala/valasourcefile.h>
#include <vala/valacodecontext.h>
#include <vala/valasymbol.h>
#include <vala/valanamespace.h>
#include <vala/valaclass.h>
#include <vala/valainterface.h>
#include <vala/valastruct.h>
#include <vala/valaenum.h>
#include <vala/valadatatype.h>
#include <vala/valamethod.h>
#include <vala/valaenumvalue.h>
#include <vala/valacreationmethod.h>
#include <vala/valadelegate.h>
#include <vala/valasignal.h>
#include <vala/valafield.h>
#include <vala/valaconstant.h>
#include <vala/valaproperty.h>
#include <vala/valaerrordomain.h>
#include <vala/valaerrorcode.h>
#include <stdlib.h>
#include <string.h>
#include <vscsymbolcompletionfilteroptions.h>
#include <vscsymbolcompletionresult.h>

G_BEGIN_DECLS


#define VSC_TYPE_COMPLETION_VISITOR (vsc_completion_visitor_get_type ())
#define VSC_COMPLETION_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_COMPLETION_VISITOR, VscCompletionVisitor))
#define VSC_COMPLETION_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_COMPLETION_VISITOR, VscCompletionVisitorClass))
#define VSC_IS_COMPLETION_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_COMPLETION_VISITOR))
#define VSC_IS_COMPLETION_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_COMPLETION_VISITOR))
#define VSC_COMPLETION_VISITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_COMPLETION_VISITOR, VscCompletionVisitorClass))

typedef struct _VscCompletionVisitor VscCompletionVisitor;
typedef struct _VscCompletionVisitorClass VscCompletionVisitorClass;
typedef struct _VscCompletionVisitorPrivate VscCompletionVisitorPrivate;

/**
 * Code visitor getting a list of completion items
 */
struct _VscCompletionVisitor {
	ValaCodeVisitor parent_instance;
	VscCompletionVisitorPrivate * priv;
};

struct _VscCompletionVisitorClass {
	ValaCodeVisitorClass parent_class;
};


VscCompletionVisitor* vsc_completion_visitor_construct (GType object_type, VscSymbolCompletionFilterOptions* options, VscSymbolCompletionResult* results, ValaSourceFile* source, ValaCodeContext* context);
VscCompletionVisitor* vsc_completion_visitor_new (VscSymbolCompletionFilterOptions* options, VscSymbolCompletionResult* results, ValaSourceFile* source, ValaCodeContext* context);
void vsc_completion_visitor_integrate_completion (VscCompletionVisitor* self, ValaSymbol* symbol);
const char* vsc_completion_visitor_get_searched_typename (VscCompletionVisitor* self);
void vsc_completion_visitor_set_searched_typename (VscCompletionVisitor* self, const char* value);
GType vsc_completion_visitor_get_type (void);


G_END_DECLS

#endif
