/*
 *  vscmethodlistvisitor.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vscmethodlistvisitor.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vala/valacodenode.h>
#include <gee/collection.h>
#include <vscsymbolcompletionitem.h>




struct _VscMethodListPrivate {
	GeeList* _methods;
};

#define VSC_METHOD_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VSC_TYPE_METHOD_LIST, VscMethodListPrivate))
enum  {
	VSC_METHOD_LIST_DUMMY_PROPERTY
};
static void vsc_method_list_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vsc_method_list_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vsc_method_list_real_visit_class (ValaCodeVisitor* base, ValaClass* cl);
static void vsc_method_list_real_visit_method (ValaCodeVisitor* base, ValaMethod* m);
static gpointer vsc_method_list_parent_class = NULL;
static void vsc_method_list_finalize (ValaCodeVisitor* obj);



VscMethodList* vsc_method_list_construct (GType object_type, GeeList* methods) {
	VscMethodList* self;
	GeeList* _tmp1;
	GeeList* _tmp0;
	g_return_val_if_fail (methods != NULL, NULL);
	self = (VscMethodList*) g_type_create_instance (object_type);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_methods = (_tmp1 = (_tmp0 = methods, (_tmp0 == NULL) ? NULL : gee_collection_object_ref (_tmp0)), (self->priv->_methods == NULL) ? NULL : (self->priv->_methods = (gee_collection_object_unref (self->priv->_methods), NULL)), _tmp1);
	return self;
}


VscMethodList* vsc_method_list_new (GeeList* methods) {
	return vsc_method_list_construct (VSC_TYPE_METHOD_LIST, methods);
}


static void vsc_method_list_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* file) {
	VscMethodList * self;
	self = (VscMethodList*) base;
	g_return_if_fail (file != NULL);
	vala_source_file_accept_children (file, (ValaCodeVisitor*) self);
}


static void vsc_method_list_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns) {
	VscMethodList * self;
	self = (VscMethodList*) base;
	g_return_if_fail (ns != NULL);
	{
		GeeList* _tmp0;
		GeeIterator* _tmp1;
		GeeIterator* _item_it;
		_tmp0 = NULL;
		_tmp1 = NULL;
		_item_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vala_namespace_get_namespaces (ns))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
		while (gee_iterator_next (_item_it)) {
			ValaNamespace* item;
			item = (ValaNamespace*) gee_iterator_get (_item_it);
			vala_code_node_accept ((ValaCodeNode*) item, (ValaCodeVisitor*) self);
			(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
	}
	{
		GeeList* _tmp2;
		GeeIterator* _tmp3;
		GeeIterator* _m_it;
		_tmp2 = NULL;
		_tmp3 = NULL;
		_m_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vala_namespace_get_methods (ns))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
		while (gee_iterator_next (_m_it)) {
			ValaMethod* m;
			m = (ValaMethod*) gee_iterator_get (_m_it);
			vala_code_node_accept ((ValaCodeNode*) m, (ValaCodeVisitor*) self);
			(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
		}
		(_m_it == NULL) ? NULL : (_m_it = (gee_collection_object_unref (_m_it), NULL));
	}
	{
		GeeList* _tmp4;
		GeeIterator* _tmp5;
		GeeIterator* _cl_it;
		_tmp4 = NULL;
		_tmp5 = NULL;
		_cl_it = (_tmp5 = gee_iterable_iterator ((GeeIterable*) (_tmp4 = vala_namespace_get_classes (ns))), (_tmp4 == NULL) ? NULL : (_tmp4 = (gee_collection_object_unref (_tmp4), NULL)), _tmp5);
		while (gee_iterator_next (_cl_it)) {
			ValaClass* cl;
			cl = (ValaClass*) gee_iterator_get (_cl_it);
			vala_code_node_accept_children ((ValaCodeNode*) cl, (ValaCodeVisitor*) self);
			(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
		}
		(_cl_it == NULL) ? NULL : (_cl_it = (gee_collection_object_unref (_cl_it), NULL));
	}
}


static void vsc_method_list_real_visit_class (ValaCodeVisitor* base, ValaClass* cl) {
	VscMethodList * self;
	self = (VscMethodList*) base;
	g_return_if_fail (cl != NULL);
	{
		GeeList* _tmp0;
		GeeIterator* _tmp1;
		GeeIterator* _item_it;
		_tmp0 = NULL;
		_tmp1 = NULL;
		_item_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vala_class_get_classes (cl))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
		while (gee_iterator_next (_item_it)) {
			ValaClass* item;
			item = (ValaClass*) gee_iterator_get (_item_it);
			vala_code_node_accept_children ((ValaCodeNode*) item, (ValaCodeVisitor*) self);
			(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
	}
}


static void vsc_method_list_real_visit_method (ValaCodeVisitor* base, ValaMethod* m) {
	VscMethodList * self;
	VscSymbolCompletionItem* _tmp0;
	self = (VscMethodList*) base;
	g_return_if_fail (m != NULL);
	_tmp0 = NULL;
	gee_collection_add ((GeeCollection*) self->priv->_methods, _tmp0 = vsc_symbol_completion_item_new_with_method (m));
	(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
}


static void vsc_method_list_class_init (VscMethodListClass * klass) {
	vsc_method_list_parent_class = g_type_class_peek_parent (klass);
	VALA_CODE_VISITOR_CLASS (klass)->finalize = vsc_method_list_finalize;
	g_type_class_add_private (klass, sizeof (VscMethodListPrivate));
	VALA_CODE_VISITOR_CLASS (klass)->visit_source_file = vsc_method_list_real_visit_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_namespace = vsc_method_list_real_visit_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_class = vsc_method_list_real_visit_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_method = vsc_method_list_real_visit_method;
}


static void vsc_method_list_instance_init (VscMethodList * self) {
	self->priv = VSC_METHOD_LIST_GET_PRIVATE (self);
}


static void vsc_method_list_finalize (ValaCodeVisitor* obj) {
	VscMethodList * self;
	self = VSC_METHOD_LIST (obj);
	(self->priv->_methods == NULL) ? NULL : (self->priv->_methods = (gee_collection_object_unref (self->priv->_methods), NULL));
	VALA_CODE_VISITOR_CLASS (vsc_method_list_parent_class)->finalize (obj);
}


GType vsc_method_list_get_type (void) {
	static GType vsc_method_list_type_id = 0;
	if (vsc_method_list_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscMethodListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_method_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscMethodList), 0, (GInstanceInitFunc) vsc_method_list_instance_init, NULL };
		vsc_method_list_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "VscMethodList", &g_define_type_info, 0);
	}
	return vsc_method_list_type_id;
}




