/*
 *  vscparsermanager.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#ifndef __VSCPARSERMANAGER_H__
#define __VSCPARSERMANAGER_H__

#include <glib.h>
#include <glib-object.h>
#include <vala/valacodecontext.h>
#include <stdlib.h>
#include <string.h>
#include <vscsourcebuffer.h>

G_BEGIN_DECLS


#define VSC_TYPE_PARSER_MANAGER (vsc_parser_manager_get_type ())
#define VSC_PARSER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_PARSER_MANAGER, VscParserManager))
#define VSC_PARSER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_PARSER_MANAGER, VscParserManagerClass))
#define VSC_IS_PARSER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_PARSER_MANAGER))
#define VSC_IS_PARSER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_PARSER_MANAGER))
#define VSC_PARSER_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_PARSER_MANAGER, VscParserManagerClass))

typedef struct _VscParserManager VscParserManager;
typedef struct _VscParserManagerClass VscParserManagerClass;
typedef struct _VscParserManagerPrivate VscParserManagerPrivate;

struct _VscParserManager {
	GObject parent_instance;
	VscParserManagerPrivate * priv;
};

struct _VscParserManagerClass {
	GObjectClass parent_class;
};


void vsc_parser_manager_lock_all_contexts (VscParserManager* self);
void vsc_parser_manager_unlock_all_contexts (VscParserManager* self);
void vsc_parser_manager_lock_pri_context (VscParserManager* self);
void vsc_parser_manager_unlock_pri_context (VscParserManager* self);
void vsc_parser_manager_lock_sec_context (VscParserManager* self);
void vsc_parser_manager_unlock_sec_context (VscParserManager* self);
void vsc_parser_manager_suspend_parsing (VscParserManager* self);
void vsc_parser_manager_resume_parsing (VscParserManager* self);
gboolean vsc_parser_manager_add_path_to_vapi_search_dir (VscParserManager* self, const char* path);
gboolean vsc_parser_manager_add_built_package (VscParserManager* self, const char* vapi_file);
gboolean vsc_parser_manager_remove_build_package (VscParserManager* self, const char* vapi_file);
gboolean vsc_parser_manager_add_package_from_namespace (VscParserManager* self, const char* namespace, gboolean auto_schedule_parse, GError** error);
void vsc_parser_manager_remove_package_from_namespace (VscParserManager* self, const char* namespace, GError** error);
void vsc_parser_manager_remove_package (VscParserManager* self, const char* package_name, GError** error);
gboolean vsc_parser_manager_try_add_package (VscParserManager* self, const char* package_name, gboolean auto_schedule_parse);
gboolean vsc_parser_manager_add_package (VscParserManager* self, const char* package_name, gboolean auto_schedule_parse, GError** error);
void vsc_parser_manager_add_source (VscParserManager* self, const char* filename, GError** error);
void vsc_parser_manager_remove_source (VscParserManager* self, const char* filename, GError** error);
gboolean vsc_parser_manager_contains_source (VscParserManager* self, const char* filename);
void vsc_parser_manager_add_source_buffer (VscParserManager* self, VscSourceBuffer* source, GError** error);
void vsc_parser_manager_remove_source_buffer_by_name (VscParserManager* self, const char* name, GError** error);
gboolean vsc_parser_manager_contains_source_buffer (VscParserManager* self, VscSourceBuffer* source);
gboolean vsc_parser_manager_contains_source_buffer_by_name (VscParserManager* self, const char* name);
void vsc_parser_manager_remove_source_buffer (VscParserManager* self, VscSourceBuffer* source);
void vsc_parser_manager_reparse_source_buffers (VscParserManager* self);
gboolean vsc_parser_manager_is_cache_building (VscParserManager* self);
void vsc_parser_manager_reparse (VscParserManager* self, gboolean all_context);
void vsc_parser_manager_parse_context (VscParserManager* self, ValaCodeContext* context);
VscParserManager* vsc_parser_manager_construct (GType object_type);
VscParserManager* vsc_parser_manager_new (void);
ValaCodeContext* vsc_parser_manager_get_pri_context (VscParserManager* self);
ValaCodeContext* vsc_parser_manager_get_sec_context (VscParserManager* self);
GType vsc_parser_manager_get_type (void);


G_END_DECLS

#endif
