/*
 *  vscsourcebuffer.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vscsourcebuffer.h>




struct _VscSourceBufferPrivate {
	char* _name;
	char* _source;
};

#define VSC_SOURCE_BUFFER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VSC_TYPE_SOURCE_BUFFER, VscSourceBufferPrivate))
enum  {
	VSC_SOURCE_BUFFER_DUMMY_PROPERTY,
	VSC_SOURCE_BUFFER_NAME,
	VSC_SOURCE_BUFFER_SOURCE
};
static gpointer vsc_source_buffer_parent_class = NULL;
static void vsc_source_buffer_finalize (GObject* obj);



VscSourceBuffer* vsc_source_buffer_construct (GType object_type, const char* name, const char* source) {
	GParameter * __params;
	GParameter * __params_it;
	VscSourceBuffer * self;
	g_return_val_if_fail (name != NULL, NULL);
	__params = g_new0 (GParameter, 2);
	__params_it = __params;
	__params_it->name = "name";
	g_value_init (&__params_it->value, G_TYPE_STRING);
	g_value_set_string (&__params_it->value, name);
	__params_it++;
	__params_it->name = "source";
	g_value_init (&__params_it->value, G_TYPE_STRING);
	g_value_set_string (&__params_it->value, source);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VscSourceBuffer* vsc_source_buffer_new (const char* name, const char* source) {
	return vsc_source_buffer_construct (VSC_TYPE_SOURCE_BUFFER, name, source);
}


const char* vsc_source_buffer_get_name (VscSourceBuffer* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_name;
}


void vsc_source_buffer_set_name (VscSourceBuffer* self, const char* value) {
	char* _tmp2;
	const char* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_name = (g_free (self->priv->_name), NULL), _tmp2);
	g_object_notify ((GObject *) self, "name");
}


const char* vsc_source_buffer_get_source (VscSourceBuffer* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_source;
}


void vsc_source_buffer_set_source (VscSourceBuffer* self, const char* value) {
	char* _tmp2;
	const char* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_source = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_source = (g_free (self->priv->_source), NULL), _tmp2);
	g_object_notify ((GObject *) self, "source");
}


static void vsc_source_buffer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VscSourceBuffer * self;
	gpointer boxed;
	self = VSC_SOURCE_BUFFER (object);
	switch (property_id) {
		case VSC_SOURCE_BUFFER_NAME:
		g_value_set_string (value, vsc_source_buffer_get_name (self));
		break;
		case VSC_SOURCE_BUFFER_SOURCE:
		g_value_set_string (value, vsc_source_buffer_get_source (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vsc_source_buffer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VscSourceBuffer * self;
	self = VSC_SOURCE_BUFFER (object);
	switch (property_id) {
		case VSC_SOURCE_BUFFER_NAME:
		vsc_source_buffer_set_name (self, g_value_get_string (value));
		break;
		case VSC_SOURCE_BUFFER_SOURCE:
		vsc_source_buffer_set_source (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vsc_source_buffer_class_init (VscSourceBufferClass * klass) {
	vsc_source_buffer_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VscSourceBufferPrivate));
	G_OBJECT_CLASS (klass)->get_property = vsc_source_buffer_get_property;
	G_OBJECT_CLASS (klass)->set_property = vsc_source_buffer_set_property;
	G_OBJECT_CLASS (klass)->finalize = vsc_source_buffer_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VSC_SOURCE_BUFFER_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VSC_SOURCE_BUFFER_SOURCE, g_param_spec_string ("source", "source", "source", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vsc_source_buffer_instance_init (VscSourceBuffer * self) {
	self->priv = VSC_SOURCE_BUFFER_GET_PRIVATE (self);
}


static void vsc_source_buffer_finalize (GObject* obj) {
	VscSourceBuffer * self;
	self = VSC_SOURCE_BUFFER (obj);
	self->priv->_name = (g_free (self->priv->_name), NULL);
	self->priv->_source = (g_free (self->priv->_source), NULL);
	G_OBJECT_CLASS (vsc_source_buffer_parent_class)->finalize (obj);
}


GType vsc_source_buffer_get_type (void) {
	static GType vsc_source_buffer_type_id = 0;
	if (vsc_source_buffer_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscSourceBufferClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_source_buffer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscSourceBuffer), 0, (GInstanceInitFunc) vsc_source_buffer_instance_init, NULL };
		vsc_source_buffer_type_id = g_type_register_static (G_TYPE_OBJECT, "VscSourceBuffer", &g_define_type_info, 0);
	}
	return vsc_source_buffer_type_id;
}




