/*
 *  vscsymbolcompletion.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#ifndef __VSCSYMBOLCOMPLETION_H__
#define __VSCSYMBOLCOMPLETION_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <vala/valadatatype.h>
#include <gee/list.h>
#include <vscsymbolcompletionresult.h>
#include <vscsymbolcompletionfilteroptions.h>
#include <vscparsermanager.h>

G_BEGIN_DECLS


#define VSC_TYPE_SYMBOL_COMPLETION (vsc_symbol_completion_get_type ())
#define VSC_SYMBOL_COMPLETION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_SYMBOL_COMPLETION, VscSymbolCompletion))
#define VSC_SYMBOL_COMPLETION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_SYMBOL_COMPLETION, VscSymbolCompletionClass))
#define VSC_IS_SYMBOL_COMPLETION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_SYMBOL_COMPLETION))
#define VSC_IS_SYMBOL_COMPLETION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_SYMBOL_COMPLETION))
#define VSC_SYMBOL_COMPLETION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_SYMBOL_COMPLETION, VscSymbolCompletionClass))

typedef struct _VscSymbolCompletion VscSymbolCompletion;
typedef struct _VscSymbolCompletionClass VscSymbolCompletionClass;
typedef struct _VscSymbolCompletionPrivate VscSymbolCompletionPrivate;

typedef enum  {
	VSC_SYMBOL_COMPLETION_ERROR_PACKAGE_FILE_NOT_FOUND,
	VSC_SYMBOL_COMPLETION_ERROR_PACKAGE_ALREADY_ADDED,
	VSC_SYMBOL_COMPLETION_ERROR_UNSUPPORTED_SYMBOL_TYPE,
	VSC_SYMBOL_COMPLETION_ERROR_UNSUPPORTED_TYPE,
	VSC_SYMBOL_COMPLETION_ERROR_SOURCE_BUFFER
} VscSymbolCompletionError;
#define VSC_SYMBOL_COMPLETION_ERROR vsc_symbol_completion_error_quark ()
struct _VscSymbolCompletion {
	GObject parent_instance;
	VscSymbolCompletionPrivate * priv;
};

struct _VscSymbolCompletionClass {
	GObjectClass parent_class;
};


GQuark vsc_symbol_completion_error_quark (void);
void vsc_symbol_completion_cleanup (VscSymbolCompletion* self);
char* vsc_symbol_completion_get_name_for_datatype (VscSymbolCompletion* self, ValaDataType* dt);
char* vsc_symbol_completion_get_qualified_name_for_datatype (VscSymbolCompletion* self, ValaDataType* dt);
GeeList* vsc_symbol_completion_get_methods_for_source (VscSymbolCompletion* self, const char* sourcefile);
char* vsc_symbol_completion_get_datatype_name_for_name (VscSymbolCompletion* self, const char* symbolname, const char* sourcefile, gint line, gint column, GError** error);
ValaDataType* vsc_symbol_completion_get_datatype_for_name (VscSymbolCompletion* self, const char* symbolname, const char* sourcefile, gint line, gint column, GError** error);
VscSymbolCompletionResult* vsc_symbol_completion_get_completions_for_name (VscSymbolCompletion* self, VscSymbolCompletionFilterOptions* options, const char* symbolname, const char* sourcefile, gint line, gint column, GError** error);
gboolean vsc_symbol_completion_symbol_has_known_namespace (const char* name);
VscSymbolCompletion* vsc_symbol_completion_construct (GType object_type);
VscSymbolCompletion* vsc_symbol_completion_new (void);
VscParserManager* vsc_symbol_completion_get_parser (VscSymbolCompletion* self);
GType vsc_symbol_completion_get_type (void);


G_END_DECLS

#endif
