/*
 *  vscsymbolcompletionitem.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vscsymbolcompletionitem.h>
#include <vala/valadatatype.h>
#include <vala/valascope.h>
#include <gee/list.h>
#include <gee/collection.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vala/valaformalparameter.h>
#include <vala/valacodenode.h>
#include <vala/valasymbol.h>
#include <vala/valasourcereference.h>




static char* string_substring (const char* self, glong offset, glong len);
static glong string_get_length (const char* self);
enum  {
	VSC_SYMBOL_COMPLETION_ITEM_DUMMY_PROPERTY
};
static char* vsc_symbol_completion_item_data_type_to_string (VscSymbolCompletionItem* self, ValaDataType* type);
static char* vsc_symbol_completion_item_formal_parameters_to_string (VscSymbolCompletionItem* self, GeeList* parameters);
static gpointer vsc_symbol_completion_item_parent_class = NULL;
static void vsc_symbol_completion_item_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct (GType object_type, const char* name) {
	VscSymbolCompletionItem * self;
	char* _tmp1;
	const char* _tmp0;
	char* _tmp3;
	const char* _tmp2;
	g_return_val_if_fail (name != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->name = (_tmp1 = (_tmp0 = name, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->name = (g_free (self->name), NULL), _tmp1);
	_tmp3 = NULL;
	_tmp2 = NULL;
	self->info = (_tmp3 = (_tmp2 = name, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), self->info = (g_free (self->info), NULL), _tmp3);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new (const char* name) {
	return vsc_symbol_completion_item_construct (VSC_TYPE_SYMBOL_COMPLETION_ITEM, name);
}


static char* vsc_symbol_completion_item_data_type_to_string (VscSymbolCompletionItem* self, ValaDataType* type) {
	char* result;
	char* _tmp0;
	GeeList* type_args;
	gboolean _tmp2;
	gboolean _tmp9;
	gboolean _tmp10;
	char* _tmp13;
	char* _tmp14;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	result = NULL;
	_tmp0 = NULL;
	result = (_tmp0 = vala_data_type_to_qualified_string (type, NULL), result = (g_free (result), NULL), _tmp0);
	if (result == NULL) {
		char* _tmp1;
		_tmp1 = NULL;
		result = (_tmp1 = g_strdup ("void"), result = (g_free (result), NULL), _tmp1);
	}
	type_args = vala_data_type_get_type_arguments (type);
	_tmp2 = FALSE;
	if (gee_collection_get_size ((GeeCollection*) type_args) > 0) {
		_tmp2 = strstr (result, "<") == NULL;
	} else {
		_tmp2 = FALSE;
	}
	if (_tmp2) {
		char* _tmp3;
		gboolean first;
		char* _tmp8;
		_tmp3 = NULL;
		result = (_tmp3 = g_strconcat (result, ("<"), NULL), result = (g_free (result), NULL), _tmp3);
		first = TRUE;
		{
			GeeIterator* _type_arg_it;
			_type_arg_it = gee_iterable_iterator ((GeeIterable*) type_args);
			while (gee_iterator_next (_type_arg_it)) {
				ValaDataType* type_arg;
				char* _tmp7;
				char* _tmp6;
				type_arg = (ValaDataType*) gee_iterator_get (_type_arg_it);
				if (!first) {
					char* _tmp4;
					_tmp4 = NULL;
					result = (_tmp4 = g_strconcat (result, (","), NULL), result = (g_free (result), NULL), _tmp4);
				} else {
					first = FALSE;
				}
				if (!vala_data_type_get_value_owned (type_arg)) {
					char* _tmp5;
					_tmp5 = NULL;
					result = (_tmp5 = g_strconcat (result, ("weak "), NULL), result = (g_free (result), NULL), _tmp5);
				}
				_tmp7 = NULL;
				_tmp6 = NULL;
				result = (_tmp7 = g_strconcat (result, _tmp6 = (vsc_symbol_completion_item_data_type_to_string (self, type_arg)), NULL), result = (g_free (result), NULL), _tmp7);
				_tmp6 = (g_free (_tmp6), NULL);
				(type_arg == NULL) ? NULL : (type_arg = (vala_code_node_unref (type_arg), NULL));
			}
			(_type_arg_it == NULL) ? NULL : (_type_arg_it = (gee_collection_object_unref (_type_arg_it), NULL));
		}
		_tmp8 = NULL;
		result = (_tmp8 = g_strconcat (result, (">"), NULL), result = (g_free (result), NULL), _tmp8);
	}
	_tmp9 = FALSE;
	_tmp10 = FALSE;
	if (vala_data_type_get_nullable (type)) {
		_tmp10 = !g_str_has_suffix (result, "?");
	} else {
		_tmp10 = FALSE;
	}
	if (_tmp10) {
		_tmp9 = !g_str_has_suffix (result, "*");
	} else {
		_tmp9 = FALSE;
	}
	if (_tmp9) {
		char* _tmp11;
		_tmp11 = NULL;
		result = (_tmp11 = g_strconcat (result, ("?"), NULL), result = (g_free (result), NULL), _tmp11);
	}
	if (vala_data_type_get_is_dynamic (type)) {
		char* _tmp12;
		_tmp12 = NULL;
		result = (_tmp12 = g_strconcat ("dynamic ", result, NULL), result = (g_free (result), NULL), _tmp12);
	}
	_tmp13 = NULL;
	result = (_tmp13 = g_markup_escape_text (result, -1), result = (g_free (result), NULL), _tmp13);
	_tmp14 = NULL;
	return (_tmp14 = result, (type_args == NULL) ? NULL : (type_args = (gee_collection_object_unref (type_args), NULL)), _tmp14);
}


static char* vsc_symbol_completion_item_formal_parameters_to_string (VscSymbolCompletionItem* self, GeeList* parameters) {
	char* params;
	char* param_sep;
	char* _tmp10;
	char* _tmp11;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (parameters != NULL, NULL);
	params = g_strdup ("");
	param_sep = g_strdup (" ");
	if (gee_collection_get_size ((GeeCollection*) parameters) > 2) {
		char* _tmp0;
		_tmp0 = NULL;
		param_sep = (_tmp0 = g_strdup ("\n\t\t"), param_sep = (g_free (param_sep), NULL), _tmp0);
	}
	{
		GeeIterator* _parameter_it;
		_parameter_it = gee_iterable_iterator ((GeeIterable*) parameters);
		while (gee_iterator_next (_parameter_it)) {
			ValaFormalParameter* parameter;
			char* direction;
			char* default_expr;
			char* parameter_type;
			parameter = (ValaFormalParameter*) gee_iterator_get (_parameter_it);
			direction = g_strdup ("");
			default_expr = g_strdup ("");
			parameter_type = g_strdup ("");
			if (vala_formal_parameter_get_ellipsis (parameter)) {
				char* _tmp1;
				_tmp1 = NULL;
				params = (_tmp1 = g_strdup_printf ("%s, %s", params, "..."), params = (g_free (params), NULL), _tmp1);
			} else {
				char* _tmp8;
				if (vala_formal_parameter_get_direction (parameter) == VALA_PARAMETER_DIRECTION_OUT) {
					char* _tmp2;
					_tmp2 = NULL;
					direction = (_tmp2 = g_strdup ("out "), direction = (g_free (direction), NULL), _tmp2);
				} else {
					if (vala_formal_parameter_get_direction (parameter) == VALA_PARAMETER_DIRECTION_REF) {
						char* _tmp3;
						_tmp3 = NULL;
						direction = (_tmp3 = g_strdup ("ref "), direction = (g_free (direction), NULL), _tmp3);
					}
				}
				if (vala_formal_parameter_get_default_expression (parameter) != NULL) {
					char* _tmp5;
					char* _tmp4;
					_tmp5 = NULL;
					_tmp4 = NULL;
					default_expr = (_tmp5 = g_strconcat (" = ", _tmp4 = vala_code_node_to_string ((ValaCodeNode*) vala_formal_parameter_get_default_expression (parameter)), NULL), default_expr = (g_free (default_expr), NULL), _tmp5);
					_tmp4 = (g_free (_tmp4), NULL);
				}
				if (vala_formal_parameter_get_parameter_type (parameter) != NULL) {
					char* _tmp6;
					_tmp6 = NULL;
					parameter_type = (_tmp6 = vsc_symbol_completion_item_data_type_to_string (self, vala_formal_parameter_get_parameter_type (parameter)), parameter_type = (g_free (parameter_type), NULL), _tmp6);
				} else {
					char* _tmp7;
					_tmp7 = NULL;
					parameter_type = (_tmp7 = g_strdup ("unknown"), parameter_type = (g_free (parameter_type), NULL), _tmp7);
				}
				_tmp8 = NULL;
				params = (_tmp8 = g_strdup_printf ("%s,%s%s%s %s%s", params, param_sep, direction, parameter_type, vala_symbol_get_name ((ValaSymbol*) parameter), default_expr), params = (g_free (params), NULL), _tmp8);
			}
			(parameter == NULL) ? NULL : (parameter = (vala_code_node_unref (parameter), NULL));
			direction = (g_free (direction), NULL);
			default_expr = (g_free (default_expr), NULL);
			parameter_type = (g_free (parameter_type), NULL);
		}
		(_parameter_it == NULL) ? NULL : (_parameter_it = (gee_collection_object_unref (_parameter_it), NULL));
	}
	if (_vala_strcmp0 (params, "") != 0) {
		char* _tmp9;
		_tmp9 = NULL;
		params = (_tmp9 = string_substring (params, (glong) 2, string_get_length (params) - 2), params = (g_free (params), NULL), _tmp9);
	}
	_tmp10 = NULL;
	params = (_tmp10 = g_markup_escape_text (params, -1), params = (g_free (params), NULL), _tmp10);
	_tmp11 = NULL;
	return (_tmp11 = params, param_sep = (g_free (param_sep), NULL), _tmp11);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_method (GType object_type, ValaMethod* item) {
	VscSymbolCompletionItem * self;
	char* _tmp1;
	const char* _tmp0;
	GeeList* _tmp6;
	gint _tmp7;
	gint param_count;
	GeeList* _tmp8;
	char* _tmp9;
	char* params;
	const char* _tmp10;
	const char* _tmp11;
	char* _tmp13;
	char* _tmp12;
	g_return_val_if_fail (item != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->name = (_tmp1 = (_tmp0 = vala_symbol_get_name ((ValaSymbol*) item), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->name = (g_free (self->name), NULL), _tmp1);
	self->line = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) item));
	if (g_str_has_prefix (self->name, "new")) {
		char* _tmp2;
		_tmp2 = NULL;
		self->name = (_tmp2 = string_substring (self->name, (glong) 3, string_get_length (self->name) - 3), self->name = (g_free (self->name), NULL), _tmp2);
		if (_vala_strcmp0 (self->name, "") == 0) {
			char* _tmp4;
			const char* _tmp3;
			_tmp4 = NULL;
			_tmp3 = NULL;
			self->name = (_tmp4 = (_tmp3 = vala_symbol_get_name (vala_symbol_get_parent_symbol ((ValaSymbol*) item)), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3)), self->name = (g_free (self->name), NULL), _tmp4);
		} else {
			if (g_str_has_prefix (self->name, ".")) {
				char* _tmp5;
				_tmp5 = NULL;
				self->name = (_tmp5 = string_substring (self->name, (glong) 1, string_get_length (self->name) - 1), self->name = (g_free (self->name), NULL), _tmp5);
			}
		}
	}
	_tmp6 = NULL;
	param_count = (_tmp7 = gee_collection_get_size ((GeeCollection*) (_tmp6 = vala_method_get_parameters (item))), (_tmp6 == NULL) ? NULL : (_tmp6 = (gee_collection_object_unref (_tmp6), NULL)), _tmp7);
	_tmp8 = NULL;
	_tmp9 = NULL;
	params = (_tmp9 = vsc_symbol_completion_item_formal_parameters_to_string (self, _tmp8 = vala_method_get_parameters (item)), (_tmp8 == NULL) ? NULL : (_tmp8 = (gee_collection_object_unref (_tmp8), NULL)), _tmp9);
	_tmp10 = NULL;
	if (param_count > 2) {
		_tmp10 = "\n";
	} else {
		_tmp10 = " ";
	}
	_tmp11 = NULL;
	if (param_count > 2) {
		_tmp11 = "\n";
	} else {
		_tmp11 = "";
	}
	_tmp13 = NULL;
	_tmp12 = NULL;
	self->info = (_tmp13 = g_strdup_printf ("Method: %s\n\n%s%s<b>%s</b> (%s%s)", self->name, _tmp12 = vsc_symbol_completion_item_data_type_to_string (self, vala_method_get_return_type (item)), (_tmp10), self->name, (_tmp11), params), self->info = (g_free (self->info), NULL), _tmp13);
	_tmp12 = (g_free (_tmp12), NULL);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_method (ValaMethod* item) {
	return vsc_symbol_completion_item_construct_with_method (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_field (GType object_type, ValaField* item) {
	VscSymbolCompletionItem * self;
	char* _tmp1;
	const char* _tmp0;
	char* default_expr;
	char* _tmp5;
	char* _tmp4;
	g_return_val_if_fail (item != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->name = (_tmp1 = (_tmp0 = vala_symbol_get_name ((ValaSymbol*) item), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->name = (g_free (self->name), NULL), _tmp1);
	self->line = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) item));
	default_expr = g_strdup ("");
	if (vala_field_get_initializer (item) != NULL) {
		char* _tmp3;
		char* _tmp2;
		_tmp3 = NULL;
		_tmp2 = NULL;
		default_expr = (_tmp3 = g_strconcat (" = ", _tmp2 = vala_code_node_to_string ((ValaCodeNode*) vala_field_get_initializer (item)), NULL), default_expr = (g_free (default_expr), NULL), _tmp3);
		_tmp2 = (g_free (_tmp2), NULL);
	}
	_tmp5 = NULL;
	_tmp4 = NULL;
	self->info = (_tmp5 = g_strdup_printf ("Field: %s\n\n%s <b>%s</b>%s", self->name, _tmp4 = vsc_symbol_completion_item_data_type_to_string (self, vala_field_get_field_type (item)), self->name, default_expr), self->info = (g_free (self->info), NULL), _tmp5);
	_tmp4 = (g_free (_tmp4), NULL);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_field (ValaField* item) {
	return vsc_symbol_completion_item_construct_with_field (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_property (GType object_type, ValaProperty* item) {
	VscSymbolCompletionItem * self;
	char* _tmp1;
	const char* _tmp0;
	char* default_expr;
	char* _tmp5;
	char* _tmp4;
	g_return_val_if_fail (item != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->name = (_tmp1 = (_tmp0 = vala_symbol_get_name ((ValaSymbol*) item), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->name = (g_free (self->name), NULL), _tmp1);
	default_expr = g_strdup ("");
	if (vala_property_get_default_expression (item) != NULL) {
		char* _tmp3;
		char* _tmp2;
		_tmp3 = NULL;
		_tmp2 = NULL;
		default_expr = (_tmp3 = g_strconcat (" = ", _tmp2 = vala_code_node_to_string ((ValaCodeNode*) vala_property_get_default_expression (item)), NULL), default_expr = (g_free (default_expr), NULL), _tmp3);
		_tmp2 = (g_free (_tmp2), NULL);
	}
	_tmp5 = NULL;
	_tmp4 = NULL;
	self->info = (_tmp5 = g_strdup_printf ("Property: %s\n\n%s <b>%s</b>%s", self->name, _tmp4 = vsc_symbol_completion_item_data_type_to_string (self, vala_property_get_property_type (item)), self->name, default_expr), self->info = (g_free (self->info), NULL), _tmp5);
	_tmp4 = (g_free (_tmp4), NULL);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_property (ValaProperty* item) {
	return vsc_symbol_completion_item_construct_with_property (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_struct (GType object_type, ValaStruct* item) {
	VscSymbolCompletionItem * self;
	char* _tmp1;
	const char* _tmp0;
	char* _tmp2;
	g_return_val_if_fail (item != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->name = (_tmp1 = (_tmp0 = vala_symbol_get_name ((ValaSymbol*) item), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->name = (g_free (self->name), NULL), _tmp1);
	_tmp2 = NULL;
	self->info = (_tmp2 = g_strdup_printf ("Struct: %s", vala_symbol_get_name ((ValaSymbol*) item)), self->info = (g_free (self->info), NULL), _tmp2);
	self->line = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) item));
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_struct (ValaStruct* item) {
	return vsc_symbol_completion_item_construct_with_struct (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_class (GType object_type, ValaClass* item) {
	VscSymbolCompletionItem * self;
	char* _tmp1;
	const char* _tmp0;
	char* _tmp2;
	g_return_val_if_fail (item != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->name = (_tmp1 = (_tmp0 = vala_symbol_get_name ((ValaSymbol*) item), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->name = (g_free (self->name), NULL), _tmp1);
	_tmp2 = NULL;
	self->info = (_tmp2 = g_strdup_printf ("Class: %s", vala_symbol_get_name ((ValaSymbol*) item)), self->info = (g_free (self->info), NULL), _tmp2);
	self->line = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) item));
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_class (ValaClass* item) {
	return vsc_symbol_completion_item_construct_with_class (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_interface (GType object_type, ValaInterface* item) {
	VscSymbolCompletionItem * self;
	char* _tmp1;
	const char* _tmp0;
	char* _tmp2;
	g_return_val_if_fail (item != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->name = (_tmp1 = (_tmp0 = vala_symbol_get_name ((ValaSymbol*) item), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->name = (g_free (self->name), NULL), _tmp1);
	_tmp2 = NULL;
	self->info = (_tmp2 = g_strdup_printf ("Interface: %s", vala_symbol_get_name ((ValaSymbol*) item)), self->info = (g_free (self->info), NULL), _tmp2);
	self->line = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) item));
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_interface (ValaInterface* item) {
	return vsc_symbol_completion_item_construct_with_interface (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_signal (GType object_type, ValaSignal* item) {
	VscSymbolCompletionItem * self;
	char* _tmp1;
	const char* _tmp0;
	char* _tmp2;
	GeeList* _tmp3;
	gint _tmp4;
	gint param_count;
	GeeList* _tmp5;
	char* _tmp6;
	char* params;
	const char* _tmp7;
	const char* _tmp8;
	char* _tmp10;
	char* _tmp9;
	g_return_val_if_fail (item != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->name = (_tmp1 = (_tmp0 = vala_symbol_get_name ((ValaSymbol*) item), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->name = (g_free (self->name), NULL), _tmp1);
	_tmp2 = NULL;
	self->info = (_tmp2 = g_strdup_printf ("Signal: %s", vala_symbol_get_name ((ValaSymbol*) item)), self->info = (g_free (self->info), NULL), _tmp2);
	self->line = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) item));
	_tmp3 = NULL;
	param_count = (_tmp4 = gee_collection_get_size ((GeeCollection*) (_tmp3 = vala_signal_get_parameters (item))), (_tmp3 == NULL) ? NULL : (_tmp3 = (gee_collection_object_unref (_tmp3), NULL)), _tmp4);
	_tmp5 = NULL;
	_tmp6 = NULL;
	params = (_tmp6 = vsc_symbol_completion_item_formal_parameters_to_string (self, _tmp5 = vala_signal_get_parameters (item)), (_tmp5 == NULL) ? NULL : (_tmp5 = (gee_collection_object_unref (_tmp5), NULL)), _tmp6);
	_tmp7 = NULL;
	if (param_count > 2) {
		_tmp7 = "\n";
	} else {
		_tmp7 = " ";
	}
	_tmp8 = NULL;
	if (param_count > 2) {
		_tmp8 = "\n";
	} else {
		_tmp8 = "";
	}
	_tmp10 = NULL;
	_tmp9 = NULL;
	self->info = (_tmp10 = g_strdup_printf ("Signal: %s\n\n%s%s<b>%s</b> (%s%s)", self->name, _tmp9 = vsc_symbol_completion_item_data_type_to_string (self, vala_signal_get_return_type (item)), (_tmp7), self->name, (_tmp8), params), self->info = (g_free (self->info), NULL), _tmp10);
	_tmp9 = (g_free (_tmp9), NULL);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_signal (ValaSignal* item) {
	return vsc_symbol_completion_item_construct_with_signal (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


static void vsc_symbol_completion_item_class_init (VscSymbolCompletionItemClass * klass) {
	vsc_symbol_completion_item_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = vsc_symbol_completion_item_finalize;
}


static void vsc_symbol_completion_item_instance_init (VscSymbolCompletionItem * self) {
	self->type_name = NULL;
	self->line = 0;
}


static void vsc_symbol_completion_item_finalize (GObject* obj) {
	VscSymbolCompletionItem * self;
	self = VSC_SYMBOL_COMPLETION_ITEM (obj);
	self->name = (g_free (self->name), NULL);
	self->type_name = (g_free (self->type_name), NULL);
	self->info = (g_free (self->info), NULL);
	G_OBJECT_CLASS (vsc_symbol_completion_item_parent_class)->finalize (obj);
}


GType vsc_symbol_completion_item_get_type (void) {
	static GType vsc_symbol_completion_item_type_id = 0;
	if (vsc_symbol_completion_item_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscSymbolCompletionItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_symbol_completion_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscSymbolCompletionItem), 0, (GInstanceInitFunc) vsc_symbol_completion_item_instance_init, NULL };
		vsc_symbol_completion_item_type_id = g_type_register_static (G_TYPE_OBJECT, "VscSymbolCompletionItem", &g_define_type_info, 0);
	}
	return vsc_symbol_completion_item_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




