/*
 *  vscsymbolcompletionitem.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#ifndef __VSCSYMBOLCOMPLETIONITEM_H__
#define __VSCSYMBOLCOMPLETIONITEM_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <vala/valamethod.h>
#include <vala/valafield.h>
#include <vala/valaproperty.h>
#include <vala/valastruct.h>
#include <vala/valaclass.h>
#include <vala/valainterface.h>
#include <vala/valasignal.h>

G_BEGIN_DECLS


#define VSC_TYPE_SYMBOL_COMPLETION_ITEM (vsc_symbol_completion_item_get_type ())
#define VSC_SYMBOL_COMPLETION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItem))
#define VSC_SYMBOL_COMPLETION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItemClass))
#define VSC_IS_SYMBOL_COMPLETION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM))
#define VSC_IS_SYMBOL_COMPLETION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_SYMBOL_COMPLETION_ITEM))
#define VSC_SYMBOL_COMPLETION_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItemClass))

typedef struct _VscSymbolCompletionItem VscSymbolCompletionItem;
typedef struct _VscSymbolCompletionItemClass VscSymbolCompletionItemClass;
typedef struct _VscSymbolCompletionItemPrivate VscSymbolCompletionItemPrivate;

struct _VscSymbolCompletionItem {
	GObject parent_instance;
	VscSymbolCompletionItemPrivate * priv;
	char* name;
	char* type_name;
	char* info;
	gint line;
};

struct _VscSymbolCompletionItemClass {
	GObjectClass parent_class;
};


VscSymbolCompletionItem* vsc_symbol_completion_item_construct (GType object_type, const char* name);
VscSymbolCompletionItem* vsc_symbol_completion_item_new (const char* name);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_method (GType object_type, ValaMethod* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_method (ValaMethod* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_field (GType object_type, ValaField* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_field (ValaField* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_property (GType object_type, ValaProperty* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_property (ValaProperty* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_struct (GType object_type, ValaStruct* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_struct (ValaStruct* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_class (GType object_type, ValaClass* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_class (ValaClass* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_interface (GType object_type, ValaInterface* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_interface (ValaInterface* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_signal (GType object_type, ValaSignal* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_signal (ValaSignal* item);
GType vsc_symbol_completion_item_get_type (void);


G_END_DECLS

#endif
