/*
 *  vscsymbolcompletionresult.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vscsymbolcompletionresult.h>
#include <gee/arraylist.h>
#include <gee/collection.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vscsymbolcompletionitem.h>




enum  {
	VSC_SYMBOL_COMPLETION_RESULT_DUMMY_PROPERTY,
	VSC_SYMBOL_COMPLETION_RESULT_IS_EMPTY,
	VSC_SYMBOL_COMPLETION_RESULT_COUNT
};
static gboolean vsc_symbol_completion_result_symbols_contain (VscSymbolCompletionResult* self, GeeList* data, const char* name);
static gpointer vsc_symbol_completion_result_parent_class = NULL;
static void vsc_symbol_completion_result_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



gboolean vsc_symbol_completion_result_namespaces_contain (VscSymbolCompletionResult* self, const char* name) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	return vsc_symbol_completion_result_symbols_contain (self, self->namespaces, name);
}


gboolean vsc_symbol_completion_result_classes_contain (VscSymbolCompletionResult* self, const char* name) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	return vsc_symbol_completion_result_symbols_contain (self, self->classes, name);
}


gboolean vsc_symbol_completion_result_interfaces_contain (VscSymbolCompletionResult* self, const char* name) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	return vsc_symbol_completion_result_symbols_contain (self, self->interfaces, name);
}


gboolean vsc_symbol_completion_result_structs_contain (VscSymbolCompletionResult* self, const char* name) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	return vsc_symbol_completion_result_symbols_contain (self, self->structs, name);
}


gboolean vsc_symbol_completion_result_constants_contain (VscSymbolCompletionResult* self, const char* name) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	return vsc_symbol_completion_result_symbols_contain (self, self->constants, name);
}


gboolean vsc_symbol_completion_result_enums_contain (VscSymbolCompletionResult* self, const char* name) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	return vsc_symbol_completion_result_symbols_contain (self, self->enums, name);
}


static gboolean vsc_symbol_completion_result_symbols_contain (VscSymbolCompletionResult* self, GeeList* data, const char* name) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (data != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	if (gee_collection_get_size ((GeeCollection*) data) == 0) {
		return FALSE;
	}
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator ((GeeIterable*) data);
		while (gee_iterator_next (_item_it)) {
			VscSymbolCompletionItem* item;
			item = (VscSymbolCompletionItem*) gee_iterator_get (_item_it);
			if (_vala_strcmp0 (item->name, name) == 0) {
				gboolean _tmp1;
				return (_tmp1 = TRUE, (item == NULL) ? NULL : (item = (g_object_unref (item), NULL)), (_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL)), _tmp1);
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
	}
	return FALSE;
}


VscSymbolCompletionResult* vsc_symbol_completion_result_construct (GType object_type) {
	VscSymbolCompletionResult * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VscSymbolCompletionResult* vsc_symbol_completion_result_new (void) {
	return vsc_symbol_completion_result_construct (VSC_TYPE_SYMBOL_COMPLETION_RESULT);
}


gboolean vsc_symbol_completion_result_get_is_empty (VscSymbolCompletionResult* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return vsc_symbol_completion_result_get_count (self) == 0;
}


gint vsc_symbol_completion_result_get_count (VscSymbolCompletionResult* self) {
	g_return_val_if_fail (self != NULL, 0);
	return ((((((((((gee_collection_get_size ((GeeCollection*) self->properties) + gee_collection_get_size ((GeeCollection*) self->classes)) + gee_collection_get_size ((GeeCollection*) self->interfaces)) + gee_collection_get_size ((GeeCollection*) self->structs)) + gee_collection_get_size ((GeeCollection*) self->methods)) + gee_collection_get_size ((GeeCollection*) self->fields)) + gee_collection_get_size ((GeeCollection*) self->signals)) + gee_collection_get_size ((GeeCollection*) self->namespaces)) + gee_collection_get_size ((GeeCollection*) self->enums)) + gee_collection_get_size ((GeeCollection*) self->error_domains)) + gee_collection_get_size ((GeeCollection*) self->constants)) + gee_collection_get_size ((GeeCollection*) self->others);
}


static void vsc_symbol_completion_result_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VscSymbolCompletionResult * self;
	gpointer boxed;
	self = VSC_SYMBOL_COMPLETION_RESULT (object);
	switch (property_id) {
		case VSC_SYMBOL_COMPLETION_RESULT_IS_EMPTY:
		g_value_set_boolean (value, vsc_symbol_completion_result_get_is_empty (self));
		break;
		case VSC_SYMBOL_COMPLETION_RESULT_COUNT:
		g_value_set_int (value, vsc_symbol_completion_result_get_count (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vsc_symbol_completion_result_class_init (VscSymbolCompletionResultClass * klass) {
	vsc_symbol_completion_result_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->get_property = vsc_symbol_completion_result_get_property;
	G_OBJECT_CLASS (klass)->finalize = vsc_symbol_completion_result_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VSC_SYMBOL_COMPLETION_RESULT_IS_EMPTY, g_param_spec_boolean ("is-empty", "is-empty", "is-empty", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VSC_SYMBOL_COMPLETION_RESULT_COUNT, g_param_spec_int ("count", "count", "count", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vsc_symbol_completion_result_instance_init (VscSymbolCompletionResult * self) {
	self->properties = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->classes = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->interfaces = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->structs = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->methods = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->fields = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->signals = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->others = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->namespaces = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->enums = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->constants = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->error_domains = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
}


static void vsc_symbol_completion_result_finalize (GObject* obj) {
	VscSymbolCompletionResult * self;
	self = VSC_SYMBOL_COMPLETION_RESULT (obj);
	(self->properties == NULL) ? NULL : (self->properties = (gee_collection_object_unref (self->properties), NULL));
	(self->classes == NULL) ? NULL : (self->classes = (gee_collection_object_unref (self->classes), NULL));
	(self->interfaces == NULL) ? NULL : (self->interfaces = (gee_collection_object_unref (self->interfaces), NULL));
	(self->structs == NULL) ? NULL : (self->structs = (gee_collection_object_unref (self->structs), NULL));
	(self->methods == NULL) ? NULL : (self->methods = (gee_collection_object_unref (self->methods), NULL));
	(self->fields == NULL) ? NULL : (self->fields = (gee_collection_object_unref (self->fields), NULL));
	(self->signals == NULL) ? NULL : (self->signals = (gee_collection_object_unref (self->signals), NULL));
	(self->others == NULL) ? NULL : (self->others = (gee_collection_object_unref (self->others), NULL));
	(self->namespaces == NULL) ? NULL : (self->namespaces = (gee_collection_object_unref (self->namespaces), NULL));
	(self->enums == NULL) ? NULL : (self->enums = (gee_collection_object_unref (self->enums), NULL));
	(self->constants == NULL) ? NULL : (self->constants = (gee_collection_object_unref (self->constants), NULL));
	(self->error_domains == NULL) ? NULL : (self->error_domains = (gee_collection_object_unref (self->error_domains), NULL));
	G_OBJECT_CLASS (vsc_symbol_completion_result_parent_class)->finalize (obj);
}


GType vsc_symbol_completion_result_get_type (void) {
	static GType vsc_symbol_completion_result_type_id = 0;
	if (vsc_symbol_completion_result_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscSymbolCompletionResultClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_symbol_completion_result_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscSymbolCompletionResult), 0, (GInstanceInitFunc) vsc_symbol_completion_result_instance_init, NULL };
		vsc_symbol_completion_result_type_id = g_type_register_static (G_TYPE_OBJECT, "VscSymbolCompletionResult", &g_define_type_info, 0);
	}
	return vsc_symbol_completion_result_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




