/*
 *  vscsymbolcompletionresult.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#ifndef __VSCSYMBOLCOMPLETIONRESULT_H__
#define __VSCSYMBOLCOMPLETIONRESULT_H__

#include <glib.h>
#include <glib-object.h>
#include <gee/list.h>
#include <stdlib.h>
#include <string.h>

G_BEGIN_DECLS


#define VSC_TYPE_SYMBOL_COMPLETION_RESULT (vsc_symbol_completion_result_get_type ())
#define VSC_SYMBOL_COMPLETION_RESULT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_SYMBOL_COMPLETION_RESULT, VscSymbolCompletionResult))
#define VSC_SYMBOL_COMPLETION_RESULT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_SYMBOL_COMPLETION_RESULT, VscSymbolCompletionResultClass))
#define VSC_IS_SYMBOL_COMPLETION_RESULT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_SYMBOL_COMPLETION_RESULT))
#define VSC_IS_SYMBOL_COMPLETION_RESULT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_SYMBOL_COMPLETION_RESULT))
#define VSC_SYMBOL_COMPLETION_RESULT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_SYMBOL_COMPLETION_RESULT, VscSymbolCompletionResultClass))

typedef struct _VscSymbolCompletionResult VscSymbolCompletionResult;
typedef struct _VscSymbolCompletionResultClass VscSymbolCompletionResultClass;
typedef struct _VscSymbolCompletionResultPrivate VscSymbolCompletionResultPrivate;

struct _VscSymbolCompletionResult {
	GObject parent_instance;
	VscSymbolCompletionResultPrivate * priv;
	GeeList* properties;
	GeeList* classes;
	GeeList* interfaces;
	GeeList* structs;
	GeeList* methods;
	GeeList* fields;
	GeeList* signals;
	GeeList* others;
	GeeList* namespaces;
	GeeList* enums;
	GeeList* constants;
	GeeList* error_domains;
};

struct _VscSymbolCompletionResultClass {
	GObjectClass parent_class;
};


gboolean vsc_symbol_completion_result_namespaces_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_classes_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_interfaces_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_structs_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_constants_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_enums_contain (VscSymbolCompletionResult* self, const char* name);
VscSymbolCompletionResult* vsc_symbol_completion_result_construct (GType object_type);
VscSymbolCompletionResult* vsc_symbol_completion_result_new (void);
gboolean vsc_symbol_completion_result_get_is_empty (VscSymbolCompletionResult* self);
gint vsc_symbol_completion_result_get_count (VscSymbolCompletionResult* self);
GType vsc_symbol_completion_result_get_type (void);


G_END_DECLS

#endif
