/*
 *  vsctypefindvisitor.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vsctypefindvisitor.h>
#include <vala/valacodenode.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vala/valaobjecttypesymbol.h>
#include <vala/valaunresolvedtype.h>
#include <vala/valascope.h>
#include <vala/valausingdirective.h>
#include <vala/valaunresolvedsymbol.h>
#include <vscsymbolcompletion.h>




struct _VscTypeFinderVisitorPrivate {
	char* _current_typename;
	ValaCodeContext* _context;
	ValaSourceFile* _current_file;
	ValaSymbol* _result;
	char* _searched_typename;
};

#define VSC_TYPEFINDER_VISITOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VSC_TYPE_TYPEFINDER_VISITOR, VscTypeFinderVisitorPrivate))
enum  {
	VSC_TYPEFINDER_VISITOR_DUMMY_PROPERTY
};
static void vsc_typefinder_visitor_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vsc_typefinder_visitor_real_visit_class (ValaCodeVisitor* base, ValaClass* cl);
static void vsc_typefinder_visitor_real_visit_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vsc_typefinder_visitor_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vsc_typefinder_visitor_real_visit_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vsc_typefinder_visitor_real_visit_error_domain (ValaCodeVisitor* base, ValaErrorDomain* ed);
static void vsc_typefinder_visitor_real_visit_method (ValaCodeVisitor* base, ValaMethod* m);
static void vsc_typefinder_visitor_real_visit_property (ValaCodeVisitor* base, ValaProperty* p);
static void vsc_typefinder_visitor_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vsc_typefinder_visitor_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
static void vsc_typefinder_visitor_real_visit_data_type (ValaCodeVisitor* base, ValaDataType* data_type);
static ValaSymbol* vsc_typefinder_visitor_resolve_type (VscTypeFinderVisitor* self, ValaUnresolvedType* unresolved_type);
static char* vsc_typefinder_visitor_get_qualified_namespace_name (VscTypeFinderVisitor* self, ValaSymbol* namespace_symbol);
static gpointer vsc_typefinder_visitor_parent_class = NULL;
static void vsc_typefinder_visitor_finalize (ValaCodeVisitor* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



VscTypeFinderVisitor* vsc_typefinder_visitor_construct (GType object_type, ValaSourceFile* source, ValaCodeContext* context) {
	VscTypeFinderVisitor* self;
	ValaCodeContext* _tmp1;
	ValaCodeContext* _tmp0;
	ValaSourceFile* _tmp3;
	ValaSourceFile* _tmp2;
	self = (VscTypeFinderVisitor*) g_type_create_instance (object_type);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_context = (_tmp1 = (_tmp0 = context, (_tmp0 == NULL) ? NULL : vala_code_context_ref (_tmp0)), (self->priv->_context == NULL) ? NULL : (self->priv->_context = (vala_code_context_unref (self->priv->_context), NULL)), _tmp1);
	_tmp3 = NULL;
	_tmp2 = NULL;
	self->priv->_current_file = (_tmp3 = (_tmp2 = source, (_tmp2 == NULL) ? NULL : vala_source_file_ref (_tmp2)), (self->priv->_current_file == NULL) ? NULL : (self->priv->_current_file = (vala_source_file_unref (self->priv->_current_file), NULL)), _tmp3);
	return self;
}


VscTypeFinderVisitor* vsc_typefinder_visitor_new (ValaSourceFile* source, ValaCodeContext* context) {
	return vsc_typefinder_visitor_construct (VSC_TYPE_TYPEFINDER_VISITOR, source, context);
}


static void vsc_typefinder_visitor_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns) {
	VscTypeFinderVisitor * self;
	const char* _tmp0;
	char* previous_typename;
	char* _tmp9;
	const char* _tmp8;
	self = (VscTypeFinderVisitor*) base;
	g_return_if_fail (ns != NULL);
	if (self->priv->_result != NULL) {
		return;
	}
	_tmp0 = NULL;
	previous_typename = (_tmp0 = self->priv->_current_typename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (self->priv->_current_typename == NULL) {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->_current_typename = (_tmp2 = (_tmp1 = vala_symbol_get_name ((ValaSymbol*) ns), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp2);
	} else {
		char* _tmp3;
		_tmp3 = NULL;
		self->priv->_current_typename = (_tmp3 = g_strdup_printf ("%s.%s", self->priv->_current_typename, vala_symbol_get_name ((ValaSymbol*) ns)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp3);
	}
	if (_vala_strcmp0 (self->priv->_current_typename, self->priv->_searched_typename) == 0) {
		ValaSymbol* _tmp5;
		ValaSymbol* _tmp4;
		char* _tmp7;
		const char* _tmp6;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->_result = (_tmp5 = (_tmp4 = (ValaSymbol*) ns, (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL)), _tmp5);
		_tmp7 = NULL;
		_tmp6 = NULL;
		self->qualified_typename = (_tmp7 = (_tmp6 = self->priv->_current_typename, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), self->qualified_typename = (g_free (self->qualified_typename), NULL), _tmp7);
	} else {
		vala_code_node_accept_children ((ValaCodeNode*) ns, (ValaCodeVisitor*) self);
	}
	_tmp9 = NULL;
	_tmp8 = NULL;
	self->priv->_current_typename = (_tmp9 = (_tmp8 = previous_typename, (_tmp8 == NULL) ? NULL : g_strdup (_tmp8)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp9);
	previous_typename = (g_free (previous_typename), NULL);
}


static void vsc_typefinder_visitor_real_visit_class (ValaCodeVisitor* base, ValaClass* cl) {
	VscTypeFinderVisitor * self;
	const char* _tmp0;
	char* previous_typename;
	char* _tmp25;
	const char* _tmp24;
	self = (VscTypeFinderVisitor*) base;
	g_return_if_fail (cl != NULL);
	if (self->priv->_result != NULL) {
		return;
	}
	_tmp0 = NULL;
	previous_typename = (_tmp0 = self->priv->_current_typename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (self->priv->_current_typename == NULL) {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->_current_typename = (_tmp2 = (_tmp1 = vala_symbol_get_name ((ValaSymbol*) cl), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp2);
	} else {
		char* _tmp3;
		_tmp3 = NULL;
		self->priv->_current_typename = (_tmp3 = g_strdup_printf ("%s.%s", self->priv->_current_typename, vala_symbol_get_name ((ValaSymbol*) cl)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp3);
	}
	if (_vala_strcmp0 (self->priv->_current_typename, self->priv->_searched_typename) == 0) {
		ValaSymbol* _tmp5;
		ValaSymbol* _tmp4;
		char* _tmp7;
		const char* _tmp6;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->_result = (_tmp5 = (_tmp4 = (ValaSymbol*) cl, (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL)), _tmp5);
		_tmp7 = NULL;
		_tmp6 = NULL;
		self->qualified_typename = (_tmp7 = (_tmp6 = self->priv->_current_typename, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), self->qualified_typename = (g_free (self->qualified_typename), NULL), _tmp7);
	} else {
		/*cl.accept_children (this);
		Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp8;
				GeeIterator* _tmp9;
				GeeIterator* _type_it;
				_tmp8 = NULL;
				_tmp9 = NULL;
				_type_it = (_tmp9 = gee_iterable_iterator ((GeeIterable*) (_tmp8 = vala_class_get_base_types (cl))), (_tmp8 == NULL) ? NULL : (_tmp8 = (gee_collection_object_unref (_tmp8), NULL)), _tmp9);
				while (gee_iterator_next (_type_it)) {
					ValaDataType* type;
					type = (ValaDataType*) gee_iterator_get (_type_it);
					if (type != NULL) {
						vala_code_node_accept ((ValaCodeNode*) type, (ValaCodeVisitor*) self);
						if (self->priv->_result != NULL) {
							(type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL));
							break;
						}
					}
					(type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL));
				}
				(_type_it == NULL) ? NULL : (_type_it = (gee_collection_object_unref (_type_it), NULL));
			}
		}
		/* process enums first to avoid order problems in C code 
		Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp10;
				GeeIterator* _tmp11;
				GeeIterator* _en_it;
				_tmp10 = NULL;
				_tmp11 = NULL;
				_en_it = (_tmp11 = gee_iterable_iterator ((GeeIterable*) (_tmp10 = vala_class_get_enums (cl))), (_tmp10 == NULL) ? NULL : (_tmp10 = (gee_collection_object_unref (_tmp10), NULL)), _tmp11);
				while (gee_iterator_next (_en_it)) {
					ValaEnum* en;
					en = (ValaEnum*) gee_iterator_get (_en_it);
					vala_code_node_accept ((ValaCodeNode*) en, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(en == NULL) ? NULL : (en = (vala_code_node_unref (en), NULL));
						break;
					}
					(en == NULL) ? NULL : (en = (vala_code_node_unref (en), NULL));
				}
				(_en_it == NULL) ? NULL : (_en_it = (gee_collection_object_unref (_en_it), NULL));
			}
		}
		/*Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp12;
				GeeIterator* _tmp13;
				GeeIterator* _f_it;
				_tmp12 = NULL;
				_tmp13 = NULL;
				_f_it = (_tmp13 = gee_iterable_iterator ((GeeIterable*) (_tmp12 = vala_class_get_fields (cl))), (_tmp12 == NULL) ? NULL : (_tmp12 = (gee_collection_object_unref (_tmp12), NULL)), _tmp13);
				while (gee_iterator_next (_f_it)) {
					ValaField* f;
					f = (ValaField*) gee_iterator_get (_f_it);
					vala_code_node_accept ((ValaCodeNode*) f, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(f == NULL) ? NULL : (f = (vala_code_node_unref (f), NULL));
						break;
					}
					(f == NULL) ? NULL : (f = (vala_code_node_unref (f), NULL));
				}
				(_f_it == NULL) ? NULL : (_f_it = (gee_collection_object_unref (_f_it), NULL));
			}
		}
		/*Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp14;
				GeeIterator* _tmp15;
				GeeIterator* _c_it;
				_tmp14 = NULL;
				_tmp15 = NULL;
				_c_it = (_tmp15 = gee_iterable_iterator ((GeeIterable*) (_tmp14 = vala_class_get_constants (cl))), (_tmp14 == NULL) ? NULL : (_tmp14 = (gee_collection_object_unref (_tmp14), NULL)), _tmp15);
				while (gee_iterator_next (_c_it)) {
					ValaConstant* c;
					c = (ValaConstant*) gee_iterator_get (_c_it);
					vala_code_node_accept ((ValaCodeNode*) c, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(c == NULL) ? NULL : (c = (vala_code_node_unref (c), NULL));
						break;
					}
					(c == NULL) ? NULL : (c = (vala_code_node_unref (c), NULL));
				}
				(_c_it == NULL) ? NULL : (_c_it = (gee_collection_object_unref (_c_it), NULL));
			}
		}
		/*Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp16;
				GeeIterator* _tmp17;
				GeeIterator* _m_it;
				_tmp16 = NULL;
				_tmp17 = NULL;
				_m_it = (_tmp17 = gee_iterable_iterator ((GeeIterable*) (_tmp16 = vala_object_type_symbol_get_methods ((ValaObjectTypeSymbol*) cl))), (_tmp16 == NULL) ? NULL : (_tmp16 = (gee_collection_object_unref (_tmp16), NULL)), _tmp17);
				while (gee_iterator_next (_m_it)) {
					ValaMethod* m;
					m = (ValaMethod*) gee_iterator_get (_m_it);
					vala_code_node_accept ((ValaCodeNode*) m, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
						break;
					}
					(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
				}
				(_m_it == NULL) ? NULL : (_m_it = (gee_collection_object_unref (_m_it), NULL));
			}
		}
		/*Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp18;
				GeeIterator* _tmp19;
				GeeIterator* _prop_it;
				_tmp18 = NULL;
				_tmp19 = NULL;
				_prop_it = (_tmp19 = gee_iterable_iterator ((GeeIterable*) (_tmp18 = vala_object_type_symbol_get_properties ((ValaObjectTypeSymbol*) cl))), (_tmp18 == NULL) ? NULL : (_tmp18 = (gee_collection_object_unref (_tmp18), NULL)), _tmp19);
				while (gee_iterator_next (_prop_it)) {
					ValaProperty* prop;
					prop = (ValaProperty*) gee_iterator_get (_prop_it);
					vala_code_node_accept ((ValaCodeNode*) prop, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
						break;
					}
					(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
				}
				(_prop_it == NULL) ? NULL : (_prop_it = (gee_collection_object_unref (_prop_it), NULL));
			}
		}
		/*Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp20;
				GeeIterator* _tmp21;
				GeeIterator* _subcl_it;
				_tmp20 = NULL;
				_tmp21 = NULL;
				_subcl_it = (_tmp21 = gee_iterable_iterator ((GeeIterable*) (_tmp20 = vala_class_get_classes (cl))), (_tmp20 == NULL) ? NULL : (_tmp20 = (gee_collection_object_unref (_tmp20), NULL)), _tmp21);
				while (gee_iterator_next (_subcl_it)) {
					ValaClass* subcl;
					subcl = (ValaClass*) gee_iterator_get (_subcl_it);
					vala_code_node_accept ((ValaCodeNode*) subcl, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(subcl == NULL) ? NULL : (subcl = (vala_code_node_unref (subcl), NULL));
						break;
					}
					(subcl == NULL) ? NULL : (subcl = (vala_code_node_unref (subcl), NULL));
				}
				(_subcl_it == NULL) ? NULL : (_subcl_it = (gee_collection_object_unref (_subcl_it), NULL));
			}
		}
		/*Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp22;
				GeeIterator* _tmp23;
				GeeIterator* _st_it;
				_tmp22 = NULL;
				_tmp23 = NULL;
				_st_it = (_tmp23 = gee_iterable_iterator ((GeeIterable*) (_tmp22 = vala_class_get_structs (cl))), (_tmp22 == NULL) ? NULL : (_tmp22 = (gee_collection_object_unref (_tmp22), NULL)), _tmp23);
				while (gee_iterator_next (_st_it)) {
					ValaStruct* st;
					st = (ValaStruct*) gee_iterator_get (_st_it);
					vala_code_node_accept ((ValaCodeNode*) st, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
						break;
					}
					(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
				}
				(_st_it == NULL) ? NULL : (_st_it = (gee_collection_object_unref (_st_it), NULL));
			}
		}
	}
	_tmp25 = NULL;
	_tmp24 = NULL;
	self->priv->_current_typename = (_tmp25 = (_tmp24 = previous_typename, (_tmp24 == NULL) ? NULL : g_strdup (_tmp24)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp25);
	previous_typename = (g_free (previous_typename), NULL);
}


static void vsc_typefinder_visitor_real_visit_interface (ValaCodeVisitor* base, ValaInterface* iface) {
	VscTypeFinderVisitor * self;
	const char* _tmp0;
	char* previous_typename;
	char* _tmp23;
	const char* _tmp22;
	self = (VscTypeFinderVisitor*) base;
	g_return_if_fail (iface != NULL);
	if (self->priv->_result != NULL) {
		return;
	}
	_tmp0 = NULL;
	previous_typename = (_tmp0 = self->priv->_current_typename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (self->priv->_current_typename == NULL) {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->_current_typename = (_tmp2 = (_tmp1 = vala_symbol_get_name ((ValaSymbol*) iface), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp2);
	} else {
		char* _tmp3;
		_tmp3 = NULL;
		self->priv->_current_typename = (_tmp3 = g_strdup_printf ("%s.%s", self->priv->_current_typename, vala_symbol_get_name ((ValaSymbol*) iface)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp3);
	}
	if (_vala_strcmp0 (self->priv->_current_typename, self->priv->_searched_typename) == 0) {
		ValaSymbol* _tmp5;
		ValaSymbol* _tmp4;
		char* _tmp7;
		const char* _tmp6;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->_result = (_tmp5 = (_tmp4 = (ValaSymbol*) iface, (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL)), _tmp5);
		_tmp7 = NULL;
		_tmp6 = NULL;
		self->qualified_typename = (_tmp7 = (_tmp6 = self->priv->_current_typename, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), self->qualified_typename = (g_free (self->qualified_typename), NULL), _tmp7);
	} else {
		/*cl.accept_children (this);
		Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp8;
				GeeIterator* _tmp9;
				GeeIterator* _type_it;
				_tmp8 = NULL;
				_tmp9 = NULL;
				_type_it = (_tmp9 = gee_iterable_iterator ((GeeIterable*) (_tmp8 = vala_interface_get_prerequisites (iface))), (_tmp8 == NULL) ? NULL : (_tmp8 = (gee_collection_object_unref (_tmp8), NULL)), _tmp9);
				while (gee_iterator_next (_type_it)) {
					ValaDataType* type;
					type = (ValaDataType*) gee_iterator_get (_type_it);
					if (type != NULL) {
						vala_code_node_accept ((ValaCodeNode*) type, (ValaCodeVisitor*) self);
						if (self->priv->_result != NULL) {
							(type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL));
							break;
						}
					}
					(type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL));
				}
				(_type_it == NULL) ? NULL : (_type_it = (gee_collection_object_unref (_type_it), NULL));
			}
		}
		/* process enums first to avoid order problems in C code 
		Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp10;
				GeeIterator* _tmp11;
				GeeIterator* _en_it;
				_tmp10 = NULL;
				_tmp11 = NULL;
				_en_it = (_tmp11 = gee_iterable_iterator ((GeeIterable*) (_tmp10 = vala_interface_get_enums (iface))), (_tmp10 == NULL) ? NULL : (_tmp10 = (gee_collection_object_unref (_tmp10), NULL)), _tmp11);
				while (gee_iterator_next (_en_it)) {
					ValaEnum* en;
					en = (ValaEnum*) gee_iterator_get (_en_it);
					vala_code_node_accept ((ValaCodeNode*) en, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(en == NULL) ? NULL : (en = (vala_code_node_unref (en), NULL));
						break;
					}
					(en == NULL) ? NULL : (en = (vala_code_node_unref (en), NULL));
				}
				(_en_it == NULL) ? NULL : (_en_it = (gee_collection_object_unref (_en_it), NULL));
			}
		}
		/*Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp12;
				GeeIterator* _tmp13;
				GeeIterator* _f_it;
				_tmp12 = NULL;
				_tmp13 = NULL;
				_f_it = (_tmp13 = gee_iterable_iterator ((GeeIterable*) (_tmp12 = vala_interface_get_fields (iface))), (_tmp12 == NULL) ? NULL : (_tmp12 = (gee_collection_object_unref (_tmp12), NULL)), _tmp13);
				while (gee_iterator_next (_f_it)) {
					ValaField* f;
					f = (ValaField*) gee_iterator_get (_f_it);
					vala_code_node_accept ((ValaCodeNode*) f, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(f == NULL) ? NULL : (f = (vala_code_node_unref (f), NULL));
						break;
					}
					(f == NULL) ? NULL : (f = (vala_code_node_unref (f), NULL));
				}
				(_f_it == NULL) ? NULL : (_f_it = (gee_collection_object_unref (_f_it), NULL));
			}
		}
		/*Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp14;
				GeeIterator* _tmp15;
				GeeIterator* _m_it;
				_tmp14 = NULL;
				_tmp15 = NULL;
				_m_it = (_tmp15 = gee_iterable_iterator ((GeeIterable*) (_tmp14 = vala_object_type_symbol_get_methods ((ValaObjectTypeSymbol*) iface))), (_tmp14 == NULL) ? NULL : (_tmp14 = (gee_collection_object_unref (_tmp14), NULL)), _tmp15);
				while (gee_iterator_next (_m_it)) {
					ValaMethod* m;
					m = (ValaMethod*) gee_iterator_get (_m_it);
					vala_code_node_accept ((ValaCodeNode*) m, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
						break;
					}
					(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
				}
				(_m_it == NULL) ? NULL : (_m_it = (gee_collection_object_unref (_m_it), NULL));
			}
		}
		/*Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp16;
				GeeIterator* _tmp17;
				GeeIterator* _prop_it;
				_tmp16 = NULL;
				_tmp17 = NULL;
				_prop_it = (_tmp17 = gee_iterable_iterator ((GeeIterable*) (_tmp16 = vala_object_type_symbol_get_properties ((ValaObjectTypeSymbol*) iface))), (_tmp16 == NULL) ? NULL : (_tmp16 = (gee_collection_object_unref (_tmp16), NULL)), _tmp17);
				while (gee_iterator_next (_prop_it)) {
					ValaProperty* prop;
					prop = (ValaProperty*) gee_iterator_get (_prop_it);
					vala_code_node_accept ((ValaCodeNode*) prop, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
						break;
					}
					(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
				}
				(_prop_it == NULL) ? NULL : (_prop_it = (gee_collection_object_unref (_prop_it), NULL));
			}
		}
		/*Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp18;
				GeeIterator* _tmp19;
				GeeIterator* _cl_it;
				_tmp18 = NULL;
				_tmp19 = NULL;
				_cl_it = (_tmp19 = gee_iterable_iterator ((GeeIterable*) (_tmp18 = vala_interface_get_classes (iface))), (_tmp18 == NULL) ? NULL : (_tmp18 = (gee_collection_object_unref (_tmp18), NULL)), _tmp19);
				while (gee_iterator_next (_cl_it)) {
					ValaClass* cl;
					cl = (ValaClass*) gee_iterator_get (_cl_it);
					vala_code_node_accept ((ValaCodeNode*) cl, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
						break;
					}
					(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
				}
				(_cl_it == NULL) ? NULL : (_cl_it = (gee_collection_object_unref (_cl_it), NULL));
			}
		}
		/*Minor optimization*/
		if (self->priv->_result == NULL) {
			{
				GeeList* _tmp20;
				GeeIterator* _tmp21;
				GeeIterator* _st_it;
				_tmp20 = NULL;
				_tmp21 = NULL;
				_st_it = (_tmp21 = gee_iterable_iterator ((GeeIterable*) (_tmp20 = vala_interface_get_structs (iface))), (_tmp20 == NULL) ? NULL : (_tmp20 = (gee_collection_object_unref (_tmp20), NULL)), _tmp21);
				while (gee_iterator_next (_st_it)) {
					ValaStruct* st;
					st = (ValaStruct*) gee_iterator_get (_st_it);
					vala_code_node_accept ((ValaCodeNode*) st, (ValaCodeVisitor*) self);
					if (self->priv->_result != NULL) {
						(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
						break;
					}
					(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
				}
				(_st_it == NULL) ? NULL : (_st_it = (gee_collection_object_unref (_st_it), NULL));
			}
		}
	}
	_tmp23 = NULL;
	_tmp22 = NULL;
	self->priv->_current_typename = (_tmp23 = (_tmp22 = previous_typename, (_tmp22 == NULL) ? NULL : g_strdup (_tmp22)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp23);
	previous_typename = (g_free (previous_typename), NULL);
}


static void vsc_typefinder_visitor_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st) {
	VscTypeFinderVisitor * self;
	const char* _tmp0;
	char* previous_typename;
	char* _tmp9;
	const char* _tmp8;
	self = (VscTypeFinderVisitor*) base;
	g_return_if_fail (st != NULL);
	if (self->priv->_result != NULL) {
		return;
	}
	_tmp0 = NULL;
	previous_typename = (_tmp0 = self->priv->_current_typename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (self->priv->_current_typename == NULL) {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->_current_typename = (_tmp2 = (_tmp1 = vala_symbol_get_name ((ValaSymbol*) st), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp2);
	} else {
		char* _tmp3;
		_tmp3 = NULL;
		self->priv->_current_typename = (_tmp3 = g_strdup_printf ("%s.%s", self->priv->_current_typename, vala_symbol_get_name ((ValaSymbol*) st)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp3);
	}
	if (_vala_strcmp0 (self->priv->_current_typename, self->priv->_searched_typename) == 0) {
		ValaSymbol* _tmp5;
		ValaSymbol* _tmp4;
		char* _tmp7;
		const char* _tmp6;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->_result = (_tmp5 = (_tmp4 = (ValaSymbol*) st, (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL)), _tmp5);
		_tmp7 = NULL;
		_tmp6 = NULL;
		self->qualified_typename = (_tmp7 = (_tmp6 = self->priv->_current_typename, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), self->qualified_typename = (g_free (self->qualified_typename), NULL), _tmp7);
	} else {
		vala_code_node_accept_children ((ValaCodeNode*) st, (ValaCodeVisitor*) self);
	}
	_tmp9 = NULL;
	_tmp8 = NULL;
	self->priv->_current_typename = (_tmp9 = (_tmp8 = previous_typename, (_tmp8 == NULL) ? NULL : g_strdup (_tmp8)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp9);
	previous_typename = (g_free (previous_typename), NULL);
}


static void vsc_typefinder_visitor_real_visit_enum (ValaCodeVisitor* base, ValaEnum* en) {
	VscTypeFinderVisitor * self;
	const char* _tmp0;
	char* previous_typename;
	char* _tmp9;
	const char* _tmp8;
	self = (VscTypeFinderVisitor*) base;
	g_return_if_fail (en != NULL);
	if (self->priv->_result != NULL) {
		return;
	}
	_tmp0 = NULL;
	previous_typename = (_tmp0 = self->priv->_current_typename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (self->priv->_current_typename == NULL) {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->_current_typename = (_tmp2 = (_tmp1 = vala_symbol_get_name ((ValaSymbol*) en), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp2);
	} else {
		char* _tmp3;
		_tmp3 = NULL;
		self->priv->_current_typename = (_tmp3 = g_strdup_printf ("%s.%s", self->priv->_current_typename, vala_symbol_get_name ((ValaSymbol*) en)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp3);
	}
	if (_vala_strcmp0 (self->priv->_current_typename, self->priv->_searched_typename) == 0) {
		ValaSymbol* _tmp5;
		ValaSymbol* _tmp4;
		char* _tmp7;
		const char* _tmp6;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->_result = (_tmp5 = (_tmp4 = (ValaSymbol*) en, (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL)), _tmp5);
		_tmp7 = NULL;
		_tmp6 = NULL;
		self->qualified_typename = (_tmp7 = (_tmp6 = self->priv->_current_typename, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), self->qualified_typename = (g_free (self->qualified_typename), NULL), _tmp7);
	} else {
		vala_code_node_accept_children ((ValaCodeNode*) en, (ValaCodeVisitor*) self);
	}
	_tmp9 = NULL;
	_tmp8 = NULL;
	self->priv->_current_typename = (_tmp9 = (_tmp8 = previous_typename, (_tmp8 == NULL) ? NULL : g_strdup (_tmp8)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp9);
	previous_typename = (g_free (previous_typename), NULL);
}


static void vsc_typefinder_visitor_real_visit_error_domain (ValaCodeVisitor* base, ValaErrorDomain* ed) {
	VscTypeFinderVisitor * self;
	const char* _tmp0;
	char* previous_typename;
	char* _tmp9;
	const char* _tmp8;
	self = (VscTypeFinderVisitor*) base;
	g_return_if_fail (ed != NULL);
	if (self->priv->_result != NULL) {
		return;
	}
	_tmp0 = NULL;
	previous_typename = (_tmp0 = self->priv->_current_typename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (self->priv->_current_typename == NULL) {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->_current_typename = (_tmp2 = (_tmp1 = vala_symbol_get_name ((ValaSymbol*) ed), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp2);
	} else {
		char* _tmp3;
		_tmp3 = NULL;
		self->priv->_current_typename = (_tmp3 = g_strdup_printf ("%s.%s", self->priv->_current_typename, vala_symbol_get_name ((ValaSymbol*) ed)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp3);
	}
	if (_vala_strcmp0 (self->priv->_current_typename, self->priv->_searched_typename) == 0) {
		ValaSymbol* _tmp5;
		ValaSymbol* _tmp4;
		char* _tmp7;
		const char* _tmp6;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->_result = (_tmp5 = (_tmp4 = (ValaSymbol*) ed, (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL)), _tmp5);
		_tmp7 = NULL;
		_tmp6 = NULL;
		self->qualified_typename = (_tmp7 = (_tmp6 = self->priv->_current_typename, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), self->qualified_typename = (g_free (self->qualified_typename), NULL), _tmp7);
	} else {
		vala_code_node_accept_children ((ValaCodeNode*) ed, (ValaCodeVisitor*) self);
	}
	_tmp9 = NULL;
	_tmp8 = NULL;
	self->priv->_current_typename = (_tmp9 = (_tmp8 = previous_typename, (_tmp8 == NULL) ? NULL : g_strdup (_tmp8)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp9);
	previous_typename = (g_free (previous_typename), NULL);
}


static void vsc_typefinder_visitor_real_visit_method (ValaCodeVisitor* base, ValaMethod* m) {
	VscTypeFinderVisitor * self;
	const char* _tmp0;
	char* previous_typename;
	char* _tmp8;
	const char* _tmp7;
	self = (VscTypeFinderVisitor*) base;
	g_return_if_fail (m != NULL);
	if (self->priv->_result != NULL) {
		return;
	}
	_tmp0 = NULL;
	previous_typename = (_tmp0 = self->priv->_current_typename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (self->priv->_current_typename == NULL) {
		char* _tmp1;
		_tmp1 = NULL;
		self->priv->_current_typename = (_tmp1 = g_strdup_printf ("%s", vala_symbol_get_name ((ValaSymbol*) m)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp1);
	} else {
		char* _tmp2;
		_tmp2 = NULL;
		self->priv->_current_typename = (_tmp2 = g_strdup_printf ("%s.%s", self->priv->_current_typename, vala_symbol_get_name ((ValaSymbol*) m)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp2);
	}
	if (_vala_strcmp0 (self->priv->_current_typename, self->priv->_searched_typename) == 0) {
		ValaSymbol* _tmp4;
		ValaSymbol* _tmp3;
		char* _tmp6;
		const char* _tmp5;
		_tmp4 = NULL;
		_tmp3 = NULL;
		self->priv->_result = (_tmp4 = (_tmp3 = (ValaSymbol*) m, (_tmp3 == NULL) ? NULL : vala_code_node_ref (_tmp3)), (self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL)), _tmp4);
		_tmp6 = NULL;
		_tmp5 = NULL;
		self->qualified_typename = (_tmp6 = (_tmp5 = self->priv->_current_typename, (_tmp5 == NULL) ? NULL : g_strdup (_tmp5)), self->qualified_typename = (g_free (self->qualified_typename), NULL), _tmp6);
	}
	_tmp8 = NULL;
	_tmp7 = NULL;
	self->priv->_current_typename = (_tmp8 = (_tmp7 = previous_typename, (_tmp7 == NULL) ? NULL : g_strdup (_tmp7)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp8);
	previous_typename = (g_free (previous_typename), NULL);
}


static void vsc_typefinder_visitor_real_visit_property (ValaCodeVisitor* base, ValaProperty* p) {
	VscTypeFinderVisitor * self;
	const char* _tmp0;
	char* previous_typename;
	char* _tmp9;
	const char* _tmp8;
	self = (VscTypeFinderVisitor*) base;
	g_return_if_fail (p != NULL);
	if (self->priv->_result != NULL) {
		return;
	}
	_tmp0 = NULL;
	previous_typename = (_tmp0 = self->priv->_current_typename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (self->priv->_current_typename == NULL) {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->_current_typename = (_tmp2 = (_tmp1 = vala_symbol_get_name ((ValaSymbol*) p), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp2);
	} else {
		char* _tmp3;
		_tmp3 = NULL;
		self->priv->_current_typename = (_tmp3 = g_strdup_printf ("%s.%s", self->priv->_current_typename, vala_symbol_get_name ((ValaSymbol*) p)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp3);
	}
	if (_vala_strcmp0 (self->priv->_current_typename, self->priv->_searched_typename) == 0) {
		ValaSymbol* _tmp5;
		ValaSymbol* _tmp4;
		char* _tmp7;
		const char* _tmp6;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->_result = (_tmp5 = (_tmp4 = (ValaSymbol*) p, (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL)), _tmp5);
		_tmp7 = NULL;
		_tmp6 = NULL;
		self->qualified_typename = (_tmp7 = (_tmp6 = self->priv->_current_typename, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), self->qualified_typename = (g_free (self->qualified_typename), NULL), _tmp7);
	}
	_tmp9 = NULL;
	_tmp8 = NULL;
	self->priv->_current_typename = (_tmp9 = (_tmp8 = previous_typename, (_tmp8 == NULL) ? NULL : g_strdup (_tmp8)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp9);
	previous_typename = (g_free (previous_typename), NULL);
}


static void vsc_typefinder_visitor_real_visit_field (ValaCodeVisitor* base, ValaField* f) {
	VscTypeFinderVisitor * self;
	const char* _tmp0;
	char* previous_typename;
	char* _tmp9;
	const char* _tmp8;
	self = (VscTypeFinderVisitor*) base;
	g_return_if_fail (f != NULL);
	if (self->priv->_result != NULL) {
		return;
	}
	_tmp0 = NULL;
	previous_typename = (_tmp0 = self->priv->_current_typename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (self->priv->_current_typename == NULL) {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->_current_typename = (_tmp2 = (_tmp1 = vala_symbol_get_name ((ValaSymbol*) f), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp2);
	} else {
		char* _tmp3;
		_tmp3 = NULL;
		self->priv->_current_typename = (_tmp3 = g_strdup_printf ("%s.%s", self->priv->_current_typename, vala_symbol_get_name ((ValaSymbol*) f)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp3);
	}
	if (_vala_strcmp0 (self->priv->_current_typename, self->priv->_searched_typename) == 0) {
		ValaSymbol* _tmp5;
		ValaSymbol* _tmp4;
		char* _tmp7;
		const char* _tmp6;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->_result = (_tmp5 = (_tmp4 = (ValaSymbol*) f, (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL)), _tmp5);
		_tmp7 = NULL;
		_tmp6 = NULL;
		self->qualified_typename = (_tmp7 = (_tmp6 = self->priv->_current_typename, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), self->qualified_typename = (g_free (self->qualified_typename), NULL), _tmp7);
	}
	_tmp9 = NULL;
	_tmp8 = NULL;
	self->priv->_current_typename = (_tmp9 = (_tmp8 = previous_typename, (_tmp8 == NULL) ? NULL : g_strdup (_tmp8)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp9);
	previous_typename = (g_free (previous_typename), NULL);
}


static void vsc_typefinder_visitor_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c) {
	VscTypeFinderVisitor * self;
	const char* _tmp0;
	char* previous_typename;
	char* _tmp9;
	const char* _tmp8;
	self = (VscTypeFinderVisitor*) base;
	g_return_if_fail (c != NULL);
	if (self->priv->_result != NULL) {
		return;
	}
	_tmp0 = NULL;
	previous_typename = (_tmp0 = self->priv->_current_typename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (self->priv->_current_typename == NULL) {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->_current_typename = (_tmp2 = (_tmp1 = vala_symbol_get_name ((ValaSymbol*) c), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp2);
	} else {
		char* _tmp3;
		_tmp3 = NULL;
		self->priv->_current_typename = (_tmp3 = g_strdup_printf ("%s.%s", self->priv->_current_typename, vala_symbol_get_name ((ValaSymbol*) c)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp3);
	}
	if (_vala_strcmp0 (self->priv->_current_typename, self->priv->_searched_typename) == 0) {
		ValaSymbol* _tmp5;
		ValaSymbol* _tmp4;
		char* _tmp7;
		const char* _tmp6;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->_result = (_tmp5 = (_tmp4 = (ValaSymbol*) c, (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL)), _tmp5);
		_tmp7 = NULL;
		_tmp6 = NULL;
		self->qualified_typename = (_tmp7 = (_tmp6 = self->priv->_current_typename, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), self->qualified_typename = (g_free (self->qualified_typename), NULL), _tmp7);
	}
	_tmp9 = NULL;
	_tmp8 = NULL;
	self->priv->_current_typename = (_tmp9 = (_tmp8 = previous_typename, (_tmp8 == NULL) ? NULL : g_strdup (_tmp8)), self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL), _tmp9);
	previous_typename = (g_free (previous_typename), NULL);
}


static void vsc_typefinder_visitor_real_visit_data_type (ValaCodeVisitor* base, ValaDataType* data_type) {
	VscTypeFinderVisitor * self;
	gboolean _tmp0;
	ValaUnresolvedType* _tmp1;
	ValaUnresolvedType* unresolved_type;
	ValaSymbol* sym;
	self = (VscTypeFinderVisitor*) base;
	g_return_if_fail (data_type != NULL);
	_tmp0 = FALSE;
	if (self->priv->_result != NULL) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = self->priv->_context == NULL;
	}
	if (_tmp0) {
		return;
	}
	if (!(VALA_IS_UNRESOLVED_TYPE (data_type))) {
		return;
	}
	_tmp1 = NULL;
	unresolved_type = (_tmp1 = VALA_UNRESOLVED_TYPE (data_type), (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1));
	sym = vsc_typefinder_visitor_resolve_type (self, unresolved_type);
	if (sym != NULL) {
		vala_code_node_accept ((ValaCodeNode*) sym, (ValaCodeVisitor*) self);
	}
	(unresolved_type == NULL) ? NULL : (unresolved_type = (vala_code_node_unref (unresolved_type), NULL));
	(sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL));
}


static ValaSymbol* vsc_typefinder_visitor_resolve_type (VscTypeFinderVisitor* self, ValaUnresolvedType* unresolved_type) {
	ValaSymbol* sym;
	char* name;
	gboolean _tmp2;
	ValaSymbol* _tmp12;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (unresolved_type != NULL, NULL);
	sym = NULL;
	name = vala_data_type_to_qualified_string ((ValaDataType*) unresolved_type, NULL);
	if (name == NULL) {
		char* _tmp1;
		const char* _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		name = (_tmp1 = (_tmp0 = vala_symbol_get_name ((ValaSymbol*) vala_unresolved_type_get_unresolved_symbol (unresolved_type)), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), name = (g_free (name), NULL), _tmp1);
	}
	_tmp2 = FALSE;
	if (sym == NULL) {
		_tmp2 = self->priv->_current_file != NULL;
	} else {
		_tmp2 = FALSE;
	}
	if (_tmp2) {
		VscTypeFinderVisitor* typefinder;
		ValaSymbol* _tmp4;
		ValaSymbol* _tmp3;
		gboolean _tmp5;
		typefinder = vsc_typefinder_visitor_new (NULL, NULL);
		/*trying on the root context*/
		vsc_typefinder_visitor_set_searched_typename (typefinder, name);
		vala_code_visitor_visit_namespace ((ValaCodeVisitor*) typefinder, vala_code_context_get_root (self->priv->_context));
		_tmp4 = NULL;
		_tmp3 = NULL;
		sym = (_tmp4 = (_tmp3 = vsc_typefinder_visitor_get_result (typefinder), (_tmp3 == NULL) ? NULL : vala_code_node_ref (_tmp3)), (sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL)), _tmp4);
		_tmp5 = FALSE;
		if (sym == NULL) {
			_tmp5 = !vsc_symbol_completion_symbol_has_known_namespace (name);
		} else {
			_tmp5 = FALSE;
		}
		if (_tmp5) {
			{
				GeeList* _tmp6;
				GeeIterator* _tmp7;
				GeeIterator* _item_it;
				_tmp6 = NULL;
				_tmp7 = NULL;
				_item_it = (_tmp7 = gee_iterable_iterator ((GeeIterable*) (_tmp6 = vala_source_file_get_using_directives (self->priv->_current_file))), (_tmp6 == NULL) ? NULL : (_tmp6 = (gee_collection_object_unref (_tmp6), NULL)), _tmp7);
				while (gee_iterator_next (_item_it)) {
					ValaUsingDirective* item;
					char* ns_name;
					char* _tmp8;
					gboolean _tmp9;
					char* using_name;
					item = (ValaUsingDirective*) gee_iterator_get (_item_it);
					ns_name = vsc_typefinder_visitor_get_qualified_namespace_name (self, vala_using_directive_get_namespace_symbol (item));
					_tmp8 = NULL;
					if ((_tmp9 = g_str_has_prefix (name, _tmp8 = g_strdup_printf ("%s.", ns_name)), _tmp8 = (g_free (_tmp8), NULL), _tmp9)) {
						(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
						ns_name = (g_free (ns_name), NULL);
						break;
					}
					/*exit since symbol is fully qualified*/
					using_name = g_strdup_printf ("%s.%s", ns_name, name);
					vsc_typefinder_visitor_set_searched_typename (typefinder, using_name);
					vala_code_visitor_visit_namespace ((ValaCodeVisitor*) typefinder, vala_code_context_get_root (self->priv->_context));
					if (vsc_typefinder_visitor_get_result (typefinder) != NULL) {
						ValaSymbol* _tmp11;
						ValaSymbol* _tmp10;
						_tmp11 = NULL;
						_tmp10 = NULL;
						sym = (_tmp11 = (_tmp10 = vsc_typefinder_visitor_get_result (typefinder), (_tmp10 == NULL) ? NULL : vala_code_node_ref (_tmp10)), (sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL)), _tmp11);
						(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
						ns_name = (g_free (ns_name), NULL);
						using_name = (g_free (using_name), NULL);
						break;
					}
					(item == NULL) ? NULL : (item = (vala_code_node_unref (item), NULL));
					ns_name = (g_free (ns_name), NULL);
					using_name = (g_free (using_name), NULL);
				}
				(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
			}
		}
		(typefinder == NULL) ? NULL : (typefinder = (vala_code_visitor_unref (typefinder), NULL));
	}
	_tmp12 = NULL;
	return (_tmp12 = sym, name = (g_free (name), NULL), _tmp12);
}


static char* vsc_typefinder_visitor_get_qualified_namespace_name (VscTypeFinderVisitor* self, ValaSymbol* namespace_symbol) {
	char* ns_name;
	ValaSymbol* _tmp0;
	ValaSymbol* ns;
	char* _tmp8;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (namespace_symbol != NULL, NULL);
	ns_name = NULL;
	_tmp0 = NULL;
	ns = (_tmp0 = namespace_symbol, (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
	while (ns != NULL) {
		if (ns_name == NULL) {
			char* _tmp2;
			const char* _tmp1;
			_tmp2 = NULL;
			_tmp1 = NULL;
			ns_name = (_tmp2 = (_tmp1 = vala_symbol_get_name (ns), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), ns_name = (g_free (ns_name), NULL), _tmp2);
		} else {
			char* _tmp3;
			_tmp3 = NULL;
			ns_name = (_tmp3 = g_strdup_printf ("%s.%s", vala_symbol_get_name (ns), ns_name), ns_name = (g_free (ns_name), NULL), _tmp3);
		}
		if (VALA_IS_UNRESOLVED_SYMBOL (ns)) {
			ValaSymbol* _tmp5;
			ValaSymbol* _tmp4;
			_tmp5 = NULL;
			_tmp4 = NULL;
			ns = (_tmp5 = (_tmp4 = (ValaSymbol*) vala_unresolved_symbol_get_inner ((VALA_UNRESOLVED_SYMBOL (ns))), (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL)), _tmp5);
		} else {
			ValaSymbol* _tmp7;
			ValaSymbol* _tmp6;
			_tmp7 = NULL;
			_tmp6 = NULL;
			ns = (_tmp7 = (_tmp6 = vala_symbol_get_parent_symbol (ns), (_tmp6 == NULL) ? NULL : vala_code_node_ref (_tmp6)), (ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL)), _tmp7);
		}
	}
	_tmp8 = NULL;
	return (_tmp8 = ns_name, (ns == NULL) ? NULL : (ns = (vala_code_node_unref (ns), NULL)), _tmp8);
}


ValaSymbol* vsc_typefinder_visitor_get_result (VscTypeFinderVisitor* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_result;
}


const char* vsc_typefinder_visitor_get_searched_typename (VscTypeFinderVisitor* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_searched_typename;
}


void vsc_typefinder_visitor_set_searched_typename (VscTypeFinderVisitor* self, const char* value) {
	char* _tmp2;
	const char* _tmp1;
	ValaSymbol* _tmp3;
	char* _tmp4;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_searched_typename = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_searched_typename = (g_free (self->priv->_searched_typename), NULL), _tmp2);
	_tmp3 = NULL;
	self->priv->_result = (_tmp3 = NULL, (self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL)), _tmp3);
	_tmp4 = NULL;
	self->qualified_typename = (_tmp4 = NULL, self->qualified_typename = (g_free (self->qualified_typename), NULL), _tmp4);
}


static void vsc_typefinder_visitor_class_init (VscTypeFinderVisitorClass * klass) {
	vsc_typefinder_visitor_parent_class = g_type_class_peek_parent (klass);
	VALA_CODE_VISITOR_CLASS (klass)->finalize = vsc_typefinder_visitor_finalize;
	g_type_class_add_private (klass, sizeof (VscTypeFinderVisitorPrivate));
	VALA_CODE_VISITOR_CLASS (klass)->visit_namespace = vsc_typefinder_visitor_real_visit_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_class = vsc_typefinder_visitor_real_visit_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_interface = vsc_typefinder_visitor_real_visit_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_struct = vsc_typefinder_visitor_real_visit_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum = vsc_typefinder_visitor_real_visit_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_error_domain = vsc_typefinder_visitor_real_visit_error_domain;
	VALA_CODE_VISITOR_CLASS (klass)->visit_method = vsc_typefinder_visitor_real_visit_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_property = vsc_typefinder_visitor_real_visit_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vsc_typefinder_visitor_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_constant = vsc_typefinder_visitor_real_visit_constant;
	VALA_CODE_VISITOR_CLASS (klass)->visit_data_type = vsc_typefinder_visitor_real_visit_data_type;
}


static void vsc_typefinder_visitor_instance_init (VscTypeFinderVisitor * self) {
	self->priv = VSC_TYPEFINDER_VISITOR_GET_PRIVATE (self);
	self->priv->_current_typename = NULL;
	self->priv->_current_file = NULL;
	self->priv->_result = NULL;
	self->priv->_searched_typename = NULL;
	self->qualified_typename = NULL;
}


static void vsc_typefinder_visitor_finalize (ValaCodeVisitor* obj) {
	VscTypeFinderVisitor * self;
	self = VSC_TYPEFINDER_VISITOR (obj);
	self->priv->_current_typename = (g_free (self->priv->_current_typename), NULL);
	(self->priv->_context == NULL) ? NULL : (self->priv->_context = (vala_code_context_unref (self->priv->_context), NULL));
	(self->priv->_current_file == NULL) ? NULL : (self->priv->_current_file = (vala_source_file_unref (self->priv->_current_file), NULL));
	(self->priv->_result == NULL) ? NULL : (self->priv->_result = (vala_code_node_unref (self->priv->_result), NULL));
	self->priv->_searched_typename = (g_free (self->priv->_searched_typename), NULL);
	self->qualified_typename = (g_free (self->qualified_typename), NULL);
	VALA_CODE_VISITOR_CLASS (vsc_typefinder_visitor_parent_class)->finalize (obj);
}


GType vsc_typefinder_visitor_get_type (void) {
	static GType vsc_typefinder_visitor_type_id = 0;
	if (vsc_typefinder_visitor_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscTypeFinderVisitorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_typefinder_visitor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscTypeFinderVisitor), 0, (GInstanceInitFunc) vsc_typefinder_visitor_instance_init, NULL };
		vsc_typefinder_visitor_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "VscTypeFinderVisitor", &g_define_type_info, 0);
	}
	return vsc_typefinder_visitor_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




