/*
 *  vsctypefindvisitor.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#ifndef __VSCTYPEFINDVISITOR_H__
#define __VSCTYPEFINDVISITOR_H__

#include <glib.h>
#include <glib-object.h>
#include <vala/valacodevisitor.h>
#include <stdlib.h>
#include <string.h>
#include <vala/valasourcefile.h>
#include <vala/valacodecontext.h>
#include <vala/valanamespace.h>
#include <vala/valaclass.h>
#include <vala/valainterface.h>
#include <vala/valastruct.h>
#include <vala/valaenum.h>
#include <vala/valaerrordomain.h>
#include <vala/valamethod.h>
#include <vala/valaproperty.h>
#include <vala/valafield.h>
#include <vala/valaconstant.h>
#include <vala/valadatatype.h>
#include <vala/valasymbol.h>

G_BEGIN_DECLS


#define VSC_TYPE_TYPEFINDER_VISITOR (vsc_typefinder_visitor_get_type ())
#define VSC_TYPEFINDER_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_TYPEFINDER_VISITOR, VscTypeFinderVisitor))
#define VSC_TYPEFINDER_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_TYPEFINDER_VISITOR, VscTypeFinderVisitorClass))
#define VSC_IS_TYPEFINDER_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_TYPEFINDER_VISITOR))
#define VSC_IS_TYPEFINDER_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_TYPEFINDER_VISITOR))
#define VSC_TYPEFINDER_VISITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_TYPEFINDER_VISITOR, VscTypeFinderVisitorClass))

typedef struct _VscTypeFinderVisitor VscTypeFinderVisitor;
typedef struct _VscTypeFinderVisitorClass VscTypeFinderVisitorClass;
typedef struct _VscTypeFinderVisitorPrivate VscTypeFinderVisitorPrivate;

/**
 * Code visitor getting the root symbol from whom
 * generate a list of items
 */
struct _VscTypeFinderVisitor {
	ValaCodeVisitor parent_instance;
	VscTypeFinderVisitorPrivate * priv;
	char* qualified_typename;
};

struct _VscTypeFinderVisitorClass {
	ValaCodeVisitorClass parent_class;
};


VscTypeFinderVisitor* vsc_typefinder_visitor_construct (GType object_type, ValaSourceFile* source, ValaCodeContext* context);
VscTypeFinderVisitor* vsc_typefinder_visitor_new (ValaSourceFile* source, ValaCodeContext* context);
ValaSymbol* vsc_typefinder_visitor_get_result (VscTypeFinderVisitor* self);
const char* vsc_typefinder_visitor_get_searched_typename (VscTypeFinderVisitor* self);
void vsc_typefinder_visitor_set_searched_typename (VscTypeFinderVisitor* self, const char* value);
GType vsc_typefinder_visitor_get_type (void);


G_END_DECLS

#endif
