/*
 *  vtgchangelog.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgchangelog.h"
#include <gee/list.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gedit/gedit-tab.h>
#include <gedit/gedit-document.h>
#include <gtk/gtk.h>
#include <time.h>
#include <glib/gi18n-lib.h>
#include "vtgprojectmanager.h"
#include "vtgprojectmanagerview.h"
#include "vtgprojectmanagerproject.h"
#include "vtgvcsbackendsivcs.h"
#include "vtgvcsbackendsgit.h"
#include "vtgvcsbackendsbzr.h"
#include "vtgvcsbackendssvn.h"
#include "vtgconfiguration.h"
#include "vtgutils.h"




static glong string_get_length (const char* self);
struct _VtgChangeLogPrivate {
	VtgPlugin* _plugin;
};

#define VTG_CHANGE_LOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_CHANGE_LOG, VtgChangeLogPrivate))
enum  {
	VTG_CHANGE_LOG_DUMMY_PROPERTY,
	VTG_CHANGE_LOG_PLUGIN
};
static VtgVcsBackendsIVcs* vtg_change_log_vcs_backend_factory (VtgChangeLog* self, VtgProjectManagerVcsTypes type);
static void vtg_change_log_set_plugin (VtgChangeLog* self, VtgPlugin* value);
static gpointer vtg_change_log_parent_class = NULL;
static void vtg_change_log_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


VtgChangeLog* vtg_change_log_construct (GType object_type, VtgPlugin* plugin) {
	GParameter * __params;
	GParameter * __params_it;
	VtgChangeLog * self;
	g_return_val_if_fail (plugin != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "plugin";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN);
	g_value_set_object (&__params_it->value, plugin);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgChangeLog* vtg_change_log_new (VtgPlugin* plugin) {
	return vtg_change_log_construct (VTG_TYPE_CHANGE_LOG, plugin);
}


gboolean vtg_change_log_prepare (VtgChangeLog* self, const char* file, GError** error) {
	GError * inner_error;
	VtgProjectManagerProject* _tmp0;
	VtgProjectManagerProject* project;
	char* file_list;
	gboolean force_add_new;
	gboolean _tmp29;
	g_return_val_if_fail (self != NULL, FALSE);
	inner_error = NULL;
	_tmp0 = NULL;
	project = (_tmp0 = vtg_project_manager_view_get_current_project (vtg_project_manager_plugin_helper_get_project_view (vtg_plugin_get_project_manager (self->priv->_plugin))), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	if (project == NULL) {
		gboolean _tmp1;
		return (_tmp1 = FALSE, (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), _tmp1);
	}
	file_list = g_strdup ("");
	force_add_new = TRUE;
	if (file != NULL) {
		char* _tmp3;
		char* _tmp2;
		_tmp3 = NULL;
		_tmp2 = NULL;
		file_list = (_tmp3 = g_strconcat (file_list, _tmp2 = (g_strdup_printf ("\t* %s:\n", file)), NULL), file_list = (g_free (file_list), NULL), _tmp3);
		_tmp2 = (g_free (_tmp2), NULL);
		force_add_new = FALSE;
	} else {
		VtgVcsBackendsIVcs* backend;
		GeeList* items;
		backend = vtg_change_log_vcs_backend_factory (self, project->vcs_type);
		if (backend == NULL) {
			gboolean _tmp4;
			return (_tmp4 = FALSE, (backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL)), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), file_list = (g_free (file_list), NULL), _tmp4);
		}
		items = vtg_vcs_backends_ivcs_get_items (backend, project->filename, &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			(backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL));
			(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
			file_list = (g_free (file_list), NULL);
			return FALSE;
		}
		{
			GeeIterator* _item_it;
			_item_it = gee_iterable_iterator ((GeeIterable*) items);
			while (gee_iterator_next (_item_it)) {
				VtgVcsBackendsItem* item;
				char* _tmp6;
				char* _tmp5;
				item = (VtgVcsBackendsItem*) gee_iterator_get (_item_it);
				_tmp6 = NULL;
				_tmp5 = NULL;
				file_list = (_tmp6 = g_strconcat (file_list, _tmp5 = (g_strdup_printf ("\t* %s:\n", item->name)), NULL), file_list = (g_free (file_list), NULL), _tmp6);
				_tmp5 = (g_free (_tmp5), NULL);
				(item == NULL) ? NULL : (item = (vtg_vcs_backends_item_unref (item), NULL));
			}
			(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
		}
		(backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL));
		(items == NULL) ? NULL : (items = (gee_collection_object_unref (items), NULL));
	}
	if (_vala_strcmp0 (file_list, "") != 0) {
		GeditTab* tab;
		GeditDocument* _tmp8;
		GeditDocument* doc;
		GMainContext* _tmp10;
		GMainContext* ctx;
		GtkTextIter iter = {0};
		const char* _tmp11;
		char* author;
		const char* _tmp12;
		char* email;
		time_t _tmp15 = {0};
		time_t _tmp13 = {0};
		struct tm _tmp14 = {0};
		struct tm today;
		char* date;
		char* header;
		char* current_header;
		char* entry;
		gint backward_chars_count;
		GtkTextMark* mark;
		gboolean _tmp28;
		tab = vtg_plugin_activate_uri (self->priv->_plugin, project->changelog_uri, 0, 0);
		if (tab == NULL) {
			gboolean _tmp7;
			return (_tmp7 = FALSE, (tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL)), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), file_list = (g_free (file_list), NULL), _tmp7);
		}
		_tmp8 = NULL;
		doc = (_tmp8 = gedit_tab_get_document (tab), (_tmp8 == NULL) ? NULL : g_object_ref (_tmp8));
		if (doc == NULL) {
			gboolean _tmp9;
			return (_tmp9 = FALSE, (tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL)), (doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL)), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), file_list = (g_free (file_list), NULL), _tmp9);
		}
		_tmp10 = NULL;
		ctx = (_tmp10 = g_main_context_default (), (_tmp10 == NULL) ? NULL : g_main_context_ref (_tmp10));
		while (g_main_context_pending (ctx)) {
			g_main_context_iteration (ctx, FALSE);
		}
		_tmp11 = NULL;
		author = (_tmp11 = vtg_configuration_get_author (vtg_plugin_get_config (self->priv->_plugin)), (_tmp11 == NULL) ? NULL : g_strdup (_tmp11));
		_tmp12 = NULL;
		email = (_tmp12 = vtg_configuration_get_email_address (vtg_plugin_get_config (self->priv->_plugin)), (_tmp12 == NULL) ? NULL : g_strdup (_tmp12));
		today = (localtime_r ((_tmp15 = (time (&_tmp13), _tmp13), &_tmp15), &_tmp14), _tmp14);
		if (vtg_string_utils_is_null_or_empty (author)) {
			char* _tmp17;
			const char* _tmp16;
			_tmp17 = NULL;
			_tmp16 = NULL;
			author = (_tmp17 = (_tmp16 = g_getenv ("REAL_NAME"), (_tmp16 == NULL) ? NULL : g_strdup (_tmp16)), author = (g_free (author), NULL), _tmp17);
		}
		if (vtg_string_utils_is_null_or_empty (email)) {
			char* _tmp19;
			const char* _tmp18;
			_tmp19 = NULL;
			_tmp18 = NULL;
			email = (_tmp19 = (_tmp18 = g_getenv ("EMAIL_ADDRESS"), (_tmp18 == NULL) ? NULL : g_strdup (_tmp18)), email = (g_free (email), NULL), _tmp19);
		}
		if (vtg_string_utils_is_null_or_empty (author)) {
			char* _tmp21;
			const char* _tmp20;
			_tmp21 = NULL;
			_tmp20 = NULL;
			author = (_tmp21 = (_tmp20 = _ ("Author Name"), (_tmp20 == NULL) ? NULL : g_strdup (_tmp20)), author = (g_free (author), NULL), _tmp21);
		}
		if (vtg_string_utils_is_null_or_empty (email)) {
			char* _tmp23;
			const char* _tmp22;
			_tmp23 = NULL;
			_tmp22 = NULL;
			email = (_tmp23 = (_tmp22 = _ ("Email Address"), (_tmp22 == NULL) ? NULL : g_strdup (_tmp22)), email = (g_free (email), NULL), _tmp23);
		}
		date = g_strdup_printf ("%04d-%02d-%02d", today.tm_year + 1900, today.tm_mon + 1, today.tm_mday);
		header = g_strdup_printf ("%s  %s  <%s>", date, author, email);
		current_header = NULL;
		entry = NULL;
		backward_chars_count = 2;
		gtk_text_buffer_get_iter_at_offset ((GtkTextBuffer*) doc, &iter, 0);
		if (!force_add_new) {
			GtkTextIter end;
			end = iter;
			if (gtk_text_iter_forward_line (&end)) {
				char* _tmp25;
				const char* _tmp24;
				gtk_text_iter_backward_char (&end);
				_tmp25 = NULL;
				_tmp24 = NULL;
				current_header = (_tmp25 = (_tmp24 = gtk_text_iter_get_text (&iter, &end), (_tmp24 == NULL) ? NULL : g_strdup (_tmp24)), current_header = (g_free (current_header), NULL), _tmp25);
			}
		}
		if (_vala_strcmp0 (current_header, header) != 0) {
			char* _tmp26;
			_tmp26 = NULL;
			entry = (_tmp26 = g_strdup_printf ("%s\n\n%s\n\t\n\n", header, file_list), entry = (g_free (entry), NULL), _tmp26);
		} else {
			char* _tmp27;
			_tmp27 = NULL;
			entry = (_tmp27 = g_strdup_printf ("\n%s\n\t\n", file_list), entry = (g_free (entry), NULL), _tmp27);
			gtk_text_iter_forward_line (&iter);
			backward_chars_count = 1;
		}
		gtk_text_buffer_place_cursor ((GtkTextBuffer*) doc, &iter);
		gtk_text_buffer_insert_interactive_at_cursor ((GtkTextBuffer*) doc, entry, (gint) string_get_length (entry), TRUE);
		mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert ((GtkTextBuffer*) doc));
		gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) doc, &iter, mark);
		gtk_text_iter_backward_chars (&iter, backward_chars_count);
		gtk_text_buffer_place_cursor ((GtkTextBuffer*) doc, &iter);
		return (_tmp28 = TRUE, (tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL)), (doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL)), (ctx == NULL) ? NULL : (ctx = (g_main_context_unref (ctx), NULL)), author = (g_free (author), NULL), email = (g_free (email), NULL), date = (g_free (date), NULL), header = (g_free (header), NULL), current_header = (g_free (current_header), NULL), entry = (g_free (entry), NULL), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), file_list = (g_free (file_list), NULL), _tmp28);
	}
	return (_tmp29 = FALSE, (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), file_list = (g_free (file_list), NULL), _tmp29);
}


static VtgVcsBackendsIVcs* vtg_change_log_vcs_backend_factory (VtgChangeLog* self, VtgProjectManagerVcsTypes type) {
	VtgVcsBackendsIVcs* backend;
	g_return_val_if_fail (self != NULL, NULL);
	backend = NULL;
	switch (type) {
		case VTG_PROJECT_MANAGER_VCS_TYPES_GIT:
		{
			VtgVcsBackendsIVcs* _tmp0;
			_tmp0 = NULL;
			backend = (_tmp0 = (VtgVcsBackendsIVcs*) vtg_vcs_backends_git_new (), (backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL)), _tmp0);
			break;
		}
		case VTG_PROJECT_MANAGER_VCS_TYPES_BZR:
		{
			VtgVcsBackendsIVcs* _tmp1;
			_tmp1 = NULL;
			backend = (_tmp1 = (VtgVcsBackendsIVcs*) vtg_vcs_backends_bzr_new (), (backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL)), _tmp1);
			break;
		}
		case VTG_PROJECT_MANAGER_VCS_TYPES_SVN:
		{
			VtgVcsBackendsIVcs* _tmp2;
			_tmp2 = NULL;
			backend = (_tmp2 = (VtgVcsBackendsIVcs*) vtg_vcs_backends_svn_new (), (backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL)), _tmp2);
			break;
		}
		default:
		{
			VtgVcsBackendsIVcs* _tmp3;
			_tmp3 = NULL;
			backend = (_tmp3 = NULL, (backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL)), _tmp3);
			break;
		}
	}
	return backend;
}


VtgPlugin* vtg_change_log_get_plugin (VtgChangeLog* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_plugin;
}


static void vtg_change_log_set_plugin (VtgChangeLog* self, VtgPlugin* value) {
	VtgPlugin* _tmp2;
	VtgPlugin* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_plugin = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "plugin");
}


static void vtg_change_log_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgChangeLog * self;
	gpointer boxed;
	self = VTG_CHANGE_LOG (object);
	switch (property_id) {
		case VTG_CHANGE_LOG_PLUGIN:
		g_value_set_object (value, vtg_change_log_get_plugin (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_change_log_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgChangeLog * self;
	self = VTG_CHANGE_LOG (object);
	switch (property_id) {
		case VTG_CHANGE_LOG_PLUGIN:
		vtg_change_log_set_plugin (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_change_log_class_init (VtgChangeLogClass * klass) {
	vtg_change_log_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgChangeLogPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_change_log_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_change_log_set_property;
	G_OBJECT_CLASS (klass)->finalize = vtg_change_log_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_CHANGE_LOG_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VTG_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vtg_change_log_instance_init (VtgChangeLog * self) {
	self->priv = VTG_CHANGE_LOG_GET_PRIVATE (self);
}


static void vtg_change_log_finalize (GObject* obj) {
	VtgChangeLog * self;
	self = VTG_CHANGE_LOG (obj);
	(self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL));
	G_OBJECT_CLASS (vtg_change_log_parent_class)->finalize (obj);
}


static GType vtg_change_log_type_id = 0;
GType vtg_change_log_get_type (void) {
	return vtg_change_log_type_id;
}


GType vtg_change_log_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgChangeLogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_change_log_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgChangeLog), 0, (GInstanceInitFunc) vtg_change_log_instance_init, NULL };
	vtg_change_log_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgChangeLog", &g_define_type_info, 0);
	return vtg_change_log_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




