/*
 *  vtgconfigurationdialog.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgconfiguration.h"
#include <gconf/gconf-client.h>
#include <gconf/gconf.h>
#include <glib/gi18n-lib.h>
#include "vtgutils.h"




struct _VtgConfigurationPrivate {
	GConfClient* _gconf;
	GtkDialog* _dialog;
	gboolean _bracket_enabled;
	gboolean _symbol_enabled;
	char* _author;
	char* _email_address;
};

#define VTG_CONFIGURATION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_CONFIGURATION, VtgConfigurationPrivate))
enum  {
	VTG_CONFIGURATION_DUMMY_PROPERTY,
	VTG_CONFIGURATION_BRACKET_ENABLED,
	VTG_CONFIGURATION_SYMBOL_ENABLED,
	VTG_CONFIGURATION_AUTHOR,
	VTG_CONFIGURATION_EMAIL_ADDRESS,
	VTG_CONFIGURATION_SAVE_BEFORE_BUILD
};
#define VTG_CONFIGURATION_VTG_BASE_KEY "/apps/gedit-2/plugins/vtg"
#define VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY VTG_CONFIGURATION_VTG_BASE_KEY "/bracket_completion_enabled"
#define VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY VTG_CONFIGURATION_VTG_BASE_KEY "/symbol_completion_enabled"
#define VTG_CONFIGURATION_VTG_AUTHOR_KEY VTG_CONFIGURATION_VTG_BASE_KEY "/author"
#define VTG_CONFIGURATION_VTG_EMAIL_ADDRESS_KEY VTG_CONFIGURATION_VTG_BASE_KEY "/email_address"
static void _vtg_configuration_on_button_close_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _vtg_configuration_on_checkbutton_toggled_gtk_toggle_button_toggled (GtkCheckButton* _sender, gpointer self);
static void _vtg_configuration_on_text_changed_g_object_notify (GtkEntry* _sender, GParamSpec* pspec, gpointer self);
static void vtg_configuration_on_button_close_clicked (VtgConfiguration* self, GtkButton* sender);
static void vtg_configuration_on_conf_value_changed (VtgConfiguration* self, GConfClient* sender, const char* key, void* value);
static void vtg_configuration_on_checkbutton_toggled (VtgConfiguration* self, GtkToggleButton* sender);
static void vtg_configuration_on_text_changed (VtgConfiguration* self, GObject* sender, GParamSpec* pspec);
static void _vtg_configuration_on_conf_value_changed_gconf_client_value_changed (GConfClient* _sender, const char* key, void* value, gpointer self);
static GObject * vtg_configuration_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_configuration_parent_class = NULL;
static void vtg_configuration_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



static void _vtg_configuration_on_button_close_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
	vtg_configuration_on_button_close_clicked (self, _sender);
}


static void _vtg_configuration_on_checkbutton_toggled_gtk_toggle_button_toggled (GtkCheckButton* _sender, gpointer self) {
	vtg_configuration_on_checkbutton_toggled (self, _sender);
}


static void _vtg_configuration_on_text_changed_g_object_notify (GtkEntry* _sender, GParamSpec* pspec, gpointer self) {
	vtg_configuration_on_text_changed (self, _sender, pspec);
}


GtkWidget* vtg_configuration_get_configuration_dialog (VtgConfiguration* self) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, NULL);
	inner_error = NULL;
	{
		GtkBuilder* builder;
		char* _tmp0;
		GtkDialog* _tmp2;
		GtkDialog* _tmp1;
		GtkButton* _tmp3;
		GtkButton* button;
		GtkCheckButton* _tmp4;
		GtkCheckButton* check;
		GtkCheckButton* _tmp6;
		GtkCheckButton* _tmp5;
		GtkEntry* _tmp7;
		GtkEntry* text;
		GtkEntry* _tmp9;
		GtkEntry* _tmp8;
		GtkWidget* _tmp10;
		builder = gtk_builder_new ();
		_tmp0 = NULL;
		gtk_builder_add_from_file (builder, _tmp0 = vtg_utils_get_ui_path ("vtg.ui"), &inner_error);
		if (inner_error != NULL) {
			(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
			goto __catch28_g_error;
			goto __finally28;
		}
		_tmp0 = (g_free (_tmp0), NULL);
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->_dialog = (_tmp2 = (_tmp1 = GTK_DIALOG (gtk_builder_get_object (builder, "dialog-settings")), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL)), _tmp2);
		g_assert (self->priv->_dialog != NULL);
		_tmp3 = NULL;
		button = (_tmp3 = GTK_BUTTON (gtk_builder_get_object (builder, "button-settings-close")), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3));
		g_signal_connect_object (button, "clicked", (GCallback) _vtg_configuration_on_button_close_clicked_gtk_button_clicked, self, 0);
		_tmp4 = NULL;
		check = (_tmp4 = GTK_CHECK_BUTTON (gtk_builder_get_object (builder, "checkbutton-settings-bracket-completion")), (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4));
		g_assert (check != NULL);
		gtk_toggle_button_set_active ((GtkToggleButton*) check, self->priv->_bracket_enabled);
		g_signal_connect_object ((GtkToggleButton*) check, "toggled", (GCallback) _vtg_configuration_on_checkbutton_toggled_gtk_toggle_button_toggled, self, 0);
		_tmp6 = NULL;
		_tmp5 = NULL;
		check = (_tmp6 = (_tmp5 = GTK_CHECK_BUTTON (gtk_builder_get_object (builder, "checkbutton-settings-symbol-completion")), (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5)), (check == NULL) ? NULL : (check = (g_object_unref (check), NULL)), _tmp6);
		g_assert (check != NULL);
		gtk_toggle_button_set_active ((GtkToggleButton*) check, self->priv->_symbol_enabled);
		g_signal_connect_object ((GtkToggleButton*) check, "toggled", (GCallback) _vtg_configuration_on_checkbutton_toggled_gtk_toggle_button_toggled, self, 0);
		_tmp7 = NULL;
		text = (_tmp7 = GTK_ENTRY (gtk_builder_get_object (builder, "entry-settings-author")), (_tmp7 == NULL) ? NULL : g_object_ref (_tmp7));
		g_assert (text != NULL);
		gtk_entry_set_text (text, self->priv->_author);
		g_signal_connect_object ((GObject*) text, "notify::text", (GCallback) _vtg_configuration_on_text_changed_g_object_notify, self, 0);
		_tmp9 = NULL;
		_tmp8 = NULL;
		text = (_tmp9 = (_tmp8 = GTK_ENTRY (gtk_builder_get_object (builder, "entry-settings-email")), (_tmp8 == NULL) ? NULL : g_object_ref (_tmp8)), (text == NULL) ? NULL : (text = (g_object_unref (text), NULL)), _tmp9);
		g_assert (text != NULL);
		gtk_entry_set_text (text, self->priv->_email_address);
		g_signal_connect_object ((GObject*) text, "notify::text", (GCallback) _vtg_configuration_on_text_changed_g_object_notify, self, 0);
		_tmp10 = NULL;
		return (_tmp10 = (GtkWidget*) self->priv->_dialog, (builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL)), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), (check == NULL) ? NULL : (check = (g_object_unref (check), NULL)), (text == NULL) ? NULL : (text = (g_object_unref (text), NULL)), _tmp10);
	}
	goto __finally28;
	__catch28_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			GtkWidget* _tmp11;
			g_warning ("vtgconfiguration.vala:153: (get_configuration_dialog): %s", err->message);
			_tmp11 = NULL;
			return (_tmp11 = NULL, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), _tmp11);
		}
	}
	__finally28:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return NULL;
	}
}


static void vtg_configuration_on_button_close_clicked (VtgConfiguration* self, GtkButton* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (self->priv->_dialog != NULL);
	gtk_object_destroy ((GtkObject*) self->priv->_dialog);
}


static void vtg_configuration_on_conf_value_changed (VtgConfiguration* self, GConfClient* sender, const char* key, void* value) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (key != NULL);
	inner_error = NULL;
	{
		if (_vala_strcmp0 (key, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY) == 0) {
			gboolean new_val;
			new_val = gconf_client_get_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch29_g_error;
				goto __finally29;
			}
			if (self->priv->_bracket_enabled != new_val) {
				vtg_configuration_set_bracket_enabled (self, new_val);
			}
		} else {
			if (_vala_strcmp0 (key, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY) == 0) {
				gboolean new_val;
				new_val = gconf_client_get_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, &inner_error);
				if (inner_error != NULL) {
					goto __catch29_g_error;
					goto __finally29;
				}
				if (self->priv->_symbol_enabled != new_val) {
					vtg_configuration_set_symbol_enabled (self, new_val);
				}
			} else {
				if (_vala_strcmp0 (key, VTG_CONFIGURATION_VTG_AUTHOR_KEY) == 0) {
					const char* _tmp0;
					char* new_val;
					_tmp0 = NULL;
					new_val = (_tmp0 = gconf_client_get_string (self->priv->_gconf, VTG_CONFIGURATION_VTG_AUTHOR_KEY, &inner_error), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
					if (inner_error != NULL) {
						goto __catch29_g_error;
						goto __finally29;
					}
					if (_vala_strcmp0 (self->priv->_author, new_val) != 0) {
						vtg_configuration_set_author (self, new_val);
					}
					new_val = (g_free (new_val), NULL);
				} else {
					if (_vala_strcmp0 (key, VTG_CONFIGURATION_VTG_EMAIL_ADDRESS_KEY) == 0) {
						const char* _tmp1;
						char* new_val;
						_tmp1 = NULL;
						new_val = (_tmp1 = gconf_client_get_string (self->priv->_gconf, VTG_CONFIGURATION_VTG_EMAIL_ADDRESS_KEY, &inner_error), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
						if (inner_error != NULL) {
							goto __catch29_g_error;
							goto __finally29;
						}
						if (_vala_strcmp0 (self->priv->_email_address, new_val) != 0) {
							vtg_configuration_set_email_address (self, new_val);
						}
						new_val = (g_free (new_val), NULL);
					}
				}
			}
		}
	}
	goto __finally29;
	__catch29_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgconfiguration.vala:189: (on_conf_value_changed): %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally29:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static void vtg_configuration_on_checkbutton_toggled (VtgConfiguration* self, GtkToggleButton* sender) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	inner_error = NULL;
	{
		gboolean new_val;
		const char* _tmp0;
		char* name;
		new_val = gtk_toggle_button_get_active (sender);
		_tmp0 = NULL;
		name = (_tmp0 = gtk_buildable_get_name ((GtkBuildable*) sender), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
		if (_vala_strcmp0 (name, "checkbutton-settings-bracket-completion") == 0) {
			gconf_client_set_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, new_val, &inner_error);
			if (inner_error != NULL) {
				name = (g_free (name), NULL);
				goto __catch30_g_error;
				goto __finally30;
			}
		} else {
			if (_vala_strcmp0 (name, "checkbutton-settings-symbol-completion") == 0) {
				gconf_client_set_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, new_val, &inner_error);
				if (inner_error != NULL) {
					name = (g_free (name), NULL);
					goto __catch30_g_error;
					goto __finally30;
				}
			}
		}
		name = (g_free (name), NULL);
	}
	goto __finally30;
	__catch30_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgconfiguration.vala:205: (on_checkbutton_toggled): %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally30:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static void vtg_configuration_on_text_changed (VtgConfiguration* self, GObject* sender, GParamSpec* pspec) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (pspec != NULL);
	inner_error = NULL;
	{
		GtkEntry* _tmp0;
		GtkEntry* entry;
		const char* _tmp1;
		char* new_val;
		const char* _tmp2;
		char* name;
		_tmp0 = NULL;
		entry = (_tmp0 = GTK_ENTRY (sender), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
		_tmp1 = NULL;
		new_val = (_tmp1 = gtk_entry_get_text (entry), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
		_tmp2 = NULL;
		name = (_tmp2 = gtk_buildable_get_name ((GtkBuildable*) entry), (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
		if (_vala_strcmp0 (name, "entry-settings-author") == 0) {
			gconf_client_set_string (self->priv->_gconf, VTG_CONFIGURATION_VTG_AUTHOR_KEY, new_val, &inner_error);
			if (inner_error != NULL) {
				(entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL));
				new_val = (g_free (new_val), NULL);
				name = (g_free (name), NULL);
				goto __catch31_g_error;
				goto __finally31;
			}
		} else {
			if (_vala_strcmp0 (name, "entry-settings-email") == 0) {
				gconf_client_set_string (self->priv->_gconf, VTG_CONFIGURATION_VTG_EMAIL_ADDRESS_KEY, new_val, &inner_error);
				if (inner_error != NULL) {
					(entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL));
					new_val = (g_free (new_val), NULL);
					name = (g_free (name), NULL);
					goto __catch31_g_error;
					goto __finally31;
				}
			}
		}
		(entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL));
		new_val = (g_free (new_val), NULL);
		name = (g_free (name), NULL);
	}
	goto __finally31;
	__catch31_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgconfiguration.vala:222: (on_text_changed): %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally31:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


VtgConfiguration* vtg_configuration_construct (GType object_type) {
	VtgConfiguration * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgConfiguration* vtg_configuration_new (void) {
	return vtg_configuration_construct (VTG_TYPE_CONFIGURATION);
}


gboolean vtg_configuration_get_bracket_enabled (VtgConfiguration* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_bracket_enabled;
}


void vtg_configuration_set_bracket_enabled (VtgConfiguration* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_bracket_enabled = value;
	g_object_notify ((GObject *) self, "bracket-enabled");
}


gboolean vtg_configuration_get_symbol_enabled (VtgConfiguration* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_symbol_enabled;
}


void vtg_configuration_set_symbol_enabled (VtgConfiguration* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_symbol_enabled = value;
	g_object_notify ((GObject *) self, "symbol-enabled");
}


const char* vtg_configuration_get_author (VtgConfiguration* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_author;
}


void vtg_configuration_set_author (VtgConfiguration* self, const char* value) {
	char* _tmp2;
	const char* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_author = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_author = (g_free (self->priv->_author), NULL), _tmp2);
	g_object_notify ((GObject *) self, "author");
}


const char* vtg_configuration_get_email_address (VtgConfiguration* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_email_address;
}


void vtg_configuration_set_email_address (VtgConfiguration* self, const char* value) {
	char* _tmp2;
	const char* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_email_address = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_email_address = (g_free (self->priv->_email_address), NULL), _tmp2);
	g_object_notify ((GObject *) self, "email-address");
}


gboolean vtg_configuration_get_save_before_build (VtgConfiguration* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return TRUE;
}


static void _vtg_configuration_on_conf_value_changed_gconf_client_value_changed (GConfClient* _sender, const char* key, void* value, gpointer self) {
	vtg_configuration_on_conf_value_changed (self, _sender, key, value);
}


/*TODO: implement me!*/
static GObject * vtg_configuration_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgConfigurationClass * klass;
	GObjectClass * parent_class;
	VtgConfiguration * self;
	GError * inner_error;
	klass = VTG_CONFIGURATION_CLASS (g_type_class_peek (VTG_TYPE_CONFIGURATION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_CONFIGURATION (obj);
	inner_error = NULL;
	{
		{
			GConfClient* _tmp1;
			GConfClient* _tmp0;
			gboolean exists_base;
			gboolean _tmp2;
			gboolean _tmp4;
			gboolean _tmp6;
			gboolean _tmp8;
			gboolean _tmp10;
			gboolean _tmp11;
			const char* _tmp12;
			char* _tmp14;
			const char* _tmp13;
			const char* _tmp15;
			char* _tmp17;
			const char* _tmp16;
			/*TODO: construct the gconf client from Engine.get_default ()
			when supported. See this bug for a similar issue
			http://bugzilla.gnome.org/show_bug.cgi?id=549061*/
			_tmp1 = NULL;
			_tmp0 = NULL;
			self->priv->_gconf = (_tmp1 = (_tmp0 = gconf_client_get_default (), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_gconf == NULL) ? NULL : (self->priv->_gconf = (g_object_unref (self->priv->_gconf), NULL)), _tmp1);
			exists_base = gconf_client_dir_exists (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_BASE_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch32_g_error;
				goto __finally32;
			}
			_tmp2 = FALSE;
			if (!exists_base) {
				_tmp2 = TRUE;
			} else {
				GConfSchema* _tmp3;
				_tmp3 = gconf_client_get_schema (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, &inner_error);
				if (inner_error != NULL) {
					goto __catch32_g_error;
					goto __finally32;
				}
				_tmp2 = _tmp3 == NULL;
			}
			if (_tmp2) {
				GConfSchema* schema;
				GConfValue* def_value;
				schema = gconf_schema_new ();
				gconf_schema_set_short_desc (schema, _ ("Enable the symbol completion module"));
				gconf_schema_set_type (schema, GCONF_VALUE_BOOL);
				def_value = gconf_value_new (GCONF_VALUE_BOOL);
				gconf_value_set_bool (def_value, TRUE);
				gconf_schema_set_default_value (schema, def_value);
				gconf_client_set_schema (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, schema, &inner_error);
				if (inner_error != NULL) {
					(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
					(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
					goto __catch32_g_error;
					goto __finally32;
				}
				gconf_client_set_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, TRUE, &inner_error);
				if (inner_error != NULL) {
					(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
					(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
					goto __catch32_g_error;
					goto __finally32;
				}
				(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
				(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
			}
			_tmp4 = FALSE;
			if (!exists_base) {
				_tmp4 = TRUE;
			} else {
				GConfSchema* _tmp5;
				_tmp5 = gconf_client_get_schema (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, &inner_error);
				if (inner_error != NULL) {
					goto __catch32_g_error;
					goto __finally32;
				}
				_tmp4 = _tmp5 == NULL;
			}
			if (_tmp4) {
				GConfSchema* schema;
				GConfValue* def_value;
				schema = gconf_schema_new ();
				gconf_schema_set_short_desc (schema, _ ("Enable the bracket completion module"));
				gconf_schema_set_type (schema, GCONF_VALUE_BOOL);
				def_value = gconf_value_new (GCONF_VALUE_BOOL);
				gconf_value_set_bool (def_value, TRUE);
				gconf_schema_set_default_value (schema, def_value);
				gconf_client_set_schema (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, schema, &inner_error);
				if (inner_error != NULL) {
					(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
					(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
					goto __catch32_g_error;
					goto __finally32;
				}
				gconf_client_set_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, TRUE, &inner_error);
				if (inner_error != NULL) {
					(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
					(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
					goto __catch32_g_error;
					goto __finally32;
				}
				(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
				(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
			}
			_tmp6 = FALSE;
			if (!exists_base) {
				_tmp6 = TRUE;
			} else {
				GConfSchema* _tmp7;
				_tmp7 = gconf_client_get_schema (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_AUTHOR_KEY, &inner_error);
				if (inner_error != NULL) {
					goto __catch32_g_error;
					goto __finally32;
				}
				_tmp6 = _tmp7 == NULL;
			}
			if (_tmp6) {
				GConfSchema* schema;
				GConfValue* def_value;
				schema = gconf_schema_new ();
				gconf_schema_set_short_desc (schema, _ ("Override the author name used in the ChangeLog entries"));
				gconf_schema_set_type (schema, GCONF_VALUE_STRING);
				def_value = gconf_value_new (GCONF_VALUE_STRING);
				gconf_value_set_string (def_value, "");
				gconf_schema_set_default_value (schema, def_value);
				gconf_client_set_schema (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_AUTHOR_KEY, schema, &inner_error);
				if (inner_error != NULL) {
					(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
					(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
					goto __catch32_g_error;
					goto __finally32;
				}
				gconf_client_set_string (self->priv->_gconf, VTG_CONFIGURATION_VTG_AUTHOR_KEY, "", &inner_error);
				if (inner_error != NULL) {
					(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
					(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
					goto __catch32_g_error;
					goto __finally32;
				}
				(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
				(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
			}
			_tmp8 = FALSE;
			if (!exists_base) {
				_tmp8 = TRUE;
			} else {
				GConfSchema* _tmp9;
				_tmp9 = gconf_client_get_schema (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_EMAIL_ADDRESS_KEY, &inner_error);
				if (inner_error != NULL) {
					goto __catch32_g_error;
					goto __finally32;
				}
				_tmp8 = _tmp9 == NULL;
			}
			if (_tmp8) {
				GConfSchema* schema;
				GConfValue* def_value;
				schema = gconf_schema_new ();
				gconf_schema_set_short_desc (schema, _ ("Override the author name used in the ChangeLog entries"));
				gconf_schema_set_type (schema, GCONF_VALUE_STRING);
				def_value = gconf_value_new (GCONF_VALUE_STRING);
				gconf_value_set_string (def_value, "");
				gconf_schema_set_default_value (schema, def_value);
				gconf_client_set_schema (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_EMAIL_ADDRESS_KEY, schema, &inner_error);
				if (inner_error != NULL) {
					(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
					(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
					goto __catch32_g_error;
					goto __finally32;
				}
				gconf_client_set_string (self->priv->_gconf, VTG_CONFIGURATION_VTG_EMAIL_ADDRESS_KEY, "", &inner_error);
				if (inner_error != NULL) {
					(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
					(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
					goto __catch32_g_error;
					goto __finally32;
				}
				(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
				(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
			}
			gconf_engine_associate_schema (self->priv->_gconf->engine, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, "/schemas" VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch32_g_error;
				goto __finally32;
			}
			gconf_engine_associate_schema (self->priv->_gconf->engine, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, "/schemas" VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch32_g_error;
				goto __finally32;
			}
			gconf_engine_associate_schema (self->priv->_gconf->engine, VTG_CONFIGURATION_VTG_AUTHOR_KEY, "/schemas" VTG_CONFIGURATION_VTG_AUTHOR_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch32_g_error;
				goto __finally32;
			}
			gconf_engine_associate_schema (self->priv->_gconf->engine, VTG_CONFIGURATION_VTG_EMAIL_ADDRESS_KEY, "/schemas" VTG_CONFIGURATION_VTG_EMAIL_ADDRESS_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch32_g_error;
				goto __finally32;
			}
			_tmp10 = gconf_client_get_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch32_g_error;
				goto __finally32;
			}
			self->priv->_symbol_enabled = _tmp10;
			_tmp11 = gconf_client_get_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch32_g_error;
				goto __finally32;
			}
			self->priv->_bracket_enabled = _tmp11;
			_tmp12 = gconf_client_get_string (self->priv->_gconf, VTG_CONFIGURATION_VTG_AUTHOR_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch32_g_error;
				goto __finally32;
			}
			_tmp14 = NULL;
			_tmp13 = NULL;
			self->priv->_author = (_tmp14 = (_tmp13 = _tmp12, (_tmp13 == NULL) ? NULL : g_strdup (_tmp13)), self->priv->_author = (g_free (self->priv->_author), NULL), _tmp14);
			_tmp15 = gconf_client_get_string (self->priv->_gconf, VTG_CONFIGURATION_VTG_EMAIL_ADDRESS_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch32_g_error;
				goto __finally32;
			}
			_tmp17 = NULL;
			_tmp16 = NULL;
			self->priv->_email_address = (_tmp17 = (_tmp16 = _tmp15, (_tmp16 == NULL) ? NULL : g_strdup (_tmp16)), self->priv->_email_address = (g_free (self->priv->_email_address), NULL), _tmp17);
			gconf_client_add_dir (self->priv->_gconf, VTG_CONFIGURATION_VTG_BASE_KEY, GCONF_CLIENT_PRELOAD_ONELEVEL, &inner_error);
			if (inner_error != NULL) {
				goto __catch32_g_error;
				goto __finally32;
			}
			g_signal_connect_object (self->priv->_gconf, "value-changed", (GCallback) _vtg_configuration_on_conf_value_changed_gconf_client_value_changed, self, 0);
		}
		goto __finally32;
		__catch32_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				g_warning ("vtgconfiguration.vala:112: (configuration): %s", err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally32:
		if (inner_error != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
	}
	return obj;
}


static void vtg_configuration_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgConfiguration * self;
	gpointer boxed;
	self = VTG_CONFIGURATION (object);
	switch (property_id) {
		case VTG_CONFIGURATION_BRACKET_ENABLED:
		g_value_set_boolean (value, vtg_configuration_get_bracket_enabled (self));
		break;
		case VTG_CONFIGURATION_SYMBOL_ENABLED:
		g_value_set_boolean (value, vtg_configuration_get_symbol_enabled (self));
		break;
		case VTG_CONFIGURATION_AUTHOR:
		g_value_set_string (value, vtg_configuration_get_author (self));
		break;
		case VTG_CONFIGURATION_EMAIL_ADDRESS:
		g_value_set_string (value, vtg_configuration_get_email_address (self));
		break;
		case VTG_CONFIGURATION_SAVE_BEFORE_BUILD:
		g_value_set_boolean (value, vtg_configuration_get_save_before_build (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_configuration_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgConfiguration * self;
	self = VTG_CONFIGURATION (object);
	switch (property_id) {
		case VTG_CONFIGURATION_BRACKET_ENABLED:
		vtg_configuration_set_bracket_enabled (self, g_value_get_boolean (value));
		break;
		case VTG_CONFIGURATION_SYMBOL_ENABLED:
		vtg_configuration_set_symbol_enabled (self, g_value_get_boolean (value));
		break;
		case VTG_CONFIGURATION_AUTHOR:
		vtg_configuration_set_author (self, g_value_get_string (value));
		break;
		case VTG_CONFIGURATION_EMAIL_ADDRESS:
		vtg_configuration_set_email_address (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_configuration_class_init (VtgConfigurationClass * klass) {
	vtg_configuration_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgConfigurationPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_configuration_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_configuration_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_configuration_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_configuration_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_CONFIGURATION_BRACKET_ENABLED, g_param_spec_boolean ("bracket-enabled", "bracket-enabled", "bracket-enabled", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_CONFIGURATION_SYMBOL_ENABLED, g_param_spec_boolean ("symbol-enabled", "symbol-enabled", "symbol-enabled", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_CONFIGURATION_AUTHOR, g_param_spec_string ("author", "author", "author", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_CONFIGURATION_EMAIL_ADDRESS, g_param_spec_string ("email-address", "email-address", "email-address", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_CONFIGURATION_SAVE_BEFORE_BUILD, g_param_spec_boolean ("save-before-build", "save-before-build", "save-before-build", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_configuration_instance_init (VtgConfiguration * self) {
	self->priv = VTG_CONFIGURATION_GET_PRIVATE (self);
}


static void vtg_configuration_finalize (GObject* obj) {
	VtgConfiguration * self;
	GError * inner_error;
	self = VTG_CONFIGURATION (obj);
	inner_error = NULL;
	{
		{
			gconf_client_suggest_sync (self->priv->_gconf, &inner_error);
			if (inner_error != NULL) {
				goto __catch33_g_error;
				goto __finally33;
			}
		}
		goto __finally33;
		__catch33_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				g_warning ("vtgconfiguration.vala:122: error %s", err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally33:
		if (inner_error != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
	}
	(self->priv->_gconf == NULL) ? NULL : (self->priv->_gconf = (g_object_unref (self->priv->_gconf), NULL));
	(self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL));
	self->priv->_author = (g_free (self->priv->_author), NULL);
	self->priv->_email_address = (g_free (self->priv->_email_address), NULL);
	G_OBJECT_CLASS (vtg_configuration_parent_class)->finalize (obj);
}


static GType vtg_configuration_type_id = 0;
GType vtg_configuration_get_type (void) {
	return vtg_configuration_type_id;
}


GType vtg_configuration_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgConfigurationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_configuration_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgConfiguration), 0, (GInstanceInitFunc) vtg_configuration_instance_init, NULL };
	vtg_configuration_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgConfiguration", &g_define_type_info, 0);
	return vtg_configuration_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




