/*
 *  vtgplugin.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgplugin.h"
#include <gee/arraylist.h>
#include <gedit/gedit-document.h>
#include <gedit/gedit-encodings.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vscparsermanager.h>
#include <gee/collection.h>
#include <gtksourceview/gtksourceview.h>
#include <gedit/gedit-app.h>
#include <glib/gi18n-lib.h>
#include <config.h>
#include "vtgbracketcompletion.h"
#include "vtgsymbolcompletionhelper.h"
#include "vtgprojectmanageroutputview.h"
#include "vtgprojectmanager.h"
#include "vtgprojectmanagerprojectmodule.h"
#include "vtgprojectmanagerprojectpackage.h"
#include "vtgprojectmanagerprojectgroup.h"
#include "vtgprojectmanagerprojecttarget.h"
#include "vtgprojectmanagerprojectsource.h"
#include "vtgprojectmanagerview.h"


#define VTG_PLUGIN_TYPE_DEACTIVATE_MODULE_OPTIONS (vtg_plugin_deactivate_module_options_get_type ())

typedef enum  {
	VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL,
	VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET,
	VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL
} VtgPluginDeactivateModuleOptions;



static char* string_substring (const char* self, glong offset, glong len);
static glong string_get_length (const char* self);
enum  {
	VTG_PROJECT_DESCRIPTOR_DUMMY_PROPERTY
};
static gpointer vtg_project_descriptor_parent_class = NULL;
static void vtg_project_descriptor_finalize (GObject* obj);
struct _VtgPluginPrivate {
	GeditWindow* _window;
	GeeList* _bcs;
	GeeList* _scs;
	GeeList* _projects;
	VtgConfiguration* _config;
	VtgProjectDescriptor* _default_project;
	VtgProjectManagerPluginHelper* _prj_man;
	VtgProjectManagerOutputView* _output_view;
};

#define VTG_PLUGIN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_PLUGIN, VtgPluginPrivate))
enum  {
	VTG_PLUGIN_DUMMY_PROPERTY,
	VTG_PLUGIN_PROJECTS,
	VTG_PLUGIN_OUTPUT_VIEW,
	VTG_PLUGIN_PROJECT_MANAGER,
	VTG_PLUGIN_CONFIG,
	VTG_PLUGIN_GEDIT_WINDOW
};
GType vtg_plugin_deactivate_module_options_get_type (void);
static void vtg_plugin_real_activate (GeditPlugin* base, GeditWindow* window);
static void vtg_plugin_real_deactivate (GeditPlugin* base, GeditWindow* window);
static gboolean vtg_plugin_real_is_configurable (GeditPlugin* base);
static GtkWidget* vtg_plugin_real_create_configure_dialog (GeditPlugin* base);
static void vtg_plugin_real_update_ui (GeditPlugin* base, GeditWindow* window);
static void vtg_plugin_on_configuration_property_changed (VtgPlugin* self, GObject* sender, GParamSpec* param);
static void vtg_plugin_deactivate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options);
static void vtg_plugin_activate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options);
static void vtg_plugin_activate_bracket (VtgPlugin* self, GeditView* view);
static void vtg_plugin_deactivate_bracket (VtgPlugin* self, VtgBracketCompletion* bc);
static void vtg_plugin_activate_symbol (VtgPlugin* self, VtgProjectDescriptor* project, GeditView* view);
static void vtg_plugin_deactivate_symbol (VtgPlugin* self, VtgSymbolCompletionHelper* sc);
static void vtg_plugin_on_tab_added (GeditWindow* sender, GeditTab* tab, VtgPlugin* instance);
static void vtg_plugin_on_tab_removed (GeditWindow* sender, GeditTab* tab, VtgPlugin* instance);
static VtgProjectDescriptor* vtg_plugin_project_descriptor_find_from_document (VtgPlugin* self, GeditDocument* document);
static gboolean vtg_plugin_scs_contains (VtgPlugin* self, GeditView* view);
static gboolean vtg_plugin_bcs_contains (VtgPlugin* self, GeditView* view);
static VtgBracketCompletion* vtg_plugin_bcs_find_from_view (VtgPlugin* self, GeditView* view);
static void vtg_plugin_initialize_views (VtgPlugin* self, GeditWindow* window);
static void vtg_plugin_initialize_view (VtgPlugin* self, VtgProjectDescriptor* project, GeditView* view);
static void vtg_plugin_initialize_document (VtgPlugin* self, GeditDocument* doc);
static void vtg_plugin_uninitialize_view (VtgPlugin* self, GeditView* view);
static void vtg_plugin_uninitialize_document (VtgPlugin* self, GeditDocument* doc);
static void vtg_plugin_on_notify_language (GeditDocument* sender, GParamSpec* pspec, VtgPlugin* instance);
static void _vtg_plugin_on_configuration_property_changed_g_object_notify (VtgConfiguration* _sender, GParamSpec* pspec, gpointer self);
static GObject * vtg_plugin_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_plugin_parent_class = NULL;
static void vtg_plugin_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


VtgProjectDescriptor* vtg_project_descriptor_construct (GType object_type) {
	VtgProjectDescriptor * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgProjectDescriptor* vtg_project_descriptor_new (void) {
	return vtg_project_descriptor_construct (VTG_TYPE_PROJECT_DESCRIPTOR);
}


static void vtg_project_descriptor_class_init (VtgProjectDescriptorClass * klass) {
	vtg_project_descriptor_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = vtg_project_descriptor_finalize;
}


static void vtg_project_descriptor_instance_init (VtgProjectDescriptor * self) {
}


static void vtg_project_descriptor_finalize (GObject* obj) {
	VtgProjectDescriptor * self;
	self = VTG_PROJECT_DESCRIPTOR (obj);
	{
		if (self->completion != NULL) {
			vsc_symbol_completion_cleanup (self->completion);
		}
	}
	(self->completion == NULL) ? NULL : (self->completion = (g_object_unref (self->completion), NULL));
	(self->project == NULL) ? NULL : (self->project = (g_object_unref (self->project), NULL));
	G_OBJECT_CLASS (vtg_project_descriptor_parent_class)->finalize (obj);
}


static GType vtg_project_descriptor_type_id = 0;
GType vtg_project_descriptor_get_type (void) {
	return vtg_project_descriptor_type_id;
}


GType vtg_project_descriptor_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectDescriptorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_descriptor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectDescriptor), 0, (GInstanceInitFunc) vtg_project_descriptor_instance_init, NULL };
	vtg_project_descriptor_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectDescriptor", &g_define_type_info, 0);
	return vtg_project_descriptor_type_id;
}



GType vtg_plugin_deactivate_module_options_get_type (void) {
	static GType vtg_plugin_deactivate_module_options_type_id = 0;
	if (G_UNLIKELY (vtg_plugin_deactivate_module_options_type_id == 0)) {
		static const GEnumValue values[] = {{VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL, "VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL", "all"}, {VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET, "VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET", "bracket"}, {VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL, "VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL", "symbol"}, {0, NULL, NULL}};
		vtg_plugin_deactivate_module_options_type_id = g_enum_register_static ("VtgPluginDeactivateModuleOptions", values);
	}
	return vtg_plugin_deactivate_module_options_type_id;
}


static void vtg_plugin_real_activate (GeditPlugin* base, GeditWindow* window) {
	VtgPlugin * self;
	GeditWindow* _tmp1;
	GeditWindow* _tmp0;
	VtgProjectManagerOutputView* _tmp3;
	VtgProjectManagerPluginHelper* _tmp4;
	self = (VtgPlugin*) base;
	g_return_if_fail (window != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_window = (_tmp1 = (_tmp0 = window, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp1);
	g_signal_connect_after (self->priv->_window, "tab-added", (GCallback) vtg_plugin_on_tab_added, self);
	g_signal_connect_after (self->priv->_window, "tab-removed", (GCallback) vtg_plugin_on_tab_removed, self);
	vtg_plugin_initialize_views (self, self->priv->_window);
	{
		GList* doc_collection;
		GList* doc_it;
		doc_collection = gedit_window_get_documents (self->priv->_window);
		for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
			GeditDocument* _tmp2;
			GeditDocument* doc;
			_tmp2 = NULL;
			doc = (_tmp2 = (GeditDocument*) doc_it->data, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
			{
				vtg_plugin_initialize_document (self, doc);
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			}
		}
	}
	_tmp3 = NULL;
	self->priv->_output_view = (_tmp3 = vtg_project_manager_output_view_new (self), (self->priv->_output_view == NULL) ? NULL : (self->priv->_output_view = (g_object_unref (self->priv->_output_view), NULL)), _tmp3);
	_tmp4 = NULL;
	self->priv->_prj_man = (_tmp4 = vtg_project_manager_plugin_helper_new (self), (self->priv->_prj_man == NULL) ? NULL : (self->priv->_prj_man = (g_object_unref (self->priv->_prj_man), NULL)), _tmp4);
}


/*_prj_man.project_loaded += this.on_project_loaded;*/
static void vtg_plugin_real_deactivate (GeditPlugin* base, GeditWindow* window) {
	VtgPlugin * self;
	VtgProjectManagerPluginHelper* _tmp0;
	VtgProjectManagerOutputView* _tmp1;
	GeditWindow* _tmp2;
	self = (VtgPlugin*) base;
	g_return_if_fail (window != NULL);
	vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL);
	_tmp0 = NULL;
	self->priv->_prj_man = (_tmp0 = NULL, (self->priv->_prj_man == NULL) ? NULL : (self->priv->_prj_man = (g_object_unref (self->priv->_prj_man), NULL)), _tmp0);
	_tmp1 = NULL;
	self->priv->_output_view = (_tmp1 = NULL, (self->priv->_output_view == NULL) ? NULL : (self->priv->_output_view = (g_object_unref (self->priv->_output_view), NULL)), _tmp1);
	_tmp2 = NULL;
	self->priv->_window = (_tmp2 = NULL, (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp2);
}


static gboolean vtg_plugin_real_is_configurable (GeditPlugin* base) {
	VtgPlugin * self;
	self = (VtgPlugin*) base;
	return TRUE;
}


static GtkWidget* vtg_plugin_real_create_configure_dialog (GeditPlugin* base) {
	VtgPlugin * self;
	self = (VtgPlugin*) base;
	return vtg_configuration_get_configuration_dialog (self->priv->_config);
}


static void vtg_plugin_real_update_ui (GeditPlugin* base, GeditWindow* window) {
	VtgPlugin * self;
	GeditDocument* _tmp0;
	GeditDocument* doc;
	self = (VtgPlugin*) base;
	g_return_if_fail (window != NULL);
	_tmp0 = NULL;
	doc = (_tmp0 = gedit_window_get_active_document (window), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	if (doc != NULL) {
		VtgProjectDescriptor* prj;
		gboolean _tmp1;
		prj = vtg_plugin_project_descriptor_find_from_document (self, doc);
		_tmp1 = FALSE;
		if (prj->project == NULL) {
			gboolean _tmp2;
			_tmp2 = FALSE;
			if (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc) == NULL) {
				_tmp2 = TRUE;
			} else {
				_tmp2 = _vala_strcmp0 (gtk_source_language_get_id (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc)), "vala") != 0;
			}
			_tmp1 = (_tmp2);
		} else {
			_tmp1 = FALSE;
		}
		if (!(_tmp1)) {
			vtg_project_manager_view_set_current_project (vtg_project_manager_plugin_helper_get_project_view (self->priv->_prj_man), prj->project);
		}
		(prj == NULL) ? NULL : (prj = (g_object_unref (prj), NULL));
	}
	(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
}


static void vtg_plugin_on_configuration_property_changed (VtgPlugin* self, GObject* sender, GParamSpec* param) {
	const char* _tmp0;
	char* name;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (param != NULL);
	_tmp0 = NULL;
	name = (_tmp0 = g_param_spec_get_name (param), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (_vala_strcmp0 (name, "bracket-enabled") == 0) {
		if (vtg_configuration_get_bracket_enabled (self->priv->_config)) {
			vtg_plugin_activate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET);
		} else {
			vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET);
		}
	} else {
		if (_vala_strcmp0 (name, "symbol-enabled") == 0) {
			if (vtg_configuration_get_bracket_enabled (self->priv->_config)) {
				vtg_plugin_activate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL);
			} else {
				vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL);
			}
		}
	}
	name = (g_free (name), NULL);
}


static void vtg_plugin_deactivate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options) {
	gint size;
	gboolean _tmp0;
	gboolean _tmp3;
	g_return_if_fail (self != NULL);
	size = 0;
	_tmp0 = FALSE;
	if (options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL;
	}
	if (_tmp0) {
		size = 0;
		while (TRUE) {
			gboolean _tmp1;
			VtgSymbolCompletionHelper* _tmp2;
			_tmp1 = FALSE;
			if (gee_collection_get_size ((GeeCollection*) self->priv->_scs) > 0) {
				_tmp1 = gee_collection_get_size ((GeeCollection*) self->priv->_scs) != size;
			} else {
				_tmp1 = FALSE;
			}
			if (!_tmp1) {
				break;
			}
			size = gee_collection_get_size ((GeeCollection*) self->priv->_scs);
			_tmp2 = NULL;
			vtg_plugin_deactivate_symbol (self, _tmp2 = (VtgSymbolCompletionHelper*) gee_list_get (self->priv->_scs, 0));
			(_tmp2 == NULL) ? NULL : (_tmp2 = (g_object_unref (_tmp2), NULL));
		}
	}
	_tmp3 = FALSE;
	if (options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL) {
		_tmp3 = TRUE;
	} else {
		_tmp3 = options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET;
	}
	if (_tmp3) {
		size = 0;
		while (TRUE) {
			gboolean _tmp4;
			VtgBracketCompletion* _tmp5;
			_tmp4 = FALSE;
			if (gee_collection_get_size ((GeeCollection*) self->priv->_bcs) > 0) {
				_tmp4 = gee_collection_get_size ((GeeCollection*) self->priv->_bcs) != size;
			} else {
				_tmp4 = FALSE;
			}
			if (!_tmp4) {
				break;
			}
			size = gee_collection_get_size ((GeeCollection*) self->priv->_bcs);
			_tmp5 = NULL;
			vtg_plugin_deactivate_bracket (self, _tmp5 = (VtgBracketCompletion*) gee_list_get (self->priv->_bcs, 0));
			(_tmp5 == NULL) ? NULL : (_tmp5 = (g_object_unref (_tmp5), NULL));
		}
	}
}


static void vtg_plugin_activate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options) {
	g_return_if_fail (self != NULL);
	vtg_plugin_initialize_views (self, self->priv->_window);
}


static void vtg_plugin_activate_bracket (VtgPlugin* self, GeditView* view) {
	VtgBracketCompletion* bc;
	g_return_if_fail (self != NULL);
	g_return_if_fail (view != NULL);
	bc = vtg_bracket_completion_new (self, view);
	gee_collection_add ((GeeCollection*) self->priv->_bcs, bc);
	(bc == NULL) ? NULL : (bc = (g_object_unref (bc), NULL));
}


static void vtg_plugin_deactivate_bracket (VtgPlugin* self, VtgBracketCompletion* bc) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (bc != NULL);
	vtg_bracket_completion_deactivate (bc);
	gee_collection_remove ((GeeCollection*) self->priv->_bcs, bc);
}


static void vtg_plugin_activate_symbol (VtgPlugin* self, VtgProjectDescriptor* project, GeditView* view) {
	VtgSymbolCompletionHelper* sc;
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	g_return_if_fail (view != NULL);
	sc = vtg_symbol_completion_helper_new (self, view, project->completion);
	gee_collection_add ((GeeCollection*) self->priv->_scs, sc);
	(sc == NULL) ? NULL : (sc = (g_object_unref (sc), NULL));
}


static void vtg_plugin_deactivate_symbol (VtgPlugin* self, VtgSymbolCompletionHelper* sc) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sc != NULL);
	vtg_symbol_completion_helper_deactivate (sc);
	gee_collection_remove ((GeeCollection*) self->priv->_scs, sc);
}


static void vtg_plugin_on_tab_added (GeditWindow* sender, GeditTab* tab, VtgPlugin* instance) {
	GeditDocument* _tmp0;
	GeditDocument* doc;
	VtgProjectDescriptor* project;
	gboolean _tmp1;
	g_return_if_fail (sender != NULL);
	g_return_if_fail (tab != NULL);
	g_return_if_fail (instance != NULL);
	_tmp0 = NULL;
	doc = (_tmp0 = gedit_tab_get_document (tab), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	project = vtg_plugin_project_descriptor_find_from_document (instance, doc);
	_tmp1 = FALSE;
	if (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc) != NULL) {
		_tmp1 = _vala_strcmp0 (gtk_source_language_get_id (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc)), "vala") == 0;
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		GeditView* _tmp2;
		GeditView* view;
		_tmp2 = NULL;
		view = (_tmp2 = gedit_tab_get_view (tab), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
		vtg_plugin_initialize_view (instance, project, view);
		(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
	}
	vtg_plugin_initialize_document (instance, doc);
	(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
}


static void vtg_plugin_on_tab_removed (GeditWindow* sender, GeditTab* tab, VtgPlugin* instance) {
	GeditView* _tmp0;
	GeditView* view;
	GeditDocument* _tmp1;
	GeditDocument* doc;
	g_return_if_fail (sender != NULL);
	g_return_if_fail (tab != NULL);
	g_return_if_fail (instance != NULL);
	_tmp0 = NULL;
	view = (_tmp0 = gedit_tab_get_view (tab), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	_tmp1 = NULL;
	doc = (_tmp1 = gedit_tab_get_document (tab), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	vtg_plugin_uninitialize_view (instance, view);
	vtg_plugin_uninitialize_document (instance, doc);
	(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
	(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
}


static VtgProjectDescriptor* vtg_plugin_project_descriptor_find_from_document (VtgPlugin* self, GeditDocument* document) {
	const char* _tmp0;
	char* file;
	VtgProjectDescriptor* _tmp6;
	VtgProjectDescriptor* _tmp7;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (document != NULL, NULL);
	_tmp0 = NULL;
	file = (_tmp0 = gedit_document_get_uri (document), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (file == NULL) {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		file = (_tmp2 = (_tmp1 = gedit_document_get_short_name_for_display (document), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), file = (g_free (file), NULL), _tmp2);
	}
	{
		GeeIterator* _project_it;
		_project_it = gee_iterable_iterator ((GeeIterable*) self->priv->_projects);
		while (gee_iterator_next (_project_it)) {
			VtgProjectDescriptor* project;
			project = (VtgProjectDescriptor*) gee_iterator_get (_project_it);
			if (vtg_project_manager_project_contains_source_file (project->project, file)) {
				VtgProjectDescriptor* _tmp3;
				_tmp3 = NULL;
				return (_tmp3 = project, (_project_it == NULL) ? NULL : (_project_it = (gee_collection_object_unref (_project_it), NULL)), file = (g_free (file), NULL), _tmp3);
			}
			(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		}
		(_project_it == NULL) ? NULL : (_project_it = (gee_collection_object_unref (_project_it), NULL));
	}
	/*return default_project anyway*/
	if (self->priv->_default_project == NULL) {
		VtgProjectDescriptor* _tmp4;
		VscSymbolCompletion* _tmp5;
		_tmp4 = NULL;
		self->priv->_default_project = (_tmp4 = vtg_project_descriptor_new (), (self->priv->_default_project == NULL) ? NULL : (self->priv->_default_project = (g_object_unref (self->priv->_default_project), NULL)), _tmp4);
		_tmp5 = NULL;
		self->priv->_default_project->completion = (_tmp5 = vsc_symbol_completion_new (), (self->priv->_default_project->completion == NULL) ? NULL : (self->priv->_default_project->completion = (g_object_unref (self->priv->_default_project->completion), NULL)), _tmp5);
		vsc_parser_manager_resume_parsing (vsc_symbol_completion_get_parser (self->priv->_default_project->completion));
	}
	_tmp6 = NULL;
	_tmp7 = NULL;
	return (_tmp7 = (_tmp6 = self->priv->_default_project, (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6)), file = (g_free (file), NULL), _tmp7);
}


static gboolean vtg_plugin_scs_contains (VtgPlugin* self, GeditView* view) {
	VtgSymbolCompletionHelper* _tmp0;
	gboolean _tmp1;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (view != NULL, FALSE);
	_tmp0 = NULL;
	return (_tmp1 = ((_tmp0 = vtg_plugin_scs_find_from_view (self, view)) != NULL), (_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)), _tmp1);
}


VtgSymbolCompletionHelper* vtg_plugin_scs_find_from_view (VtgPlugin* self, GeditView* view) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (view != NULL, NULL);
	{
		GeeIterator* _sc_it;
		_sc_it = gee_iterable_iterator ((GeeIterable*) self->priv->_scs);
		while (gee_iterator_next (_sc_it)) {
			VtgSymbolCompletionHelper* sc;
			sc = (VtgSymbolCompletionHelper*) gee_iterator_get (_sc_it);
			if (vtg_symbol_completion_helper_get_view (sc) == view) {
				VtgSymbolCompletionHelper* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = sc, (_sc_it == NULL) ? NULL : (_sc_it = (gee_collection_object_unref (_sc_it), NULL)), _tmp0);
			}
			(sc == NULL) ? NULL : (sc = (g_object_unref (sc), NULL));
		}
		(_sc_it == NULL) ? NULL : (_sc_it = (gee_collection_object_unref (_sc_it), NULL));
	}
	return NULL;
}


static gboolean vtg_plugin_bcs_contains (VtgPlugin* self, GeditView* view) {
	VtgBracketCompletion* _tmp0;
	gboolean _tmp1;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (view != NULL, FALSE);
	_tmp0 = NULL;
	return (_tmp1 = ((_tmp0 = vtg_plugin_bcs_find_from_view (self, view)) != NULL), (_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)), _tmp1);
}


static VtgBracketCompletion* vtg_plugin_bcs_find_from_view (VtgPlugin* self, GeditView* view) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (view != NULL, NULL);
	{
		GeeIterator* _bc_it;
		_bc_it = gee_iterable_iterator ((GeeIterable*) self->priv->_bcs);
		while (gee_iterator_next (_bc_it)) {
			VtgBracketCompletion* bc;
			bc = (VtgBracketCompletion*) gee_iterator_get (_bc_it);
			if (vtg_bracket_completion_get_view (bc) == view) {
				VtgBracketCompletion* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = bc, (_bc_it == NULL) ? NULL : (_bc_it = (gee_collection_object_unref (_bc_it), NULL)), _tmp0);
			}
			(bc == NULL) ? NULL : (bc = (g_object_unref (bc), NULL));
		}
		(_bc_it == NULL) ? NULL : (_bc_it = (gee_collection_object_unref (_bc_it), NULL));
	}
	return NULL;
}


static void vtg_plugin_initialize_views (VtgPlugin* self, GeditWindow* window) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (window != NULL);
	{
		GList* view_collection;
		GList* view_it;
		view_collection = gedit_window_get_views (window);
		for (view_it = view_collection; view_it != NULL; view_it = view_it->next) {
			GeditView* _tmp2;
			GeditView* view;
			_tmp2 = NULL;
			view = (_tmp2 = (GeditView*) view_it->data, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
			{
				GeditDocument* _tmp0;
				GeditDocument* doc;
				gboolean _tmp1;
				_tmp0 = NULL;
				doc = (_tmp0 = GEDIT_DOCUMENT ((gtk_text_view_get_buffer ((GtkTextView*) view))), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
				_tmp1 = FALSE;
				if (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc) != NULL) {
					_tmp1 = _vala_strcmp0 (gtk_source_language_get_id (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc)), "vala") == 0;
				} else {
					_tmp1 = FALSE;
				}
				if (_tmp1) {
					VtgProjectDescriptor* project;
					project = vtg_plugin_project_descriptor_find_from_document (self, doc);
					vtg_plugin_initialize_view (self, project, view);
					(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
				}
				(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			}
		}
	}
}


static void vtg_plugin_initialize_view (VtgPlugin* self, VtgProjectDescriptor* project, GeditView* view) {
	gboolean _tmp0;
	gboolean _tmp1;
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	g_return_if_fail (view != NULL);
	_tmp0 = FALSE;
	if (vtg_configuration_get_symbol_enabled (self->priv->_config)) {
		_tmp0 = !vtg_plugin_scs_contains (self, view);
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		vtg_plugin_activate_symbol (self, project, view);
	} else {
		g_warning ("vtgplugin.vala:294: sc already initialized for view");
	}
	_tmp1 = FALSE;
	if (vtg_configuration_get_bracket_enabled (self->priv->_config)) {
		_tmp1 = !vtg_plugin_bcs_contains (self, view);
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		vtg_plugin_activate_bracket (self, view);
	} else {
		g_warning ("vtgplugin.vala:300: bc already initialized vor view");
	}
}


static void vtg_plugin_initialize_document (VtgPlugin* self, GeditDocument* doc) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (doc != NULL);
	g_signal_connect (doc, "notify::language", (GCallback) vtg_plugin_on_notify_language, self);
}


static void vtg_plugin_uninitialize_view (VtgPlugin* self, GeditView* view) {
	VtgSymbolCompletionHelper* sc;
	VtgBracketCompletion* bc;
	g_return_if_fail (self != NULL);
	g_return_if_fail (view != NULL);
	sc = vtg_plugin_scs_find_from_view (self, view);
	if (sc != NULL) {
		vtg_plugin_deactivate_symbol (self, sc);
	} else {
		g_warning ("vtgplugin.vala:315: sc not found");
	}
	bc = vtg_plugin_bcs_find_from_view (self, view);
	if (bc != NULL) {
		vtg_plugin_deactivate_bracket (self, bc);
	} else {
		g_warning ("vtgplugin.vala:322: bc not found");
	}
	(sc == NULL) ? NULL : (sc = (g_object_unref (sc), NULL));
	(bc == NULL) ? NULL : (bc = (g_object_unref (bc), NULL));
}


static void vtg_plugin_uninitialize_document (VtgPlugin* self, GeditDocument* doc) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (doc != NULL);
	g_signal_handlers_disconnect_by_func (doc, (void*) vtg_plugin_on_notify_language, self);
}


static void vtg_plugin_on_notify_language (GeditDocument* sender, GParamSpec* pspec, VtgPlugin* instance) {
	GeditApp* _tmp0;
	GeditApp* app;
	g_return_if_fail (sender != NULL);
	g_return_if_fail (pspec != NULL);
	g_return_if_fail (instance != NULL);
	/*search the view*/
	_tmp0 = NULL;
	app = (_tmp0 = gedit_app_get_default (), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	{
		GList* view_collection;
		GList* view_it;
		view_collection = gedit_app_get_views (app);
		for (view_it = view_collection; view_it != NULL; view_it = view_it->next) {
			GeditView* _tmp2;
			GeditView* view;
			_tmp2 = NULL;
			view = (_tmp2 = (GeditView*) view_it->data, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
			{
				if (gtk_text_view_get_buffer ((GtkTextView*) view) == GTK_TEXT_BUFFER (sender)) {
					gboolean _tmp1;
					_tmp1 = FALSE;
					if (gtk_source_buffer_get_language ((GtkSourceBuffer*) sender) == NULL) {
						_tmp1 = TRUE;
					} else {
						_tmp1 = _vala_strcmp0 (gtk_source_language_get_id (gtk_source_buffer_get_language ((GtkSourceBuffer*) sender)), "vala") != 0;
					}
					if (_tmp1) {
						vtg_plugin_uninitialize_view (instance, view);
					} else {
						VtgProjectDescriptor* project;
						project = vtg_plugin_project_descriptor_find_from_document (instance, sender);
						vtg_plugin_initialize_view (instance, project, view);
						(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
					}
					(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
					break;
				}
				(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
			}
		}
	}
	(app == NULL) ? NULL : (app = (g_object_unref (app), NULL));
}


void vtg_plugin_activate_display_name (VtgPlugin* self, const char* display_name, gint line, gint col) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (display_name != NULL);
	{
		GList* doc_collection;
		GList* doc_it;
		doc_collection = gedit_window_get_documents (self->priv->_window);
		for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
			GeditDocument* _tmp1;
			GeditDocument* doc;
			_tmp1 = NULL;
			doc = (_tmp1 = (GeditDocument*) doc_it->data, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
			{
				if (_vala_strcmp0 (gedit_document_get_short_name_for_display (doc), display_name) == 0) {
					GeditTab* _tmp0;
					GeditTab* tab;
					_tmp0 = NULL;
					tab = (_tmp0 = gedit_tab_get_from_document (doc), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
					gedit_window_set_active_tab (self->priv->_window, tab);
					gedit_document_goto_line (doc, line - 1);
					gedit_view_scroll_to_cursor (gedit_tab_get_view (tab));
					(tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL));
				}
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			}
		}
	}
}


GeditTab* vtg_plugin_activate_uri (VtgPlugin* self, const char* uri, gint line, gint col) {
	GeditTab* tab;
	GeditDocument* existing_doc;
	GeditTab* _tmp8;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (uri != NULL, NULL);
	tab = NULL;
	existing_doc = NULL;
	{
		GList* doc_collection;
		GList* doc_it;
		doc_collection = gedit_window_get_documents (self->priv->_window);
		for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
			GeditDocument* _tmp4;
			GeditDocument* doc;
			_tmp4 = NULL;
			doc = (_tmp4 = (GeditDocument*) doc_it->data, (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4));
			{
				if (_vala_strcmp0 (gedit_document_get_uri (doc), uri) == 0) {
					GeditTab* _tmp1;
					GeditTab* _tmp0;
					GeditDocument* _tmp3;
					GeditDocument* _tmp2;
					_tmp1 = NULL;
					_tmp0 = NULL;
					tab = (_tmp1 = (_tmp0 = gedit_tab_get_from_document (doc), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL)), _tmp1);
					_tmp3 = NULL;
					_tmp2 = NULL;
					existing_doc = (_tmp3 = (_tmp2 = doc, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (existing_doc == NULL) ? NULL : (existing_doc = (g_object_unref (existing_doc), NULL)), _tmp3);
					(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
					break;
				}
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			}
		}
	}
	if (tab == NULL) {
		GeditTab* _tmp6;
		GeditTab* _tmp5;
		_tmp6 = NULL;
		_tmp5 = NULL;
		tab = (_tmp6 = (_tmp5 = gedit_window_create_tab_from_uri (self->priv->_window, uri, gedit_encoding_get_utf8 (), line, TRUE, TRUE), (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5)), (tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL)), _tmp6);
	} else {
		gboolean _tmp7;
		gedit_window_set_active_tab (self->priv->_window, tab);
		_tmp7 = FALSE;
		if (existing_doc != NULL) {
			_tmp7 = line > 0;
		} else {
			_tmp7 = FALSE;
		}
		if (_tmp7) {
			gedit_document_goto_line (existing_doc, line - 1);
			gedit_view_scroll_to_cursor (gedit_tab_get_view (tab));
		}
	}
	_tmp8 = NULL;
	return (_tmp8 = tab, (existing_doc == NULL) ? NULL : (existing_doc = (g_object_unref (existing_doc), NULL)), _tmp8);
}


void vtg_plugin_on_project_closed (VtgPlugin* self, VtgProjectManagerPluginHelper* sender, VtgProjectManagerProject* project) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (project != NULL);
	g_return_if_fail (project != self->priv->_default_project->project);
	{
		GeeIterator* _descriptor_it;
		_descriptor_it = gee_iterable_iterator ((GeeIterable*) self->priv->_projects);
		while (gee_iterator_next (_descriptor_it)) {
			VtgProjectDescriptor* descriptor;
			descriptor = (VtgProjectDescriptor*) gee_iterator_get (_descriptor_it);
			if (descriptor->project == project) {
				gee_collection_remove ((GeeCollection*) self->priv->_projects, descriptor);
				(descriptor == NULL) ? NULL : (descriptor = (g_object_unref (descriptor), NULL));
				break;
			}
			(descriptor == NULL) ? NULL : (descriptor = (g_object_unref (descriptor), NULL));
		}
		(_descriptor_it == NULL) ? NULL : (_descriptor_it = (gee_collection_object_unref (_descriptor_it), NULL));
	}
}


void vtg_plugin_on_project_loaded (VtgPlugin* self, VtgProjectManagerPluginHelper* sender, VtgProjectManagerProject* project) {
	GError * inner_error;
	VtgProjectDescriptor* prj;
	VscSymbolCompletion* completion;
	VscSymbolCompletion* _tmp2;
	VscSymbolCompletion* _tmp1;
	VtgProjectManagerProject* _tmp4;
	VtgProjectManagerProject* _tmp3;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (project != NULL);
	inner_error = NULL;
	prj = vtg_project_descriptor_new ();
	completion = vsc_symbol_completion_new ();
	/* setup referenced packages */
	{
		GeeIterator* _module_it;
		/* setup referenced packages */
		_module_it = gee_iterable_iterator ((GeeIterable*) project->modules);
		/* setup referenced packages */
		while (gee_iterator_next (_module_it)) {
			VtgProjectManagerProjectModule* module;
			/* setup referenced packages */
			module = (VtgProjectManagerProjectModule*) gee_iterator_get (_module_it);
			{
				GeeIterator* _package_it;
				_package_it = gee_iterable_iterator ((GeeIterable*) module->packages);
				while (gee_iterator_next (_package_it)) {
					VtgProjectManagerProjectPackage* package;
					package = (VtgProjectManagerProjectPackage*) gee_iterator_get (_package_it);
					vsc_parser_manager_try_add_package (vsc_symbol_completion_get_parser (completion), package->name, TRUE);
					(package == NULL) ? NULL : (package = (g_object_unref (package), NULL));
				}
				(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
			}
			(module == NULL) ? NULL : (module = (g_object_unref (module), NULL));
		}
		(_module_it == NULL) ? NULL : (_module_it = (gee_collection_object_unref (_module_it), NULL));
	}
	/* setup vapidir, built libraries and local packages */
	{
		GeeIterator* _group_it;
		/* setup vapidir, built libraries and local packages */
		_group_it = gee_iterable_iterator ((GeeIterable*) project->groups);
		/* setup vapidir, built libraries and local packages */
		while (gee_iterator_next (_group_it)) {
			VtgProjectManagerProjectGroup* group;
			/* setup vapidir, built libraries and local packages */
			group = (VtgProjectManagerProjectGroup*) gee_iterator_get (_group_it);
			{
				GeeIterator* _package_it;
				_package_it = gee_iterable_iterator ((GeeIterable*) group->built_libraries);
				while (gee_iterator_next (_package_it)) {
					char* package;
					package = (char*) gee_iterator_get (_package_it);
					vsc_parser_manager_add_built_package (vsc_symbol_completion_get_parser (completion), package);
					package = (g_free (package), NULL);
				}
				(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
			}
			{
				GeeIterator* _path_it;
				_path_it = gee_iterable_iterator ((GeeIterable*) group->vapidirs);
				while (gee_iterator_next (_path_it)) {
					char* path;
					path = (char*) gee_iterator_get (_path_it);
					vsc_parser_manager_add_path_to_vapi_search_dir (vsc_symbol_completion_get_parser (completion), path);
					path = (g_free (path), NULL);
				}
				(_path_it == NULL) ? NULL : (_path_it = (gee_collection_object_unref (_path_it), NULL));
			}
			{
				GeeIterator* _package_it;
				_package_it = gee_iterable_iterator ((GeeIterable*) group->packages);
				while (gee_iterator_next (_package_it)) {
					char* package;
					package = (char*) gee_iterator_get (_package_it);
					vsc_parser_manager_try_add_package (vsc_symbol_completion_get_parser (completion), package, TRUE);
					package = (g_free (package), NULL);
				}
				(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	/* setup source files */
	{
		GeeIterator* _group_it;
		/* setup source files */
		_group_it = gee_iterable_iterator ((GeeIterable*) project->groups);
		/* setup source files */
		while (gee_iterator_next (_group_it)) {
			VtgProjectManagerProjectGroup* group;
			/* setup source files */
			group = (VtgProjectManagerProjectGroup*) gee_iterator_get (_group_it);
			{
				GeeIterator* _target_it;
				_target_it = gee_iterable_iterator ((GeeIterable*) group->targets);
				while (gee_iterator_next (_target_it)) {
					VtgProjectManagerProjectTarget* target;
					target = (VtgProjectManagerProjectTarget*) gee_iterator_get (_target_it);
					{
						GeeIterator* _source_it;
						_source_it = gee_iterable_iterator ((GeeIterable*) target->sources);
						while (gee_iterator_next (_source_it)) {
							VtgProjectManagerProjectSource* source;
							gboolean _tmp0;
							source = (VtgProjectManagerProjectSource*) gee_iterator_get (_source_it);
							_tmp0 = FALSE;
							if (g_str_has_suffix (source->uri, ".vala")) {
								_tmp0 = g_str_has_prefix (source->uri, "file://");
							} else {
								_tmp0 = FALSE;
							}
							if (_tmp0) {
								char* filename;
								filename = string_substring (source->uri, (glong) 7, string_get_length (source->uri) - 7);
								{
									vsc_parser_manager_add_source (vsc_symbol_completion_get_parser (completion), filename, &inner_error);
									if (inner_error != NULL) {
										goto __catch34_g_error;
										goto __finally34;
									}
								}
								goto __finally34;
								__catch34_g_error:
								{
									GError * err;
									err = inner_error;
									inner_error = NULL;
									{
										g_warning ("vtgplugin.vala:432: Error adding source %s: %s", filename, err->message);
										(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
									}
								}
								__finally34:
								if (inner_error != NULL) {
									filename = (g_free (filename), NULL);
									(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
									(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
									(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
									(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
									(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
									(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
									(prj == NULL) ? NULL : (prj = (g_object_unref (prj), NULL));
									(completion == NULL) ? NULL : (completion = (g_object_unref (completion), NULL));
									g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
									g_clear_error (&inner_error);
									return;
								}
								filename = (g_free (filename), NULL);
							}
							(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
						}
						(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	_tmp2 = NULL;
	_tmp1 = NULL;
	prj->completion = (_tmp2 = (_tmp1 = completion, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (prj->completion == NULL) ? NULL : (prj->completion = (g_object_unref (prj->completion), NULL)), _tmp2);
	_tmp4 = NULL;
	_tmp3 = NULL;
	prj->project = (_tmp4 = (_tmp3 = project, (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3)), (prj->project == NULL) ? NULL : (prj->project = (g_object_unref (prj->project), NULL)), _tmp4);
	gee_collection_add ((GeeCollection*) self->priv->_projects, prj);
	vsc_parser_manager_resume_parsing (vsc_symbol_completion_get_parser (completion));
	(prj == NULL) ? NULL : (prj = (g_object_unref (prj), NULL));
	(completion == NULL) ? NULL : (completion = (g_object_unref (completion), NULL));
}


VtgPlugin* vtg_plugin_construct (GType object_type) {
	VtgPlugin * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgPlugin* vtg_plugin_new (void) {
	return vtg_plugin_construct (VTG_TYPE_PLUGIN);
}


GeeList* vtg_plugin_get_projects (VtgPlugin* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_projects;
}


VtgProjectManagerOutputView* vtg_plugin_get_output_view (VtgPlugin* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_output_view;
}


void vtg_plugin_set_output_view (VtgPlugin* self, VtgProjectManagerOutputView* value) {
	VtgProjectManagerOutputView* _tmp2;
	VtgProjectManagerOutputView* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_output_view = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_output_view == NULL) ? NULL : (self->priv->_output_view = (g_object_unref (self->priv->_output_view), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "output-view");
}


VtgProjectManagerPluginHelper* vtg_plugin_get_project_manager (VtgPlugin* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_prj_man;
}


VtgConfiguration* vtg_plugin_get_config (VtgPlugin* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_config;
}


GeditWindow* vtg_plugin_get_gedit_window (VtgPlugin* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_window;
}


static void _vtg_plugin_on_configuration_property_changed_g_object_notify (VtgConfiguration* _sender, GParamSpec* pspec, gpointer self) {
	vtg_plugin_on_configuration_property_changed (self, _sender, pspec);
}


static GObject * vtg_plugin_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgPluginClass * klass;
	GObjectClass * parent_class;
	VtgPlugin * self;
	klass = VTG_PLUGIN_CLASS (g_type_class_peek (VTG_TYPE_PLUGIN));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_PLUGIN (obj);
	{
		VtgConfiguration* _tmp0;
		_tmp0 = NULL;
		self->priv->_config = (_tmp0 = vtg_configuration_new (), (self->priv->_config == NULL) ? NULL : (self->priv->_config = (g_object_unref (self->priv->_config), NULL)), _tmp0);
		g_signal_connect_object ((GObject*) self->priv->_config, "notify", (GCallback) _vtg_plugin_on_configuration_property_changed_g_object_notify, self, 0);
		bindtextdomain (GETTEXT_PACKAGE, NULL);
	}
	return obj;
}


static void vtg_plugin_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgPlugin * self;
	gpointer boxed;
	self = VTG_PLUGIN (object);
	switch (property_id) {
		case VTG_PLUGIN_PROJECTS:
		g_value_set_object (value, vtg_plugin_get_projects (self));
		break;
		case VTG_PLUGIN_OUTPUT_VIEW:
		g_value_set_object (value, vtg_plugin_get_output_view (self));
		break;
		case VTG_PLUGIN_PROJECT_MANAGER:
		g_value_set_object (value, vtg_plugin_get_project_manager (self));
		break;
		case VTG_PLUGIN_CONFIG:
		g_value_set_object (value, vtg_plugin_get_config (self));
		break;
		case VTG_PLUGIN_GEDIT_WINDOW:
		g_value_set_object (value, vtg_plugin_get_gedit_window (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_plugin_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgPlugin * self;
	self = VTG_PLUGIN (object);
	switch (property_id) {
		case VTG_PLUGIN_OUTPUT_VIEW:
		vtg_plugin_set_output_view (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_plugin_class_init (VtgPluginClass * klass) {
	vtg_plugin_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgPluginPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_plugin_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_plugin_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_plugin_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_plugin_finalize;
	GEDIT_PLUGIN_CLASS (klass)->activate = vtg_plugin_real_activate;
	GEDIT_PLUGIN_CLASS (klass)->deactivate = vtg_plugin_real_deactivate;
	GEDIT_PLUGIN_CLASS (klass)->is_configurable = vtg_plugin_real_is_configurable;
	GEDIT_PLUGIN_CLASS (klass)->create_configure_dialog = vtg_plugin_real_create_configure_dialog;
	GEDIT_PLUGIN_CLASS (klass)->update_ui = vtg_plugin_real_update_ui;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_PROJECTS, gee_param_spec_collection_object ("projects", "projects", "projects", GEE_TYPE_LIST, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_OUTPUT_VIEW, g_param_spec_object ("output-view", "output-view", "output-view", VTG_PROJECT_MANAGER_TYPE_OUTPUT_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_PROJECT_MANAGER, g_param_spec_object ("project-manager", "project-manager", "project-manager", VTG_PROJECT_MANAGER_TYPE_PLUGIN_HELPER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_CONFIG, g_param_spec_object ("config", "config", "config", VTG_TYPE_CONFIGURATION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_GEDIT_WINDOW, g_param_spec_object ("gedit-window", "gedit-window", "gedit-window", GEDIT_TYPE_WINDOW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_plugin_instance_init (VtgPlugin * self) {
	self->priv = VTG_PLUGIN_GET_PRIVATE (self);
	self->priv->_window = NULL;
	self->priv->_bcs = (GeeList*) gee_array_list_new (VTG_TYPE_BRACKET_COMPLETION, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->_scs = (GeeList*) gee_array_list_new (VTG_TYPE_SYMBOL_COMPLETION_HELPER, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->_projects = (GeeList*) gee_array_list_new (VTG_TYPE_PROJECT_DESCRIPTOR, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->_config = NULL;
	self->priv->_default_project = NULL;
}


static void vtg_plugin_finalize (GObject* obj) {
	VtgPlugin * self;
	self = VTG_PLUGIN (obj);
	{
		vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL);
	}
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	(self->priv->_bcs == NULL) ? NULL : (self->priv->_bcs = (gee_collection_object_unref (self->priv->_bcs), NULL));
	(self->priv->_scs == NULL) ? NULL : (self->priv->_scs = (gee_collection_object_unref (self->priv->_scs), NULL));
	(self->priv->_projects == NULL) ? NULL : (self->priv->_projects = (gee_collection_object_unref (self->priv->_projects), NULL));
	(self->priv->_config == NULL) ? NULL : (self->priv->_config = (g_object_unref (self->priv->_config), NULL));
	(self->priv->_default_project == NULL) ? NULL : (self->priv->_default_project = (g_object_unref (self->priv->_default_project), NULL));
	(self->priv->_prj_man == NULL) ? NULL : (self->priv->_prj_man = (g_object_unref (self->priv->_prj_man), NULL));
	(self->priv->_output_view == NULL) ? NULL : (self->priv->_output_view = (g_object_unref (self->priv->_output_view), NULL));
	G_OBJECT_CLASS (vtg_plugin_parent_class)->finalize (obj);
}


static GType vtg_plugin_type_id = 0;
GType vtg_plugin_get_type (void) {
	return vtg_plugin_type_id;
}


GType vtg_plugin_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgPluginClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_plugin_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgPlugin), 0, (GInstanceInitFunc) vtg_plugin_instance_init, NULL };
	vtg_plugin_type_id = g_type_module_register_type (module, GEDIT_TYPE_PLUGIN, "VtgPlugin", &g_define_type_info, 0);
	return vtg_plugin_type_id;
}


GType register_gedit_plugin (GTypeModule* module) {
	g_return_val_if_fail (module != NULL, 0UL);
	vtg_bracket_completion_register_type (module);
	vtg_symbol_completion_provider_register_type (module);
	vtg_symbol_completion_trigger_register_type (module);
	vtg_symbol_completion_helper_register_type (module);
	vtg_project_manager_plugin_helper_register_type (module);
	vtg_project_manager_view_register_type (module);
	vtg_project_manager_builder_register_type (module);
	vtg_project_manager_executer_register_type (module);
	vtg_project_manager_executer_dialog_register_type (module);
	vtg_project_manager_output_view_register_type (module);
	vtg_project_manager_process_watch_info_register_type (module);
	vtg_project_manager_build_log_view_register_type (module);
	vtg_project_manager_project_register_type (module);
	vtg_project_manager_project_module_register_type (module);
	vtg_project_manager_project_package_register_type (module);
	vtg_project_manager_project_group_register_type (module);
	vtg_project_manager_project_target_register_type (module);
	vtg_project_manager_project_source_register_type (module);
	vtg_project_manager_packages_dialog_register_type (module);
	vtg_project_manager_filtered_list_dialog_register_type (module);
	vtg_vcs_backends_item_register_type (module);
	vtg_vcs_backends_git_register_type (module);
	vtg_vcs_backends_bzr_register_type (module);
	vtg_vcs_backends_svn_register_type (module);
	vtg_change_log_register_type (module);
	vtg_interaction_parameters_dialog_register_type (module);
	vtg_configuration_register_type (module);
	vtg_project_descriptor_register_type (module);
	vtg_plugin_register_type (module);
	return VTG_TYPE_PLUGIN;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




