/*
 *  vtgplugin.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#ifndef __VTGPLUGIN_H__
#define __VTGPLUGIN_H__

#include <glib.h>
#include <glib-object.h>
#include <gedit/gedit-plugin.h>
#include <gedit/gedit-window.h>
#include <vscsymbolcompletion.h>
#include <gedit/gedit-tab.h>
#include <stdlib.h>
#include <string.h>
#include <gee/list.h>
#include <gedit/gedit-view.h>
#include <gtk/gtk.h>
#include "vtgprojectmanagerproject.h"
#include "vtgconfiguration.h"

G_BEGIN_DECLS

typedef struct _VtgProjectDescriptor VtgProjectDescriptor;
typedef struct _VtgProjectDescriptorClass VtgProjectDescriptorClass;
typedef struct _VtgPlugin VtgPlugin;
typedef struct _VtgPluginClass VtgPluginClass;
typedef struct _VtgProjectManagerOutputView VtgProjectManagerOutputView;
typedef struct _VtgProjectManagerOutputViewClass VtgProjectManagerOutputViewClass;
typedef struct _VtgProjectManagerPluginHelper VtgProjectManagerPluginHelper;
typedef struct _VtgProjectManagerPluginHelperClass VtgProjectManagerPluginHelperClass;
typedef struct _VtgProjectManagerView VtgProjectManagerView;
typedef struct _VtgProjectManagerViewClass VtgProjectManagerViewClass;
typedef struct _VtgSymbolCompletionHelper VtgSymbolCompletionHelper;
typedef struct _VtgSymbolCompletionHelperClass VtgSymbolCompletionHelperClass;

#define VTG_TYPE_PROJECT_DESCRIPTOR (vtg_project_descriptor_get_type ())
#define VTG_PROJECT_DESCRIPTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptor))
#define VTG_PROJECT_DESCRIPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptorClass))
#define VTG_IS_PROJECT_DESCRIPTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_DESCRIPTOR))
#define VTG_IS_PROJECT_DESCRIPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_DESCRIPTOR))
#define VTG_PROJECT_DESCRIPTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptorClass))

typedef struct _VtgProjectDescriptorPrivate VtgProjectDescriptorPrivate;

#define VTG_TYPE_PLUGIN (vtg_plugin_get_type ())
#define VTG_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PLUGIN, VtgPlugin))
#define VTG_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PLUGIN, VtgPluginClass))
#define VTG_IS_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PLUGIN))
#define VTG_IS_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PLUGIN))
#define VTG_PLUGIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PLUGIN, VtgPluginClass))

typedef struct _VtgPluginPrivate VtgPluginPrivate;

struct _VtgProjectDescriptor {
	GObject parent_instance;
	VtgProjectDescriptorPrivate * priv;
	VscSymbolCompletion* completion;
	VtgProjectManagerProject* project;
};

struct _VtgProjectDescriptorClass {
	GObjectClass parent_class;
};

struct _VtgPlugin {
	GeditPlugin parent_instance;
	VtgPluginPrivate * priv;
};

struct _VtgPluginClass {
	GeditPluginClass parent_class;
};


VtgProjectDescriptor* vtg_project_descriptor_construct (GType object_type);
VtgProjectDescriptor* vtg_project_descriptor_new (void);
GType vtg_project_descriptor_get_type (void);
GType vtg_project_descriptor_register_type (GTypeModule * module);
VtgSymbolCompletionHelper* vtg_plugin_scs_find_from_view (VtgPlugin* self, GeditView* view);
void vtg_plugin_activate_display_name (VtgPlugin* self, const char* display_name, gint line, gint col);
GeditTab* vtg_plugin_activate_uri (VtgPlugin* self, const char* uri, gint line, gint col);
void vtg_plugin_on_project_closed (VtgPlugin* self, VtgProjectManagerPluginHelper* sender, VtgProjectManagerProject* project);
void vtg_plugin_on_project_loaded (VtgPlugin* self, VtgProjectManagerPluginHelper* sender, VtgProjectManagerProject* project);
VtgPlugin* vtg_plugin_construct (GType object_type);
VtgPlugin* vtg_plugin_new (void);
GeeList* vtg_plugin_get_projects (VtgPlugin* self);
VtgProjectManagerOutputView* vtg_plugin_get_output_view (VtgPlugin* self);
void vtg_plugin_set_output_view (VtgPlugin* self, VtgProjectManagerOutputView* value);
VtgProjectManagerPluginHelper* vtg_plugin_get_project_manager (VtgPlugin* self);
VtgConfiguration* vtg_plugin_get_config (VtgPlugin* self);
GeditWindow* vtg_plugin_get_gedit_window (VtgPlugin* self);
GType vtg_plugin_get_type (void);
GType vtg_plugin_register_type (GTypeModule * module);
GType register_gedit_plugin (GTypeModule* module);


G_END_DECLS

#endif
