/*
 *  vtgprojectmanager.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanager.h"
#include <stdlib.h>
#include <string.h>
#include <gee/arraylist.h>
#include <gee/list.h>
#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>
#include <sys/wait.h>
#include <gee/collection.h>
#include <gedit/gedit-window.h>
#include <gedit/gedit-document.h>
#include <gedit/gedit-tab.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gedit/gedit-view.h>
#include <vscsymbolcompletion.h>
#include <vscsymbolcompletionitem.h>
#include <config.h>
#include <gtksourceview/gtksourceview.h>
#include "vtgprojectmanagerproject.h"
#include "vtgprojectmanagerview.h"
#include "vtgprojectmanagerbuilder.h"
#include "vtgprojectmanagerexecuter.h"
#include "vtgchangelog.h"
#include "vtgprojectmanageroutputview.h"
#include "vtgutils.h"
#include "vtgparametersdialog.h"
#include "vtgconfiguration.h"
#include "vtgprojectmanagerexecuterdialog.h"
#include "vtgprojectmanagerprojectsource.h"
#include "vtgprojectmanagerfilteredlistdialog.h"
#include "vtgsymbolcompletionhelper.h"
#include "vtgsymbolcompletiontrigger.h"
#include "vtgprojectmanagerbuildlogview.h"




static glong string_get_length (const char* self);
static char* string_replace (const char* self, const char* old, const char* replacement);
struct _VtgProjectManagerPluginHelperPrivate {
	char* _ui_def;
	guint _ui_id;
	GeeList* _projects;
	GtkActionGroup* _actions;
	VtgPlugin* _plugin;
	VtgProjectManagerView* _prj_view;
	VtgProjectManagerBuilder* _prj_builder;
	VtgProjectManagerExecuter* _prj_executer;
	VtgChangeLog* _changelog;
};

#define VTG_PROJECT_MANAGER_PLUGIN_HELPER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_PLUGIN_HELPER, VtgProjectManagerPluginHelperPrivate))
enum  {
	VTG_PROJECT_MANAGER_PLUGIN_HELPER_DUMMY_PROPERTY,
	VTG_PROJECT_MANAGER_PLUGIN_HELPER_PLUGIN,
	VTG_PROJECT_MANAGER_PLUGIN_HELPER_PROJECT_VIEW
};
static void _vtg_project_manager_plugin_helper_on_project_new_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_open_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_close_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_build_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_clean_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_clean_stamps_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_configure_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_standalone_file_compile_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_error_next_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_error_previuos_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_execute_process_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_kill_process_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_goto_document_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_project_goto_method_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_complete_word_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_prepare_changelog_gtk_action_callback (GtkAction* action, gpointer self);
static void _vtg_project_manager_plugin_helper_on_prepare_single_file_changelog_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_plugin_helper_initialize_ui (VtgProjectManagerPluginHelper* self);
static void vtg_project_manager_plugin_helper_on_prepare_changelog (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_prepare_single_file_changelog (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_complete_word (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_open (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_close (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_new (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_goto_document (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_goto_method (VtgProjectManagerPluginHelper* self, GtkAction* action);
static VtgProjectDescriptor* vtg_project_manager_plugin_helper_get_projectdescriptor_for_project (VtgProjectManagerPluginHelper* self, VtgProjectManagerProject* project);
static void vtg_project_manager_plugin_helper_on_standalone_file_compile (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_build (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_configure (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_clean (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_clean_stamps (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_execute_process (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_kill_process (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_clean_project (VtgProjectManagerPluginHelper* self, gboolean stamps);
static void vtg_project_manager_plugin_helper_on_project_error_next (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_project_error_previuos (VtgProjectManagerPluginHelper* self, GtkAction* action);
static void vtg_project_manager_plugin_helper_on_current_project_changed (VtgProjectManagerPluginHelper* self, GObject* sender, GParamSpec* pspec);
static void vtg_project_manager_plugin_helper_update_ui (VtgProjectManagerPluginHelper* self, gboolean default_project);
static void vtg_project_manager_plugin_helper_open_project (VtgProjectManagerPluginHelper* self, const char* name);
static void vtg_project_manager_plugin_helper_close_project (VtgProjectManagerPluginHelper* self, VtgProjectManagerProject* project);
static void vtg_project_manager_plugin_helper_project_save_all (VtgProjectManagerPluginHelper* self, VtgProjectManagerProject* project);
static void _vtg_project_manager_plugin_helper_on_child_watch_gchild_watch_func (GPid pid, gint status, gpointer self);
static void vtg_project_manager_plugin_helper_create_project (VtgProjectManagerPluginHelper* self, const char* project_path);
static gboolean vtg_project_manager_plugin_helper_is_dir_empty (VtgProjectManagerPluginHelper* self, const char* dir_path);
static void vtg_project_manager_plugin_helper_on_child_watch (VtgProjectManagerPluginHelper* self, GPid pid, gint status);
static void vtg_project_manager_plugin_helper_set_plugin (VtgProjectManagerPluginHelper* self, VtgPlugin* value);
static void _vtg_project_manager_plugin_helper_on_current_project_changed_g_object_notify (VtgProjectManagerView* _sender, GParamSpec* pspec, gpointer self);
static void __lambda0 (VtgProjectManagerExecuter* sender, VtgProjectManagerPluginHelper* self);
static void ___lambda0_vtg_project_manager_executer_process_start (VtgProjectManagerExecuter* _sender, gpointer self);
static void __lambda1 (VtgProjectManagerExecuter* sender, gint exit_status, VtgProjectManagerPluginHelper* self);
static void ___lambda1_vtg_project_manager_executer_process_exit (VtgProjectManagerExecuter* _sender, gint exit_status, gpointer self);
static void __lambda2 (VtgProjectManagerBuilder* sender, VtgProjectManagerPluginHelper* self);
static void ___lambda2_vtg_project_manager_builder_build_start (VtgProjectManagerBuilder* _sender, gpointer self);
static void __lambda3 (VtgProjectManagerBuilder* sender, gint exit_status, VtgProjectManagerPluginHelper* self);
static void ___lambda3_vtg_project_manager_builder_build_exit (VtgProjectManagerBuilder* _sender, gint exit_status, gpointer self);
static GObject * vtg_project_manager_plugin_helper_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_project_manager_plugin_helper_parent_class = NULL;
static void vtg_project_manager_plugin_helper_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const GtkActionEntry VTG_PROJECT_MANAGER_PLUGIN_HELPER__action_entries[] = {{"ProjectNew", NULL, N_ ("_New Project..."), NULL, N_ ("Create a new project"), (GCallback) _vtg_project_manager_plugin_helper_on_project_new_gtk_action_callback}, {"ProjectOpen", NULL, N_ ("Op_en Project..."), "<control><alt>O", N_ ("Open an existing project"), (GCallback) _vtg_project_manager_plugin_helper_on_project_open_gtk_action_callback}, {"ProjectClose", NULL, N_ ("_Close Current Project"), NULL, N_ ("Close current selected project"), (GCallback) _vtg_project_manager_plugin_helper_on_project_close_gtk_action_callback}, {"ProjectBuildMenuAction", NULL, N_ ("Build"), NULL, N_ ("Build menu"), (GCallback) NULL}, {"ProjectBuild", GTK_STOCK_EXECUTE, N_ ("_Build Project"), "<control><shift>B", N_ ("Build the current project using 'make'"), (GCallback) _vtg_project_manager_plugin_helper_on_project_build_gtk_action_callback}, {"ProjectBuildClean", GTK_STOCK_CLEAR, N_ ("_Clean Project"), NULL, N_ ("Clean the current project using 'make clean'"), (GCallback) _vtg_project_manager_plugin_helper_on_project_clean_gtk_action_callback}, {"ProjectBuildCleanStamps", NULL, N_ ("_Clean Project and Vala 'Stamp' Files"), NULL, N_ ("Clean the current project stamp files"), (GCallback) _vtg_project_manager_plugin_helper_on_project_clean_stamps_gtk_action_callback}, {"ProjectBuildConfigure", NULL, N_ ("C_onfigure Project"), NULL, N_ ("Configure or reconfigure the current project"), (GCallback) _vtg_project_manager_plugin_helper_on_project_configure_gtk_action_callback}, {"ProjectBuildCompileFile", NULL, N_ ("_Compile File"), "<control>B", N_ ("Compile the current file with the vala compiler"), (GCallback) _vtg_project_manager_plugin_helper_on_standalone_file_compile_gtk_action_callback}, {"ProjectBuildNextError", GTK_STOCK_GO_FORWARD, N_ ("_Next Error"), "<control><shift>F12", N_ ("Go to next error source line"), (GCallback) _vtg_project_manager_plugin_helper_on_project_error_next_gtk_action_callback}, {"ProjectBuildPreviousError", GTK_STOCK_GO_BACK, N_ ("_Previuos Error"), NULL, N_ ("Go to previous error source line"), (GCallback) _vtg_project_manager_plugin_helper_on_project_error_previuos_gtk_action_callback}, {"ProjectBuildExecute", GTK_STOCK_EXECUTE, N_ ("_Execute"), "F5", N_ ("Excute target program"), (GCallback) _vtg_project_manager_plugin_helper_on_project_execute_process_gtk_action_callback}, {"ProjectBuildKill", GTK_STOCK_STOP, N_ ("_Stop process"), NULL, N_ ("Stop (kill) executing program"), (GCallback) _vtg_project_manager_plugin_helper_on_project_kill_process_gtk_action_callback}, {"ProjectGotoDocument", GTK_STOCK_JUMP_TO, N_ ("_Go To Document..."), "<control>J", N_ ("Open a document that belong to this project"), (GCallback) _vtg_project_manager_plugin_helper_on_project_goto_document_gtk_action_callback}, {"ProjectGotoMethod", NULL, N_ ("_Go To Method..."), "<control>M", N_ ("Goto to a specific method in the current source document"), (GCallback) _vtg_project_manager_plugin_helper_on_project_goto_method_gtk_action_callback}, {"ProjectCompleteWord", NULL, N_ ("Complete _Word"), "<control>space", N_ ("Try to complete the word in the current source document"), (GCallback) _vtg_project_manager_plugin_helper_on_complete_word_gtk_action_callback}, {"ProjectPrepareChangeLog", NULL, N_ ("_Prepare ChangeLog"), NULL, N_ ("Add an entry to the ChangeLog with all added/modified files"), (GCallback) _vtg_project_manager_plugin_helper_on_prepare_changelog_gtk_action_callback}, {"ProjectPrepareSingleFileChangeLog", NULL, N_ ("_Add Current File To ChangeLog"), NULL, N_ ("Add the current file to the ChangeLog"), (GCallback) _vtg_project_manager_plugin_helper_on_prepare_single_file_changelog_gtk_action_callback}};


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static char* string_replace (const char* self, const char* old, const char* replacement) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (old != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	inner_error = NULL;
	{
		char* _tmp0;
		GRegex* _tmp1;
		GRegex* regex;
		char* _tmp2;
		char* _tmp3;
		_tmp0 = NULL;
		_tmp1 = NULL;
		regex = (_tmp1 = g_regex_new (_tmp0 = g_regex_escape_string (old, -1), 0, 0, &inner_error), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
		if (inner_error != NULL) {
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch3_g_regex_error;
			}
			goto __finally3;
		}
		_tmp2 = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &inner_error);
		if (inner_error != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch3_g_regex_error;
			}
			goto __finally3;
		}
		_tmp3 = NULL;
		return (_tmp3 = _tmp2, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp3);
	}
	goto __finally3;
	__catch3_g_regex_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_assert_not_reached ();
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally3:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return NULL;
	}
}


static void _vtg_project_manager_plugin_helper_on_project_new_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_new (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_open_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_open (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_close_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_close (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_build_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_build (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_clean_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_clean (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_clean_stamps_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_clean_stamps (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_configure_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_configure (self, action);
}


static void _vtg_project_manager_plugin_helper_on_standalone_file_compile_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_standalone_file_compile (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_error_next_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_error_next (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_error_previuos_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_error_previuos (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_execute_process_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_execute_process (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_kill_process_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_kill_process (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_goto_document_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_goto_document (self, action);
}


static void _vtg_project_manager_plugin_helper_on_project_goto_method_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_project_goto_method (self, action);
}


static void _vtg_project_manager_plugin_helper_on_complete_word_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_complete_word (self, action);
}


static void _vtg_project_manager_plugin_helper_on_prepare_changelog_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_prepare_changelog (self, action);
}


static void _vtg_project_manager_plugin_helper_on_prepare_single_file_changelog_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_plugin_helper_on_prepare_single_file_changelog (self, action);
}


/*public signal void project_loaded (Project project);*/
VtgProjectManagerPluginHelper* vtg_project_manager_plugin_helper_construct (GType object_type, VtgPlugin* plugin) {
	GParameter * __params;
	GParameter * __params_it;
	VtgProjectManagerPluginHelper * self;
	g_return_val_if_fail (plugin != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "plugin";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN);
	g_value_set_object (&__params_it->value, plugin);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgProjectManagerPluginHelper* vtg_project_manager_plugin_helper_new (VtgPlugin* plugin) {
	return vtg_project_manager_plugin_helper_construct (VTG_PROJECT_MANAGER_TYPE_PLUGIN_HELPER, plugin);
}


static void vtg_project_manager_plugin_helper_initialize_ui (VtgProjectManagerPluginHelper* self) {
	GError * inner_error;
	GtkActionGroup* _tmp0;
	GtkUIManager* _tmp1;
	GtkUIManager* manager;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	_tmp0 = NULL;
	self->priv->_actions = (_tmp0 = gtk_action_group_new ("ProjectManagerActionGroup"), (self->priv->_actions == NULL) ? NULL : (self->priv->_actions = (g_object_unref (self->priv->_actions), NULL)), _tmp0);
	gtk_action_group_set_translation_domain (self->priv->_actions, GETTEXT_PACKAGE);
	gtk_action_group_add_actions (self->priv->_actions, VTG_PROJECT_MANAGER_PLUGIN_HELPER__action_entries, G_N_ELEMENTS (VTG_PROJECT_MANAGER_PLUGIN_HELPER__action_entries), self);
	_tmp1 = NULL;
	manager = (_tmp1 = gedit_window_get_ui_manager (vtg_plugin_get_gedit_window (vtg_project_manager_plugin_helper_get_plugin (self))), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	gtk_ui_manager_insert_action_group (manager, self->priv->_actions, -1);
	{
		guint _tmp2;
		_tmp2 = gtk_ui_manager_add_ui_from_string (manager, self->priv->_ui_def, (gssize) (-1), &inner_error);
		if (inner_error != NULL) {
			goto __catch4_g_error;
			goto __finally4;
		}
		self->priv->_ui_id = _tmp2;
	}
	goto __finally4;
	__catch4_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgprojectmanager.vala:204: Error %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally4:
	if (inner_error != NULL) {
		(manager == NULL) ? NULL : (manager = (g_object_unref (manager), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	(manager == NULL) ? NULL : (manager = (g_object_unref (manager), NULL));
}


void vtg_project_manager_plugin_helper_deactivate (VtgProjectManagerPluginHelper* self) {
	g_return_if_fail (self != NULL);
}


static void vtg_project_manager_plugin_helper_on_prepare_changelog (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	inner_error = NULL;
	{
		vtg_change_log_prepare (self->priv->_changelog, NULL, &inner_error);
		if (inner_error != NULL) {
			goto __catch5_g_error;
			goto __finally5;
		}
	}
	goto __finally5;
	__catch5_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			vtg_interaction_error_message (_ ("Can't prepare the ChangeLog entry"), err);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally5:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static void vtg_project_manager_plugin_helper_on_prepare_single_file_changelog (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	inner_error = NULL;
	{
		GeditDocument* _tmp0;
		GeditDocument* doc;
		_tmp0 = NULL;
		doc = (_tmp0 = gedit_window_get_active_document (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
		if (doc != NULL) {
			VtgProjectManagerProject* _tmp1;
			VtgProjectManagerProject* prj;
			const char* _tmp2;
			char* uri;
			const char* _tmp3;
			char* file;
			_tmp1 = NULL;
			prj = (_tmp1 = vtg_project_manager_view_get_current_project (self->priv->_prj_view), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
			_tmp2 = NULL;
			uri = (_tmp2 = gedit_document_get_uri (doc), (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
			_tmp3 = NULL;
			file = (_tmp3 = gedit_document_get_short_name_for_display (doc), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
			if (prj != NULL) {
				VtgProjectManagerProjectSource* src;
				src = vtg_project_manager_project_get_source_file_from_uri (prj, uri);
				if (src != NULL) {
					char* _tmp5;
					const char* _tmp4;
					_tmp5 = NULL;
					_tmp4 = NULL;
					file = (_tmp5 = (_tmp4 = src->name, (_tmp4 == NULL) ? NULL : g_strdup (_tmp4)), file = (g_free (file), NULL), _tmp5);
				}
				(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
			}
			vtg_change_log_prepare (self->priv->_changelog, file, &inner_error);
			if (inner_error != NULL) {
				(prj == NULL) ? NULL : (prj = (g_object_unref (prj), NULL));
				uri = (g_free (uri), NULL);
				file = (g_free (file), NULL);
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				goto __catch6_g_error;
				goto __finally6;
			}
			(prj == NULL) ? NULL : (prj = (g_object_unref (prj), NULL));
			uri = (g_free (uri), NULL);
			file = (g_free (file), NULL);
		}
		(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
	}
	goto __finally6;
	__catch6_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			vtg_interaction_error_message (_ ("Can't prepare the ChangeLog entry"), err);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally6:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static void vtg_project_manager_plugin_helper_on_complete_word (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	VtgProjectManagerProject* _tmp0;
	VtgProjectManagerProject* project;
	GeditView* _tmp1;
	GeditView* view;
	VtgSymbolCompletionHelper* sch;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_tmp0 = NULL;
	project = (_tmp0 = vtg_project_manager_view_get_current_project (self->priv->_prj_view), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	g_return_if_fail (project != NULL);
	_tmp1 = NULL;
	view = (_tmp1 = gedit_window_get_active_view (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	if (view == NULL) {
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
		return;
	}
	sch = vtg_plugin_scs_find_from_view (self->priv->_plugin, view);
	if (sch == NULL) {
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
		(sch == NULL) ? NULL : (sch = (g_object_unref (sch), NULL));
		return;
	}
	vtg_symbol_completion_trigger_complete_word (vtg_symbol_completion_helper_get_trigger (sch));
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
	(sch == NULL) ? NULL : (sch = (g_object_unref (sch), NULL));
}


static void vtg_project_manager_plugin_helper_on_project_open (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	GtkFileChooserDialog* dialog;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	dialog = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new (_ ("Open Project"), (GtkWindow*) vtg_plugin_get_gedit_window (self->priv->_plugin), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL, NULL));
	if (gtk_dialog_run ((GtkDialog*) dialog) == GTK_RESPONSE_ACCEPT) {
		const char* _tmp0;
		char* foldername;
		gtk_widget_hide_all ((GtkWidget*) dialog);
		_tmp0 = NULL;
		foldername = (_tmp0 = gtk_file_chooser_get_filename ((GtkFileChooser*) dialog), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
		vtg_project_manager_plugin_helper_open_project (self, foldername);
		foldername = (g_free (foldername), NULL);
	}
	gtk_object_destroy ((GtkObject*) dialog);
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


static void vtg_project_manager_plugin_helper_on_project_close (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	VtgProjectManagerProject* _tmp0;
	VtgProjectManagerProject* project;
	gboolean save_required;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_tmp0 = NULL;
	project = (_tmp0 = vtg_project_manager_view_get_current_project (self->priv->_prj_view), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	g_return_if_fail (project != NULL);
	save_required = FALSE;
	{
		GList* doc_collection;
		GList* doc_it;
		doc_collection = gedit_window_get_unsaved_documents (vtg_plugin_get_gedit_window (self->priv->_plugin));
		for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
			GeditDocument* _tmp1;
			GeditDocument* doc;
			_tmp1 = NULL;
			doc = (_tmp1 = (GeditDocument*) doc_it->data, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
			{
				if (vtg_project_manager_project_contains_source_file (project, gedit_document_get_uri (doc))) {
					save_required = TRUE;
				}
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			}
		}
	}
	/*there are some files that require saving: ask it!*/
	if (save_required) {
		GtkMessageDialog* dialog;
		gint response;
		dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) vtg_plugin_get_gedit_window (self->priv->_plugin), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, _ ("Project files need to be saved"), NULL));
		gtk_dialog_add_buttons ((GtkDialog*) dialog, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
		response = gtk_dialog_run ((GtkDialog*) dialog);
		gtk_object_destroy ((GtkObject*) dialog);
		if (response == GTK_RESPONSE_CANCEL) {
			(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
			(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
			return;
		} else {
			if (response == GTK_RESPONSE_ACCEPT) {
				vtg_project_manager_plugin_helper_project_save_all (self, project);
			}
		}
		(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	}
	/*close project*/
	vtg_project_manager_plugin_helper_close_project (self, project);
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
}


static void vtg_project_manager_plugin_helper_on_project_new (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	GtkFileChooserDialog* dialog;
	char* foldername;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	/*save dialog*/
	dialog = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new (_ ("Save Project"), (GtkWindow*) vtg_plugin_get_gedit_window (self->priv->_plugin), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL, NULL));
	foldername = NULL;
	if (gtk_dialog_run ((GtkDialog*) dialog) == GTK_RESPONSE_ACCEPT) {
		char* _tmp1;
		const char* _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		foldername = (_tmp1 = (_tmp0 = gtk_file_chooser_get_filename ((GtkFileChooser*) dialog), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), foldername = (g_free (foldername), NULL), _tmp1);
	}
	gtk_object_destroy ((GtkObject*) dialog);
	while (g_main_context_pending (g_main_context_default ())) {
		g_main_context_iteration (g_main_context_default (), FALSE);
	}
	if (foldername != NULL) {
		vtg_project_manager_plugin_helper_create_project (self, foldername);
		vtg_project_manager_plugin_helper_open_project (self, foldername);
	}
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	foldername = (g_free (foldername), NULL);
}


static void vtg_project_manager_plugin_helper_on_project_goto_document (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	VtgProjectManagerProject* _tmp0;
	VtgProjectManagerProject* project;
	GtkTreeIter iter = {0};
	GtkListStore* model;
	VtgProjectManagerFilteredListDialog* dialog;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_tmp0 = NULL;
	project = (_tmp0 = vtg_project_manager_view_get_current_project (self->priv->_prj_view), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	g_return_if_fail (project != NULL);
	model = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_OBJECT, NULL);
	{
		GeeIterator* _src_it;
		_src_it = gee_iterable_iterator ((GeeIterable*) project->all_vala_sources);
		while (gee_iterator_next (_src_it)) {
			VtgProjectManagerProjectSource* src;
			src = (VtgProjectManagerProjectSource*) gee_iterator_get (_src_it);
			gtk_list_store_append (model, &iter);
			gtk_list_store_set (model, &iter, 0, src->name, 1, src, -1);
			(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
		}
		(_src_it == NULL) ? NULL : (_src_it = (gee_collection_object_unref (_src_it), NULL));
	}
	dialog = vtg_project_manager_filtered_list_dialog_new ((GtkTreeModel*) model);
	if (vtg_project_manager_filtered_list_dialog_run (dialog)) {
		VtgProjectManagerProjectSource* src;
		GeditTab* _tmp1;
		src = NULL;
		gtk_tree_model_get ((GtkTreeModel*) model, &dialog->selected_iter, 1, &src, -1);
		_tmp1 = NULL;
		_tmp1 = vtg_plugin_activate_uri (self->priv->_plugin, src->uri, 0, 0);
		(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
		(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
	}
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


static void vtg_project_manager_plugin_helper_on_project_goto_method (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	VtgProjectManagerProject* _tmp0;
	VtgProjectManagerProject* project;
	VtgProjectDescriptor* pdes;
	GeditView* _tmp1;
	GeditView* view;
	GeditDocument* _tmp2;
	GeditDocument* doc;
	const char* _tmp3;
	char* uri;
	GeeList* methods;
	GtkTreeIter iter = {0};
	GtkListStore* model;
	VtgProjectManagerFilteredListDialog* dialog;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_tmp0 = NULL;
	project = (_tmp0 = vtg_project_manager_view_get_current_project (self->priv->_prj_view), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	g_return_if_fail (project != NULL);
	pdes = vtg_project_manager_plugin_helper_get_projectdescriptor_for_project (self, project);
	g_return_if_fail (pdes != NULL);
	_tmp1 = NULL;
	view = (_tmp1 = gedit_window_get_active_view (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	if (view == NULL) {
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		(pdes == NULL) ? NULL : (pdes = (g_object_unref (pdes), NULL));
		(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
		return;
	}
	_tmp2 = NULL;
	doc = (_tmp2 = GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) view)), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
	g_return_if_fail (doc != NULL);
	_tmp3 = NULL;
	uri = (_tmp3 = gedit_document_get_uri (doc), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
	if (uri == NULL) {
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		(pdes == NULL) ? NULL : (pdes = (g_object_unref (pdes), NULL));
		(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
		(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
		uri = (g_free (uri), NULL);
		return;
	}
	methods = vsc_symbol_completion_get_methods_for_source (pdes->completion, uri);
	if (gee_collection_get_size ((GeeCollection*) methods) <= 0) {
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		(pdes == NULL) ? NULL : (pdes = (g_object_unref (pdes), NULL));
		(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
		(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
		uri = (g_free (uri), NULL);
		(methods == NULL) ? NULL : (methods = (gee_collection_object_unref (methods), NULL));
		return;
	}
	model = gtk_list_store_new (2, G_TYPE_STRING, VSC_TYPE_SYMBOL_COMPLETION_ITEM, NULL);
	{
		GeeIterator* _method_it;
		_method_it = gee_iterable_iterator ((GeeIterable*) methods);
		while (gee_iterator_next (_method_it)) {
			VscSymbolCompletionItem* method;
			method = (VscSymbolCompletionItem*) gee_iterator_get (_method_it);
			gtk_list_store_append (model, &iter);
			gtk_list_store_set (model, &iter, 0, method->name, 1, method, -1);
			(method == NULL) ? NULL : (method = (g_object_unref (method), NULL));
		}
		(_method_it == NULL) ? NULL : (_method_it = (gee_collection_object_unref (_method_it), NULL));
	}
	dialog = vtg_project_manager_filtered_list_dialog_new ((GtkTreeModel*) model);
	if (vtg_project_manager_filtered_list_dialog_run (dialog)) {
		VscSymbolCompletionItem* method;
		method = NULL;
		gtk_tree_model_get ((GtkTreeModel*) model, &dialog->selected_iter, 1, &method, -1);
		gedit_document_goto_line (doc, method->line - 1);
		gedit_view_scroll_to_cursor (view);
		(method == NULL) ? NULL : (method = (g_object_unref (method), NULL));
	}
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	(pdes == NULL) ? NULL : (pdes = (g_object_unref (pdes), NULL));
	(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
	(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
	uri = (g_free (uri), NULL);
	(methods == NULL) ? NULL : (methods = (gee_collection_object_unref (methods), NULL));
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


static VtgProjectDescriptor* vtg_project_manager_plugin_helper_get_projectdescriptor_for_project (VtgProjectManagerPluginHelper* self, VtgProjectManagerProject* project) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (project != NULL, NULL);
	{
		GeeIterator* _current_it;
		_current_it = gee_iterable_iterator ((GeeIterable*) vtg_plugin_get_projects (self->priv->_plugin));
		while (gee_iterator_next (_current_it)) {
			VtgProjectDescriptor* current;
			current = (VtgProjectDescriptor*) gee_iterator_get (_current_it);
			if (current->project == project) {
				VtgProjectDescriptor* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = current, (_current_it == NULL) ? NULL : (_current_it = (gee_collection_object_unref (_current_it), NULL)), _tmp0);
			}
			(current == NULL) ? NULL : (current = (g_object_unref (current), NULL));
		}
		(_current_it == NULL) ? NULL : (_current_it = (gee_collection_object_unref (_current_it), NULL));
	}
	return NULL;
}


static void vtg_project_manager_plugin_helper_on_standalone_file_compile (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	GeditDocument* _tmp0;
	GeditDocument* doc;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_tmp0 = NULL;
	doc = (_tmp0 = gedit_window_get_active_document (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	if (doc != NULL) {
		const char* _tmp1;
		char* file;
		VtgProjectManagerProject* _tmp2;
		VtgProjectManagerProject* project;
		GtkListStore* cache;
		VtgInteractionParametersDialog* params_dialog;
		_tmp1 = NULL;
		file = (_tmp1 = gedit_document_get_uri (doc), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
		_tmp2 = NULL;
		project = (_tmp2 = vtg_project_manager_view_get_current_project (self->priv->_prj_view), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
		if (project != NULL) {
			if (vtg_project_manager_project_contains_source_file (project, file)) {
				/*TODO: we should get the group an issue a make in that subfolder*/
				g_warning ("vtgprojectmanager.vala:415: Can't compile a project file (for now)");
				file = (g_free (file), NULL);
				(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				return;
			}
		}
		cache = vtg_caches_get_compile_cache ();
		params_dialog = vtg_interaction_parameters_dialog_new (_ ("Compile File"), (GtkWindow*) vtg_plugin_get_gedit_window (self->priv->_plugin), cache);
		if (vtg_interaction_parameters_dialog_run (params_dialog) == GTK_RESPONSE_OK) {
			const char* _tmp3;
			char* params;
			char* _tmp4;
			gboolean _tmp5;
			_tmp3 = NULL;
			params = (_tmp3 = vtg_interaction_parameters_dialog_get_parameters (params_dialog), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
			if (!vtg_caches_cache_contains (cache, params)) {
				vtg_caches_cache_add (cache, params);
			}
			_tmp4 = NULL;
			file = (_tmp4 = string_replace (file, "file://", ""), file = (g_free (file), NULL), _tmp4);
			_tmp5 = FALSE;
			if (!gedit_document_is_untouched (doc)) {
				_tmp5 = vtg_configuration_get_save_before_build (vtg_plugin_get_config (self->priv->_plugin));
			} else {
				_tmp5 = FALSE;
			}
			/*HACK*/
			if (_tmp5) {
				gedit_document_save (doc, GEDIT_DOCUMENT_SAVE_IGNORE_MTIME);
			}
			vtg_project_manager_builder_compile_file (self->priv->_prj_builder, file, params);
			params = (g_free (params), NULL);
		}
		file = (g_free (file), NULL);
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		(cache == NULL) ? NULL : (cache = (g_object_unref (cache), NULL));
		(params_dialog == NULL) ? NULL : (params_dialog = (vtg_interaction_parameters_dialog_unref (params_dialog), NULL));
	}
	(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
}


static void vtg_project_manager_plugin_helper_on_project_build (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	if (vtg_project_manager_view_get_current_project (self->priv->_prj_view) != NULL) {
		char* pars;
		GtkListStore* cache;
		VtgProjectManagerProject* _tmp2;
		VtgProjectManagerProject* project;
		pars = NULL;
		cache = vtg_caches_get_build_cache ();
		if (vtg_project_manager_builder_get_is_building (self->priv->_prj_builder)) {
			GtkMessageDialog* dialog;
			gint res;
			/*ask if stop the current build process and restart a new one*/
			dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) vtg_plugin_get_gedit_window (self->priv->_plugin), GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, _ ("Stop the current build process and restart a new one?"), NULL));
			g_object_set (dialog, "secondary-text", _ ("Stop the current build process and start a new one with the same command line parameters"), NULL);
			res = gtk_dialog_run ((GtkDialog*) dialog);
			gtk_object_destroy ((GtkObject*) dialog);
			if (res == GTK_RESPONSE_YES) {
				GtkTreeIter iter = {0};
				vtg_project_manager_builder_stop_build (self->priv->_prj_builder);
				if (gtk_tree_model_get_iter_first ((GtkTreeModel*) cache, &iter)) {
					gtk_tree_model_get ((GtkTreeModel*) cache, &iter, 0, &pars, -1);
				}
			} else {
				(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
				pars = (g_free (pars), NULL);
				(cache == NULL) ? NULL : (cache = (g_object_unref (cache), NULL));
				return;
			}
			(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
		}
		if (pars == NULL) {
			VtgInteractionParametersDialog* params_dialog;
			char* _tmp1;
			const char* _tmp0;
			params_dialog = vtg_interaction_parameters_dialog_new (_ ("Build Project"), (GtkWindow*) vtg_plugin_get_gedit_window (self->priv->_plugin), cache);
			if (vtg_interaction_parameters_dialog_run (params_dialog) != GTK_RESPONSE_OK) {
				(params_dialog == NULL) ? NULL : (params_dialog = (vtg_interaction_parameters_dialog_unref (params_dialog), NULL));
				pars = (g_free (pars), NULL);
				(cache == NULL) ? NULL : (cache = (g_object_unref (cache), NULL));
				return;
			}
			_tmp1 = NULL;
			_tmp0 = NULL;
			pars = (_tmp1 = (_tmp0 = vtg_interaction_parameters_dialog_get_parameters (params_dialog), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), pars = (g_free (pars), NULL), _tmp1);
			if (!vtg_caches_cache_contains (cache, pars)) {
				vtg_caches_cache_add (cache, pars);
			}
			(params_dialog == NULL) ? NULL : (params_dialog = (vtg_interaction_parameters_dialog_unref (params_dialog), NULL));
		}
		_tmp2 = NULL;
		project = (_tmp2 = vtg_project_manager_view_get_current_project (self->priv->_prj_view), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
		vtg_project_manager_plugin_helper_project_save_all (self, project);
		vtg_project_manager_builder_build (self->priv->_prj_builder, project, pars);
		pars = (g_free (pars), NULL);
		(cache == NULL) ? NULL : (cache = (g_object_unref (cache), NULL));
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	}
}


static void vtg_project_manager_plugin_helper_on_project_configure (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	if (vtg_project_manager_view_get_current_project (self->priv->_prj_view) != NULL) {
		GtkListStore* cache;
		VtgInteractionParametersDialog* params_dialog;
		cache = vtg_caches_get_configure_cache ();
		params_dialog = vtg_interaction_parameters_dialog_new (_ ("Configure Project"), (GtkWindow*) vtg_plugin_get_gedit_window (self->priv->_plugin), cache);
		if (vtg_interaction_parameters_dialog_run (params_dialog) == GTK_RESPONSE_OK) {
			VtgProjectManagerProject* _tmp0;
			VtgProjectManagerProject* project;
			const char* _tmp1;
			char* params;
			_tmp0 = NULL;
			project = (_tmp0 = vtg_project_manager_view_get_current_project (self->priv->_prj_view), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
			_tmp1 = NULL;
			params = (_tmp1 = vtg_interaction_parameters_dialog_get_parameters (params_dialog), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
			if (!vtg_caches_cache_contains (cache, params)) {
				vtg_caches_cache_add (cache, params);
			}
			vtg_project_manager_plugin_helper_project_save_all (self, project);
			vtg_project_manager_builder_configure (self->priv->_prj_builder, project, params);
			(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
			params = (g_free (params), NULL);
		}
		(cache == NULL) ? NULL : (cache = (g_object_unref (cache), NULL));
		(params_dialog == NULL) ? NULL : (params_dialog = (vtg_interaction_parameters_dialog_unref (params_dialog), NULL));
	}
}


static void vtg_project_manager_plugin_helper_on_project_clean (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_manager_plugin_helper_clean_project (self, FALSE);
}


static void vtg_project_manager_plugin_helper_on_project_clean_stamps (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_manager_plugin_helper_clean_project (self, TRUE);
}


static void vtg_project_manager_plugin_helper_on_project_execute_process (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	if (vtg_project_manager_view_get_current_project (self->priv->_prj_view) != NULL) {
		VtgProjectManagerProject* _tmp0;
		VtgProjectManagerProject* project;
		VtgProjectManagerExecuterDialog* exec_dialog;
		_tmp0 = NULL;
		project = (_tmp0 = vtg_project_manager_view_get_current_project (self->priv->_prj_view), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
		exec_dialog = vtg_project_manager_executer_dialog_new ((GtkWindow*) vtg_plugin_get_gedit_window (self->priv->_plugin), project);
		if (vtg_project_manager_executer_dialog_run (exec_dialog) == GTK_RESPONSE_OK) {
			const char* _tmp1;
			char* command_line;
			_tmp1 = NULL;
			command_line = (_tmp1 = vtg_project_manager_executer_dialog_get_command_line (exec_dialog), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
			vtg_project_manager_executer_execute (self->priv->_prj_executer, project, command_line);
			command_line = (g_free (command_line), NULL);
		}
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		(exec_dialog == NULL) ? NULL : (exec_dialog = (vtg_project_manager_executer_dialog_unref (exec_dialog), NULL));
	}
}


static void vtg_project_manager_plugin_helper_on_project_kill_process (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	/*TODO: implement a kill (project);*/
	vtg_project_manager_executer_kill_last (self->priv->_prj_executer);
}


static void vtg_project_manager_plugin_helper_clean_project (VtgProjectManagerPluginHelper* self, gboolean stamps) {
	g_return_if_fail (self != NULL);
	if (vtg_project_manager_view_get_current_project (self->priv->_prj_view) != NULL) {
		VtgProjectManagerProject* _tmp0;
		VtgProjectManagerProject* project;
		_tmp0 = NULL;
		project = (_tmp0 = vtg_project_manager_view_get_current_project (self->priv->_prj_view), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
		vtg_project_manager_builder_clean (self->priv->_prj_builder, project, stamps);
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	}
}


static void vtg_project_manager_plugin_helper_on_project_error_next (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_manager_builder_next_error (self->priv->_prj_builder);
}


static void vtg_project_manager_plugin_helper_on_project_error_previuos (VtgProjectManagerPluginHelper* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_manager_builder_previous_error (self->priv->_prj_builder);
}


static void vtg_project_manager_plugin_helper_on_current_project_changed (VtgProjectManagerPluginHelper* self, GObject* sender, GParamSpec* pspec) {
	VtgProjectManagerView* _tmp0;
	VtgProjectManagerView* view;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (pspec != NULL);
	_tmp0 = NULL;
	view = (_tmp0 = VTG_PROJECT_MANAGER_VIEW (sender), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	vtg_project_manager_plugin_helper_update_ui (self, vtg_project_manager_view_get_current_project (view) == NULL);
	(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
}


static void vtg_project_manager_plugin_helper_update_ui (VtgProjectManagerPluginHelper* self, gboolean default_project) {
	GtkAction* _tmp0;
	GtkAction* action;
	GtkAction* _tmp2;
	GtkAction* _tmp1;
	GtkAction* _tmp4;
	GtkAction* _tmp3;
	GtkAction* _tmp6;
	GtkAction* _tmp5;
	GeditDocument* _tmp7;
	GeditDocument* doc;
	gboolean _tmp8;
	gboolean _tmp9;
	gboolean is_vala_source;
	GtkAction* _tmp11;
	GtkAction* _tmp10;
	gboolean _tmp12;
	GtkAction* _tmp14;
	GtkAction* _tmp13;
	GtkAction* _tmp16;
	GtkAction* _tmp15;
	gboolean has_errors;
	GtkAction* _tmp18;
	GtkAction* _tmp17;
	GtkAction* _tmp20;
	GtkAction* _tmp19;
	GtkAction* _tmp22;
	GtkAction* _tmp21;
	gboolean _tmp23;
	GtkAction* _tmp25;
	GtkAction* _tmp24;
	gboolean _tmp26;
	gboolean can_complete;
	GeditView* _tmp27;
	GeditView* view;
	GtkAction* _tmp29;
	GtkAction* _tmp28;
	gboolean has_changelog;
	gboolean has_vcs_backend;
	gboolean _tmp30;
	GtkAction* _tmp32;
	GtkAction* _tmp31;
	gboolean _tmp33;
	GtkAction* _tmp35;
	GtkAction* _tmp34;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	action = (_tmp0 = gtk_action_group_get_action (self->priv->_actions, "ProjectClose"), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	gtk_action_set_sensitive (action, !default_project);
	_tmp2 = NULL;
	_tmp1 = NULL;
	action = (_tmp2 = (_tmp1 = gtk_action_group_get_action (self->priv->_actions, "ProjectBuild"), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp2);
	gtk_action_set_sensitive (action, !default_project);
	_tmp4 = NULL;
	_tmp3 = NULL;
	action = (_tmp4 = (_tmp3 = gtk_action_group_get_action (self->priv->_actions, "ProjectBuildClean"), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp4);
	gtk_action_set_sensitive (action, !default_project);
	_tmp6 = NULL;
	_tmp5 = NULL;
	action = (_tmp6 = (_tmp5 = gtk_action_group_get_action (self->priv->_actions, "ProjectBuildCleanStamps"), (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp6);
	gtk_action_set_sensitive (action, !default_project);
	_tmp7 = NULL;
	doc = (_tmp7 = gedit_window_get_active_document (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp7 == NULL) ? NULL : g_object_ref (_tmp7));
	_tmp8 = FALSE;
	_tmp9 = FALSE;
	if (doc != NULL) {
		_tmp9 = gtk_source_buffer_get_language ((GtkSourceBuffer*) doc) != NULL;
	} else {
		_tmp9 = FALSE;
	}
	if (_tmp9) {
		_tmp8 = _vala_strcmp0 (gtk_source_language_get_id (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc)), "vala") == 0;
	} else {
		_tmp8 = FALSE;
	}
	is_vala_source = (_tmp8);
	_tmp11 = NULL;
	_tmp10 = NULL;
	action = (_tmp11 = (_tmp10 = gtk_action_group_get_action (self->priv->_actions, "ProjectBuildCompileFile"), (_tmp10 == NULL) ? NULL : g_object_ref (_tmp10)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp11);
	_tmp12 = FALSE;
	if (default_project) {
		_tmp12 = is_vala_source;
	} else {
		_tmp12 = FALSE;
	}
	gtk_action_set_sensitive (action, _tmp12);
	_tmp14 = NULL;
	_tmp13 = NULL;
	action = (_tmp14 = (_tmp13 = gtk_action_group_get_action (self->priv->_actions, "ProjectGotoMethod"), (_tmp13 == NULL) ? NULL : g_object_ref (_tmp13)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp14);
	gtk_action_set_sensitive (action, is_vala_source);
	_tmp16 = NULL;
	_tmp15 = NULL;
	action = (_tmp16 = (_tmp15 = gtk_action_group_get_action (self->priv->_actions, "ProjectGotoDocument"), (_tmp15 == NULL) ? NULL : g_object_ref (_tmp15)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp16);
	gtk_action_set_sensitive (action, !default_project);
	has_errors = vtg_project_manager_build_log_view_get_error_count (vtg_project_manager_builder_get_error_pane (self->priv->_prj_builder)) > 0;
	_tmp18 = NULL;
	_tmp17 = NULL;
	action = (_tmp18 = (_tmp17 = gtk_action_group_get_action (self->priv->_actions, "ProjectBuildNextError"), (_tmp17 == NULL) ? NULL : g_object_ref (_tmp17)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp18);
	gtk_action_set_sensitive (action, has_errors);
	_tmp20 = NULL;
	_tmp19 = NULL;
	action = (_tmp20 = (_tmp19 = gtk_action_group_get_action (self->priv->_actions, "ProjectBuildPreviousError"), (_tmp19 == NULL) ? NULL : g_object_ref (_tmp19)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp20);
	gtk_action_set_sensitive (action, has_errors);
	_tmp22 = NULL;
	_tmp21 = NULL;
	action = (_tmp22 = (_tmp21 = gtk_action_group_get_action (self->priv->_actions, "ProjectBuildExecute"), (_tmp21 == NULL) ? NULL : g_object_ref (_tmp21)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp22);
	_tmp23 = FALSE;
	if (!vtg_project_manager_executer_get_is_executing (self->priv->_prj_executer)) {
		_tmp23 = !default_project;
	} else {
		_tmp23 = FALSE;
	}
	gtk_action_set_sensitive (action, _tmp23);
	_tmp25 = NULL;
	_tmp24 = NULL;
	action = (_tmp25 = (_tmp24 = gtk_action_group_get_action (self->priv->_actions, "ProjectBuildKill"), (_tmp24 == NULL) ? NULL : g_object_ref (_tmp24)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp25);
	_tmp26 = FALSE;
	if (vtg_project_manager_executer_get_is_executing (self->priv->_prj_executer)) {
		_tmp26 = !default_project;
	} else {
		_tmp26 = FALSE;
	}
	gtk_action_set_sensitive (action, _tmp26);
	can_complete = FALSE;
	_tmp27 = NULL;
	view = (_tmp27 = gedit_window_get_active_view (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp27 == NULL) ? NULL : g_object_ref (_tmp27));
	if (view != NULL) {
		VtgSymbolCompletionHelper* sch;
		sch = vtg_plugin_scs_find_from_view (self->priv->_plugin, view);
		can_complete = (sch != NULL);
		(sch == NULL) ? NULL : (sch = (g_object_unref (sch), NULL));
	}
	_tmp29 = NULL;
	_tmp28 = NULL;
	action = (_tmp29 = (_tmp28 = gtk_action_group_get_action (self->priv->_actions, "ProjectCompleteWord"), (_tmp28 == NULL) ? NULL : g_object_ref (_tmp28)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp29);
	gtk_action_set_sensitive (action, can_complete);
	has_changelog = FALSE;
	has_vcs_backend = FALSE;
	_tmp30 = FALSE;
	if (vtg_project_manager_view_get_current_project (self->priv->_prj_view) != NULL) {
		_tmp30 = vtg_project_manager_view_get_current_project (self->priv->_prj_view)->changelog_uri != NULL;
	} else {
		_tmp30 = FALSE;
	}
	if (_tmp30) {
		has_changelog = TRUE;
		if (vtg_project_manager_view_get_current_project (self->priv->_prj_view)->vcs_type != VTG_PROJECT_MANAGER_VCS_TYPES_NONE) {
			has_vcs_backend = TRUE;
		}
	}
	_tmp32 = NULL;
	_tmp31 = NULL;
	action = (_tmp32 = (_tmp31 = gtk_action_group_get_action (self->priv->_actions, "ProjectPrepareChangeLog"), (_tmp31 == NULL) ? NULL : g_object_ref (_tmp31)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp32);
	_tmp33 = FALSE;
	if (has_changelog) {
		_tmp33 = has_vcs_backend;
	} else {
		_tmp33 = FALSE;
	}
	gtk_action_set_sensitive (action, _tmp33);
	_tmp35 = NULL;
	_tmp34 = NULL;
	action = (_tmp35 = (_tmp34 = gtk_action_group_get_action (self->priv->_actions, "ProjectPrepareSingleFileChangeLog"), (_tmp34 == NULL) ? NULL : g_object_ref (_tmp34)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp35);
	gtk_action_set_sensitive (action, has_changelog);
	(action == NULL) ? NULL : (action = (g_object_unref (action), NULL));
	(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
	(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
}


static void vtg_project_manager_plugin_helper_open_project (VtgProjectManagerPluginHelper* self, const char* name) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (name != NULL);
	inner_error = NULL;
	{
		VtgProjectManagerProject* project;
		gboolean _tmp0;
		project = vtg_project_manager_project_new ();
		_tmp0 = vtg_project_manager_project_open (project, name, &inner_error);
		if (inner_error != NULL) {
			(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
			goto __catch7_g_error;
			goto __finally7;
		}
		if (_tmp0) {
			gee_collection_add ((GeeCollection*) self->priv->_projects, project);
			/*HACK: why the signal isn't working?!?!
			this.project_loaded (project);*/
			vtg_plugin_on_project_loaded (self->priv->_plugin, self, project);
			vtg_project_manager_view_add_project (self->priv->_prj_view, project);
		}
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	}
	goto __finally7;
	__catch7_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			char* _tmp1;
			_tmp1 = NULL;
			vtg_interaction_error_message (_tmp1 = g_strdup_printf (_ ("Error opening project %s"), name), err);
			_tmp1 = (g_free (_tmp1), NULL);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally7:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static void vtg_project_manager_plugin_helper_close_project (VtgProjectManagerPluginHelper* self, VtgProjectManagerProject* project) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	{
		GList* doc_collection;
		GList* doc_it;
		doc_collection = gedit_window_get_documents (vtg_plugin_get_gedit_window (self->priv->_plugin));
		for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
			GeditDocument* _tmp1;
			GeditDocument* doc;
			_tmp1 = NULL;
			doc = (_tmp1 = (GeditDocument*) doc_it->data, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
			{
				if (vtg_project_manager_project_contains_source_file (project, gedit_document_get_uri (doc))) {
					GeditTab* _tmp0;
					GeditTab* tab;
					/*close tab*/
					_tmp0 = NULL;
					tab = (_tmp0 = gedit_tab_get_from_document (doc), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
					gedit_window_close_tab (vtg_plugin_get_gedit_window (self->priv->_plugin), tab);
					(tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL));
				}
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			}
		}
	}
	vtg_project_manager_view_remove_project (self->priv->_prj_view, project);
	vtg_plugin_on_project_closed (self->priv->_plugin, self, project);
	vtg_project_manager_project_close (project);
	gee_collection_remove ((GeeCollection*) self->priv->_projects, project);
}


static void vtg_project_manager_plugin_helper_project_save_all (VtgProjectManagerPluginHelper* self, VtgProjectManagerProject* project) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	{
		GList* doc_collection;
		GList* doc_it;
		doc_collection = gedit_window_get_unsaved_documents (vtg_plugin_get_gedit_window (self->priv->_plugin));
		for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
			GeditDocument* _tmp0;
			GeditDocument* doc;
			_tmp0 = NULL;
			doc = (_tmp0 = (GeditDocument*) doc_it->data, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
			{
				if (vtg_project_manager_project_contains_source_file (project, gedit_document_get_uri (doc))) {
					gedit_document_save (doc, GEDIT_DOCUMENT_SAVE_IGNORE_MTIME);
				}
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			}
		}
	}
}


static void _vtg_project_manager_plugin_helper_on_child_watch_gchild_watch_func (GPid pid, gint status, gpointer self) {
	vtg_project_manager_plugin_helper_on_child_watch (self, pid, status);
}


static void vtg_project_manager_plugin_helper_create_project (VtgProjectManagerPluginHelper* self, const char* project_path) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (project_path != NULL);
	inner_error = NULL;
	{
		VtgProjectManagerOutputView* _tmp0;
		VtgProjectManagerOutputView* log;
		char* process_file;
		gint status;
		gint stdo;
		gint stde;
		GPid child_pid;
		gint _tmp5_length1;
		char** _tmp5;
		char** _tmp4;
		const char* _tmp3;
		const char* _tmp2;
		gboolean _tmp6;
		gboolean _tmp7;
		_tmp0 = NULL;
		log = (_tmp0 = vtg_plugin_get_output_view (self->priv->_plugin), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
		if (!vtg_project_manager_plugin_helper_is_dir_empty (self, project_path)) {
			char* _tmp1;
			_tmp1 = NULL;
			vtg_project_manager_output_view_log_message (log, _tmp1 = g_strdup_printf ("project directory %s not empty\n", project_path));
			_tmp1 = (g_free (_tmp1), NULL);
			(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
			return;
		}
		process_file = g_strdup ("vala-gen-project");
		status = 0;
		stdo = 0;
		stde = 0;
		child_pid = 0;
		_tmp5 = NULL;
		_tmp4 = NULL;
		_tmp3 = NULL;
		_tmp2 = NULL;
		_tmp7 = (_tmp6 = g_spawn_sync (project_path, (_tmp5 = (_tmp4 = g_new0 (char*, 3 + 1), _tmp4[0] = (_tmp2 = process_file, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), _tmp4[1] = g_strdup ("--projectdir"), _tmp4[2] = (_tmp3 = project_path, (_tmp3 == NULL) ? NULL : g_strdup (_tmp3)), _tmp4), _tmp5_length1 = 3, _tmp5), NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL, &status, &inner_error), _tmp5 = (_vala_array_free (_tmp5, _tmp5_length1, (GDestroyNotify) g_free), NULL), _tmp6);
		if (inner_error != NULL) {
			(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
			process_file = (g_free (process_file), NULL);
			goto __catch8_g_error;
			goto __finally8;
		}
		/*vala-gen-project*/
		if (_tmp7) {
			if (WEXITSTATUS (status) == 0) {
				char* start_message;
				char* _tmp9;
				char* _tmp8;
				gint _tmp11_length1;
				char** _tmp11;
				char** _tmp10;
				/*autogen*/
				start_message = g_strdup_printf (_ ("Autogenerating project: %s\n"), project_path);
				vtg_project_manager_output_view_log_message (log, start_message);
				_tmp9 = NULL;
				_tmp8 = NULL;
				vtg_project_manager_output_view_log_message (log, _tmp9 = g_strdup_printf ("%s\n\n", _tmp8 = g_strnfill ((gulong) (string_get_length (start_message) - 1), '-')));
				_tmp9 = (g_free (_tmp9), NULL);
				_tmp8 = (g_free (_tmp8), NULL);
				_tmp11 = NULL;
				_tmp10 = NULL;
				g_spawn_async_with_pipes (project_path, (_tmp11 = (_tmp10 = g_new0 (char*, 1 + 1), _tmp10[0] = g_strdup ("./autogen.sh"), _tmp10), _tmp11_length1 = 1, _tmp11), NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &child_pid, NULL, &stdo, &stde, &inner_error);
				if (inner_error != NULL) {
					start_message = (g_free (start_message), NULL);
					(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
					process_file = (g_free (process_file), NULL);
					goto __catch8_g_error;
					goto __finally8;
				}
				_tmp11 = (_vala_array_free (_tmp11, _tmp11_length1, (GDestroyNotify) g_free), NULL);
				if (child_pid != ((GPid) NULL)) {
					g_child_watch_add (child_pid, _vtg_project_manager_plugin_helper_on_child_watch_gchild_watch_func, self);
					vtg_project_manager_output_view_start_watch (log, (guint) child_pid, stdo, stde, -1);
					vtg_project_manager_output_view_activate (log);
				} else {
					vtg_project_manager_output_view_log_message (log, "error spawning ./autogen.sh process\n");
				}
				start_message = (g_free (start_message), NULL);
			} else {
				vtg_project_manager_output_view_log_message (log, "error executing vala-gen-project process\n");
			}
		} else {
			vtg_project_manager_output_view_log_message (log, "error spawning vala-gen-project process\n");
		}
		(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
		process_file = (g_free (process_file), NULL);
	}
	goto __finally8;
	__catch8_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgprojectmanager.vala:683: error creating project: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally8:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static gboolean vtg_project_manager_plugin_helper_is_dir_empty (VtgProjectManagerPluginHelper* self, const char* dir_path) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (dir_path != NULL, FALSE);
	inner_error = NULL;
	{
		GDir* dir;
		gboolean _tmp0;
		dir = g_dir_open (dir_path, 0, &inner_error);
		if (inner_error != NULL) {
			goto __catch9_g_error;
			goto __finally9;
		}
		return (_tmp0 = g_dir_read_name (dir) == NULL, (dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL)), _tmp0);
	}
	goto __finally9;
	__catch9_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			gboolean _tmp1;
			g_warning ("vtgprojectmanager.vala:693: cannot open directort %s", dir_path);
			return (_tmp1 = FALSE, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), _tmp1);
		}
	}
	__finally9:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return FALSE;
	}
}


static void vtg_project_manager_plugin_helper_on_child_watch (VtgProjectManagerPluginHelper* self, GPid pid, gint status) {
	VtgProjectManagerOutputView* _tmp0;
	VtgProjectManagerOutputView* log;
	char* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	log = (_tmp0 = vtg_plugin_get_output_view (self->priv->_plugin), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	g_spawn_close_pid (pid);
	vtg_project_manager_output_view_stop_watch (log, (guint) pid);
	_tmp1 = NULL;
	vtg_project_manager_output_view_log_message (log, _tmp1 = g_strdup_printf (_ ("\nautogeneration end with exit status %d\n"), status));
	_tmp1 = (g_free (_tmp1), NULL);
	(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
}


VtgPlugin* vtg_project_manager_plugin_helper_get_plugin (VtgProjectManagerPluginHelper* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_plugin;
}


static void vtg_project_manager_plugin_helper_set_plugin (VtgProjectManagerPluginHelper* self, VtgPlugin* value) {
	VtgPlugin* _tmp2;
	VtgPlugin* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_plugin = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "plugin");
}


VtgProjectManagerView* vtg_project_manager_plugin_helper_get_project_view (VtgProjectManagerPluginHelper* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_prj_view;
}


static void _vtg_project_manager_plugin_helper_on_current_project_changed_g_object_notify (VtgProjectManagerView* _sender, GParamSpec* pspec, gpointer self) {
	vtg_project_manager_plugin_helper_on_current_project_changed (self, _sender, pspec);
}


static void __lambda0 (VtgProjectManagerExecuter* sender, VtgProjectManagerPluginHelper* self) {
	g_return_if_fail (sender != NULL);
	vtg_project_manager_plugin_helper_update_ui (self, vtg_project_manager_view_get_current_project (self->priv->_prj_view) == NULL);
}


static void ___lambda0_vtg_project_manager_executer_process_start (VtgProjectManagerExecuter* _sender, gpointer self) {
	__lambda0 (_sender, self);
}


static void __lambda1 (VtgProjectManagerExecuter* sender, gint exit_status, VtgProjectManagerPluginHelper* self) {
	g_return_if_fail (sender != NULL);
	vtg_project_manager_plugin_helper_update_ui (self, vtg_project_manager_view_get_current_project (self->priv->_prj_view) == NULL);
}


static void ___lambda1_vtg_project_manager_executer_process_exit (VtgProjectManagerExecuter* _sender, gint exit_status, gpointer self) {
	__lambda1 (_sender, exit_status, self);
}


static void __lambda2 (VtgProjectManagerBuilder* sender, VtgProjectManagerPluginHelper* self) {
	g_return_if_fail (sender != NULL);
	vtg_project_manager_plugin_helper_update_ui (self, vtg_project_manager_view_get_current_project (self->priv->_prj_view) == NULL);
}


static void ___lambda2_vtg_project_manager_builder_build_start (VtgProjectManagerBuilder* _sender, gpointer self) {
	__lambda2 (_sender, self);
}


static void __lambda3 (VtgProjectManagerBuilder* sender, gint exit_status, VtgProjectManagerPluginHelper* self) {
	g_return_if_fail (sender != NULL);
	vtg_project_manager_plugin_helper_update_ui (self, vtg_project_manager_view_get_current_project (self->priv->_prj_view) == NULL);
}


static void ___lambda3_vtg_project_manager_builder_build_exit (VtgProjectManagerBuilder* _sender, gint exit_status, gpointer self) {
	__lambda3 (_sender, exit_status, self);
}


static GObject * vtg_project_manager_plugin_helper_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgProjectManagerPluginHelperClass * klass;
	GObjectClass * parent_class;
	VtgProjectManagerPluginHelper * self;
	klass = VTG_PROJECT_MANAGER_PLUGIN_HELPER_CLASS (g_type_class_peek (VTG_PROJECT_MANAGER_TYPE_PLUGIN_HELPER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_PROJECT_MANAGER_PLUGIN_HELPER (obj);
	{
		VtgProjectManagerView* _tmp0;
		VtgProjectManagerBuilder* _tmp1;
		VtgProjectManagerExecuter* _tmp2;
		VtgChangeLog* _tmp3;
		_tmp0 = NULL;
		self->priv->_prj_view = (_tmp0 = vtg_project_manager_view_new (self->priv->_plugin), (self->priv->_prj_view == NULL) ? NULL : (self->priv->_prj_view = (g_object_unref (self->priv->_prj_view), NULL)), _tmp0);
		g_signal_connect_object ((GObject*) self->priv->_prj_view, "notify::current-project", (GCallback) _vtg_project_manager_plugin_helper_on_current_project_changed_g_object_notify, self, 0);
		_tmp1 = NULL;
		self->priv->_prj_builder = (_tmp1 = vtg_project_manager_builder_new (self->priv->_plugin), (self->priv->_prj_builder == NULL) ? NULL : (self->priv->_prj_builder = (g_object_unref (self->priv->_prj_builder), NULL)), _tmp1);
		_tmp2 = NULL;
		self->priv->_prj_executer = (_tmp2 = vtg_project_manager_executer_new (self->priv->_plugin), (self->priv->_prj_executer == NULL) ? NULL : (self->priv->_prj_executer = (g_object_unref (self->priv->_prj_executer), NULL)), _tmp2);
		g_signal_connect (self->priv->_prj_executer, "process-start", (GCallback) ___lambda0_vtg_project_manager_executer_process_start, self);
		g_signal_connect (self->priv->_prj_executer, "process-exit", (GCallback) ___lambda1_vtg_project_manager_executer_process_exit, self);
		g_signal_connect (self->priv->_prj_builder, "build-start", (GCallback) ___lambda2_vtg_project_manager_builder_build_start, self);
		g_signal_connect (self->priv->_prj_builder, "build-exit", (GCallback) ___lambda3_vtg_project_manager_builder_build_exit, self);
		vtg_project_manager_plugin_helper_initialize_ui (self);
		_tmp3 = NULL;
		self->priv->_changelog = (_tmp3 = vtg_change_log_new (self->priv->_plugin), (self->priv->_changelog == NULL) ? NULL : (self->priv->_changelog = (g_object_unref (self->priv->_changelog), NULL)), _tmp3);
		vtg_project_manager_plugin_helper_update_ui (self, vtg_project_manager_view_get_current_project (self->priv->_prj_view) == NULL);
	}
	return obj;
}


static void vtg_project_manager_plugin_helper_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectManagerPluginHelper * self;
	gpointer boxed;
	self = VTG_PROJECT_MANAGER_PLUGIN_HELPER (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_PLUGIN_HELPER_PLUGIN:
		g_value_set_object (value, vtg_project_manager_plugin_helper_get_plugin (self));
		break;
		case VTG_PROJECT_MANAGER_PLUGIN_HELPER_PROJECT_VIEW:
		g_value_set_object (value, vtg_project_manager_plugin_helper_get_project_view (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_plugin_helper_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgProjectManagerPluginHelper * self;
	self = VTG_PROJECT_MANAGER_PLUGIN_HELPER (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_PLUGIN_HELPER_PLUGIN:
		vtg_project_manager_plugin_helper_set_plugin (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_plugin_helper_class_init (VtgProjectManagerPluginHelperClass * klass) {
	vtg_project_manager_plugin_helper_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerPluginHelperPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_manager_plugin_helper_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_project_manager_plugin_helper_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_project_manager_plugin_helper_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_plugin_helper_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_PLUGIN_HELPER_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VTG_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_PLUGIN_HELPER_PROJECT_VIEW, g_param_spec_object ("project-view", "project-view", "project-view", VTG_PROJECT_MANAGER_TYPE_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_project_manager_plugin_helper_instance_init (VtgProjectManagerPluginHelper * self) {
	self->priv = VTG_PROJECT_MANAGER_PLUGIN_HELPER_GET_PRIVATE (self);
	self->priv->_ui_def = g_strdup ("<ui>\n                                            <menubar name=\"MenuBar\">\n                                                <menu name=\"FileMenu\" action=\"File\">\n                                                    <placeholder name=\"FileOps_2\">\n                                                        <separator />\n                                                        <menuitem name=\"ProjectNew\" action=\"ProjectNew\"/>\n                                                        <menuitem name=\"ProjectOpen\" action=\"ProjectOpen\"/>\n                                                        <separator />\n                                                        <menuitem name=\"ProjectClose\" action=\"ProjectClose\"/>\n                                                        <separator />\n                                                    </placeholder>\n                                                </menu>\n                                            </menubar>\n\n                                            <menubar name=\"MenuBar\">\n                                              <menu name=\"EditMenu\" action=\"Edit\">\n                                                 <placeholder name=\"EditOps_3\">\n                                                     <separator />\n                                                     <menuitem name=\"ProjectCompleteWord\" action=\"ProjectCompleteWord\"/>\n                                                 </placeholder>\n                                              </menu>\n                                            </menubar>\n\n                                            <menubar name=\"MenuBar\">\n                                              <placeholder name=\"ExtraMenu_1\">\n                                                <menu name=\"BuildMenu\" action=\"ProjectBuildMenuAction\">\n                                                    <placeholder name=\"BuildMenuOps_1\">\n                                                        <menuitem name=\"ProjectBuild\" action=\"ProjectBuild\"/>\n                                                        <menuitem name=\"ProjectBuildClean\" action=\"ProjectBuildClean\"/>\n                                                        <separator />\n                                                        <menuitem name=\"ProjectBuildCleanStamps\" action=\"ProjectBuildCleanStamps\"/>\n                                                        <separator />\n                                                        <menuitem name=\"ProjectBuildConfigure\" action=\"ProjectBuildConfigure\"/>\n                                                        <separator />\n                                                        <menuitem name=\"ProjectBuildCompileFile\" action=\"ProjectBuildCompileFile\"/>\n                                                    </placeholder>\n                                                    <placeholder name=\"BuildMenuOps_2\">\n                                                        <separator />\n                                                        <menuitem name=\"NextError\" action=\"ProjectBuildNextError\"/>\n                                                        <menuitem name=\"PreviousError\" action=\"ProjectBuildPreviousError\"/>\n                                                    </placeholder>\n                                                    <placeholder name=\"BuildMenuOps_3\">\n                                                        <separator />\n                                                        <menuitem name=\"Execute\" action=\"ProjectBuildExecute\"/>\n                                                        <menuitem name=\"KillProcess\" action=\"ProjectBuildKill\"/>\n                                                    </placeholder>\n                                                </menu>\n                                              </placeholder>                                              \n                                            </menubar>\n                                            \n                                            <menubar name=\"MenuBar\">\n                                                <menu name=\"DocumentsMenu\" action=\"Documents\">\n                                                    <placeholder name=\"DocumentsOps_3\">\n                                                        <menuitem name=\"GotoDocument\" action=\"ProjectGotoDocument\"/>\n                                                        <separator />\n                                                    </placeholder>\n                                                </menu>\n                                            </menubar>\n\n                                             <menubar name=\"MenuBar\">\n                                                <menu name=\"SearchMenu\" action=\"Search\">\n                                                    <placeholder name=\"SearchOps_8\">\n                                                    \t<separator />\n                                                        <menuitem name=\"GotoMethod\" action=\"ProjectGotoMethod\"/>\n\n                                                    </placeholder>\n                                                </menu>\n                                            </menubar>\n                                            \n                                            <menubar name=\"MenuBar\">\n                                            \t <menu name=\"ToolsMenu\" action=\"Tools\">\n\t\t\t\t\t\t \t<placeholder name=\"ToolsOps_1\"> \n\t\t\t\t\t\t \t\t<separator />\n\t\t\t\t\t\t\t \t<menuitem name=\"PrepareSingleFileChangeLog\" action=\"ProjectPrepareSingleFileChangeLog\"/>\n\t\t\t\t\t\t\t \t<menuitem name=\"PrepareChangeLog\" action=\"ProjectPrepareChangeLog\"/>\n\t\t\t\t\t\t \t\t<separator />\t\t\t\t\t\t\t \t\n                                                \t</placeholder>\n                                                 </menu>\n                                            </menubar>\n                                        </ui>");
	self->priv->_projects = (GeeList*) gee_array_list_new (VTG_PROJECT_MANAGER_TYPE_PROJECT, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->_actions = NULL;
	self->priv->_prj_view = NULL;
	self->priv->_prj_builder = NULL;
	self->priv->_prj_executer = NULL;
	self->priv->_changelog = NULL;
}


static void vtg_project_manager_plugin_helper_finalize (GObject* obj) {
	VtgProjectManagerPluginHelper * self;
	self = VTG_PROJECT_MANAGER_PLUGIN_HELPER (obj);
	{
		GtkUIManager* _tmp4;
		GtkUIManager* manager;
		_tmp4 = NULL;
		manager = (_tmp4 = gedit_window_get_ui_manager (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4));
		gtk_ui_manager_remove_ui (manager, self->priv->_ui_id);
		gtk_ui_manager_remove_action_group (manager, self->priv->_actions);
		(manager == NULL) ? NULL : (manager = (g_object_unref (manager), NULL));
	}
	self->priv->_ui_def = (g_free (self->priv->_ui_def), NULL);
	(self->priv->_projects == NULL) ? NULL : (self->priv->_projects = (gee_collection_object_unref (self->priv->_projects), NULL));
	(self->priv->_actions == NULL) ? NULL : (self->priv->_actions = (g_object_unref (self->priv->_actions), NULL));
	(self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL));
	(self->priv->_prj_view == NULL) ? NULL : (self->priv->_prj_view = (g_object_unref (self->priv->_prj_view), NULL));
	(self->priv->_prj_builder == NULL) ? NULL : (self->priv->_prj_builder = (g_object_unref (self->priv->_prj_builder), NULL));
	(self->priv->_prj_executer == NULL) ? NULL : (self->priv->_prj_executer = (g_object_unref (self->priv->_prj_executer), NULL));
	(self->priv->_changelog == NULL) ? NULL : (self->priv->_changelog = (g_object_unref (self->priv->_changelog), NULL));
	G_OBJECT_CLASS (vtg_project_manager_plugin_helper_parent_class)->finalize (obj);
}


static GType vtg_project_manager_plugin_helper_type_id = 0;
GType vtg_project_manager_plugin_helper_get_type (void) {
	return vtg_project_manager_plugin_helper_type_id;
}


GType vtg_project_manager_plugin_helper_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerPluginHelperClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_plugin_helper_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerPluginHelper), 0, (GInstanceInitFunc) vtg_project_manager_plugin_helper_instance_init, NULL };
	vtg_project_manager_plugin_helper_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerPluginHelper", &g_define_type_info, 0);
	return vtg_project_manager_plugin_helper_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




