/*
 *  vtgprojectmanagerbuilder.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#ifndef __VTGPROJECTMANAGERBUILDER_H__
#define __VTGPROJECTMANAGERBUILDER_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include "vtgplugin.h"
#include "vtgprojectmanagerproject.h"
#include "vtgprojectmanagerbuildlogview.h"

G_BEGIN_DECLS


#define VTG_PROJECT_MANAGER_TYPE_BUILDER (vtg_project_manager_builder_get_type ())
#define VTG_PROJECT_MANAGER_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_PROJECT_MANAGER_TYPE_BUILDER, VtgProjectManagerBuilder))
#define VTG_PROJECT_MANAGER_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_PROJECT_MANAGER_TYPE_BUILDER, VtgProjectManagerBuilderClass))
#define VTG_PROJECT_MANAGER_IS_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_PROJECT_MANAGER_TYPE_BUILDER))
#define VTG_PROJECT_MANAGER_IS_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_PROJECT_MANAGER_TYPE_BUILDER))
#define VTG_PROJECT_MANAGER_BUILDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_PROJECT_MANAGER_TYPE_BUILDER, VtgProjectManagerBuilderClass))

typedef struct _VtgProjectManagerBuilder VtgProjectManagerBuilder;
typedef struct _VtgProjectManagerBuilderClass VtgProjectManagerBuilderClass;
typedef struct _VtgProjectManagerBuilderPrivate VtgProjectManagerBuilderPrivate;

struct _VtgProjectManagerBuilder {
	GObject parent_instance;
	VtgProjectManagerBuilderPrivate * priv;
};

struct _VtgProjectManagerBuilderClass {
	GObjectClass parent_class;
};


VtgProjectManagerBuilder* vtg_project_manager_builder_construct (GType object_type, VtgPlugin* plugin);
VtgProjectManagerBuilder* vtg_project_manager_builder_new (VtgPlugin* plugin);
gboolean vtg_project_manager_builder_compile_file (VtgProjectManagerBuilder* self, const char* filename, const char* params);
gboolean vtg_project_manager_builder_build (VtgProjectManagerBuilder* self, VtgProjectManagerProject* project, const char* params);
gboolean vtg_project_manager_builder_configure (VtgProjectManagerBuilder* self, VtgProjectManagerProject* project, const char* params);
gboolean vtg_project_manager_builder_clean (VtgProjectManagerBuilder* self, VtgProjectManagerProject* project, gboolean vala_stamp);
void vtg_project_manager_builder_next_error (VtgProjectManagerBuilder* self);
void vtg_project_manager_builder_previous_error (VtgProjectManagerBuilder* self);
void vtg_project_manager_builder_stop_build (VtgProjectManagerBuilder* self);
VtgPlugin* vtg_project_manager_builder_get_plugin (VtgProjectManagerBuilder* self);
VtgProjectManagerBuildLogView* vtg_project_manager_builder_get_error_pane (VtgProjectManagerBuilder* self);
gboolean vtg_project_manager_builder_get_is_building (VtgProjectManagerBuilder* self);
GType vtg_project_manager_builder_get_type (void);
GType vtg_project_manager_builder_register_type (GTypeModule * module);


G_END_DECLS

#endif
