/*
 *  vtgprojectmanagerbuildlogview.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerbuildlogview.h"
#include <gedit/gedit-window.h>
#include <gedit/gedit-panel.h>
#include <gedit/gedit-view.h>
#include <float.h>
#include <math.h>
#include <glib/gi18n-lib.h>




struct _VtgProjectManagerBuildLogViewPrivate {
	GtkVBox* _ui;
	GtkListStore* _model;
	GtkTreeView* _build_view;
	gint current_error_row;
	gint _error_count;
	VtgPlugin* _plugin;
	VtgProjectManagerProject* _project;
};

#define VTG_PROJECT_MANAGER_BUILD_LOG_VIEW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_BUILD_LOG_VIEW, VtgProjectManagerBuildLogViewPrivate))
enum  {
	VTG_PROJECT_MANAGER_BUILD_LOG_VIEW_DUMMY_PROPERTY,
	VTG_PROJECT_MANAGER_BUILD_LOG_VIEW_PLUGIN,
	VTG_PROJECT_MANAGER_BUILD_LOG_VIEW_ERROR_COUNT
};
static void vtg_project_manager_build_log_view_activate_path (VtgProjectManagerBuildLogView* self, const GtkTreePath* path);
static void vtg_project_manager_build_log_view_add_message (VtgProjectManagerBuildLogView* self, const char* file, const char* message);
static void vtg_project_manager_build_log_view_set_plugin (VtgProjectManagerBuildLogView* self, VtgPlugin* value);
static void _vtg_project_manager_build_log_view_on_build_view_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static gboolean _vtg_project_manager_build_log_view_on_message_added_vtg_project_manager_output_view_message_added (VtgProjectManagerOutputView* _sender, const char* message, gpointer self);
static GObject * vtg_project_manager_build_log_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_project_manager_build_log_view_parent_class = NULL;
static void vtg_project_manager_build_log_view_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);



VtgProjectManagerBuildLogView* vtg_project_manager_build_log_view_construct (GType object_type, VtgPlugin* plugin) {
	GParameter * __params;
	GParameter * __params_it;
	VtgProjectManagerBuildLogView * self;
	g_return_val_if_fail (plugin != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "plugin";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN);
	g_value_set_object (&__params_it->value, plugin);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgProjectManagerBuildLogView* vtg_project_manager_build_log_view_new (VtgPlugin* plugin) {
	return vtg_project_manager_build_log_view_construct (VTG_PROJECT_MANAGER_TYPE_BUILD_LOG_VIEW, plugin);
}


void vtg_project_manager_build_log_view_initialize (VtgProjectManagerBuildLogView* self, VtgProjectManagerProject* project) {
	VtgProjectManagerProject* _tmp1;
	VtgProjectManagerProject* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_project = (_tmp1 = (_tmp0 = project, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL)), _tmp1);
	self->priv->current_error_row = 0;
	self->priv->_error_count = 0;
	gtk_list_store_clear (self->priv->_model);
}


void vtg_project_manager_build_log_view_activate (VtgProjectManagerBuildLogView* self) {
	GeditPanel* _tmp0;
	GeditPanel* panel;
	GeditView* _tmp1;
	GeditView* view;
	gboolean _tmp2;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	panel = (_tmp0 = gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	gedit_panel_activate_item (panel, (GtkWidget*) self->priv->_ui);
	_tmp1 = NULL;
	view = (_tmp1 = gedit_window_get_active_view (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	_tmp2 = FALSE;
	if (view != NULL) {
		gboolean _tmp3;
		_tmp2 = !(g_object_get ((GtkWidget*) view, "is-focus", &_tmp3, NULL), _tmp3);
	} else {
		_tmp2 = FALSE;
	}
	if (_tmp2) {
		gtk_widget_grab_focus ((GtkWidget*) view);
	}
	(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
	(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
}


gboolean vtg_project_manager_build_log_view_on_message_added (VtgProjectManagerBuildLogView* self, VtgProjectManagerOutputView* sender, const char* message) {
	char** _tmp1;
	gint lines_size;
	gint lines_length1;
	char** _tmp0;
	char** lines;
	gint idx;
	gboolean _tmp6;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (sender != NULL, FALSE);
	g_return_val_if_fail (message != NULL, FALSE);
	_tmp1 = NULL;
	_tmp0 = NULL;
	lines = (_tmp1 = _tmp0 = g_strsplit (message, "\n", 0), lines_length1 = _vala_array_length (_tmp0), lines_size = lines_length1, _tmp1);
	idx = 0;
	while (lines[idx] != NULL) {
		char** _tmp3;
		gint tmp_size;
		gint tmp_length1;
		char** _tmp2;
		char** tmp;
		gboolean _tmp4;
		_tmp3 = NULL;
		_tmp2 = NULL;
		tmp = (_tmp3 = _tmp2 = g_strsplit (lines[idx], ":", 2), tmp_length1 = _vala_array_length (_tmp2), tmp_size = tmp_length1, _tmp3);
		_tmp4 = FALSE;
		if (tmp[0] != NULL) {
			gboolean _tmp5;
			_tmp5 = FALSE;
			if (g_str_has_suffix (tmp[0], ".vala")) {
				_tmp5 = TRUE;
			} else {
				_tmp5 = g_str_has_suffix (tmp[0], ".vapi");
			}
			_tmp4 = (_tmp5);
		} else {
			_tmp4 = FALSE;
		}
		if (_tmp4) {
			vtg_project_manager_build_log_view_add_message (self, tmp[0], tmp[1]);
		}
		idx++;
		tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL);
	}
	return (_tmp6 = TRUE, lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL), _tmp6);
}


void vtg_project_manager_build_log_view_on_build_view_row_activated (VtgProjectManagerBuildLogView* self, GtkWidget* sender, const GtkTreePath* path, GtkTreeViewColumn* column) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (path != NULL);
	g_return_if_fail (column != NULL);
	vtg_project_manager_build_log_view_activate_path (self, path);
}


static void vtg_project_manager_build_log_view_activate_path (VtgProjectManagerBuildLogView* self, const GtkTreePath* path) {
	GtkTreeIter iter = {0};
	g_return_if_fail (self != NULL);
	g_return_if_fail (path != NULL);
	if (gtk_tree_model_get_iter ((GtkTreeModel*) self->priv->_model, &iter, path)) {
		char* name;
		gint line;
		gint col;
		VtgProjectManagerProject* proj;
		name = NULL;
		line = 0;
		col = 0;
		proj = NULL;
		gtk_tree_model_get ((GtkTreeModel*) self->priv->_model, &iter, 2, &name, 3, &line, 4, &col, 5, &proj, -1);
		if (proj != NULL) {
			char* uri;
			uri = vtg_project_manager_project_source_uri_for_name (proj, name);
			if (uri != NULL) {
				GeditTab* _tmp0;
				_tmp0 = NULL;
				_tmp0 = vtg_plugin_activate_uri (self->priv->_plugin, uri, line, col);
				(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
			} else {
				g_warning ("vtgprojectmanagerbuildlogview.vala:155: Couldn't find uri for source: %s", name);
			}
			uri = (g_free (uri), NULL);
		} else {
			vtg_plugin_activate_display_name (self->priv->_plugin, name, line, col);
		}
		name = (g_free (name), NULL);
		(proj == NULL) ? NULL : (proj = (g_object_unref (proj), NULL));
	}
}


void vtg_project_manager_build_log_view_next_error (VtgProjectManagerBuildLogView* self) {
	char* _tmp0;
	GtkTreePath* _tmp1;
	GtkTreePath* path;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	_tmp1 = NULL;
	path = (_tmp1 = gtk_tree_path_new_from_string (_tmp0 = g_strdup_printf ("%i", self->priv->current_error_row)), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
	if (path != NULL) {
		vtg_project_manager_build_log_view_activate_path (self, path);
		gtk_tree_view_scroll_to_cell (self->priv->_build_view, path, NULL, FALSE, (float) 0, (float) 0);
		gtk_tree_selection_select_path (gtk_tree_view_get_selection (self->priv->_build_view), path);
		if (self->priv->current_error_row < (self->priv->_error_count - 1)) {
			self->priv->current_error_row++;
		} else {
			self->priv->current_error_row = 0;
		}
	}
	(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
}


void vtg_project_manager_build_log_view_previous_error (VtgProjectManagerBuildLogView* self) {
	char* _tmp0;
	GtkTreePath* _tmp1;
	GtkTreePath* path;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	_tmp1 = NULL;
	path = (_tmp1 = gtk_tree_path_new_from_string (_tmp0 = g_strdup_printf ("%i", self->priv->current_error_row)), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
	if (path != NULL) {
		vtg_project_manager_build_log_view_activate_path (self, path);
		gtk_tree_view_scroll_to_cell (self->priv->_build_view, path, NULL, FALSE, (float) 0, (float) 0);
		gtk_tree_selection_select_path (gtk_tree_view_get_selection (self->priv->_build_view), path);
		if (self->priv->current_error_row > 0) {
			self->priv->current_error_row--;
		} else {
			self->priv->current_error_row = self->priv->_error_count - 1;
		}
	}
	(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
}


/* 
  Example valac output:

  vtgprojectmanagerbuilder.vala:37.3-37.16: error: missing return type in method `BuildLogView.LogView */
static void vtg_project_manager_build_log_view_add_message (VtgProjectManagerBuildLogView* self, const char* file, const char* message) {
	char** _tmp1;
	gint parts_size;
	gint parts_length1;
	char** _tmp0;
	char** parts;
	char** _tmp6;
	gint src_ref_size;
	gint src_ref_length1;
	char** _tmp4;
	gint _tmp3_length1;
	char** _tmp3;
	char** _tmp2;
	char** _tmp5;
	char** src_ref;
	gint line;
	gint col;
	char* stock_id;
	g_return_if_fail (self != NULL);
	g_return_if_fail (file != NULL);
	g_return_if_fail (message != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	parts = (_tmp1 = _tmp0 = g_strsplit (message, ":", 3), parts_length1 = _vala_array_length (_tmp0), parts_size = parts_length1, _tmp1);
	_tmp6 = NULL;
	_tmp4 = NULL;
	_tmp3 = NULL;
	_tmp2 = NULL;
	_tmp5 = NULL;
	src_ref = (_tmp6 = (_tmp5 = _tmp4 = g_strsplit ((_tmp3 = _tmp2 = g_strsplit (parts[0], "-", 0), _tmp3_length1 = _vala_array_length (_tmp2), _tmp3)[0], ".", 0), _tmp3 = (_vala_array_free (_tmp3, _tmp3_length1, (GDestroyNotify) g_free), NULL), _tmp5), src_ref_length1 = _vala_array_length (_tmp4), src_ref_size = src_ref_length1, _tmp6);
	line = atoi (src_ref[0]);
	col = 0;
	if (src_ref[1] != NULL) {
		col = atoi (src_ref[1]);
	}
	stock_id = NULL;
	if (parts[1] != NULL) {
		if (g_str_has_suffix (parts[1], "error")) {
			char* _tmp7;
			_tmp7 = NULL;
			stock_id = (_tmp7 = g_strdup (GTK_STOCK_DIALOG_ERROR), stock_id = (g_free (stock_id), NULL), _tmp7);
		} else {
			if (g_str_has_suffix (parts[1], "warning")) {
				char* _tmp8;
				_tmp8 = NULL;
				stock_id = (_tmp8 = g_strdup (GTK_STOCK_DIALOG_WARNING), stock_id = (g_free (stock_id), NULL), _tmp8);
			}
		}
		if (parts[2] != NULL) {
			GtkTreeIter iter = {0};
			gtk_list_store_append (self->priv->_model, &iter);
			gtk_list_store_set (self->priv->_model, &iter, 0, stock_id, 1, parts[2], 2, file, 3, line, 4, col, 5, self->priv->_project, -1);
			self->priv->_error_count++;
		}
	}
	parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
	src_ref = (_vala_array_free (src_ref, src_ref_length1, (GDestroyNotify) g_free), NULL);
	stock_id = (g_free (stock_id), NULL);
}


VtgPlugin* vtg_project_manager_build_log_view_get_plugin (VtgProjectManagerBuildLogView* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_plugin;
}


static void vtg_project_manager_build_log_view_set_plugin (VtgProjectManagerBuildLogView* self, VtgPlugin* value) {
	VtgPlugin* _tmp2;
	VtgPlugin* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_plugin = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "plugin");
}


gint vtg_project_manager_build_log_view_get_error_count (VtgProjectManagerBuildLogView* self) {
	g_return_val_if_fail (self != NULL, 0);
	return self->priv->_error_count;
}


static void _vtg_project_manager_build_log_view_on_build_view_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
	vtg_project_manager_build_log_view_on_build_view_row_activated (self, _sender, path, column);
}


static gboolean _vtg_project_manager_build_log_view_on_message_added_vtg_project_manager_output_view_message_added (VtgProjectManagerOutputView* _sender, const char* message, gpointer self) {
	return vtg_project_manager_build_log_view_on_message_added (self, _sender, message);
}


static GObject * vtg_project_manager_build_log_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgProjectManagerBuildLogViewClass * klass;
	GObjectClass * parent_class;
	VtgProjectManagerBuildLogView * self;
	klass = VTG_PROJECT_MANAGER_BUILD_LOG_VIEW_CLASS (g_type_class_peek (VTG_PROJECT_MANAGER_TYPE_BUILD_LOG_VIEW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_PROJECT_MANAGER_BUILD_LOG_VIEW (obj);
	{
		GeditPanel* _tmp0;
		GeditPanel* panel;
		GtkVBox* _tmp1;
		GtkListStore* _tmp2;
		GtkTreeView* _tmp3;
		GtkCellRenderer* renderer;
		GtkTreeViewColumn* column;
		GtkCellRenderer* _tmp4;
		GtkCellRenderer* _tmp5;
		GtkTreeViewColumn* _tmp6;
		GtkCellRenderer* _tmp7;
		GtkTreeViewColumn* _tmp8;
		GtkCellRenderer* _tmp9;
		GtkTreeViewColumn* _tmp10;
		GtkScrolledWindow* scroll;
		_tmp0 = NULL;
		panel = (_tmp0 = gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
		_tmp1 = NULL;
		self->priv->_ui = (_tmp1 = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 8)), (self->priv->_ui == NULL) ? NULL : (self->priv->_ui = (g_object_unref (self->priv->_ui), NULL)), _tmp1);
		_tmp2 = NULL;
		self->priv->_model = (_tmp2 = gtk_list_store_new (6, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_OBJECT, NULL), (self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL)), _tmp2);
		_tmp3 = NULL;
		self->priv->_build_view = (_tmp3 = g_object_ref_sink ((GtkTreeView*) gtk_tree_view_new_with_model ((GtkTreeModel*) self->priv->_model)), (self->priv->_build_view == NULL) ? NULL : (self->priv->_build_view = (g_object_unref (self->priv->_build_view), NULL)), _tmp3);
		renderer = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ());
		column = g_object_ref_sink (gtk_tree_view_column_new ());
		gtk_tree_view_column_set_title (column, _ ("Message"));
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, FALSE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "stock-id", 0);
		_tmp4 = NULL;
		renderer = (_tmp4 = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp4);
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, TRUE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "text", 1);
		gtk_tree_view_append_column (self->priv->_build_view, column);
		_tmp5 = NULL;
		renderer = (_tmp5 = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp5);
		_tmp6 = NULL;
		column = (_tmp6 = g_object_ref_sink (gtk_tree_view_column_new ()), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp6);
		gtk_tree_view_column_set_title (column, _ ("File"));
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, FALSE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "text", 2);
		gtk_tree_view_append_column (self->priv->_build_view, column);
		_tmp7 = NULL;
		renderer = (_tmp7 = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp7);
		_tmp8 = NULL;
		column = (_tmp8 = g_object_ref_sink (gtk_tree_view_column_new ()), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp8);
		gtk_tree_view_column_set_title (column, _ ("Line"));
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, FALSE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "text", 3);
		gtk_tree_view_append_column (self->priv->_build_view, column);
		_tmp9 = NULL;
		renderer = (_tmp9 = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp9);
		_tmp10 = NULL;
		column = (_tmp10 = g_object_ref_sink (gtk_tree_view_column_new ()), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp10);
		gtk_tree_view_column_set_title (column, _ ("Column"));
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, FALSE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "text", 4);
		gtk_tree_view_append_column (self->priv->_build_view, column);
		g_signal_connect_object (self->priv->_build_view, "row-activated", (GCallback) _vtg_project_manager_build_log_view_on_build_view_row_activated_gtk_tree_view_row_activated, self, 0);
		gtk_tree_view_set_rules_hint (self->priv->_build_view, TRUE);
		scroll = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL));
		gtk_container_add ((GtkContainer*) scroll, (GtkWidget*) self->priv->_build_view);
		gtk_box_pack_start ((GtkBox*) self->priv->_ui, (GtkWidget*) scroll, TRUE, TRUE, (guint) 4);
		gtk_widget_show_all ((GtkWidget*) self->priv->_ui);
		gedit_panel_add_item_with_stock_icon (panel, (GtkWidget*) self->priv->_ui, _ ("Build results"), GTK_STOCK_EXECUTE);
		g_signal_connect_object (vtg_plugin_get_output_view (self->priv->_plugin), "message-added", (GCallback) _vtg_project_manager_build_log_view_on_message_added_vtg_project_manager_output_view_message_added, self, 0);
		(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
		(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
		(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
		(scroll == NULL) ? NULL : (scroll = (g_object_unref (scroll), NULL));
	}
	return obj;
}


static void vtg_project_manager_build_log_view_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectManagerBuildLogView * self;
	gpointer boxed;
	self = VTG_PROJECT_MANAGER_BUILD_LOG_VIEW (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_BUILD_LOG_VIEW_PLUGIN:
		g_value_set_object (value, vtg_project_manager_build_log_view_get_plugin (self));
		break;
		case VTG_PROJECT_MANAGER_BUILD_LOG_VIEW_ERROR_COUNT:
		g_value_set_int (value, vtg_project_manager_build_log_view_get_error_count (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_build_log_view_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgProjectManagerBuildLogView * self;
	self = VTG_PROJECT_MANAGER_BUILD_LOG_VIEW (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_BUILD_LOG_VIEW_PLUGIN:
		vtg_project_manager_build_log_view_set_plugin (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_build_log_view_class_init (VtgProjectManagerBuildLogViewClass * klass) {
	vtg_project_manager_build_log_view_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerBuildLogViewPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_manager_build_log_view_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_project_manager_build_log_view_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_project_manager_build_log_view_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_build_log_view_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_BUILD_LOG_VIEW_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VTG_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_BUILD_LOG_VIEW_ERROR_COUNT, g_param_spec_int ("error-count", "error-count", "error-count", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_project_manager_build_log_view_instance_init (VtgProjectManagerBuildLogView * self) {
	self->priv = VTG_PROJECT_MANAGER_BUILD_LOG_VIEW_GET_PRIVATE (self);
	self->priv->_model = NULL;
	self->priv->_build_view = NULL;
	self->priv->current_error_row = 0;
	self->priv->_error_count = 0;
}


static void vtg_project_manager_build_log_view_finalize (GObject* obj) {
	VtgProjectManagerBuildLogView * self;
	self = VTG_PROJECT_MANAGER_BUILD_LOG_VIEW (obj);
	{
		GeditPanel* _tmp11;
		GeditPanel* panel;
		_tmp11 = NULL;
		panel = (_tmp11 = gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->priv->_plugin)), (_tmp11 == NULL) ? NULL : g_object_ref (_tmp11));
		gedit_panel_remove_item (panel, (GtkWidget*) self->priv->_ui);
		(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
	}
	(self->priv->_ui == NULL) ? NULL : (self->priv->_ui = (g_object_unref (self->priv->_ui), NULL));
	(self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL));
	(self->priv->_build_view == NULL) ? NULL : (self->priv->_build_view = (g_object_unref (self->priv->_build_view), NULL));
	(self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL));
	(self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL));
	G_OBJECT_CLASS (vtg_project_manager_build_log_view_parent_class)->finalize (obj);
}


static GType vtg_project_manager_build_log_view_type_id = 0;
GType vtg_project_manager_build_log_view_get_type (void) {
	return vtg_project_manager_build_log_view_type_id;
}


GType vtg_project_manager_build_log_view_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerBuildLogViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_build_log_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerBuildLogView), 0, (GInstanceInitFunc) vtg_project_manager_build_log_view_instance_init, NULL };
	vtg_project_manager_build_log_view_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerBuildLogView", &g_define_type_info, 0);
	return vtg_project_manager_build_log_view_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}




