/*
 *  vtgprojectmanagerexecuter.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerexecuter.h"
#include <glib/gi18n-lib.h>
#include <sys/wait.h>
#include <signal.h>
#include <sys/types.h>
#include "vtgprojectmanagerbuildlogview.h"
#include "vtgprojectmanageroutputview.h"




static glong string_get_length (const char* self);
struct _VtgProjectManagerExecuterPrivate {
	VtgPlugin* _plugin;
	VtgProjectManagerBuildLogView* _build_view;
	guint _child_watch_id;
	GPid child_pid;
};

#define VTG_PROJECT_MANAGER_EXECUTER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_EXECUTER, VtgProjectManagerExecuterPrivate))
enum  {
	VTG_PROJECT_MANAGER_EXECUTER_DUMMY_PROPERTY,
	VTG_PROJECT_MANAGER_EXECUTER_PLUGIN,
	VTG_PROJECT_MANAGER_EXECUTER_IS_EXECUTING
};
static void _vtg_project_manager_executer_on_child_watch_gchild_watch_func (GPid pid, gint status, gpointer self);
static void vtg_project_manager_executer_on_child_watch (VtgProjectManagerExecuter* self, GPid pid, gint status);
static void vtg_project_manager_executer_set_plugin (VtgProjectManagerExecuter* self, VtgPlugin* value);
static gpointer vtg_project_manager_executer_parent_class = NULL;
static void vtg_project_manager_executer_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);



static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


VtgProjectManagerExecuter* vtg_project_manager_executer_construct (GType object_type, VtgPlugin* plugin) {
	GParameter * __params;
	GParameter * __params_it;
	VtgProjectManagerExecuter * self;
	g_return_val_if_fail (plugin != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "plugin";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN);
	g_value_set_object (&__params_it->value, plugin);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgProjectManagerExecuter* vtg_project_manager_executer_new (VtgPlugin* plugin) {
	return vtg_project_manager_executer_construct (VTG_PROJECT_MANAGER_TYPE_EXECUTER, plugin);
}


static void _vtg_project_manager_executer_on_child_watch_gchild_watch_func (GPid pid, gint status, gpointer self) {
	vtg_project_manager_executer_on_child_watch (self, pid, status);
}


gboolean vtg_project_manager_executer_execute (VtgProjectManagerExecuter* self, VtgProjectManagerProject* project, const char* command_line) {
	GError * inner_error;
	const char* _tmp1;
	char* working_dir;
	gint stdo;
	gint stde;
	gint stdi;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project != NULL, FALSE);
	g_return_val_if_fail (command_line != NULL, FALSE);
	inner_error = NULL;
	if (self->priv->_child_watch_id != 0) {
		return FALSE;
	}
	_tmp1 = NULL;
	working_dir = (_tmp1 = project->filename, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
	stdo = 0;
	stde = 0;
	stdi = 0;
	{
		VtgProjectManagerOutputView* _tmp2;
		VtgProjectManagerOutputView* log;
		char* cmd;
		gint cmds_size;
		gint cmds_length1;
		char** cmds;
		char* start_message;
		char* _tmp7;
		char* _tmp6;
		gboolean _tmp8;
		_tmp2 = NULL;
		log = (_tmp2 = vtg_plugin_get_output_view (self->priv->_plugin), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
		cmd = NULL;
		vtg_project_manager_output_view_clean_output (log);
		if (command_line == NULL) {
			char* _tmp3;
			gboolean _tmp4;
			_tmp3 = NULL;
			vtg_project_manager_output_view_log_message (log, _tmp3 = g_strdup_printf ("No command line specified for project %s", project->name));
			_tmp3 = (g_free (_tmp3), NULL);
			return (_tmp4 = FALSE, (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), cmd = (g_free (cmd), NULL), working_dir = (g_free (working_dir), NULL), _tmp4);
		} else {
			char* _tmp5;
			_tmp5 = NULL;
			cmd = (_tmp5 = g_build_filename (project->filename, command_line, NULL), cmd = (g_free (cmd), NULL), _tmp5);
		}
		cmds = (cmds_length1 = 0, NULL);
		g_shell_parse_argv (cmd, &cmds_length1, &cmds, &inner_error);
		if (inner_error != NULL) {
			(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
			cmd = (g_free (cmd), NULL);
			cmds = (_vala_array_free (cmds, cmds_length1, (GDestroyNotify) g_free), NULL);
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch15_g_spawn_error;
			}
			goto __finally15;
		}
		start_message = g_strdup_printf (_ ("Starting from project %s executable: %s\n"), project->name, cmd);
		vtg_project_manager_output_view_log_message (log, start_message);
		_tmp7 = NULL;
		_tmp6 = NULL;
		vtg_project_manager_output_view_log_message (log, _tmp7 = g_strdup_printf ("%s\n\n", _tmp6 = g_strnfill ((gulong) (string_get_length (start_message) - 1), '-')));
		_tmp7 = (g_free (_tmp7), NULL);
		_tmp6 = (g_free (_tmp6), NULL);
		g_spawn_async_with_pipes (working_dir, cmds, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &self->priv->child_pid, &stdi, &stdo, &stde, &inner_error);
		if (inner_error != NULL) {
			(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
			cmd = (g_free (cmd), NULL);
			cmds = (_vala_array_free (cmds, cmds_length1, (GDestroyNotify) g_free), NULL);
			start_message = (g_free (start_message), NULL);
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch15_g_spawn_error;
			}
			goto __finally15;
		}
		if (self->priv->child_pid != ((GPid) 0)) {
			self->priv->_child_watch_id = g_child_watch_add (self->priv->child_pid, _vtg_project_manager_executer_on_child_watch_gchild_watch_func, self);
			vtg_project_manager_output_view_start_watch (log, self->priv->_child_watch_id, stdo, stde, stdi);
			vtg_project_manager_output_view_activate (log);
			g_signal_emit_by_name (self, "process-start");
		} else {
			vtg_project_manager_output_view_log_message (log, "error spawning process\n");
		}
		return (_tmp8 = TRUE, (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), cmd = (g_free (cmd), NULL), cmds = (_vala_array_free (cmds, cmds_length1, (GDestroyNotify) g_free), NULL), start_message = (g_free (start_message), NULL), working_dir = (g_free (working_dir), NULL), _tmp8);
	}
	goto __finally15;
	__catch15_g_spawn_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			gboolean _tmp9;
			g_warning ("vtgprojectmanagerexecuter.vala:86: Error spawning build process: %s", err->message);
			return (_tmp9 = FALSE, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), working_dir = (g_free (working_dir), NULL), _tmp9);
		}
	}
	__finally15:
	if (inner_error != NULL) {
		working_dir = (g_free (working_dir), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return FALSE;
	}
	working_dir = (g_free (working_dir), NULL);
}


void vtg_project_manager_executer_kill_last (VtgProjectManagerExecuter* self) {
	g_return_if_fail (self != NULL);
	if (((gint) self->priv->child_pid) != 0) {
		if (kill ((gint) self->priv->child_pid, 9) != 0) {
			g_warning ("vtgprojectmanagerexecuter.vala:95: exec error: kill failed");
		}
	}
}


static void vtg_project_manager_executer_on_child_watch (VtgProjectManagerExecuter* self, GPid pid, gint status) {
	VtgProjectManagerOutputView* _tmp0;
	VtgProjectManagerOutputView* log;
	char* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	log = (_tmp0 = vtg_plugin_get_output_view (self->priv->_plugin), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	g_spawn_close_pid (self->priv->child_pid);
	vtg_project_manager_output_view_stop_watch (log, self->priv->_child_watch_id);
	_tmp1 = NULL;
	vtg_project_manager_output_view_log_message (log, _tmp1 = g_strdup_printf (_ ("\nprocess terminated with exit status %d\n"), status));
	_tmp1 = (g_free (_tmp1), NULL);
	vtg_project_manager_build_log_view_activate (self->priv->_build_view);
	self->priv->_child_watch_id = (guint) 0;
	g_signal_emit_by_name (self, "process-exit", WEXITSTATUS (status));
	self->priv->child_pid = (GPid) 0;
	(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
}


VtgPlugin* vtg_project_manager_executer_get_plugin (VtgProjectManagerExecuter* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_plugin;
}


static void vtg_project_manager_executer_set_plugin (VtgProjectManagerExecuter* self, VtgPlugin* value) {
	VtgPlugin* _tmp2;
	VtgPlugin* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_plugin = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "plugin");
}


gboolean vtg_project_manager_executer_get_is_executing (VtgProjectManagerExecuter* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_child_watch_id != 0;
}


static void vtg_project_manager_executer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectManagerExecuter * self;
	gpointer boxed;
	self = VTG_PROJECT_MANAGER_EXECUTER (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_EXECUTER_PLUGIN:
		g_value_set_object (value, vtg_project_manager_executer_get_plugin (self));
		break;
		case VTG_PROJECT_MANAGER_EXECUTER_IS_EXECUTING:
		g_value_set_boolean (value, vtg_project_manager_executer_get_is_executing (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_executer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgProjectManagerExecuter * self;
	self = VTG_PROJECT_MANAGER_EXECUTER (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_EXECUTER_PLUGIN:
		vtg_project_manager_executer_set_plugin (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_executer_class_init (VtgProjectManagerExecuterClass * klass) {
	vtg_project_manager_executer_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerExecuterPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_manager_executer_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_project_manager_executer_set_property;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_executer_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_EXECUTER_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VTG_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_EXECUTER_IS_EXECUTING, g_param_spec_boolean ("is-executing", "is-executing", "is-executing", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("process_start", VTG_PROJECT_MANAGER_TYPE_EXECUTER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("process_exit", VTG_PROJECT_MANAGER_TYPE_EXECUTER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
}


static void vtg_project_manager_executer_instance_init (VtgProjectManagerExecuter * self) {
	self->priv = VTG_PROJECT_MANAGER_EXECUTER_GET_PRIVATE (self);
	self->priv->_build_view = NULL;
	self->priv->_child_watch_id = (guint) 0;
	self->priv->child_pid = (GPid) 0;
}


static void vtg_project_manager_executer_finalize (GObject* obj) {
	VtgProjectManagerExecuter * self;
	self = VTG_PROJECT_MANAGER_EXECUTER (obj);
	(self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL));
	(self->priv->_build_view == NULL) ? NULL : (self->priv->_build_view = (g_object_unref (self->priv->_build_view), NULL));
	G_OBJECT_CLASS (vtg_project_manager_executer_parent_class)->finalize (obj);
}


static GType vtg_project_manager_executer_type_id = 0;
GType vtg_project_manager_executer_get_type (void) {
	return vtg_project_manager_executer_type_id;
}


GType vtg_project_manager_executer_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerExecuterClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_executer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerExecuter), 0, (GInstanceInitFunc) vtg_project_manager_executer_instance_init, NULL };
	vtg_project_manager_executer_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerExecuter", &g_define_type_info, 0);
	return vtg_project_manager_executer_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}




