/*
 *  vtgprojectexecuterdialog.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerexecuterdialog.h"
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <float.h>
#include <math.h>
#include <gee/list.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include "vtgutils.h"
#include <gobject/gvaluecollector.h>




struct _VtgProjectManagerExecuterDialogPrivate {
	GtkEntryCompletion* _completion;
	GtkDialog* _dialog;
	GtkTreeView* _tree;
	GtkButton* _button_exec;
};

#define VTG_PROJECT_MANAGER_EXECUTER_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_EXECUTER_DIALOG, VtgProjectManagerExecuterDialogPrivate))
enum  {
	VTG_PROJECT_MANAGER_EXECUTER_DIALOG_DUMMY_PROPERTY
};
static gboolean _vtg_project_manager_executer_dialog_on_entry_key_press_gtk_widget_key_press_event (GtkEntry* _sender, const GdkEventKey* event, gpointer self);
static void _vtg_project_manager_executer_dialog_on_command_line_changed_g_object_notify (GtkEntry* _sender, GParamSpec* pspec, gpointer self);
static void _vtg_project_manager_executer_dialog_on_tree_selection_changed_gtk_tree_selection_changed (GtkTreeSelection* _sender, gpointer self);
static void vtg_project_manager_executer_dialog_initialize_ui (VtgProjectManagerExecuterDialog* self, GtkWindow* parent, VtgProjectManagerProject* project);
static void vtg_project_manager_executer_dialog_on_tree_selection_changed (VtgProjectManagerExecuterDialog* self, GtkTreeSelection* treeselection);
static void vtg_project_manager_executer_dialog_on_command_line_changed (VtgProjectManagerExecuterDialog* self, GObject* pspec, GParamSpec* gobject);
static gboolean vtg_project_manager_executer_dialog_on_entry_key_press (VtgProjectManagerExecuterDialog* self, GtkWidget* sender, const GdkEventKey* evt);
static gpointer vtg_project_manager_executer_dialog_parent_class = NULL;
static void vtg_project_manager_executer_dialog_finalize (VtgProjectManagerExecuterDialog* obj);



VtgProjectManagerExecuterDialog* vtg_project_manager_executer_dialog_construct (GType object_type, GtkWindow* parent, VtgProjectManagerProject* project) {
	VtgProjectManagerExecuterDialog* self;
	g_return_val_if_fail (parent != NULL, NULL);
	g_return_val_if_fail (project != NULL, NULL);
	self = (VtgProjectManagerExecuterDialog*) g_type_create_instance (object_type);
	vtg_project_manager_executer_dialog_initialize_ui (self, parent, project);
	return self;
}


VtgProjectManagerExecuterDialog* vtg_project_manager_executer_dialog_new (GtkWindow* parent, VtgProjectManagerProject* project) {
	return vtg_project_manager_executer_dialog_construct (VTG_PROJECT_MANAGER_TYPE_EXECUTER_DIALOG, parent, project);
}


static gboolean _vtg_project_manager_executer_dialog_on_entry_key_press_gtk_widget_key_press_event (GtkEntry* _sender, const GdkEventKey* event, gpointer self) {
	return vtg_project_manager_executer_dialog_on_entry_key_press (self, _sender, event);
}


static void _vtg_project_manager_executer_dialog_on_command_line_changed_g_object_notify (GtkEntry* _sender, GParamSpec* pspec, gpointer self) {
	vtg_project_manager_executer_dialog_on_command_line_changed (self, _sender, pspec);
}


static void _vtg_project_manager_executer_dialog_on_tree_selection_changed_gtk_tree_selection_changed (GtkTreeSelection* _sender, gpointer self) {
	vtg_project_manager_executer_dialog_on_tree_selection_changed (self, _sender);
}


static void vtg_project_manager_executer_dialog_initialize_ui (VtgProjectManagerExecuterDialog* self, GtkWindow* parent, VtgProjectManagerProject* project) {
	GError * inner_error;
	GtkListStore* completions;
	GtkBuilder* builder;
	GtkDialog* _tmp2;
	GtkDialog* _tmp1;
	GtkEntryCompletion* _tmp3;
	GtkEntry* _tmp4;
	GtkEntry* entry;
	GtkButton* _tmp6;
	GtkButton* _tmp5;
	GtkTreeView* _tmp8;
	GtkTreeView* _tmp7;
	GtkTreeViewColumn* column;
	GtkCellRendererText* renderer;
	GeeList* _tmp9;
	GeeList* programs;
	GtkTreeIter iter = {0};
	GtkListStore* list;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	g_return_if_fail (project != NULL);
	inner_error = NULL;
	completions = vtg_caches_get_executer_cache ();
	builder = gtk_builder_new ();
	{
		char* _tmp0;
		_tmp0 = NULL;
		gtk_builder_add_from_file (builder, _tmp0 = vtg_utils_get_ui_path ("vtg.ui"), &inner_error);
		if (inner_error != NULL) {
			goto __catch16_g_error;
			goto __finally16;
		}
		_tmp0 = (g_free (_tmp0), NULL);
	}
	goto __finally16;
	__catch16_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgprojectmanagerexecuterdialog.vala:53: initialize_ui: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally16:
	if (inner_error != NULL) {
		(completions == NULL) ? NULL : (completions = (g_object_unref (completions), NULL));
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_dialog = (_tmp2 = (_tmp1 = GTK_DIALOG (gtk_builder_get_object (builder, "dialog-run")), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL)), _tmp2);
	g_assert (self->priv->_dialog != NULL);
	gtk_window_set_transient_for ((GtkWindow*) self->priv->_dialog, parent);
	_tmp3 = NULL;
	self->priv->_completion = (_tmp3 = gtk_entry_completion_new (), (self->priv->_completion == NULL) ? NULL : (self->priv->_completion = (g_object_unref (self->priv->_completion), NULL)), _tmp3);
	gtk_entry_completion_set_model (self->priv->_completion, (GtkTreeModel*) completions);
	gtk_entry_completion_set_text_column (self->priv->_completion, 0);
	_tmp4 = NULL;
	entry = (_tmp4 = GTK_ENTRY (gtk_builder_get_object (builder, "entry-command-line")), (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4));
	g_assert (entry != NULL);
	gtk_entry_set_completion (entry, self->priv->_completion);
	g_signal_connect ((GtkWidget*) entry, "key-press-event", (GCallback) _vtg_project_manager_executer_dialog_on_entry_key_press_gtk_widget_key_press_event, self);
	g_signal_connect ((GObject*) entry, "notify::text", (GCallback) _vtg_project_manager_executer_dialog_on_command_line_changed_g_object_notify, self);
	_tmp6 = NULL;
	_tmp5 = NULL;
	self->priv->_button_exec = (_tmp6 = (_tmp5 = GTK_BUTTON (gtk_builder_get_object (builder, "button-run-execute")), (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5)), (self->priv->_button_exec == NULL) ? NULL : (self->priv->_button_exec = (g_object_unref (self->priv->_button_exec), NULL)), _tmp6);
	g_assert (self->priv->_button_exec != NULL);
	_tmp8 = NULL;
	_tmp7 = NULL;
	self->priv->_tree = (_tmp8 = (_tmp7 = GTK_TREE_VIEW (gtk_builder_get_object (builder, "treeview-executables")), (_tmp7 == NULL) ? NULL : g_object_ref (_tmp7)), (self->priv->_tree == NULL) ? NULL : (self->priv->_tree = (g_object_unref (self->priv->_tree), NULL)), _tmp8);
	g_assert (self->priv->_tree != NULL);
	column = g_object_ref_sink (gtk_tree_view_column_new ());
	renderer = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ());
	gtk_cell_layout_pack_start ((GtkCellLayout*) column, (GtkCellRenderer*) renderer, TRUE);
	gtk_cell_layout_add_attribute ((GtkCellLayout*) column, (GtkCellRenderer*) renderer, "text", 0);
	gtk_tree_view_append_column (self->priv->_tree, column);
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (self->priv->_tree), GTK_SELECTION_SINGLE);
	g_signal_connect (gtk_tree_view_get_selection (self->priv->_tree), "changed", (GCallback) _vtg_project_manager_executer_dialog_on_tree_selection_changed_gtk_tree_selection_changed, self);
	/*initialize the project list*/
	_tmp9 = NULL;
	programs = (_tmp9 = project->exec_targets, (_tmp9 == NULL) ? NULL : gee_collection_object_ref (_tmp9));
	list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING, NULL);
	{
		GeeIterator* _program_it;
		_program_it = gee_iterable_iterator ((GeeIterable*) programs);
		while (gee_iterator_next (_program_it)) {
			char* program;
			char* _tmp10;
			program = (char*) gee_iterator_get (_program_it);
			gtk_list_store_append (list, &iter);
			_tmp10 = NULL;
			gtk_list_store_set (list, &iter, 0, _tmp10 = g_path_get_basename (program), 1, program, -1);
			_tmp10 = (g_free (_tmp10), NULL);
			program = (g_free (program), NULL);
		}
		(_program_it == NULL) ? NULL : (_program_it = (gee_collection_object_unref (_program_it), NULL));
	}
	gtk_tree_view_set_model (self->priv->_tree, (GtkTreeModel*) list);
	if (gtk_tree_model_get_iter_first ((GtkTreeModel*) completions, &iter)) {
		char* val;
		val = NULL;
		gtk_tree_model_get ((GtkTreeModel*) completions, &iter, 0, &val, -1);
		gtk_entry_set_text (entry, val);
		gtk_editable_set_position ((GtkEditable*) entry, -1);
		val = (g_free (val), NULL);
	} else {
		if (gtk_tree_model_get_iter_first ((GtkTreeModel*) list, &iter)) {
			char* program;
			program = NULL;
			gtk_tree_model_get ((GtkTreeModel*) list, &iter, 1, &program, -1);
			gtk_entry_set_text (entry, program);
			gtk_editable_set_position ((GtkEditable*) entry, -1);
			program = (g_free (program), NULL);
		}
	}
	(completions == NULL) ? NULL : (completions = (g_object_unref (completions), NULL));
	(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
	(entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL));
	(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
	(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
	(programs == NULL) ? NULL : (programs = (gee_collection_object_unref (programs), NULL));
	(list == NULL) ? NULL : (list = (g_object_unref (list), NULL));
}


gint vtg_project_manager_executer_dialog_run (VtgProjectManagerExecuterDialog* self) {
	gint res;
	const char* _tmp0;
	char* cmd;
	GtkListStore* cache;
	gboolean _tmp1;
	gint _tmp2;
	g_return_val_if_fail (self != NULL, 0);
	res = gtk_dialog_run (self->priv->_dialog);
	_tmp0 = NULL;
	cmd = (_tmp0 = vtg_project_manager_executer_dialog_get_command_line (self), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	cache = vtg_caches_get_executer_cache ();
	_tmp1 = FALSE;
	if (!vtg_string_utils_is_null_or_empty (cmd)) {
		_tmp1 = !vtg_caches_cache_contains (cache, cmd);
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		vtg_caches_cache_add (cache, cmd);
	}
	return (_tmp2 = res, cmd = (g_free (cmd), NULL), (cache == NULL) ? NULL : (cache = (g_object_unref (cache), NULL)), _tmp2);
}


static void vtg_project_manager_executer_dialog_on_tree_selection_changed (VtgProjectManagerExecuterDialog* self, GtkTreeSelection* treeselection) {
	GtkListStore* model;
	GtkTreeIter iter = {0};
	GtkListStore* _tmp3;
	GtkListStore* _tmp2;
	gboolean _tmp1;
	GtkTreeModel* _tmp0;
	g_return_if_fail (self != NULL);
	g_return_if_fail (treeselection != NULL);
	model = NULL;
	_tmp3 = NULL;
	_tmp2 = NULL;
	_tmp0 = NULL;
	if ((_tmp1 = gtk_tree_selection_get_selected (treeselection, &_tmp0, &iter), model = (_tmp2 = (_tmp3 = (GtkListStore*) _tmp0, (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp2), _tmp1)) {
		char* program;
		GtkEntry* _tmp4;
		GtkEntry* entry;
		program = NULL;
		gtk_tree_model_get ((GtkTreeModel*) model, &iter, 1, &program, -1);
		_tmp4 = NULL;
		entry = (_tmp4 = GTK_ENTRY (gtk_entry_completion_get_entry (self->priv->_completion)), (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4));
		gtk_entry_set_text (entry, program);
		gtk_editable_set_position ((GtkEditable*) entry, -1);
		program = (g_free (program), NULL);
		(entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL));
	}
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
}


static void vtg_project_manager_executer_dialog_on_command_line_changed (VtgProjectManagerExecuterDialog* self, GObject* pspec, GParamSpec* gobject) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (pspec != NULL);
	g_return_if_fail (gobject != NULL);
	gtk_widget_set_sensitive ((GtkWidget*) self->priv->_button_exec, !vtg_string_utils_is_null_or_empty (gtk_entry_get_text ((GTK_ENTRY (gtk_entry_completion_get_entry (self->priv->_completion))))));
}


static gboolean vtg_project_manager_executer_dialog_on_entry_key_press (VtgProjectManagerExecuterDialog* self, GtkWidget* sender, const GdkEventKey* evt) {
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (sender != NULL, FALSE);
	_tmp0 = FALSE;
	if ((*evt).keyval == GDK_Down) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = (*evt).keyval == GDK_Up;
	}
	if (_tmp0) {
		GtkTreeIter sel = {0};
		GtkTreeModel* model;
		GtkTreePath* path;
		GtkTreeModel* _tmp4;
		GtkTreeModel* _tmp3;
		gboolean _tmp2;
		GtkTreeModel* _tmp1;
		GtkTreePath* _tmp8;
		gboolean _tmp9;
		model = NULL;
		path = NULL;
		_tmp4 = NULL;
		_tmp3 = NULL;
		_tmp1 = NULL;
		if ((_tmp2 = gtk_tree_selection_get_selected (gtk_tree_view_get_selection (self->priv->_tree), &_tmp1, &sel), model = (_tmp3 = (_tmp4 = _tmp1, (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp3), _tmp2)) {
			if ((*evt).keyval == GDK_Down) {
				gtk_tree_model_iter_next (model, &sel);
			} else {
				GtkTreePath* _tmp5;
				_tmp5 = NULL;
				path = (_tmp5 = gtk_tree_model_get_path (model, &sel), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp5);
				if (gtk_tree_path_prev (path)) {
					gtk_tree_model_get_iter (model, &sel, path);
				} else {
					gtk_tree_selection_select_iter (gtk_tree_view_get_selection (self->priv->_tree), &sel);
				}
			}
		} else {
			GtkTreeModel* _tmp7;
			GtkTreeModel* _tmp6;
			_tmp7 = NULL;
			_tmp6 = NULL;
			model = (_tmp7 = (_tmp6 = gtk_tree_view_get_model (self->priv->_tree), (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp7);
			gtk_tree_model_get_iter_first (model, &sel);
		}
		_tmp8 = NULL;
		path = (_tmp8 = gtk_tree_model_get_path (model, &sel), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp8);
		gtk_tree_selection_select_iter (gtk_tree_view_get_selection (self->priv->_tree), &sel);
		gtk_tree_view_scroll_to_cell (self->priv->_tree, path, NULL, FALSE, (float) 0, (float) 0);
		return (_tmp9 = TRUE, (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp9);
	}
	return FALSE;
}


const char* vtg_project_manager_executer_dialog_get_command_line (VtgProjectManagerExecuterDialog* self) {
	GtkEntry* _tmp0;
	GtkEntry* en;
	char* _tmp1;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0 = NULL;
	en = (_tmp0 = GTK_ENTRY (gtk_entry_completion_get_entry (self->priv->_completion)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	_tmp1 = NULL;
	return (_tmp1 = gtk_entry_get_text (en), (en == NULL) ? NULL : (en = (g_object_unref (en), NULL)), _tmp1);
}


static void vtg_project_manager_value_executer_dialog_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vtg_project_manager_value_executer_dialog_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vtg_project_manager_executer_dialog_unref (value->data[0].v_pointer);
	}
}


static void vtg_project_manager_value_executer_dialog_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vtg_project_manager_executer_dialog_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vtg_project_manager_value_executer_dialog_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vtg_project_manager_value_executer_dialog_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		VtgProjectManagerExecuterDialog* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vtg_project_manager_executer_dialog_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vtg_project_manager_value_executer_dialog_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	VtgProjectManagerExecuterDialog** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vtg_project_manager_executer_dialog_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vtg_project_manager_param_spec_executer_dialog (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	VtgProjectManagerParamSpecExecuterDialog* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VTG_PROJECT_MANAGER_TYPE_EXECUTER_DIALOG), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vtg_project_manager_value_get_executer_dialog (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_PROJECT_MANAGER_TYPE_EXECUTER_DIALOG), NULL);
	return value->data[0].v_pointer;
}


void vtg_project_manager_value_set_executer_dialog (GValue* value, gpointer v_object) {
	VtgProjectManagerExecuterDialog* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_PROJECT_MANAGER_TYPE_EXECUTER_DIALOG));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VTG_PROJECT_MANAGER_TYPE_EXECUTER_DIALOG));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vtg_project_manager_executer_dialog_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vtg_project_manager_executer_dialog_unref (old);
	}
}


static void vtg_project_manager_executer_dialog_class_init (VtgProjectManagerExecuterDialogClass * klass) {
	vtg_project_manager_executer_dialog_parent_class = g_type_class_peek_parent (klass);
	VTG_PROJECT_MANAGER_EXECUTER_DIALOG_CLASS (klass)->finalize = vtg_project_manager_executer_dialog_finalize;
	g_type_class_add_private (klass, sizeof (VtgProjectManagerExecuterDialogPrivate));
}


static void vtg_project_manager_executer_dialog_instance_init (VtgProjectManagerExecuterDialog * self) {
	self->priv = VTG_PROJECT_MANAGER_EXECUTER_DIALOG_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void vtg_project_manager_executer_dialog_finalize (VtgProjectManagerExecuterDialog* obj) {
	VtgProjectManagerExecuterDialog * self;
	self = VTG_PROJECT_MANAGER_EXECUTER_DIALOG (obj);
	{
		gtk_object_destroy ((GtkObject*) self->priv->_dialog);
	}
	(self->priv->_completion == NULL) ? NULL : (self->priv->_completion = (g_object_unref (self->priv->_completion), NULL));
	(self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL));
	(self->priv->_tree == NULL) ? NULL : (self->priv->_tree = (g_object_unref (self->priv->_tree), NULL));
	(self->priv->_button_exec == NULL) ? NULL : (self->priv->_button_exec = (g_object_unref (self->priv->_button_exec), NULL));
}


static GType vtg_project_manager_executer_dialog_type_id = 0;
GType vtg_project_manager_executer_dialog_get_type (void) {
	return vtg_project_manager_executer_dialog_type_id;
}


GType vtg_project_manager_executer_dialog_register_type (GTypeModule * module) {
	static const GTypeValueTable g_define_type_value_table = { vtg_project_manager_value_executer_dialog_init, vtg_project_manager_value_executer_dialog_free_value, vtg_project_manager_value_executer_dialog_copy_value, vtg_project_manager_value_executer_dialog_peek_pointer, "p", vtg_project_manager_value_executer_dialog_collect_value, "p", vtg_project_manager_value_executer_dialog_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerExecuterDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_executer_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerExecuterDialog), 0, (GInstanceInitFunc) vtg_project_manager_executer_dialog_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	vtg_project_manager_executer_dialog_type_id = g_type_register_fundamental (g_type_fundamental_next (), "VtgProjectManagerExecuterDialog", &g_define_type_info, &g_define_type_fundamental_info, 0);
	return vtg_project_manager_executer_dialog_type_id;
}


gpointer vtg_project_manager_executer_dialog_ref (gpointer instance) {
	VtgProjectManagerExecuterDialog* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vtg_project_manager_executer_dialog_unref (gpointer instance) {
	VtgProjectManagerExecuterDialog* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VTG_PROJECT_MANAGER_EXECUTER_DIALOG_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




