/*
 *  vtgprojectmanagerfilteredlistdialog.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerfilteredlistdialog.h"
#include <stdlib.h>
#include <string.h>
#include <gdk/gdkkeysyms.h>
#include <float.h>
#include <math.h>
#include "vtgutils.h"




struct _VtgProjectManagerFilteredListDialogPrivate {
	GtkDialog* _dialog;
	GtkTreeView* _treeview;
	GtkEntry* _entry;
	GtkTreeModelFilter* _model;
	GtkTreeModelSort* _sorted;
	GtkTreeModel* _child_model;
	GPatternSpec* _current_pattern;
	GtkButton* _button_ok;
};

#define VTG_PROJECT_MANAGER_FILTERED_LIST_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_FILTERED_LIST_DIALOG, VtgProjectManagerFilteredListDialogPrivate))
enum  {
	VTG_PROJECT_MANAGER_FILTERED_LIST_DIALOG_DUMMY_PROPERTY
};
static gboolean _vtg_project_manager_filtered_list_dialog_on_entry_key_press_gtk_widget_key_press_event (GtkEntry* _sender, const GdkEventKey* event, gpointer self);
static void _vtg_project_manager_filtered_list_dialog_on_entry_text_changed_g_object_notify (GtkEntry* _sender, GParamSpec* pspec, gpointer self);
static gboolean _vtg_project_manager_filtered_list_dialog_filter_model_gtk_tree_model_filter_visible_func (GtkTreeModel* model, const GtkTreeIter* iter, gpointer self);
static void _vtg_project_manager_filtered_list_dialog_on_row_changed_gtk_tree_model_row_changed (GtkTreeModel* _sender, const GtkTreePath* path, const GtkTreeIter* iter, gpointer self);
static gint _vtg_project_manager_filtered_list_dialog_sort_model_gtk_tree_iter_compare_func (GtkTreeModel* model, const GtkTreeIter* a, const GtkTreeIter* b, gpointer self);
static gboolean _vtg_project_manager_filtered_list_dialog_on_treeview_key_press_gtk_widget_key_press_event (GtkTreeView* _sender, const GdkEventKey* event, gpointer self);
static void vtg_project_manager_filtered_list_dialog_initialize_ui (VtgProjectManagerFilteredListDialog* self);
static void vtg_project_manager_filtered_list_dialog_on_row_changed (VtgProjectManagerFilteredListDialog* self, GtkTreeModel* tree_model, const GtkTreePath* path, const GtkTreeIter* iter);
static void vtg_project_manager_filtered_list_dialog_on_entry_text_changed (VtgProjectManagerFilteredListDialog* self, GObject* pspec, GParamSpec* gobject);
static void vtg_project_manager_filtered_list_dialog_select_result (VtgProjectManagerFilteredListDialog* self);
static gboolean vtg_project_manager_filtered_list_dialog_filter_model (VtgProjectManagerFilteredListDialog* self, GtkTreeModel* model, const GtkTreeIter* iter);
static gint vtg_project_manager_filtered_list_dialog_sort_model (VtgProjectManagerFilteredListDialog* self, GtkTreeModel* model, const GtkTreeIter* a, const GtkTreeIter* b);
static gpointer vtg_project_manager_filtered_list_dialog_parent_class = NULL;
static void vtg_project_manager_filtered_list_dialog_finalize (GObject* obj);



VtgProjectManagerFilteredListDialog* vtg_project_manager_filtered_list_dialog_construct (GType object_type, GtkTreeModel* model) {
	VtgProjectManagerFilteredListDialog * self;
	GtkTreeModel* _tmp1;
	GtkTreeModel* _tmp0;
	g_return_val_if_fail (model != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_child_model = (_tmp1 = (_tmp0 = model, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_child_model == NULL) ? NULL : (self->priv->_child_model = (g_object_unref (self->priv->_child_model), NULL)), _tmp1);
	vtg_project_manager_filtered_list_dialog_initialize_ui (self);
	return self;
}


VtgProjectManagerFilteredListDialog* vtg_project_manager_filtered_list_dialog_new (GtkTreeModel* model) {
	return vtg_project_manager_filtered_list_dialog_construct (VTG_PROJECT_MANAGER_TYPE_FILTERED_LIST_DIALOG, model);
}


static gboolean _vtg_project_manager_filtered_list_dialog_on_entry_key_press_gtk_widget_key_press_event (GtkEntry* _sender, const GdkEventKey* event, gpointer self) {
	return vtg_project_manager_filtered_list_dialog_on_entry_key_press (self, _sender, event);
}


static void _vtg_project_manager_filtered_list_dialog_on_entry_text_changed_g_object_notify (GtkEntry* _sender, GParamSpec* pspec, gpointer self) {
	vtg_project_manager_filtered_list_dialog_on_entry_text_changed (self, _sender, pspec);
}


static gboolean _vtg_project_manager_filtered_list_dialog_filter_model_gtk_tree_model_filter_visible_func (GtkTreeModel* model, const GtkTreeIter* iter, gpointer self) {
	return vtg_project_manager_filtered_list_dialog_filter_model (self, model, iter);
}


static void _vtg_project_manager_filtered_list_dialog_on_row_changed_gtk_tree_model_row_changed (GtkTreeModel* _sender, const GtkTreePath* path, const GtkTreeIter* iter, gpointer self) {
	vtg_project_manager_filtered_list_dialog_on_row_changed (self, _sender, path, iter);
}


static gint _vtg_project_manager_filtered_list_dialog_sort_model_gtk_tree_iter_compare_func (GtkTreeModel* model, const GtkTreeIter* a, const GtkTreeIter* b, gpointer self) {
	return vtg_project_manager_filtered_list_dialog_sort_model (self, model, a, b);
}


static gboolean _vtg_project_manager_filtered_list_dialog_on_treeview_key_press_gtk_widget_key_press_event (GtkTreeView* _sender, const GdkEventKey* event, gpointer self) {
	return vtg_project_manager_filtered_list_dialog_on_treeview_key_press (self, _sender, event);
}


static void vtg_project_manager_filtered_list_dialog_initialize_ui (VtgProjectManagerFilteredListDialog* self) {
	GError * inner_error;
	GtkBuilder* builder;
	GtkDialog* _tmp2;
	GtkDialog* _tmp1;
	GtkButton* _tmp4;
	GtkButton* _tmp3;
	GtkTreeView* _tmp6;
	GtkTreeView* _tmp5;
	GtkEntry* _tmp8;
	GtkEntry* _tmp7;
	GtkTreeModelFilter* _tmp9;
	GtkTreeViewColumn* column;
	GtkCellRenderer* renderer;
	GtkTreeModelSort* _tmp10;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	builder = gtk_builder_new ();
	{
		char* _tmp0;
		_tmp0 = NULL;
		gtk_builder_add_from_file (builder, _tmp0 = vtg_utils_get_ui_path ("vtg.ui"), &inner_error);
		if (inner_error != NULL) {
			goto __catch25_g_error;
			goto __finally25;
		}
		_tmp0 = (g_free (_tmp0), NULL);
	}
	goto __finally25;
	__catch25_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgprojectmanagerfilteredlistdialog.vala:54: initialize_ui: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally25:
	if (inner_error != NULL) {
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_dialog = (_tmp2 = (_tmp1 = GTK_DIALOG (gtk_builder_get_object (builder, "dialog-db")), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL)), _tmp2);
	g_assert (self->priv->_dialog != NULL);
	_tmp4 = NULL;
	_tmp3 = NULL;
	self->priv->_button_ok = (_tmp4 = (_tmp3 = GTK_BUTTON (gtk_builder_get_object (builder, "button-db-ok")), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3)), (self->priv->_button_ok == NULL) ? NULL : (self->priv->_button_ok = (g_object_unref (self->priv->_button_ok), NULL)), _tmp4);
	g_assert (self->priv->_button_ok != NULL);
	_tmp6 = NULL;
	_tmp5 = NULL;
	self->priv->_treeview = (_tmp6 = (_tmp5 = GTK_TREE_VIEW (gtk_builder_get_object (builder, "treeview-db-docs")), (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5)), (self->priv->_treeview == NULL) ? NULL : (self->priv->_treeview = (g_object_unref (self->priv->_treeview), NULL)), _tmp6);
	g_assert (self->priv->_treeview != NULL);
	_tmp8 = NULL;
	_tmp7 = NULL;
	self->priv->_entry = (_tmp8 = (_tmp7 = GTK_ENTRY (gtk_builder_get_object (builder, "entry-db-filter")), (_tmp7 == NULL) ? NULL : g_object_ref (_tmp7)), (self->priv->_entry == NULL) ? NULL : (self->priv->_entry = (g_object_unref (self->priv->_entry), NULL)), _tmp8);
	g_assert (self->priv->_entry != NULL);
	g_signal_connect_object ((GtkWidget*) self->priv->_entry, "key-press-event", (GCallback) _vtg_project_manager_filtered_list_dialog_on_entry_key_press_gtk_widget_key_press_event, self, 0);
	g_signal_connect_object ((GObject*) self->priv->_entry, "notify::text", (GCallback) _vtg_project_manager_filtered_list_dialog_on_entry_text_changed_g_object_notify, self, 0);
	_tmp9 = NULL;
	self->priv->_model = (_tmp9 = (GtkTreeModelFilter*) gtk_tree_model_filter_new (self->priv->_child_model, NULL), (self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL)), _tmp9);
	gtk_tree_model_filter_set_visible_func (self->priv->_model, _vtg_project_manager_filtered_list_dialog_filter_model_gtk_tree_model_filter_visible_func, g_object_ref (self), g_object_unref);
	g_signal_connect_object (self->priv->_child_model, "row-changed", (GCallback) _vtg_project_manager_filtered_list_dialog_on_row_changed_gtk_tree_model_row_changed, self, 0);
	column = g_object_ref_sink (gtk_tree_view_column_new ());
	renderer = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ());
	gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, TRUE);
	gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "text", 0);
	gtk_tree_view_append_column (self->priv->_treeview, column);
	_tmp10 = NULL;
	self->priv->_sorted = (_tmp10 = (GtkTreeModelSort*) gtk_tree_model_sort_new_with_model ((GtkTreeModel*) self->priv->_model), (self->priv->_sorted == NULL) ? NULL : (self->priv->_sorted = (g_object_unref (self->priv->_sorted), NULL)), _tmp10);
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) self->priv->_sorted, 0, GTK_SORT_ASCENDING);
	gtk_tree_sortable_set_sort_func ((GtkTreeSortable*) self->priv->_sorted, 0, _vtg_project_manager_filtered_list_dialog_sort_model_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
	gtk_tree_view_set_model (self->priv->_treeview, (GtkTreeModel*) self->priv->_sorted);
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (self->priv->_treeview), GTK_SELECTION_SINGLE);
	g_signal_connect_object ((GtkWidget*) self->priv->_treeview, "key-press-event", (GCallback) _vtg_project_manager_filtered_list_dialog_on_treeview_key_press_gtk_widget_key_press_event, self, 0);
	vtg_project_manager_filtered_list_dialog_select_result (self);
	(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
	(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
	(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
}


gboolean vtg_project_manager_filtered_list_dialog_run (VtgProjectManagerFilteredListDialog* self) {
	gint dialog_result;
	g_return_val_if_fail (self != NULL, FALSE);
	gtk_widget_show_all ((GtkWidget*) self->priv->_dialog);
	dialog_result = gtk_dialog_run (self->priv->_dialog);
	if (dialog_result > 0) {
		GtkTreeIter iter = {0};
		if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection (self->priv->_treeview), NULL, &iter)) {
			GtkTreeIter sort = {0};
			gtk_tree_model_sort_convert_iter_to_child_iter (self->priv->_sorted, &sort, &iter);
			gtk_tree_model_filter_convert_iter_to_child_iter (self->priv->_model, &self->selected_iter, &sort);
		} else {
			dialog_result = 0;
		}
	}
	gtk_object_destroy ((GtkObject*) self->priv->_dialog);
	return dialog_result > 0;
}


static void vtg_project_manager_filtered_list_dialog_on_row_changed (VtgProjectManagerFilteredListDialog* self, GtkTreeModel* tree_model, const GtkTreePath* path, const GtkTreeIter* iter) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (tree_model != NULL);
	g_return_if_fail (path != NULL);
	if (!gtk_tree_selection_get_selected (gtk_tree_view_get_selection (self->priv->_treeview), NULL, NULL)) {
		GtkTreeIter sel = {0};
		gtk_tree_model_get_iter_first (tree_model, &sel);
		gtk_tree_selection_select_iter (gtk_tree_view_get_selection (self->priv->_treeview), &sel);
	}
}


gboolean vtg_project_manager_filtered_list_dialog_on_entry_key_press (VtgProjectManagerFilteredListDialog* self, GtkWidget* sender, const GdkEventKey* evt) {
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (sender != NULL, FALSE);
	_tmp0 = FALSE;
	if ((*evt).keyval == GDK_Down) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = (*evt).keyval == GDK_Up;
	}
	if (_tmp0) {
		GtkTreeIter sel = {0};
		GtkTreeModel* model;
		GtkTreePath* path;
		GtkTreeModel* _tmp4;
		GtkTreeModel* _tmp3;
		gboolean _tmp2;
		GtkTreeModel* _tmp1;
		GtkTreePath* _tmp8;
		gboolean _tmp9;
		model = NULL;
		path = NULL;
		_tmp4 = NULL;
		_tmp3 = NULL;
		_tmp1 = NULL;
		if ((_tmp2 = gtk_tree_selection_get_selected (gtk_tree_view_get_selection (self->priv->_treeview), &_tmp1, &sel), model = (_tmp3 = (_tmp4 = _tmp1, (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp3), _tmp2)) {
			if ((*evt).keyval == GDK_Down) {
				gtk_tree_model_iter_next (model, &sel);
			} else {
				GtkTreePath* _tmp5;
				_tmp5 = NULL;
				path = (_tmp5 = gtk_tree_model_get_path (model, &sel), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp5);
				if (gtk_tree_path_prev (path)) {
					gtk_tree_model_get_iter (model, &sel, path);
				} else {
					gtk_tree_selection_select_iter (gtk_tree_view_get_selection (self->priv->_treeview), &sel);
				}
			}
		} else {
			GtkTreeModel* _tmp7;
			GtkTreeModel* _tmp6;
			_tmp7 = NULL;
			_tmp6 = NULL;
			model = (_tmp7 = (_tmp6 = gtk_tree_view_get_model (self->priv->_treeview), (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp7);
			gtk_tree_model_get_iter_first (model, &sel);
		}
		_tmp8 = NULL;
		path = (_tmp8 = gtk_tree_model_get_path (model, &sel), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp8);
		gtk_tree_selection_select_iter (gtk_tree_view_get_selection (self->priv->_treeview), &sel);
		gtk_tree_view_scroll_to_cell (self->priv->_treeview, path, NULL, FALSE, (float) 0, (float) 0);
		return (_tmp9 = TRUE, (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp9);
	}
	return FALSE;
}


gboolean vtg_project_manager_filtered_list_dialog_on_treeview_key_press (VtgProjectManagerFilteredListDialog* self, GtkWidget* sender, const GdkEventKey* evt) {
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (sender != NULL, FALSE);
	_tmp0 = FALSE;
	if (((*evt).state & GDK_MOD1_MASK) == 0) {
		_tmp0 = (*evt).keyval == GDK_Return;
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection (self->priv->_treeview), NULL, NULL)) {
			gtk_dialog_response (self->priv->_dialog, 2);
		}
	}
	return FALSE;
}


static void vtg_project_manager_filtered_list_dialog_on_entry_text_changed (VtgProjectManagerFilteredListDialog* self, GObject* pspec, GParamSpec* gobject) {
	const char* _tmp0;
	char* filter;
	g_return_if_fail (self != NULL);
	g_return_if_fail (pspec != NULL);
	g_return_if_fail (gobject != NULL);
	_tmp0 = NULL;
	filter = (_tmp0 = gtk_entry_get_text (self->priv->_entry), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (vtg_string_utils_is_null_or_empty (filter)) {
		GPatternSpec* _tmp1;
		_tmp1 = NULL;
		self->priv->_current_pattern = (_tmp1 = NULL, (self->priv->_current_pattern == NULL) ? NULL : (self->priv->_current_pattern = (g_pattern_spec_free (self->priv->_current_pattern), NULL)), _tmp1);
	} else {
		char* _tmp2;
		GPatternSpec* _tmp5;
		_tmp2 = NULL;
		filter = (_tmp2 = vtg_string_utils_replace (filter, " ", "*"), filter = (g_free (filter), NULL), _tmp2);
		if (!g_str_has_suffix (filter, "*")) {
			char* _tmp3;
			_tmp3 = NULL;
			filter = (_tmp3 = g_strconcat (filter, ("*"), NULL), filter = (g_free (filter), NULL), _tmp3);
		}
		if (!g_str_has_prefix (filter, "*")) {
			char* _tmp4;
			_tmp4 = NULL;
			filter = (_tmp4 = g_strconcat ("*", filter, NULL), filter = (g_free (filter), NULL), _tmp4);
		}
		_tmp5 = NULL;
		self->priv->_current_pattern = (_tmp5 = g_pattern_spec_new (filter), (self->priv->_current_pattern == NULL) ? NULL : (self->priv->_current_pattern = (g_pattern_spec_free (self->priv->_current_pattern), NULL)), _tmp5);
	}
	gtk_tree_model_filter_refilter (self->priv->_model);
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) self->priv->_sorted, 0, GTK_SORT_ASCENDING);
	vtg_project_manager_filtered_list_dialog_select_result (self);
	filter = (g_free (filter), NULL);
}


static void vtg_project_manager_filtered_list_dialog_select_result (VtgProjectManagerFilteredListDialog* self) {
	g_return_if_fail (self != NULL);
	if (!gtk_tree_selection_get_selected (gtk_tree_view_get_selection (self->priv->_treeview), NULL, NULL)) {
		GtkTreePath* path;
		path = gtk_tree_path_new_from_indices (0, -1);
		gtk_tree_selection_select_path (gtk_tree_view_get_selection (self->priv->_treeview), path);
		(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
	}
	gtk_widget_set_sensitive ((GtkWidget*) self->priv->_button_ok, gtk_tree_selection_get_selected (gtk_tree_view_get_selection (self->priv->_treeview), NULL, NULL));
}


static gboolean vtg_project_manager_filtered_list_dialog_filter_model (VtgProjectManagerFilteredListDialog* self, GtkTreeModel* model, const GtkTreeIter* iter) {
	char* val;
	gboolean _tmp1;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (model != NULL, FALSE);
	if (self->priv->_current_pattern == NULL) {
		return TRUE;
	}
	val = NULL;
	gtk_tree_model_get (model, &(*iter), 0, &val, -1);
	return (_tmp1 = g_pattern_match_string (self->priv->_current_pattern, val), val = (g_free (val), NULL), _tmp1);
}


static gint vtg_project_manager_filtered_list_dialog_sort_model (VtgProjectManagerFilteredListDialog* self, GtkTreeModel* model, const GtkTreeIter* a, const GtkTreeIter* b) {
	char* vala;
	char* valb;
	gint _tmp0;
	g_return_val_if_fail (self != NULL, 0);
	g_return_val_if_fail (model != NULL, 0);
	vala = NULL;
	valb = NULL;
	gtk_tree_model_get (model, &(*a), 0, &vala, -1);
	gtk_tree_model_get (model, &(*b), 0, &valb, -1);
	return (_tmp0 = vtg_path_utils_compare_vala_filenames (vala, valb), vala = (g_free (vala), NULL), valb = (g_free (valb), NULL), _tmp0);
}


static void vtg_project_manager_filtered_list_dialog_class_init (VtgProjectManagerFilteredListDialogClass * klass) {
	vtg_project_manager_filtered_list_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerFilteredListDialogPrivate));
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_filtered_list_dialog_finalize;
}


static void vtg_project_manager_filtered_list_dialog_instance_init (VtgProjectManagerFilteredListDialog * self) {
	self->priv = VTG_PROJECT_MANAGER_FILTERED_LIST_DIALOG_GET_PRIVATE (self);
	self->priv->_current_pattern = NULL;
}


static void vtg_project_manager_filtered_list_dialog_finalize (GObject* obj) {
	VtgProjectManagerFilteredListDialog * self;
	self = VTG_PROJECT_MANAGER_FILTERED_LIST_DIALOG (obj);
	(self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL));
	(self->priv->_treeview == NULL) ? NULL : (self->priv->_treeview = (g_object_unref (self->priv->_treeview), NULL));
	(self->priv->_entry == NULL) ? NULL : (self->priv->_entry = (g_object_unref (self->priv->_entry), NULL));
	(self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL));
	(self->priv->_sorted == NULL) ? NULL : (self->priv->_sorted = (g_object_unref (self->priv->_sorted), NULL));
	(self->priv->_child_model == NULL) ? NULL : (self->priv->_child_model = (g_object_unref (self->priv->_child_model), NULL));
	(self->priv->_current_pattern == NULL) ? NULL : (self->priv->_current_pattern = (g_pattern_spec_free (self->priv->_current_pattern), NULL));
	(self->priv->_button_ok == NULL) ? NULL : (self->priv->_button_ok = (g_object_unref (self->priv->_button_ok), NULL));
	G_OBJECT_CLASS (vtg_project_manager_filtered_list_dialog_parent_class)->finalize (obj);
}


static GType vtg_project_manager_filtered_list_dialog_type_id = 0;
GType vtg_project_manager_filtered_list_dialog_get_type (void) {
	return vtg_project_manager_filtered_list_dialog_type_id;
}


GType vtg_project_manager_filtered_list_dialog_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerFilteredListDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_filtered_list_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerFilteredListDialog), 0, (GInstanceInitFunc) vtg_project_manager_filtered_list_dialog_instance_init, NULL };
	vtg_project_manager_filtered_list_dialog_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerFilteredListDialog", &g_define_type_info, 0);
	return vtg_project_manager_filtered_list_dialog_type_id;
}




