/*
 *  vtgprojectmanageroutputview.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanageroutputview.h"
#include <gee/arraylist.h>
#include <gee/list.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gee/collection.h>
#include <gedit/gedit-window.h>
#include <gedit/gedit-panel.h>
#include <pango/pango.h>
#include <glib/gi18n-lib.h>
#include "vtgprojectmanagerprocesswatchinfo.h"
#include "vtgutils.h"




static glong string_get_length (const char* self);
struct _VtgProjectManagerOutputViewPrivate {
	GeeList* _processes;
	GString* line;
	GtkTextBuffer* _messages;
	GtkTextView* _textview;
	GtkScrolledWindow* _ui;
};

#define VTG_PROJECT_MANAGER_OUTPUT_VIEW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_OUTPUT_VIEW, VtgProjectManagerOutputViewPrivate))
enum  {
	VTG_PROJECT_MANAGER_OUTPUT_VIEW_DUMMY_PROPERTY,
	VTG_PROJECT_MANAGER_OUTPUT_VIEW_PLUGIN
};
static gboolean vtg_project_manager_output_view_on_textview_key_press (VtgProjectManagerOutputView* self, GtkTextView* sender, const GdkEventKey* evt);
static VtgProjectManagerProcessWatchInfo* vtg_project_manager_output_view_find_process_by_id (VtgProjectManagerOutputView* self, guint id);
static VtgProjectManagerProcessWatchInfo* vtg_project_manager_output_view_add_process_view (VtgProjectManagerOutputView* self, guint id);
static gboolean _vtg_project_manager_output_view_on_messages_gio_func (GIOChannel* source, GIOCondition condition, gpointer self);
static void vtg_project_manager_output_view_real_start_watch (VtgProjectManagerOutputView* self, guint id, gint stdo, gint stde, gint stdi);
static void vtg_project_manager_output_view_real_stop_watch (VtgProjectManagerOutputView* self, guint id);
static gboolean vtg_project_manager_output_view_on_messages (VtgProjectManagerOutputView* self, GIOChannel* source, GIOCondition condition);
static void vtg_project_manager_output_view_set_plugin (VtgProjectManagerOutputView* self, VtgPlugin* value);
static gboolean _vtg_project_manager_output_view_on_textview_key_press_gtk_widget_key_press_event (GtkTextView* _sender, const GdkEventKey* event, gpointer self);
static GObject * vtg_project_manager_output_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_project_manager_output_view_parent_class = NULL;
static void vtg_project_manager_output_view_finalize (GObject* obj);


static void g_cclosure_user_marshal_BOOLEAN__STRING (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);

static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


VtgProjectManagerOutputView* vtg_project_manager_output_view_construct (GType object_type, VtgPlugin* plugin) {
	GParameter * __params;
	GParameter * __params_it;
	VtgProjectManagerOutputView * self;
	g_return_val_if_fail (plugin != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "plugin";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN);
	g_value_set_object (&__params_it->value, plugin);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgProjectManagerOutputView* vtg_project_manager_output_view_new (VtgPlugin* plugin) {
	return vtg_project_manager_output_view_construct (VTG_PROJECT_MANAGER_TYPE_OUTPUT_VIEW, plugin);
}


static gboolean vtg_project_manager_output_view_on_textview_key_press (VtgProjectManagerOutputView* self, GtkTextView* sender, const GdkEventKey* evt) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (sender != NULL, FALSE);
	inner_error = NULL;
	if ((*evt).keyval == GDK_Return) {
		char* buffer;
		buffer = NULL;
		if (self->priv->line->len == 0) {
			char* _tmp0;
			_tmp0 = NULL;
			buffer = (_tmp0 = g_strdup ("\n"), buffer = (g_free (buffer), NULL), _tmp0);
		} else {
			char* _tmp1;
			_tmp1 = NULL;
			buffer = (_tmp1 = g_strdup_printf ("%s\n", self->priv->line->str), buffer = (g_free (buffer), NULL), _tmp1);
		}
		/*TODO: find a way to select the target process*/
		{
			GeeIterator* _item_it;
			/*TODO: find a way to select the target process*/
			_item_it = gee_iterable_iterator ((GeeIterable*) self->priv->_processes);
			/*TODO: find a way to select the target process*/
			while (gee_iterator_next (_item_it)) {
				VtgProjectManagerProcessWatchInfo* item;
				/*TODO: find a way to select the target process*/
				item = (VtgProjectManagerProcessWatchInfo*) gee_iterator_get (_item_it);
				if (item->stdin != NULL) {
					gsize bw;
					bw = 0UL;
					{
						g_io_channel_write_chars (item->stdin, (gchar*) buffer, -1, &bw, &inner_error);
						if (inner_error != NULL) {
							goto __catch17_g_error;
							goto __finally17;
						}
						g_io_channel_flush (item->stdin, &inner_error);
						if (inner_error != NULL) {
							goto __catch17_g_error;
							goto __finally17;
						}
					}
					goto __finally17;
					__catch17_g_error:
					{
						GError * err;
						err = inner_error;
						inner_error = NULL;
						{
							g_warning ("vtgprojectmanageroutputview.vala:90: on_textview_key_press - error: %s", err->message);
							(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
						}
					}
					__finally17:
					if (inner_error != NULL) {
						(item == NULL) ? NULL : (item = (vtg_project_manager_process_watch_info_unref (item), NULL));
						(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
						buffer = (g_free (buffer), NULL);
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
						g_clear_error (&inner_error);
						return FALSE;
					}
				}
				(item == NULL) ? NULL : (item = (vtg_project_manager_process_watch_info_unref (item), NULL));
			}
			(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
		}
		g_string_erase (self->priv->line, (glong) 0, (glong) (-1));
		buffer = (g_free (buffer), NULL);
	} else {
		gunichar ch;
		ch = (gunichar) gdk_keyval_to_unicode ((*evt).keyval);
		g_string_append_unichar (self->priv->line, ch);
	}
	return FALSE;
}


static VtgProjectManagerProcessWatchInfo* vtg_project_manager_output_view_find_process_by_id (VtgProjectManagerOutputView* self, guint id) {
	g_return_val_if_fail (self != NULL, NULL);
	{
		GeeIterator* _target_it;
		_target_it = gee_iterable_iterator ((GeeIterable*) self->priv->_processes);
		while (gee_iterator_next (_target_it)) {
			VtgProjectManagerProcessWatchInfo* target;
			target = (VtgProjectManagerProcessWatchInfo*) gee_iterator_get (_target_it);
			if (target->id == id) {
				VtgProjectManagerProcessWatchInfo* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = target, (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), _tmp0);
			}
			(target == NULL) ? NULL : (target = (vtg_project_manager_process_watch_info_unref (target), NULL));
		}
		(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
	}
	return NULL;
}


static VtgProjectManagerProcessWatchInfo* vtg_project_manager_output_view_add_process_view (VtgProjectManagerOutputView* self, guint id) {
	VtgProjectManagerProcessWatchInfo* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = vtg_project_manager_process_watch_info_new (id);
	gee_collection_add ((GeeCollection*) self->priv->_processes, result);
	return result;
}


static gboolean _vtg_project_manager_output_view_on_messages_gio_func (GIOChannel* source, GIOCondition condition, gpointer self) {
	return vtg_project_manager_output_view_on_messages (self, source, condition);
}


static void vtg_project_manager_output_view_real_start_watch (VtgProjectManagerOutputView* self, guint id, gint stdo, gint stde, gint stdi) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	{
		VtgProjectManagerProcessWatchInfo* target;
		VtgProjectManagerProcessWatchInfo* _tmp0;
		GIOChannel* _tmp2;
		GIOChannel* _tmp3;
		GeditPanel* _tmp4;
		GeditPanel* panel;
		gboolean _tmp5;
		target = vtg_project_manager_output_view_find_process_by_id (self, id);
		if (target != NULL) {
			vtg_project_manager_output_view_stop_watch (self, id);
		}
		_tmp0 = NULL;
		target = (_tmp0 = vtg_project_manager_output_view_add_process_view (self, id), (target == NULL) ? NULL : (target = (vtg_project_manager_process_watch_info_unref (target), NULL)), _tmp0);
		if (stdi != (-1)) {
			GIOChannel* _tmp1;
			_tmp1 = NULL;
			target->stdin = (_tmp1 = g_io_channel_unix_new (stdi), (target->stdin == NULL) ? NULL : (target->stdin = (g_io_channel_unref (target->stdin), NULL)), _tmp1);
		}
		_tmp2 = NULL;
		target->stdout = (_tmp2 = g_io_channel_unix_new (stdo), (target->stdout == NULL) ? NULL : (target->stdout = (g_io_channel_unref (target->stdout), NULL)), _tmp2);
		target->stdout_watch_id = g_io_add_watch (target->stdout, G_IO_IN, _vtg_project_manager_output_view_on_messages_gio_func, self);
		g_io_channel_set_flags (target->stdout, g_io_channel_get_flags (target->stdout) | G_IO_FLAG_NONBLOCK, &inner_error);
		if (inner_error != NULL) {
			(target == NULL) ? NULL : (target = (vtg_project_manager_process_watch_info_unref (target), NULL));
			goto __catch18_g_error;
			goto __finally18;
		}
		/*target.stdout.set_buffered (false);*/
		_tmp3 = NULL;
		target->stderr = (_tmp3 = g_io_channel_unix_new (stde), (target->stderr == NULL) ? NULL : (target->stderr = (g_io_channel_unref (target->stderr), NULL)), _tmp3);
		target->stderr_watch_id = g_io_add_watch (target->stderr, G_IO_IN, _vtg_project_manager_output_view_on_messages_gio_func, self);
		g_io_channel_set_flags (target->stderr, g_io_channel_get_flags (target->stderr) | G_IO_FLAG_NONBLOCK, &inner_error);
		if (inner_error != NULL) {
			(target == NULL) ? NULL : (target = (vtg_project_manager_process_watch_info_unref (target), NULL));
			goto __catch18_g_error;
			goto __finally18;
		}
		/*target.stderr.set_buffered (false);*/
		g_string_erase (self->priv->line, (glong) 0, (glong) (-1));
		/*activate bottom pane if not visible*/
		_tmp4 = NULL;
		panel = (_tmp4 = gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->_plugin)), (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4));
		if (!(g_object_get ((GtkWidget*) panel, "visible", &_tmp5, NULL), _tmp5)) {
			gtk_widget_show_all ((GtkWidget*) panel);
		}
		(target == NULL) ? NULL : (target = (vtg_project_manager_process_watch_info_unref (target), NULL));
		(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
	}
	goto __finally18;
	__catch18_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgprojectmanageroutputview.vala:149: error during watch setup: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally18:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


void vtg_project_manager_output_view_start_watch (VtgProjectManagerOutputView* self, guint id, gint stdo, gint stde, gint stdi) {
	VTG_PROJECT_MANAGER_OUTPUT_VIEW_GET_CLASS (self)->start_watch (self, id, stdo, stde, stdi);
}


static void vtg_project_manager_output_view_real_stop_watch (VtgProjectManagerOutputView* self, guint id) {
	VtgProjectManagerProcessWatchInfo* target;
	g_return_if_fail (self != NULL);
	target = vtg_project_manager_output_view_find_process_by_id (self, id);
	if (target == NULL) {
		g_warning ("vtgprojectmanageroutputview.vala:158: stop_watch: no target with id %u found", id);
		(target == NULL) ? NULL : (target = (vtg_project_manager_process_watch_info_unref (target), NULL));
		return;
	}
	vtg_project_manager_process_watch_info_cleanup (target);
	gee_list_remove_at (self->priv->_processes, gee_list_index_of (self->priv->_processes, target));
	(target == NULL) ? NULL : (target = (vtg_project_manager_process_watch_info_unref (target), NULL));
}


void vtg_project_manager_output_view_stop_watch (VtgProjectManagerOutputView* self, guint id) {
	VTG_PROJECT_MANAGER_OUTPUT_VIEW_GET_CLASS (self)->stop_watch (self, id);
}


void vtg_project_manager_output_view_clean_output (VtgProjectManagerOutputView* self) {
	g_return_if_fail (self != NULL);
	gtk_text_buffer_set_text (self->priv->_messages, "", 0);
}


static gboolean vtg_project_manager_output_view_on_messages (VtgProjectManagerOutputView* self, GIOChannel* source, GIOCondition condition) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (source != NULL, FALSE);
	inner_error = NULL;
	{
		if (condition == G_IO_IN) {
			char* message;
			gsize len;
			gchar* _tmp0;
			gint buff_size;
			gint buff_length1;
			gchar* buff;
			message = NULL;
			len = (gsize) 0;
			_tmp0 = NULL;
			buff = (_tmp0 = g_new0 (gchar, 1024), buff_length1 = 1024, buff_size = buff_length1, _tmp0);
			g_io_channel_read_chars (source, buff, buff_length1, &len, &inner_error);
			if (inner_error != NULL) {
				message = (g_free (message), NULL);
				buff = (g_free (buff), NULL);
				goto __catch19_g_error;
				goto __finally19;
			}
			while (len > 0) {
				if (message == NULL) {
					char* _tmp2;
					const char* _tmp1;
					_tmp2 = NULL;
					_tmp1 = NULL;
					message = (_tmp2 = (_tmp1 = (const char*) buff, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), message = (g_free (message), NULL), _tmp2);
				} else {
					char* _tmp3;
					_tmp3 = NULL;
					message = (_tmp3 = g_strconcat (message, (const char*) buff, NULL), message = (g_free (message), NULL), _tmp3);
				}
				g_io_channel_read_chars (source, buff, buff_length1, &len, &inner_error);
				if (inner_error != NULL) {
					message = (g_free (message), NULL);
					buff = (g_free (buff), NULL);
					goto __catch19_g_error;
					goto __finally19;
				}
			}
			if (!vtg_string_utils_is_null_or_empty (message)) {
				vtg_project_manager_output_view_log_message (self, message);
			}
			message = (g_free (message), NULL);
			buff = (g_free (buff), NULL);
		}
		return TRUE;
	}
	goto __finally19;
	__catch19_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			gboolean _tmp5;
			g_warning ("vtgprojectmanageroutputview.vala:193: Error reading from process %s", err->message);
			return (_tmp5 = FALSE, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), _tmp5);
		}
	}
	__finally19:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return FALSE;
	}
}


void vtg_project_manager_output_view_log_message (VtgProjectManagerOutputView* self, const char* message) {
	gboolean _tmp0;
	g_return_if_fail (self != NULL);
	g_return_if_fail (message != NULL);
	_tmp0 = FALSE;
	if (message != NULL) {
		gboolean _tmp1;
		_tmp0 = (g_signal_emit_by_name (self, "message-added", message, &_tmp1), _tmp1);
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		char* str;
		char* _tmp2;
		str = vtg_string_utils_replace (message, "[1m", "");
		_tmp2 = NULL;
		str = (_tmp2 = vtg_string_utils_replace (str, "[m", ""), str = (g_free (str), NULL), _tmp2);
		gtk_text_buffer_insert_at_cursor (self->priv->_messages, str, (gint) string_get_length (str));
		gtk_text_view_scroll_mark_onscreen (self->priv->_textview, gtk_text_buffer_get_insert (self->priv->_messages));
		str = (g_free (str), NULL);
	}
}


void vtg_project_manager_output_view_activate (VtgProjectManagerOutputView* self) {
	GeditPanel* _tmp0;
	GeditPanel* panel;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	panel = (_tmp0 = gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->_plugin)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	gedit_panel_activate_item (panel, (GtkWidget*) self->priv->_ui);
	(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
}


VtgPlugin* vtg_project_manager_output_view_get_plugin (VtgProjectManagerOutputView* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->_plugin;
}


static void vtg_project_manager_output_view_set_plugin (VtgProjectManagerOutputView* self, VtgPlugin* value) {
	VtgPlugin* _tmp2;
	VtgPlugin* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->_plugin = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->_plugin == NULL) ? NULL : (self->_plugin = (g_object_unref (self->_plugin), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "plugin");
}


static gboolean _vtg_project_manager_output_view_on_textview_key_press_gtk_widget_key_press_event (GtkTextView* _sender, const GdkEventKey* event, gpointer self) {
	return vtg_project_manager_output_view_on_textview_key_press (self, _sender, event);
}


static GObject * vtg_project_manager_output_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgProjectManagerOutputViewClass * klass;
	GObjectClass * parent_class;
	VtgProjectManagerOutputView * self;
	klass = VTG_PROJECT_MANAGER_OUTPUT_VIEW_CLASS (g_type_class_peek (VTG_PROJECT_MANAGER_TYPE_OUTPUT_VIEW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_PROJECT_MANAGER_OUTPUT_VIEW (obj);
	{
		GeditPanel* _tmp0;
		GeditPanel* panel;
		GtkTextBuffer* _tmp1;
		GtkTextView* _tmp2;
		PangoFontDescription* font_desc;
		GtkScrolledWindow* _tmp3;
		_tmp0 = NULL;
		panel = (_tmp0 = gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->_plugin)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
		_tmp1 = NULL;
		self->priv->_messages = (_tmp1 = gtk_text_buffer_new (NULL), (self->priv->_messages == NULL) ? NULL : (self->priv->_messages = (g_object_unref (self->priv->_messages), NULL)), _tmp1);
		_tmp2 = NULL;
		self->priv->_textview = (_tmp2 = g_object_ref_sink ((GtkTextView*) gtk_text_view_new_with_buffer (self->priv->_messages)), (self->priv->_textview == NULL) ? NULL : (self->priv->_textview = (g_object_unref (self->priv->_textview), NULL)), _tmp2);
		g_signal_connect_object ((GtkWidget*) self->priv->_textview, "key-press-event", (GCallback) _vtg_project_manager_output_view_on_textview_key_press_gtk_widget_key_press_event, self, 0);
		/* Change default font throughout the widget */
		font_desc = pango_font_description_from_string ("Monospace");
		gtk_widget_modify_font ((GtkWidget*) self->priv->_textview, font_desc);
		gtk_text_view_set_wrap_mode (self->priv->_textview, GTK_WRAP_CHAR);
		_tmp3 = NULL;
		self->priv->_ui = (_tmp3 = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL)), (self->priv->_ui == NULL) ? NULL : (self->priv->_ui = (g_object_unref (self->priv->_ui), NULL)), _tmp3);
		gtk_container_add ((GtkContainer*) self->priv->_ui, (GtkWidget*) self->priv->_textview);
		gtk_widget_show_all ((GtkWidget*) self->priv->_ui);
		gedit_panel_add_item (panel, (GtkWidget*) self->priv->_ui, _ ("Output"), NULL);
		(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
	}
	return obj;
}


static void vtg_project_manager_output_view_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectManagerOutputView * self;
	gpointer boxed;
	self = VTG_PROJECT_MANAGER_OUTPUT_VIEW (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_OUTPUT_VIEW_PLUGIN:
		g_value_set_object (value, vtg_project_manager_output_view_get_plugin (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_output_view_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgProjectManagerOutputView * self;
	self = VTG_PROJECT_MANAGER_OUTPUT_VIEW (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_OUTPUT_VIEW_PLUGIN:
		vtg_project_manager_output_view_set_plugin (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_output_view_class_init (VtgProjectManagerOutputViewClass * klass) {
	vtg_project_manager_output_view_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerOutputViewPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_manager_output_view_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_project_manager_output_view_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_project_manager_output_view_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_output_view_finalize;
	VTG_PROJECT_MANAGER_OUTPUT_VIEW_CLASS (klass)->start_watch = vtg_project_manager_output_view_real_start_watch;
	VTG_PROJECT_MANAGER_OUTPUT_VIEW_CLASS (klass)->stop_watch = vtg_project_manager_output_view_real_stop_watch;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_OUTPUT_VIEW_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VTG_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_signal_new ("message_added", VTG_PROJECT_MANAGER_TYPE_OUTPUT_VIEW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_BOOLEAN__STRING, G_TYPE_BOOLEAN, 1, G_TYPE_STRING);
}


static void vtg_project_manager_output_view_instance_init (VtgProjectManagerOutputView * self) {
	self->priv = VTG_PROJECT_MANAGER_OUTPUT_VIEW_GET_PRIVATE (self);
	self->priv->_processes = (GeeList*) gee_array_list_new (VTG_PROJECT_MANAGER_TYPE_PROCESS_WATCH_INFO, (GBoxedCopyFunc) vtg_project_manager_process_watch_info_ref, vtg_project_manager_process_watch_info_unref, g_direct_equal);
	self->priv->line = g_string_new ("");
	self->priv->_ui = NULL;
}


static void vtg_project_manager_output_view_finalize (GObject* obj) {
	VtgProjectManagerOutputView * self;
	self = VTG_PROJECT_MANAGER_OUTPUT_VIEW (obj);
	{
		GeditPanel* _tmp4;
		GeditPanel* panel;
		_tmp4 = NULL;
		panel = (_tmp4 = gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->_plugin)), (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4));
		gedit_panel_remove_item (panel, (GtkWidget*) self->priv->_ui);
		(panel == NULL) ? NULL : (panel = (g_object_unref (panel), NULL));
	}
	(self->_plugin == NULL) ? NULL : (self->_plugin = (g_object_unref (self->_plugin), NULL));
	(self->priv->_processes == NULL) ? NULL : (self->priv->_processes = (gee_collection_object_unref (self->priv->_processes), NULL));
	(self->priv->line == NULL) ? NULL : (self->priv->line = (g_string_free (self->priv->line, TRUE), NULL));
	(self->priv->_messages == NULL) ? NULL : (self->priv->_messages = (g_object_unref (self->priv->_messages), NULL));
	(self->priv->_textview == NULL) ? NULL : (self->priv->_textview = (g_object_unref (self->priv->_textview), NULL));
	(self->priv->_ui == NULL) ? NULL : (self->priv->_ui = (g_object_unref (self->priv->_ui), NULL));
	G_OBJECT_CLASS (vtg_project_manager_output_view_parent_class)->finalize (obj);
}


static GType vtg_project_manager_output_view_type_id = 0;
GType vtg_project_manager_output_view_get_type (void) {
	return vtg_project_manager_output_view_type_id;
}


GType vtg_project_manager_output_view_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerOutputViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_output_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerOutputView), 0, (GInstanceInitFunc) vtg_project_manager_output_view_instance_init, NULL };
	vtg_project_manager_output_view_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerOutputView", &g_define_type_info, 0);
	return vtg_project_manager_output_view_type_id;
}



static void g_cclosure_user_marshal_BOOLEAN__STRING (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef gboolean (*GMarshalFunc_BOOLEAN__STRING) (gpointer data1, const char* arg_1, gpointer data2);
	register GMarshalFunc_BOOLEAN__STRING callback;
	register GCClosure * cc;
	register gpointer data1, data2;
	gboolean v_return;
	cc = (GCClosure *) closure;
	g_return_if_fail (return_value != NULL);
	g_return_if_fail (n_param_values == 2);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_BOOLEAN__STRING) (marshal_data ? marshal_data : cc->callback);
	v_return = callback (data1, g_value_get_string (param_values + 1), data2);
	g_value_set_boolean (return_value, v_return);
}



