/*
 *  vtgprojectmanagermodulesdialog.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerpackagesdialog.h"
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gbf/gbf-project.h>
#include "vtgutils.h"
#include "vtgprojectmanagerproject.h"
#include <gobject/gvaluecollector.h>




struct _VtgProjectManagerPackagesDialogPrivate {
	VtgProjectManagerProjectModule* _module;
	GeeList* _modules;
	GtkDialog* _dialog;
};

#define VTG_PROJECT_MANAGER_PACKAGES_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_PACKAGES_DIALOG, VtgProjectManagerPackagesDialogPrivate))
enum  {
	VTG_PROJECT_MANAGER_PACKAGES_DIALOG_DUMMY_PROPERTY
};
static void vtg_project_manager_packages_dialog_initialize_ui (VtgProjectManagerPackagesDialog* self);
static gpointer vtg_project_manager_packages_dialog_parent_class = NULL;
static void vtg_project_manager_packages_dialog_finalize (VtgProjectManagerPackagesDialog* obj);



VtgProjectManagerPackagesDialog* vtg_project_manager_packages_dialog_construct (GType object_type, VtgProjectManagerProjectModule* module, GeeList* modules) {
	VtgProjectManagerPackagesDialog* self;
	GeeList* _tmp1;
	GeeList* _tmp0;
	VtgProjectManagerProjectModule* _tmp3;
	VtgProjectManagerProjectModule* _tmp2;
	g_return_val_if_fail (module != NULL, NULL);
	g_return_val_if_fail (modules != NULL, NULL);
	self = (VtgProjectManagerPackagesDialog*) g_type_create_instance (object_type);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_modules = (_tmp1 = (_tmp0 = modules, (_tmp0 == NULL) ? NULL : gee_collection_object_ref (_tmp0)), (self->priv->_modules == NULL) ? NULL : (self->priv->_modules = (gee_collection_object_unref (self->priv->_modules), NULL)), _tmp1);
	_tmp3 = NULL;
	_tmp2 = NULL;
	self->priv->_module = (_tmp3 = (_tmp2 = module, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (self->priv->_module == NULL) ? NULL : (self->priv->_module = (g_object_unref (self->priv->_module), NULL)), _tmp3);
	vtg_project_manager_packages_dialog_initialize_ui (self);
	return self;
}


VtgProjectManagerPackagesDialog* vtg_project_manager_packages_dialog_new (VtgProjectManagerProjectModule* module, GeeList* modules) {
	return vtg_project_manager_packages_dialog_construct (VTG_PROJECT_MANAGER_TYPE_PACKAGES_DIALOG, module, modules);
}


static void vtg_project_manager_packages_dialog_initialize_ui (VtgProjectManagerPackagesDialog* self) {
	GError * inner_error;
	GtkBuilder* builder;
	GtkDialog* _tmp2;
	GtkDialog* _tmp1;
	GtkVBox* _tmp3;
	GtkVBox* vbox;
	GtkWidget* _tmp4;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	builder = gtk_builder_new ();
	{
		char* _tmp0;
		_tmp0 = NULL;
		gtk_builder_add_from_file (builder, _tmp0 = vtg_utils_get_ui_path ("vtg.ui"), &inner_error);
		if (inner_error != NULL) {
			goto __catch24_g_error;
			goto __finally24;
		}
		_tmp0 = (g_free (_tmp0), NULL);
	}
	goto __finally24;
	__catch24_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgprojectmanagerpackagesdialog.vala:49: initialize_ui: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally24:
	if (inner_error != NULL) {
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_dialog = (_tmp2 = (_tmp1 = GTK_DIALOG (gtk_builder_get_object (builder, "dialog-lm")), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL)), _tmp2);
	g_assert (self->priv->_dialog != NULL);
	_tmp3 = NULL;
	vbox = (_tmp3 = GTK_VBOX (gtk_builder_get_object (builder, "dialog-vbox-lm")), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3));
	g_assert (vbox != NULL);
	_tmp4 = gbf_project_configure (vtg_project_manager_project_get_gbf_project (vtg_project_manager_project_module_get_project (self->priv->_module)), &inner_error);
	if (inner_error != NULL) {
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
		(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	gtk_box_pack_start ((GtkBox*) vbox, _tmp4, TRUE, TRUE, (guint) 8);
	(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
	(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
}


void vtg_project_manager_packages_dialog_show (VtgProjectManagerPackagesDialog* self) {
	GtkDialog* _tmp0;
	g_return_if_fail (self != NULL);
	gtk_dialog_run (self->priv->_dialog);
	gtk_object_destroy ((GtkObject*) self->priv->_dialog);
	_tmp0 = NULL;
	self->priv->_dialog = (_tmp0 = NULL, (self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL)), _tmp0);
}


static void vtg_project_manager_value_packages_dialog_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vtg_project_manager_value_packages_dialog_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vtg_project_manager_packages_dialog_unref (value->data[0].v_pointer);
	}
}


static void vtg_project_manager_value_packages_dialog_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vtg_project_manager_packages_dialog_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vtg_project_manager_value_packages_dialog_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vtg_project_manager_value_packages_dialog_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		VtgProjectManagerPackagesDialog* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vtg_project_manager_packages_dialog_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vtg_project_manager_value_packages_dialog_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	VtgProjectManagerPackagesDialog** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vtg_project_manager_packages_dialog_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vtg_project_manager_param_spec_packages_dialog (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	VtgProjectManagerParamSpecPackagesDialog* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VTG_PROJECT_MANAGER_TYPE_PACKAGES_DIALOG), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vtg_project_manager_value_get_packages_dialog (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_PROJECT_MANAGER_TYPE_PACKAGES_DIALOG), NULL);
	return value->data[0].v_pointer;
}


void vtg_project_manager_value_set_packages_dialog (GValue* value, gpointer v_object) {
	VtgProjectManagerPackagesDialog* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_PROJECT_MANAGER_TYPE_PACKAGES_DIALOG));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VTG_PROJECT_MANAGER_TYPE_PACKAGES_DIALOG));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vtg_project_manager_packages_dialog_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vtg_project_manager_packages_dialog_unref (old);
	}
}


static void vtg_project_manager_packages_dialog_class_init (VtgProjectManagerPackagesDialogClass * klass) {
	vtg_project_manager_packages_dialog_parent_class = g_type_class_peek_parent (klass);
	VTG_PROJECT_MANAGER_PACKAGES_DIALOG_CLASS (klass)->finalize = vtg_project_manager_packages_dialog_finalize;
	g_type_class_add_private (klass, sizeof (VtgProjectManagerPackagesDialogPrivate));
}


static void vtg_project_manager_packages_dialog_instance_init (VtgProjectManagerPackagesDialog * self) {
	self->priv = VTG_PROJECT_MANAGER_PACKAGES_DIALOG_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void vtg_project_manager_packages_dialog_finalize (VtgProjectManagerPackagesDialog* obj) {
	VtgProjectManagerPackagesDialog * self;
	self = VTG_PROJECT_MANAGER_PACKAGES_DIALOG (obj);
	(self->priv->_module == NULL) ? NULL : (self->priv->_module = (g_object_unref (self->priv->_module), NULL));
	(self->priv->_modules == NULL) ? NULL : (self->priv->_modules = (gee_collection_object_unref (self->priv->_modules), NULL));
	(self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL));
}


static GType vtg_project_manager_packages_dialog_type_id = 0;
GType vtg_project_manager_packages_dialog_get_type (void) {
	return vtg_project_manager_packages_dialog_type_id;
}


GType vtg_project_manager_packages_dialog_register_type (GTypeModule * module) {
	static const GTypeValueTable g_define_type_value_table = { vtg_project_manager_value_packages_dialog_init, vtg_project_manager_value_packages_dialog_free_value, vtg_project_manager_value_packages_dialog_copy_value, vtg_project_manager_value_packages_dialog_peek_pointer, "p", vtg_project_manager_value_packages_dialog_collect_value, "p", vtg_project_manager_value_packages_dialog_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerPackagesDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_packages_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerPackagesDialog), 0, (GInstanceInitFunc) vtg_project_manager_packages_dialog_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	vtg_project_manager_packages_dialog_type_id = g_type_register_fundamental (g_type_fundamental_next (), "VtgProjectManagerPackagesDialog", &g_define_type_info, &g_define_type_fundamental_info, 0);
	return vtg_project_manager_packages_dialog_type_id;
}


gpointer vtg_project_manager_packages_dialog_ref (gpointer instance) {
	VtgProjectManagerPackagesDialog* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vtg_project_manager_packages_dialog_unref (gpointer instance) {
	VtgProjectManagerPackagesDialog* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VTG_PROJECT_MANAGER_PACKAGES_DIALOG_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




