/*
 *  vtgprojectmanagerprocesswatchinfo.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerprocesswatchinfo.h"
#include <stdlib.h>
#include <string.h>
#include <gobject/gvaluecollector.h>




enum  {
	VTG_PROJECT_MANAGER_PROCESS_WATCH_INFO_DUMMY_PROPERTY
};
static gpointer vtg_project_manager_process_watch_info_parent_class = NULL;
static void vtg_project_manager_process_watch_info_finalize (VtgProjectManagerProcessWatchInfo* obj);



VtgProjectManagerProcessWatchInfo* vtg_project_manager_process_watch_info_construct (GType object_type, guint id) {
	VtgProjectManagerProcessWatchInfo* self;
	self = (VtgProjectManagerProcessWatchInfo*) g_type_create_instance (object_type);
	self->id = id;
	return self;
}


VtgProjectManagerProcessWatchInfo* vtg_project_manager_process_watch_info_new (guint id) {
	return vtg_project_manager_process_watch_info_construct (VTG_PROJECT_MANAGER_TYPE_PROCESS_WATCH_INFO, id);
}


void vtg_project_manager_process_watch_info_cleanup (VtgProjectManagerProcessWatchInfo* self) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	{
		GIOChannel* _tmp0;
		GIOChannel* _tmp1;
		GIOChannel* _tmp2;
		if (self->stdin != NULL) {
			g_io_channel_flush (self->stdin, &inner_error);
			if (inner_error != NULL) {
				goto __catch20_g_error;
				goto __finally20;
			}
		}
		g_io_channel_flush (self->stdout, &inner_error);
		if (inner_error != NULL) {
			goto __catch20_g_error;
			goto __finally20;
		}
		g_io_channel_flush (self->stderr, &inner_error);
		if (inner_error != NULL) {
			goto __catch20_g_error;
			goto __finally20;
		}
		if (self->stdout_watch_id != 0) {
			g_source_remove (self->stdout_watch_id);
		}
		if (self->stderr_watch_id != 0) {
			g_source_remove (self->stderr_watch_id);
		}
		_tmp0 = NULL;
		self->stdin = (_tmp0 = NULL, (self->stdin == NULL) ? NULL : (self->stdin = (g_io_channel_unref (self->stdin), NULL)), _tmp0);
		_tmp1 = NULL;
		self->stdout = (_tmp1 = NULL, (self->stdout == NULL) ? NULL : (self->stdout = (g_io_channel_unref (self->stdout), NULL)), _tmp1);
		_tmp2 = NULL;
		self->stderr = (_tmp2 = NULL, (self->stderr == NULL) ? NULL : (self->stderr = (g_io_channel_unref (self->stderr), NULL)), _tmp2);
	}
	goto __finally20;
	__catch20_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgprojectmanagerprocesswatchinfo.vala:63: cleanup - error: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally20:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static void vtg_project_manager_value_process_watch_info_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vtg_project_manager_value_process_watch_info_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vtg_project_manager_process_watch_info_unref (value->data[0].v_pointer);
	}
}


static void vtg_project_manager_value_process_watch_info_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vtg_project_manager_process_watch_info_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vtg_project_manager_value_process_watch_info_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vtg_project_manager_value_process_watch_info_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		VtgProjectManagerProcessWatchInfo* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vtg_project_manager_process_watch_info_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vtg_project_manager_value_process_watch_info_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	VtgProjectManagerProcessWatchInfo** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vtg_project_manager_process_watch_info_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vtg_project_manager_param_spec_process_watch_info (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	VtgProjectManagerParamSpecProcessWatchInfo* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VTG_PROJECT_MANAGER_TYPE_PROCESS_WATCH_INFO), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vtg_project_manager_value_get_process_watch_info (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_PROJECT_MANAGER_TYPE_PROCESS_WATCH_INFO), NULL);
	return value->data[0].v_pointer;
}


void vtg_project_manager_value_set_process_watch_info (GValue* value, gpointer v_object) {
	VtgProjectManagerProcessWatchInfo* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_PROJECT_MANAGER_TYPE_PROCESS_WATCH_INFO));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VTG_PROJECT_MANAGER_TYPE_PROCESS_WATCH_INFO));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vtg_project_manager_process_watch_info_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vtg_project_manager_process_watch_info_unref (old);
	}
}


static void vtg_project_manager_process_watch_info_class_init (VtgProjectManagerProcessWatchInfoClass * klass) {
	vtg_project_manager_process_watch_info_parent_class = g_type_class_peek_parent (klass);
	VTG_PROJECT_MANAGER_PROCESS_WATCH_INFO_CLASS (klass)->finalize = vtg_project_manager_process_watch_info_finalize;
}


static void vtg_project_manager_process_watch_info_instance_init (VtgProjectManagerProcessWatchInfo * self) {
	self->id = (guint) 0;
	self->stdin = NULL;
	self->stdout = NULL;
	self->stderr = NULL;
	self->stdout_watch_id = (guint) 0;
	self->stderr_watch_id = (guint) 0;
	self->ref_count = 1;
}


static void vtg_project_manager_process_watch_info_finalize (VtgProjectManagerProcessWatchInfo* obj) {
	VtgProjectManagerProcessWatchInfo * self;
	self = VTG_PROJECT_MANAGER_PROCESS_WATCH_INFO (obj);
	(self->stdin == NULL) ? NULL : (self->stdin = (g_io_channel_unref (self->stdin), NULL));
	(self->stdout == NULL) ? NULL : (self->stdout = (g_io_channel_unref (self->stdout), NULL));
	(self->stderr == NULL) ? NULL : (self->stderr = (g_io_channel_unref (self->stderr), NULL));
}


static GType vtg_project_manager_process_watch_info_type_id = 0;
GType vtg_project_manager_process_watch_info_get_type (void) {
	return vtg_project_manager_process_watch_info_type_id;
}


GType vtg_project_manager_process_watch_info_register_type (GTypeModule * module) {
	static const GTypeValueTable g_define_type_value_table = { vtg_project_manager_value_process_watch_info_init, vtg_project_manager_value_process_watch_info_free_value, vtg_project_manager_value_process_watch_info_copy_value, vtg_project_manager_value_process_watch_info_peek_pointer, "p", vtg_project_manager_value_process_watch_info_collect_value, "p", vtg_project_manager_value_process_watch_info_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerProcessWatchInfoClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_process_watch_info_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerProcessWatchInfo), 0, (GInstanceInitFunc) vtg_project_manager_process_watch_info_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	vtg_project_manager_process_watch_info_type_id = g_type_register_fundamental (g_type_fundamental_next (), "VtgProjectManagerProcessWatchInfo", &g_define_type_info, &g_define_type_fundamental_info, 0);
	return vtg_project_manager_process_watch_info_type_id;
}


gpointer vtg_project_manager_process_watch_info_ref (gpointer instance) {
	VtgProjectManagerProcessWatchInfo* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vtg_project_manager_process_watch_info_unref (gpointer instance) {
	VtgProjectManagerProcessWatchInfo* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VTG_PROJECT_MANAGER_PROCESS_WATCH_INFO_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




