/*
 *  vtgprojectmanagerproject.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerproject.h"
#include <gbf/gbf-backend.h>
#include <gee/arraylist.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gee/collection.h>
#include <glib/gi18n-lib.h>
#include "vtgprojectmanagerprojectmodule.h"
#include "vtgprojectmanagerprojectgroup.h"
#include "vtgprojectmanagerprojecttarget.h"
#include "vtgprojectmanagerprojectpackage.h"
#include "vtgutils.h"
#include "vtgvcsbackendsgit.h"
#include "vtgvcsbackendsivcs.h"
#include "vtgvcsbackendsbzr.h"
#include "vtgvcsbackendssvn.h"




static glong string_get_length (const char* self);
static char* string_substring (const char* self, glong offset, glong len);
struct _VtgProjectManagerProjectPrivate {
	GbfBackend* _backend;
	GbfProject* _gbf_project;
	GtkTreeStore* _model;
	gboolean in_update;
};

#define VTG_PROJECT_MANAGER_PROJECT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_PROJECT, VtgProjectManagerProjectPrivate))
enum  {
	VTG_PROJECT_MANAGER_PROJECT_DUMMY_PROPERTY,
	VTG_PROJECT_MANAGER_PROJECT_MODEL,
	VTG_PROJECT_MANAGER_PROJECT_GBF_PROJECT
};
static void _vtg_project_manager_project_on_project_updated_gbf_project_project_updated (GbfProject* _sender, gpointer self);
static void vtg_project_manager_project_vcs_test (VtgProjectManagerProject* self, const char* filename);
static void vtg_project_manager_project_on_project_updated (VtgProjectManagerProject* self, GbfProject* sender);
static gint _vtg_project_manager_project_sort_model_gtk_tree_iter_compare_func (GtkTreeModel* model, const GtkTreeIter* a, const GtkTreeIter* b, gpointer self);
static void vtg_project_manager_project_build_tree_model (VtgProjectManagerProject* self);
static void vtg_project_manager_project_parse_project (VtgProjectManagerProject* self);
static gint vtg_project_manager_project_sort_model (VtgProjectManagerProject* self, GtkTreeModel* model, const GtkTreeIter* a, const GtkTreeIter* b);
static gpointer vtg_project_manager_project_parent_class = NULL;
static void vtg_project_manager_project_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


GQuark vtg_project_manager_project_manager_error_quark (void) {
	return g_quark_from_static_string ("vtg_project_manager_project_manager_error-quark");
}



GType vtg_project_manager_vcs_types_get_type (void) {
	static GType vtg_project_manager_vcs_types_type_id = 0;
	if (G_UNLIKELY (vtg_project_manager_vcs_types_type_id == 0)) {
		static const GEnumValue values[] = {{VTG_PROJECT_MANAGER_VCS_TYPES_NONE, "VTG_PROJECT_MANAGER_VCS_TYPES_NONE", "none"}, {VTG_PROJECT_MANAGER_VCS_TYPES_GIT, "VTG_PROJECT_MANAGER_VCS_TYPES_GIT", "git"}, {VTG_PROJECT_MANAGER_VCS_TYPES_BZR, "VTG_PROJECT_MANAGER_VCS_TYPES_BZR", "bzr"}, {VTG_PROJECT_MANAGER_VCS_TYPES_SVN, "VTG_PROJECT_MANAGER_VCS_TYPES_SVN", "svn"}, {0, NULL, NULL}};
		vtg_project_manager_vcs_types_type_id = g_enum_register_static ("VtgProjectManagerVcsTypes", values);
	}
	return vtg_project_manager_vcs_types_type_id;
}


VtgProjectManagerProjectGroup* vtg_project_manager_project_find_group (VtgProjectManagerProject* self, const char* id) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	{
		GeeIterator* _group_it;
		_group_it = gee_iterable_iterator ((GeeIterable*) self->groups);
		while (gee_iterator_next (_group_it)) {
			VtgProjectManagerProjectGroup* group;
			group = (VtgProjectManagerProjectGroup*) gee_iterator_get (_group_it);
			if (_vala_strcmp0 (group->id, id) == 0) {
				VtgProjectManagerProjectGroup* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = group, (_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL)), _tmp0);
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	return NULL;
}


gboolean vtg_project_manager_project_contains_source_file (VtgProjectManagerProject* self, const char* uri) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (uri != NULL, FALSE);
	{
		GeeIterator* _group_it;
		_group_it = gee_iterable_iterator ((GeeIterable*) self->groups);
		while (gee_iterator_next (_group_it)) {
			VtgProjectManagerProjectGroup* group;
			group = (VtgProjectManagerProjectGroup*) gee_iterator_get (_group_it);
			{
				GeeIterator* _target_it;
				_target_it = gee_iterable_iterator ((GeeIterable*) group->targets);
				while (gee_iterator_next (_target_it)) {
					VtgProjectManagerProjectTarget* target;
					target = (VtgProjectManagerProjectTarget*) gee_iterator_get (_target_it);
					{
						GeeIterator* _source_it;
						_source_it = gee_iterable_iterator ((GeeIterable*) target->sources);
						while (gee_iterator_next (_source_it)) {
							VtgProjectManagerProjectSource* source;
							source = (VtgProjectManagerProjectSource*) gee_iterator_get (_source_it);
							if (_vala_strcmp0 (source->uri, uri) == 0) {
								gboolean _tmp0;
								return (_tmp0 = TRUE, (source == NULL) ? NULL : (source = (g_object_unref (source), NULL)), (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), (group == NULL) ? NULL : (group = (g_object_unref (group), NULL)), (_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL)), _tmp0);
							}
							(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
						}
						(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	return FALSE;
}


VtgProjectManagerProjectSource* vtg_project_manager_project_get_source_file_from_uri (VtgProjectManagerProject* self, const char* uri) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (uri != NULL, NULL);
	{
		GeeIterator* _group_it;
		_group_it = gee_iterable_iterator ((GeeIterable*) self->groups);
		while (gee_iterator_next (_group_it)) {
			VtgProjectManagerProjectGroup* group;
			group = (VtgProjectManagerProjectGroup*) gee_iterator_get (_group_it);
			{
				GeeIterator* _target_it;
				_target_it = gee_iterable_iterator ((GeeIterable*) group->targets);
				while (gee_iterator_next (_target_it)) {
					VtgProjectManagerProjectTarget* target;
					target = (VtgProjectManagerProjectTarget*) gee_iterator_get (_target_it);
					{
						GeeIterator* _source_it;
						_source_it = gee_iterable_iterator ((GeeIterable*) target->sources);
						while (gee_iterator_next (_source_it)) {
							VtgProjectManagerProjectSource* source;
							source = (VtgProjectManagerProjectSource*) gee_iterator_get (_source_it);
							if (_vala_strcmp0 (source->uri, uri) == 0) {
								VtgProjectManagerProjectSource* _tmp0;
								_tmp0 = NULL;
								return (_tmp0 = source, (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), (group == NULL) ? NULL : (group = (g_object_unref (group), NULL)), (_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL)), _tmp0);
							}
							(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
						}
						(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	return NULL;
}


gboolean vtg_project_manager_project_contains_vala_source_file (VtgProjectManagerProject* self, const char* uri) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (uri != NULL, FALSE);
	{
		GeeIterator* _source_it;
		_source_it = gee_iterable_iterator ((GeeIterable*) self->all_vala_sources);
		while (gee_iterator_next (_source_it)) {
			VtgProjectManagerProjectSource* source;
			source = (VtgProjectManagerProjectSource*) gee_iterator_get (_source_it);
			if (_vala_strcmp0 (source->uri, uri) == 0) {
				gboolean _tmp0;
				return (_tmp0 = TRUE, (source == NULL) ? NULL : (source = (g_object_unref (source), NULL)), (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), _tmp0);
			}
			(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
		}
		(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
	}
	return FALSE;
}


char* vtg_project_manager_project_source_uri_for_name (VtgProjectManagerProject* self, const char* name) {
	char** _tmp1;
	gint name_parts_size;
	gint name_parts_length1;
	char** _tmp0;
	char** name_parts;
	gint name_count;
	char* _tmp8;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	name_parts = (_tmp1 = _tmp0 = g_strsplit (name, "/", 0), name_parts_length1 = _vala_array_length (_tmp0), name_parts_size = name_parts_length1, _tmp1);
	name_count = 0;
	while (name_parts[name_count] != NULL) {
		name_count++;
	}
	{
		GeeIterator* _group_it;
		_group_it = gee_iterable_iterator ((GeeIterable*) self->groups);
		while (gee_iterator_next (_group_it)) {
			VtgProjectManagerProjectGroup* group;
			group = (VtgProjectManagerProjectGroup*) gee_iterator_get (_group_it);
			{
				GeeIterator* _target_it;
				_target_it = gee_iterable_iterator ((GeeIterable*) group->targets);
				while (gee_iterator_next (_target_it)) {
					VtgProjectManagerProjectTarget* target;
					target = (VtgProjectManagerProjectTarget*) gee_iterator_get (_target_it);
					{
						GeeIterator* _source_it;
						_source_it = gee_iterable_iterator ((GeeIterable*) target->sources);
						while (gee_iterator_next (_source_it)) {
							VtgProjectManagerProjectSource* source;
							source = (VtgProjectManagerProjectSource*) gee_iterator_get (_source_it);
							if (name_count == 1) {
								if (_vala_strcmp0 (source->name, name) == 0) {
									const char* _tmp2;
									char* _tmp3;
									_tmp2 = NULL;
									_tmp3 = NULL;
									return (_tmp3 = (_tmp2 = source->uri, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), (source == NULL) ? NULL : (source = (g_object_unref (source), NULL)), (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), (group == NULL) ? NULL : (group = (g_object_unref (group), NULL)), (_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL)), name_parts = (_vala_array_free (name_parts, name_parts_length1, (GDestroyNotify) g_free), NULL), _tmp3);
								}
							} else {
								char** _tmp5;
								gint src_parts_size;
								gint src_parts_length1;
								char** _tmp4;
								char** src_parts;
								gint src_count;
								_tmp5 = NULL;
								_tmp4 = NULL;
								src_parts = (_tmp5 = _tmp4 = g_strsplit (source->uri, "/", 0), src_parts_length1 = _vala_array_length (_tmp4), src_parts_size = src_parts_length1, _tmp5);
								src_count = 0;
								while (src_parts[src_count] != NULL) {
									src_count++;
								}
								if (name_count <= src_count) {
									gboolean equals;
									equals = TRUE;
									{
										gint idx;
										idx = 0;
										for (; idx < name_count; idx++) {
											if (_vala_strcmp0 (src_parts[src_count - idx], name_parts[name_count - idx]) != 0) {
												equals = FALSE;
												break;
											}
										}
									}
									if (equals) {
										const char* _tmp6;
										char* _tmp7;
										_tmp6 = NULL;
										_tmp7 = NULL;
										return (_tmp7 = (_tmp6 = source->uri, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), src_parts = (_vala_array_free (src_parts, src_parts_length1, (GDestroyNotify) g_free), NULL), (source == NULL) ? NULL : (source = (g_object_unref (source), NULL)), (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), (group == NULL) ? NULL : (group = (g_object_unref (group), NULL)), (_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL)), name_parts = (_vala_array_free (name_parts, name_parts_length1, (GDestroyNotify) g_free), NULL), _tmp7);
									}
								}
								src_parts = (_vala_array_free (src_parts, src_parts_length1, (GDestroyNotify) g_free), NULL);
							}
							(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
						}
						(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	_tmp8 = NULL;
	return (_tmp8 = NULL, name_parts = (_vala_array_free (name_parts, name_parts_length1, (GDestroyNotify) g_free), NULL), _tmp8);
}


static void _vtg_project_manager_project_on_project_updated_gbf_project_project_updated (GbfProject* _sender, gpointer self) {
	vtg_project_manager_project_on_project_updated (self, _sender);
}


gboolean vtg_project_manager_project_open (VtgProjectManagerProject* self, const char* project_filename, GError** error) {
	GError * inner_error;
	char* _tmp1;
	const char* _tmp0;
	char** _tmp3;
	gint tmp_size;
	gint tmp_length1;
	char** _tmp2;
	char** tmp;
	gint count;
	GbfProject* _tmp9;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project_filename != NULL, FALSE);
	inner_error = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->filename = (_tmp1 = (_tmp0 = project_filename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->filename = (g_free (self->filename), NULL), _tmp1);
	_tmp3 = NULL;
	_tmp2 = NULL;
	tmp = (_tmp3 = _tmp2 = g_strsplit (self->filename, "/", 0), tmp_length1 = _vala_array_length (_tmp2), tmp_size = tmp_length1, _tmp3);
	count = 0;
	while (tmp[count] != NULL) {
		count++;
	}
	if (count == 0) {
		char* _tmp5;
		const char* _tmp4;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->name = (_tmp5 = (_tmp4 = self->filename, (_tmp4 == NULL) ? NULL : g_strdup (_tmp4)), self->name = (g_free (self->name), NULL), _tmp5);
	} else {
		char* _tmp7;
		const char* _tmp6;
		_tmp7 = NULL;
		_tmp6 = NULL;
		self->name = (_tmp7 = (_tmp6 = tmp[count - 1], (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), self->name = (g_free (self->name), NULL), _tmp7);
	}
	gbf_backend_init ();
	{
		GSList* item_collection;
		GSList* item_it;
		item_collection = gbf_backend_get_backends ();
		for (item_it = item_collection; item_it != NULL; item_it = item_it->next) {
			GbfBackend* item;
			item = (GbfBackend*) item_it->data;
			{
				GbfProject* proj;
				gboolean _tmp8;
				proj = gbf_backend_new_project (item->id);
				_tmp8 = gbf_project_probe (proj, self->filename, &inner_error);
				if (inner_error != NULL) {
					g_propagate_error (error, inner_error);
					(proj == NULL) ? NULL : (proj = (g_object_unref (proj), NULL));
					tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL);
					return FALSE;
				}
				if (_tmp8) {
					self->priv->_backend = item;
					(proj == NULL) ? NULL : (proj = (g_object_unref (proj), NULL));
					break;
				}
				(proj == NULL) ? NULL : (proj = (g_object_unref (proj), NULL));
			}
		}
	}
	_tmp9 = NULL;
	self->priv->_gbf_project = (_tmp9 = NULL, (self->priv->_gbf_project == NULL) ? NULL : (self->priv->_gbf_project = (g_object_unref (self->priv->_gbf_project), NULL)), _tmp9);
	if (self->priv->_backend != NULL) {
		GbfProject* _tmp10;
		gboolean _tmp11;
		_tmp10 = NULL;
		self->priv->_gbf_project = (_tmp10 = gbf_backend_new_project (self->priv->_backend->id), (self->priv->_gbf_project == NULL) ? NULL : (self->priv->_gbf_project = (g_object_unref (self->priv->_gbf_project), NULL)), _tmp10);
		gbf_project_load (self->priv->_gbf_project, self->filename, &inner_error);
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL);
			return FALSE;
		}
		vtg_project_manager_project_parse_project (self);
		vtg_project_manager_project_build_tree_model (self);
		vtg_project_manager_project_vcs_test (self, self->filename);
		g_signal_connect_object (self->priv->_gbf_project, "project-updated", (GCallback) _vtg_project_manager_project_on_project_updated_gbf_project_project_updated, self, 0);
		return (_tmp11 = TRUE, tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL), _tmp11);
	} else {
		inner_error = g_error_new_literal (VTG_PROJECT_MANAGER_PROJECT_MANAGER_ERROR, VTG_PROJECT_MANAGER_PROJECT_MANAGER_ERROR_NO_BACKEND, _ ("Can't load project, no suitable backend found"));
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL);
			return FALSE;
		}
	}
	tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL);
}


static void vtg_project_manager_project_vcs_test (VtgProjectManagerProject* self, const char* filename) {
	VtgVcsBackendsIVcs* backend;
	g_return_if_fail (self != NULL);
	g_return_if_fail (filename != NULL);
	/*test if the project is under some known revision control system*/
	backend = (VtgVcsBackendsIVcs*) vtg_vcs_backends_git_new ();
	self->vcs_type = VTG_PROJECT_MANAGER_VCS_TYPES_NONE;
	if (vtg_vcs_backends_ivcs_test (backend, filename)) {
		self->vcs_type = VTG_PROJECT_MANAGER_VCS_TYPES_GIT;
	} else {
		VtgVcsBackendsIVcs* _tmp0;
		_tmp0 = NULL;
		backend = (_tmp0 = (VtgVcsBackendsIVcs*) vtg_vcs_backends_bzr_new (), (backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL)), _tmp0);
		if (vtg_vcs_backends_ivcs_test (backend, filename)) {
			self->vcs_type = VTG_PROJECT_MANAGER_VCS_TYPES_BZR;
		} else {
			VtgVcsBackendsIVcs* _tmp1;
			_tmp1 = NULL;
			backend = (_tmp1 = (VtgVcsBackendsIVcs*) vtg_vcs_backends_svn_new (), (backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL)), _tmp1);
			if (vtg_vcs_backends_ivcs_test (backend, filename)) {
				self->vcs_type = VTG_PROJECT_MANAGER_VCS_TYPES_SVN;
			}
		}
	}
	(backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL));
}


void vtg_project_manager_project_close (VtgProjectManagerProject* self) {
	GtkTreeStore* _tmp0;
	GbfProject* _tmp1;
	g_return_if_fail (self != NULL);
	gee_collection_clear ((GeeCollection*) self->modules);
	gee_collection_clear ((GeeCollection*) self->groups);
	gee_collection_clear ((GeeCollection*) self->exec_targets);
	gee_collection_clear ((GeeCollection*) self->all_vala_sources);
	_tmp0 = NULL;
	self->priv->_model = (_tmp0 = NULL, (self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL)), _tmp0);
	self->priv->_backend = NULL;
	_tmp1 = NULL;
	self->priv->_gbf_project = (_tmp1 = NULL, (self->priv->_gbf_project == NULL) ? NULL : (self->priv->_gbf_project = (g_object_unref (self->priv->_gbf_project), NULL)), _tmp1);
}


static void vtg_project_manager_project_on_project_updated (VtgProjectManagerProject* self, GbfProject* sender) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	inner_error = NULL;
	if (self->priv->in_update) {
		return;
	}
	self->priv->in_update = TRUE;
	{
		gbf_project_refresh (self->priv->_gbf_project, &inner_error);
		if (inner_error != NULL) {
			goto __catch21_g_error;
			goto __finally21;
		}
	}
	goto __finally21;
	__catch21_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgprojectmanagerproject.vala:236: on_project_upadted: error: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally21:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	vtg_project_manager_project_parse_project (self);
	vtg_project_manager_project_build_tree_model (self);
	g_signal_emit_by_name (self, "updated");
	self->priv->in_update = FALSE;
}


static gint _vtg_project_manager_project_sort_model_gtk_tree_iter_compare_func (GtkTreeModel* model, const GtkTreeIter* a, const GtkTreeIter* b, gpointer self) {
	return vtg_project_manager_project_sort_model (self, model, a, b);
}


static void vtg_project_manager_project_build_tree_model (VtgProjectManagerProject* self) {
	GtkTreeIter project_iter = {0};
	GtkTreeIter modules_iter = {0};
	GtkTreeIter groups_iter = {0};
	GtkTreeStore* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	self->priv->_model = (_tmp0 = gtk_tree_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_OBJECT, G_TYPE_STRING, NULL), (self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL)), _tmp0);
	gtk_tree_store_append (self->priv->_model, &project_iter, NULL);
	gtk_tree_store_set (self->priv->_model, &project_iter, 0, GTK_STOCK_DIRECTORY, 1, self->name, 2, "project-root", 4, "", -1);
	gtk_tree_store_append (self->priv->_model, &modules_iter, &project_iter);
	gtk_tree_store_set (self->priv->_model, &modules_iter, 0, GTK_STOCK_DIRECTORY, 1, _ ("References"), 2, "project-reference", 4, "1", -1);
	{
		GeeIterator* _module_it;
		_module_it = gee_iterable_iterator ((GeeIterable*) self->modules);
		while (gee_iterator_next (_module_it)) {
			VtgProjectManagerProjectModule* module;
			GtkTreeIter module_iter = {0};
			module = (VtgProjectManagerProjectModule*) gee_iterator_get (_module_it);
			gtk_tree_store_append (self->priv->_model, &module_iter, &modules_iter);
			gtk_tree_store_set (self->priv->_model, &module_iter, 0, GTK_STOCK_DIRECTORY, 1, module->name, 2, module->id, 3, module, 4, module->name, -1);
			{
				GeeIterator* _package_it;
				_package_it = gee_iterable_iterator ((GeeIterable*) module->packages);
				while (gee_iterator_next (_package_it)) {
					VtgProjectManagerProjectPackage* package;
					GtkTreeIter package_iter = {0};
					package = (VtgProjectManagerProjectPackage*) gee_iterator_get (_package_it);
					gtk_tree_store_append (self->priv->_model, &package_iter, &module_iter);
					gtk_tree_store_set (self->priv->_model, &package_iter, 0, GTK_STOCK_FILE, 1, package->name, 2, package->id, 3, package, 4, package->name, -1);
					(package == NULL) ? NULL : (package = (g_object_unref (package), NULL));
				}
				(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
			}
			(module == NULL) ? NULL : (module = (g_object_unref (module), NULL));
		}
		(_module_it == NULL) ? NULL : (_module_it = (gee_collection_object_unref (_module_it), NULL));
	}
	gtk_tree_store_append (self->priv->_model, &groups_iter, &project_iter);
	gtk_tree_store_set (self->priv->_model, &groups_iter, 0, GTK_STOCK_DIRECTORY, 1, _ ("Files"), 2, "project-files", 4, "2", -1);
	{
		GeeIterator* _group_it;
		_group_it = gee_iterable_iterator ((GeeIterable*) self->groups);
		while (gee_iterator_next (_group_it)) {
			VtgProjectManagerProjectGroup* group;
			group = (VtgProjectManagerProjectGroup*) gee_iterator_get (_group_it);
			{
				GeeIterator* _target_it;
				_target_it = gee_iterable_iterator ((GeeIterable*) group->targets);
				while (gee_iterator_next (_target_it)) {
					VtgProjectManagerProjectTarget* target;
					gboolean _tmp1;
					target = (VtgProjectManagerProjectTarget*) gee_iterator_get (_target_it);
					_tmp1 = FALSE;
					if (vtg_project_manager_project_target_get_simple (target)) {
						_tmp1 = TRUE;
					} else {
						_tmp1 = target->vala_sources;
					}
					if (_tmp1) {
						GtkTreeIter target_iter;
						gboolean target_added;
						target_iter = groups_iter;
						target_added = FALSE;
						{
							GeeIterator* _source_it;
							_source_it = gee_iterable_iterator ((GeeIterable*) target->sources);
							while (gee_iterator_next (_source_it)) {
								VtgProjectManagerProjectSource* source;
								gboolean _tmp2;
								gboolean _tmp3;
								gboolean _tmp4;
								GtkTreeIter source_iter = {0};
								source = (VtgProjectManagerProjectSource*) gee_iterator_get (_source_it);
								_tmp2 = FALSE;
								_tmp3 = FALSE;
								_tmp4 = FALSE;
								if (g_str_has_prefix (source->name, ".")) {
									_tmp4 = TRUE;
								} else {
									_tmp4 = g_str_has_suffix (source->name, ".c");
								}
								if (_tmp4) {
									_tmp3 = TRUE;
								} else {
									_tmp3 = g_str_has_suffix (source->name, ".h");
								}
								if (_tmp3) {
									_tmp2 = TRUE;
								} else {
									_tmp2 = g_str_has_suffix (source->name, ".stamp");
								}
								if (_tmp2) {
									(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
									continue;
								}
								if (!target_added) {
									gtk_tree_store_append (self->priv->_model, &target_iter, &groups_iter);
									gtk_tree_store_set (self->priv->_model, &target_iter, 0, GTK_STOCK_DIRECTORY, 1, group->name, 2, target->id, 3, target, 4, group->name, -1);
									target_added = TRUE;
								}
								gtk_tree_store_append (self->priv->_model, &source_iter, &target_iter);
								gtk_tree_store_set (self->priv->_model, &source_iter, 0, GTK_STOCK_FILE, 1, source->name, 2, source->uri, 3, source, 4, source->name, -1);
								(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
							}
							(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
						}
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) self->priv->_model, 4, GTK_SORT_ASCENDING);
	gtk_tree_sortable_set_sort_func ((GtkTreeSortable*) self->priv->_model, 4, _vtg_project_manager_project_sort_model_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
}


static void vtg_project_manager_project_parse_project (VtgProjectManagerProject* self) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	{
		char* _tmp0;
		GList* _tmp1;
		GList* _tmp6;
		gee_collection_clear ((GeeCollection*) self->modules);
		gee_collection_clear ((GeeCollection*) self->groups);
		gee_collection_clear ((GeeCollection*) self->exec_targets);
		gee_collection_clear ((GeeCollection*) self->all_vala_sources);
		_tmp0 = NULL;
		self->changelog_uri = (_tmp0 = NULL, self->changelog_uri = (g_free (self->changelog_uri), NULL), _tmp0);
		_tmp1 = gbf_project_get_config_modules (self->priv->_gbf_project, &inner_error);
		if (inner_error != NULL) {
			goto __catch22_g_error;
			goto __finally22;
		}
		{
			GList* mod_id_collection;
			GList* mod_id_it;
			mod_id_collection = _tmp1;
			for (mod_id_it = mod_id_collection; mod_id_it != NULL; mod_id_it = mod_id_it->next) {
				const char* _tmp5;
				char* mod_id;
				_tmp5 = NULL;
				mod_id = (_tmp5 = (const char*) mod_id_it->data, (_tmp5 == NULL) ? NULL : g_strdup (_tmp5));
				{
					VtgProjectManagerProjectModule* module;
					GList* _tmp2;
					module = vtg_project_manager_project_module_new (self, mod_id);
					gee_collection_add ((GeeCollection*) self->modules, module);
					_tmp2 = gbf_project_get_config_packages (self->priv->_gbf_project, mod_id, &inner_error);
					if (inner_error != NULL) {
						mod_id = (g_free (mod_id), NULL);
						(module == NULL) ? NULL : (module = (g_object_unref (module), NULL));
						goto __catch22_g_error;
						goto __finally22;
					}
					{
						GList* pkg_id_collection;
						GList* pkg_id_it;
						pkg_id_collection = _tmp2;
						for (pkg_id_it = pkg_id_collection; pkg_id_it != NULL; pkg_id_it = pkg_id_it->next) {
							const char* _tmp4;
							char* pkg_id;
							_tmp4 = NULL;
							pkg_id = (_tmp4 = (const char*) pkg_id_it->data, (_tmp4 == NULL) ? NULL : g_strdup (_tmp4));
							{
								VtgProjectManagerProjectPackage* _tmp3;
								_tmp3 = NULL;
								gee_collection_add ((GeeCollection*) module->packages, _tmp3 = vtg_project_manager_project_package_new (pkg_id));
								(_tmp3 == NULL) ? NULL : (_tmp3 = (g_object_unref (_tmp3), NULL));
								pkg_id = (g_free (pkg_id), NULL);
							}
						}
					}
					mod_id = (g_free (mod_id), NULL);
					(module == NULL) ? NULL : (module = (g_object_unref (module), NULL));
				}
			}
		}
		_tmp6 = gbf_project_get_all_groups (self->priv->_gbf_project, &inner_error);
		if (inner_error != NULL) {
			goto __catch22_g_error;
			goto __finally22;
		}
		{
			GList* grp_id_collection;
			GList* grp_id_it;
			grp_id_collection = _tmp6;
			for (grp_id_it = grp_id_collection; grp_id_it != NULL; grp_id_it = grp_id_it->next) {
				const char* _tmp24;
				char* grp_id;
				_tmp24 = NULL;
				grp_id = (_tmp24 = (const char*) grp_id_it->data, (_tmp24 == NULL) ? NULL : g_strdup (_tmp24));
				{
					char* grp_name;
					VtgProjectManagerProjectGroup* group;
					grp_name = vtg_path_utils_normalize_path (grp_id);
					group = vtg_project_manager_project_find_group (self, grp_name);
					if (group == NULL) {
						VtgProjectManagerProjectGroup* _tmp7;
						GbfProjectGroup* _tmp8;
						_tmp7 = NULL;
						group = (_tmp7 = vtg_project_manager_project_group_new (grp_name, self), (group == NULL) ? NULL : (group = (g_object_unref (group), NULL)), _tmp7);
						gee_collection_add ((GeeCollection*) self->groups, group);
						_tmp8 = gbf_project_get_group (self->priv->_gbf_project, grp_id, &inner_error);
						if (inner_error != NULL) {
							grp_id = (g_free (grp_id), NULL);
							grp_name = (g_free (grp_name), NULL);
							(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
							goto __catch22_g_error;
							goto __finally22;
						}
						{
							GList* tgt_id_collection;
							GList* tgt_id_it;
							tgt_id_collection = _tmp8->targets;
							for (tgt_id_it = tgt_id_collection; tgt_id_it != NULL; tgt_id_it = tgt_id_it->next) {
								const char* _tmp23;
								char* tgt_id;
								_tmp23 = NULL;
								tgt_id = (_tmp23 = (const char*) tgt_id_it->data, (_tmp23 == NULL) ? NULL : g_strdup (_tmp23));
								{
									char* tgt_name;
									char** _tmp10;
									gint tgt_parts_size;
									gint tgt_parts_length1;
									char** _tmp9;
									char** tgt_parts;
									char* _tmp12;
									const char* _tmp11;
									VtgProjectManagerProjectTarget* target;
									tgt_name = string_substring (tgt_id, string_get_length (grp_id), string_get_length (tgt_id) - string_get_length (grp_id));
									_tmp10 = NULL;
									_tmp9 = NULL;
									tgt_parts = (_tmp10 = _tmp9 = g_strsplit (tgt_name, ":", 0), tgt_parts_length1 = _vala_array_length (_tmp9), tgt_parts_size = tgt_parts_length1, _tmp10);
									_tmp12 = NULL;
									_tmp11 = NULL;
									tgt_name = (_tmp12 = (_tmp11 = tgt_parts[0], (_tmp11 == NULL) ? NULL : g_strdup (_tmp11)), tgt_name = (g_free (tgt_name), NULL), _tmp12);
									target = vtg_project_manager_project_group_find_target (group, tgt_name);
									if (target == NULL) {
										GbfProjectTarget* tgt;
										VtgProjectManagerProjectTarget* _tmp13;
										tgt = gbf_project_get_target (self->priv->_gbf_project, tgt_id, &inner_error);
										if (inner_error != NULL) {
											tgt_id = (g_free (tgt_id), NULL);
											tgt_name = (g_free (tgt_name), NULL);
											tgt_parts = (_vala_array_free (tgt_parts, tgt_parts_length1, (GDestroyNotify) g_free), NULL);
											(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
											grp_id = (g_free (grp_id), NULL);
											grp_name = (g_free (grp_name), NULL);
											(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
											goto __catch22_g_error;
											goto __finally22;
										}
										_tmp13 = NULL;
										target = (_tmp13 = vtg_project_manager_project_target_new (tgt_id, group), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), _tmp13);
										vtg_project_manager_project_target_set_type_from_string (target, tgt->type);
										if (target->type == VTG_PROJECT_MANAGER_TARGET_TYPES_EXECUTABLE) {
											char** _tmp15;
											gint tmp_size;
											gint tmp_length1;
											char** _tmp14;
											char** tmp;
											_tmp15 = NULL;
											_tmp14 = NULL;
											tmp = (_tmp15 = _tmp14 = g_strsplit (tgt_id, ":", 0), tmp_length1 = _vala_array_length (_tmp14), tmp_size = tmp_length1, _tmp15);
											gee_collection_add ((GeeCollection*) self->exec_targets, tmp[0]);
											tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL);
										}
										gee_collection_add ((GeeCollection*) group->targets, target);
										{
											GList* src_id_collection;
											GList* src_id_it;
											src_id_collection = tgt->sources;
											for (src_id_it = src_id_collection; src_id_it != NULL; src_id_it = src_id_it->next) {
												const char* _tmp22;
												char* src_id;
												_tmp22 = NULL;
												src_id = (_tmp22 = (const char*) src_id_it->data, (_tmp22 == NULL) ? NULL : g_strdup (_tmp22));
												{
													char* src_uri;
													VtgProjectManagerProjectSource* src;
													src_uri = string_substring (src_id, string_get_length (tgt_id) + 1, (string_get_length (src_id) - string_get_length (tgt_id)) - 1);
													src = vtg_project_manager_project_target_find_source (target, src_uri);
													if (src == NULL) {
														VtgProjectManagerProjectSource* _tmp16;
														gboolean _tmp17;
														char* _tmp18;
														gboolean _tmp19;
														_tmp16 = NULL;
														src = (_tmp16 = vtg_project_manager_project_source_new (src_uri), (src == NULL) ? NULL : (src = (g_object_unref (src), NULL)), _tmp16);
														_tmp17 = FALSE;
														if (src->is_vala_source) {
															_tmp17 = !vtg_project_manager_project_contains_vala_source_file (self, src_uri);
														} else {
															_tmp17 = FALSE;
														}
														if (_tmp17) {
															gee_collection_add ((GeeCollection*) self->all_vala_sources, src);
														}
														vtg_project_manager_project_target_add_source (target, src);
														_tmp18 = NULL;
														if ((_tmp19 = _vala_strcmp0 (_tmp18 = g_utf8_strdown (src->name, -1), "changelog") == 0, _tmp18 = (g_free (_tmp18), NULL), _tmp19)) {
															char* _tmp21;
															const char* _tmp20;
															_tmp21 = NULL;
															_tmp20 = NULL;
															self->changelog_uri = (_tmp21 = (_tmp20 = src_uri, (_tmp20 == NULL) ? NULL : g_strdup (_tmp20)), self->changelog_uri = (g_free (self->changelog_uri), NULL), _tmp21);
														}
													}
													src_id = (g_free (src_id), NULL);
													src_uri = (g_free (src_uri), NULL);
													(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
												}
											}
										}
									}
									tgt_id = (g_free (tgt_id), NULL);
									tgt_name = (g_free (tgt_name), NULL);
									tgt_parts = (_vala_array_free (tgt_parts, tgt_parts_length1, (GDestroyNotify) g_free), NULL);
									(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
								}
							}
						}
					}
					grp_id = (g_free (grp_id), NULL);
					grp_name = (g_free (grp_name), NULL);
					(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
				}
			}
		}
	}
	goto __finally22;
	__catch22_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgprojectmanagerproject.vala:352: error %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally22:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static gint vtg_project_manager_project_sort_model (VtgProjectManagerProject* self, GtkTreeModel* model, const GtkTreeIter* a, const GtkTreeIter* b) {
	char* vala;
	char* valb;
	gint _tmp0;
	g_return_val_if_fail (self != NULL, 0);
	g_return_val_if_fail (model != NULL, 0);
	vala = NULL;
	valb = NULL;
	gtk_tree_model_get (model, &(*a), 4, &vala, -1);
	gtk_tree_model_get (model, &(*b), 4, &valb, -1);
	return (_tmp0 = vtg_path_utils_compare_vala_filenames (vala, valb), vala = (g_free (vala), NULL), valb = (g_free (valb), NULL), _tmp0);
}


VtgProjectManagerProject* vtg_project_manager_project_construct (GType object_type) {
	VtgProjectManagerProject * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgProjectManagerProject* vtg_project_manager_project_new (void) {
	return vtg_project_manager_project_construct (VTG_PROJECT_MANAGER_TYPE_PROJECT);
}


GtkTreeModel* vtg_project_manager_project_get_model (VtgProjectManagerProject* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GtkTreeModel*) self->priv->_model;
}


GbfProject* vtg_project_manager_project_get_gbf_project (VtgProjectManagerProject* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_gbf_project;
}


static void vtg_project_manager_project_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectManagerProject * self;
	gpointer boxed;
	self = VTG_PROJECT_MANAGER_PROJECT (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_PROJECT_MODEL:
		g_value_set_object (value, vtg_project_manager_project_get_model (self));
		break;
		case VTG_PROJECT_MANAGER_PROJECT_GBF_PROJECT:
		g_value_set_object (value, vtg_project_manager_project_get_gbf_project (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_project_class_init (VtgProjectManagerProjectClass * klass) {
	vtg_project_manager_project_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerProjectPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_manager_project_get_property;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_project_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_PROJECT_MODEL, g_param_spec_object ("model", "model", "model", GTK_TYPE_TREE_MODEL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_PROJECT_GBF_PROJECT, g_param_spec_object ("gbf-project", "gbf-project", "gbf-project", GBF_TYPE_PROJECT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("updated", VTG_PROJECT_MANAGER_TYPE_PROJECT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void vtg_project_manager_project_instance_init (VtgProjectManagerProject * self) {
	self->priv = VTG_PROJECT_MANAGER_PROJECT_GET_PRIVATE (self);
	self->priv->_backend = NULL;
	self->priv->_gbf_project = NULL;
	self->priv->in_update = FALSE;
	self->name = NULL;
	self->filename = NULL;
	self->exec_targets = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
	self->modules = (GeeList*) gee_array_list_new (VTG_PROJECT_MANAGER_TYPE_PROJECT_MODULE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->groups = (GeeList*) gee_array_list_new (VTG_PROJECT_MANAGER_TYPE_PROJECT_GROUP, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->all_vala_sources = (GeeList*) gee_array_list_new (VTG_PROJECT_MANAGER_TYPE_PROJECT_SOURCE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->vcs_type = VTG_PROJECT_MANAGER_VCS_TYPES_NONE;
	self->changelog_uri = NULL;
}


static void vtg_project_manager_project_finalize (GObject* obj) {
	VtgProjectManagerProject * self;
	self = VTG_PROJECT_MANAGER_PROJECT (obj);
	(self->priv->_gbf_project == NULL) ? NULL : (self->priv->_gbf_project = (g_object_unref (self->priv->_gbf_project), NULL));
	(self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL));
	self->name = (g_free (self->name), NULL);
	self->filename = (g_free (self->filename), NULL);
	(self->exec_targets == NULL) ? NULL : (self->exec_targets = (gee_collection_object_unref (self->exec_targets), NULL));
	(self->modules == NULL) ? NULL : (self->modules = (gee_collection_object_unref (self->modules), NULL));
	(self->groups == NULL) ? NULL : (self->groups = (gee_collection_object_unref (self->groups), NULL));
	(self->all_vala_sources == NULL) ? NULL : (self->all_vala_sources = (gee_collection_object_unref (self->all_vala_sources), NULL));
	self->changelog_uri = (g_free (self->changelog_uri), NULL);
	G_OBJECT_CLASS (vtg_project_manager_project_parent_class)->finalize (obj);
}


static GType vtg_project_manager_project_type_id = 0;
GType vtg_project_manager_project_get_type (void) {
	return vtg_project_manager_project_type_id;
}


GType vtg_project_manager_project_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerProjectClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_project_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerProject), 0, (GInstanceInitFunc) vtg_project_manager_project_instance_init, NULL };
	vtg_project_manager_project_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerProject", &g_define_type_info, 0);
	return vtg_project_manager_project_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




