/*
 *  vtgprojectmanagerproject.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#ifndef __VTGPROJECTMANAGERPROJECT_H__
#define __VTGPROJECTMANAGERPROJECT_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee/list.h>
#include <gtk/gtk.h>
#include <gbf/gbf-project.h>
#include "vtgprojectmanagerprojectsource.h"

G_BEGIN_DECLS

typedef struct _VtgProjectManagerProject VtgProjectManagerProject;
typedef struct _VtgProjectManagerProjectClass VtgProjectManagerProjectClass;
typedef struct _VtgProjectManagerProjectGroup VtgProjectManagerProjectGroup;
typedef struct _VtgProjectManagerProjectGroupClass VtgProjectManagerProjectGroupClass;
typedef struct _VtgProjectManagerProjectTarget VtgProjectManagerProjectTarget;
typedef struct _VtgProjectManagerProjectTargetClass VtgProjectManagerProjectTargetClass;

#define VTG_PROJECT_MANAGER_TYPE_VCS_TYPES (vtg_project_manager_vcs_types_get_type ())

#define VTG_PROJECT_MANAGER_TYPE_PROJECT (vtg_project_manager_project_get_type ())
#define VTG_PROJECT_MANAGER_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_PROJECT_MANAGER_TYPE_PROJECT, VtgProjectManagerProject))
#define VTG_PROJECT_MANAGER_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_PROJECT_MANAGER_TYPE_PROJECT, VtgProjectManagerProjectClass))
#define VTG_PROJECT_MANAGER_IS_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_PROJECT_MANAGER_TYPE_PROJECT))
#define VTG_PROJECT_MANAGER_IS_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_PROJECT_MANAGER_TYPE_PROJECT))
#define VTG_PROJECT_MANAGER_PROJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_PROJECT_MANAGER_TYPE_PROJECT, VtgProjectManagerProjectClass))

typedef struct _VtgProjectManagerProjectPrivate VtgProjectManagerProjectPrivate;

typedef enum  {
	VTG_PROJECT_MANAGER_PROJECT_MANAGER_ERROR_NO_BACKEND
} VtgProjectManagerProjectManagerError;
#define VTG_PROJECT_MANAGER_PROJECT_MANAGER_ERROR vtg_project_manager_project_manager_error_quark ()
typedef enum  {
	VTG_PROJECT_MANAGER_VCS_TYPES_NONE,
	VTG_PROJECT_MANAGER_VCS_TYPES_GIT,
	VTG_PROJECT_MANAGER_VCS_TYPES_BZR,
	VTG_PROJECT_MANAGER_VCS_TYPES_SVN
} VtgProjectManagerVcsTypes;

struct _VtgProjectManagerProject {
	GObject parent_instance;
	VtgProjectManagerProjectPrivate * priv;
	char* name;
	char* filename;
	GeeList* exec_targets;
	GeeList* modules;
	GeeList* groups;
	GeeList* all_vala_sources;
	VtgProjectManagerVcsTypes vcs_type;
	char* changelog_uri;
};

struct _VtgProjectManagerProjectClass {
	GObjectClass parent_class;
};


GQuark vtg_project_manager_project_manager_error_quark (void);
GType vtg_project_manager_vcs_types_get_type (void);
VtgProjectManagerProjectGroup* vtg_project_manager_project_find_group (VtgProjectManagerProject* self, const char* id);
gboolean vtg_project_manager_project_contains_source_file (VtgProjectManagerProject* self, const char* uri);
VtgProjectManagerProjectSource* vtg_project_manager_project_get_source_file_from_uri (VtgProjectManagerProject* self, const char* uri);
gboolean vtg_project_manager_project_contains_vala_source_file (VtgProjectManagerProject* self, const char* uri);
char* vtg_project_manager_project_source_uri_for_name (VtgProjectManagerProject* self, const char* name);
gboolean vtg_project_manager_project_open (VtgProjectManagerProject* self, const char* project_filename, GError** error);
void vtg_project_manager_project_close (VtgProjectManagerProject* self);
VtgProjectManagerProject* vtg_project_manager_project_construct (GType object_type);
VtgProjectManagerProject* vtg_project_manager_project_new (void);
GtkTreeModel* vtg_project_manager_project_get_model (VtgProjectManagerProject* self);
GbfProject* vtg_project_manager_project_get_gbf_project (VtgProjectManagerProject* self);
GType vtg_project_manager_project_get_type (void);
GType vtg_project_manager_project_register_type (GTypeModule * module);


G_END_DECLS

#endif
