/*
 *  vtgprojectmanagerprojectgroup.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerprojectgroup.h"
#include <gee/arraylist.h>
#include <glib/gi18n-lib.h>
#include <glib/gstdio.h>
#include <gee/collection.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include "vtgprojectmanagerprojecttarget.h"




static glong string_get_length (const char* self);
static char* string_substring (const char* self, glong offset, glong len);
enum  {
	VTG_PROJECT_MANAGER_PROJECT_GROUP_DUMMY_PROPERTY
};
static void vtg_project_manager_project_group_initialize_vapis (VtgProjectManagerProjectGroup* self, const char* project_path);
static gpointer vtg_project_manager_project_group_parent_class = NULL;
static void vtg_project_manager_project_group_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const char* VTG_PROJECT_MANAGER_PROJECT_GROUP_files_to_scan[] = {"Makefile.am", "Makefile"};


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


VtgProjectManagerProjectGroup* vtg_project_manager_project_group_construct (GType object_type, const char* id, VtgProjectManagerProject* project) {
	VtgProjectManagerProjectGroup * self;
	VtgProjectManagerProject* _tmp1;
	VtgProjectManagerProject* _tmp0;
	gboolean _tmp2;
	g_return_val_if_fail (id != NULL, NULL);
	g_return_val_if_fail (project != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->project = (_tmp1 = (_tmp0 = project, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->project == NULL) ? NULL : (self->project = (g_object_unref (self->project), NULL)), _tmp1);
	_tmp2 = FALSE;
	if (id == NULL) {
		_tmp2 = TRUE;
	} else {
		_tmp2 = string_get_length (id) == 0;
	}
	if (_tmp2) {
		char* _tmp4;
		const char* _tmp3;
		char* _tmp6;
		const char* _tmp5;
		_tmp4 = NULL;
		_tmp3 = NULL;
		self->id = (_tmp4 = (_tmp3 = _ ("other"), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3)), self->id = (g_free (self->id), NULL), _tmp4);
		_tmp6 = NULL;
		_tmp5 = NULL;
		self->name = (_tmp6 = (_tmp5 = self->id, (_tmp5 == NULL) ? NULL : g_strdup (_tmp5)), self->name = (g_free (self->name), NULL), _tmp6);
	} else {
		char* _tmp8;
		const char* _tmp7;
		_tmp8 = NULL;
		_tmp7 = NULL;
		self->id = (_tmp8 = (_tmp7 = id, (_tmp7 == NULL) ? NULL : g_strdup (_tmp7)), self->id = (g_free (self->id), NULL), _tmp8);
		if (string_get_length (id) > 2) {
			char* _tmp9;
			_tmp9 = NULL;
			self->name = (_tmp9 = string_substring (id, (glong) 1, string_get_length (id) - 2), self->name = (g_free (self->name), NULL), _tmp9);
		} else {
			char* _tmp11;
			const char* _tmp10;
			_tmp11 = NULL;
			_tmp10 = NULL;
			self->name = (_tmp11 = (_tmp10 = id, (_tmp10 == NULL) ? NULL : g_strdup (_tmp10)), self->name = (g_free (self->name), NULL), _tmp11);
		}
	}
	vtg_project_manager_project_group_initialize_vapis (self, project->filename);
	return self;
}


VtgProjectManagerProjectGroup* vtg_project_manager_project_group_new (const char* id, VtgProjectManagerProject* project) {
	return vtg_project_manager_project_group_construct (VTG_PROJECT_MANAGER_TYPE_PROJECT_GROUP, id, project);
}


VtgProjectManagerProjectTarget* vtg_project_manager_project_group_find_target (VtgProjectManagerProjectGroup* self, const char* id) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	{
		GeeIterator* _target_it;
		_target_it = gee_iterable_iterator ((GeeIterable*) self->targets);
		while (gee_iterator_next (_target_it)) {
			VtgProjectManagerProjectTarget* target;
			target = (VtgProjectManagerProjectTarget*) gee_iterator_get (_target_it);
			if (_vala_strcmp0 (target->id, id) == 0) {
				VtgProjectManagerProjectTarget* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = target, (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), _tmp0);
			}
			(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
		}
		(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
	}
	return NULL;
}


static void vtg_project_manager_project_group_initialize_vapis (VtgProjectManagerProjectGroup* self, const char* project_path) {
	GError * inner_error;
	char* buffer;
	g_return_if_fail (self != NULL);
	g_return_if_fail (project_path != NULL);
	inner_error = NULL;
	buffer = NULL;
	{
		const char** file_collection;
		int file_collection_length1;
		int file_it;
		file_collection = VTG_PROJECT_MANAGER_PROJECT_GROUP_files_to_scan;
		file_collection_length1 = G_N_ELEMENTS (VTG_PROJECT_MANAGER_PROJECT_GROUP_files_to_scan);
		for (file_it = 0; file_it < G_N_ELEMENTS (VTG_PROJECT_MANAGER_PROJECT_GROUP_files_to_scan); file_it = file_it + 1) {
			const char* _tmp16;
			char* file;
			_tmp16 = NULL;
			file = (_tmp16 = file_collection[file_it], (_tmp16 == NULL) ? NULL : g_strdup (_tmp16));
			{
				char* filename;
				filename = g_build_filename (project_path, self->id, file, NULL);
				{
					char* _tmp2;
					gboolean _tmp1;
					char* _tmp0;
					gboolean _tmp3;
					_tmp2 = NULL;
					_tmp0 = NULL;
					_tmp3 = (_tmp1 = g_file_get_contents (filename, &_tmp0, NULL, &inner_error), buffer = (_tmp2 = _tmp0, buffer = (g_free (buffer), NULL), _tmp2), _tmp1);
					if (inner_error != NULL) {
						if (inner_error->domain == G_FILE_ERROR) {
							goto __catch23_g_file_error;
						}
						goto __finally23;
					}
					if (_tmp3) {
						char** _tmp5;
						gint lines_size;
						gint lines_length1;
						char** _tmp4;
						char** lines;
						char* _tmp6;
						_tmp5 = NULL;
						_tmp4 = NULL;
						lines = (_tmp5 = _tmp4 = g_strsplit (buffer, "\n", 0), lines_length1 = _vala_array_length (_tmp4), lines_size = lines_length1, _tmp5);
						_tmp6 = NULL;
						buffer = (_tmp6 = NULL, buffer = (g_free (buffer), NULL), _tmp6);
						/*this frees some memory*/
						{
							char** line_collection;
							int line_collection_length1;
							int line_it;
							line_collection = lines;
							line_collection_length1 = lines_length1;
							for (line_it = 0; line_it < lines_length1; line_it = line_it + 1) {
								const char* _tmp15;
								char* line;
								_tmp15 = NULL;
								line = (_tmp15 = line_collection[line_it], (_tmp15 == NULL) ? NULL : g_strdup (_tmp15));
								{
									char** _tmp8;
									gint tmps_size;
									gint tmps_length1;
									char** _tmp7;
									char** tmps;
									gint count;
									_tmp8 = NULL;
									_tmp7 = NULL;
									tmps = (_tmp8 = _tmp7 = g_strsplit (line, " ", 0), tmps_length1 = _vala_array_length (_tmp7), tmps_size = tmps_length1, _tmp8);
									count = 0;
									while (tmps[count] != NULL) {
										count++;
									}
									{
										gint idx;
										idx = 0;
										for (; idx < count; idx++) {
											gboolean _tmp9;
											_tmp9 = FALSE;
											if (_vala_strcmp0 (tmps[idx], "--vapidir") == 0) {
												_tmp9 = (idx + 1) < count;
											} else {
												_tmp9 = FALSE;
											}
											if (_tmp9) {
												const char* _tmp10;
												char* tmp;
												_tmp10 = NULL;
												tmp = (_tmp10 = tmps[idx + 1], (_tmp10 == NULL) ? NULL : g_strdup (_tmp10));
												if (g_str_has_prefix (tmp, ".")) {
													char* _tmp11;
													_tmp11 = NULL;
													tmp = (_tmp11 = g_build_filename (project_path, self->id, tmp, NULL), tmp = (g_free (tmp), NULL), _tmp11);
												}
												gee_collection_add ((GeeCollection*) self->vapidirs, tmp);
												idx++;
												tmp = (g_free (tmp), NULL);
											} else {
												gboolean _tmp12;
												_tmp12 = FALSE;
												if (_vala_strcmp0 (tmps[idx], "--pkg") == 0) {
													_tmp12 = (idx + 1) < count;
												} else {
													_tmp12 = FALSE;
												}
												if (_tmp12) {
													const char* _tmp13;
													char* tmp;
													_tmp13 = NULL;
													tmp = (_tmp13 = tmps[idx + 1], (_tmp13 == NULL) ? NULL : g_strdup (_tmp13));
													gee_collection_add ((GeeCollection*) self->packages, tmp);
													idx++;
													tmp = (g_free (tmp), NULL);
												} else {
													if (_vala_strcmp0 (tmps[idx], "--library") == 0) {
														const char* _tmp14;
														char* tmp;
														_tmp14 = NULL;
														tmp = (_tmp14 = tmps[idx + 1], (_tmp14 == NULL) ? NULL : g_strdup (_tmp14));
														gee_collection_add ((GeeCollection*) self->built_libraries, tmp);
														idx++;
														tmp = (g_free (tmp), NULL);
													}
												}
											}
										}
									}
									line = (g_free (line), NULL);
									tmps = (_vala_array_free (tmps, tmps_length1, (GDestroyNotify) g_free), NULL);
								}
							}
						}
						lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
						file = (g_free (file), NULL);
						filename = (g_free (filename), NULL);
						buffer = (g_free (buffer), NULL);
						return;
					}
				}
				goto __finally23;
				__catch23_g_file_error:
				{
					GError * err;
					err = inner_error;
					inner_error = NULL;
					{
						g_warning ("vtgprojectmanagerprojectgroup.vala:107: Error reading file %s: %s", self->id, err->message);
						(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
					}
				}
				__finally23:
				if (inner_error != NULL) {
					file = (g_free (file), NULL);
					filename = (g_free (filename), NULL);
					buffer = (g_free (buffer), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
					g_clear_error (&inner_error);
					return;
				}
				file = (g_free (file), NULL);
				filename = (g_free (filename), NULL);
			}
		}
	}
	buffer = (g_free (buffer), NULL);
}


static void vtg_project_manager_project_group_class_init (VtgProjectManagerProjectGroupClass * klass) {
	vtg_project_manager_project_group_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_project_group_finalize;
}


static void vtg_project_manager_project_group_instance_init (VtgProjectManagerProjectGroup * self) {
	self->targets = (GeeList*) gee_array_list_new (VTG_PROJECT_MANAGER_TYPE_PROJECT_TARGET, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->vapidirs = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
	self->packages = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
	self->built_libraries = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
}


static void vtg_project_manager_project_group_finalize (GObject* obj) {
	VtgProjectManagerProjectGroup * self;
	self = VTG_PROJECT_MANAGER_PROJECT_GROUP (obj);
	self->name = (g_free (self->name), NULL);
	self->id = (g_free (self->id), NULL);
	(self->targets == NULL) ? NULL : (self->targets = (gee_collection_object_unref (self->targets), NULL));
	(self->vapidirs == NULL) ? NULL : (self->vapidirs = (gee_collection_object_unref (self->vapidirs), NULL));
	(self->packages == NULL) ? NULL : (self->packages = (gee_collection_object_unref (self->packages), NULL));
	(self->built_libraries == NULL) ? NULL : (self->built_libraries = (gee_collection_object_unref (self->built_libraries), NULL));
	(self->project == NULL) ? NULL : (self->project = (g_object_unref (self->project), NULL));
	G_OBJECT_CLASS (vtg_project_manager_project_group_parent_class)->finalize (obj);
}


static GType vtg_project_manager_project_group_type_id = 0;
GType vtg_project_manager_project_group_get_type (void) {
	return vtg_project_manager_project_group_type_id;
}


GType vtg_project_manager_project_group_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerProjectGroupClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_project_group_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerProjectGroup), 0, (GInstanceInitFunc) vtg_project_manager_project_group_instance_init, NULL };
	vtg_project_manager_project_group_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerProjectGroup", &g_define_type_info, 0);
	return vtg_project_manager_project_group_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




