/*
 *  vtgprojectmanagerprojecttarget.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerprojecttarget.h"
#include <gee/arraylist.h>
#include <gee/collection.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include "vtgprojectmanagerprojectgroup.h"




struct _VtgProjectManagerProjectTargetPrivate {
	gboolean _simple;
};

#define VTG_PROJECT_MANAGER_PROJECT_TARGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_PROJECT_TARGET, VtgProjectManagerProjectTargetPrivate))
enum  {
	VTG_PROJECT_MANAGER_PROJECT_TARGET_DUMMY_PROPERTY,
	VTG_PROJECT_MANAGER_PROJECT_TARGET_SIMPLE
};
static gpointer vtg_project_manager_project_target_parent_class = NULL;
static void vtg_project_manager_project_target_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);




GType vtg_project_manager_target_types_get_type (void) {
	static GType vtg_project_manager_target_types_type_id = 0;
	if (G_UNLIKELY (vtg_project_manager_target_types_type_id == 0)) {
		static const GEnumValue values[] = {{VTG_PROJECT_MANAGER_TARGET_TYPES_UNKNOWN, "VTG_PROJECT_MANAGER_TARGET_TYPES_UNKNOWN", "unknown"}, {VTG_PROJECT_MANAGER_TARGET_TYPES_SHARED_LIBRARY, "VTG_PROJECT_MANAGER_TARGET_TYPES_SHARED_LIBRARY", "shared-library"}, {VTG_PROJECT_MANAGER_TARGET_TYPES_STATIC_LIBRARY, "VTG_PROJECT_MANAGER_TARGET_TYPES_STATIC_LIBRARY", "static-library"}, {VTG_PROJECT_MANAGER_TARGET_TYPES_EXECUTABLE, "VTG_PROJECT_MANAGER_TARGET_TYPES_EXECUTABLE", "executable"}, {0, NULL, NULL}};
		vtg_project_manager_target_types_type_id = g_enum_register_static ("VtgProjectManagerTargetTypes", values);
	}
	return vtg_project_manager_target_types_type_id;
}


VtgProjectManagerProjectTarget* vtg_project_manager_project_target_construct (GType object_type, const char* id, VtgProjectManagerProjectGroup* group) {
	VtgProjectManagerProjectTarget * self;
	char* _tmp1;
	const char* _tmp0;
	char* _tmp3;
	const char* _tmp2;
	VtgProjectManagerProjectGroup* _tmp5;
	VtgProjectManagerProjectGroup* _tmp4;
	g_return_val_if_fail (id != NULL, NULL);
	g_return_val_if_fail (group != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->id = (_tmp1 = (_tmp0 = id, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->id = (g_free (self->id), NULL), _tmp1);
	_tmp3 = NULL;
	_tmp2 = NULL;
	self->name = (_tmp3 = (_tmp2 = id, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), self->name = (g_free (self->name), NULL), _tmp3);
	_tmp5 = NULL;
	_tmp4 = NULL;
	self->group = (_tmp5 = (_tmp4 = group, (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4)), (self->group == NULL) ? NULL : (self->group = (g_object_unref (self->group), NULL)), _tmp5);
	return self;
}


VtgProjectManagerProjectTarget* vtg_project_manager_project_target_new (const char* id, VtgProjectManagerProjectGroup* group) {
	return vtg_project_manager_project_target_construct (VTG_PROJECT_MANAGER_TYPE_PROJECT_TARGET, id, group);
}


void vtg_project_manager_project_target_add_source (VtgProjectManagerProjectTarget* self, VtgProjectManagerProjectSource* source) {
	gboolean _tmp2;
	gboolean _tmp3;
	g_return_if_fail (self != NULL);
	g_return_if_fail (source != NULL);
	if (gee_collection_get_size ((GeeCollection*) self->sources) == 0) {
		gboolean _tmp0;
		gboolean _tmp1;
		/*defaults*/
		self->vala_sources = TRUE;
		self->generated_sources = TRUE;
		_tmp0 = FALSE;
		_tmp1 = FALSE;
		if (g_str_has_suffix (self->name, "/other:extra")) {
			_tmp1 = TRUE;
		} else {
			_tmp1 = g_str_has_suffix (self->name, ":rule");
		}
		if (_tmp1) {
			_tmp0 = TRUE;
		} else {
			_tmp0 = g_str_has_suffix (self->name, ":intltool_rule");
		}
		if (_tmp0) {
			self->priv->_simple = FALSE;
		} else {
			self->priv->_simple = TRUE;
		}
	}
	_tmp2 = FALSE;
	if (g_str_has_suffix (source->uri, ".vala")) {
		_tmp2 = TRUE;
	} else {
		_tmp2 = g_str_has_suffix (source->uri, ".vapi");
	}
	if (!(_tmp2)) {
		self->vala_sources = FALSE;
	}
	_tmp3 = FALSE;
	if (g_str_has_suffix (source->uri, ".c")) {
		_tmp3 = TRUE;
	} else {
		_tmp3 = g_str_has_suffix (source->uri, ".h");
	}
	if (!(_tmp3)) {
		self->generated_sources = FALSE;
	}
	gee_collection_add ((GeeCollection*) self->sources, source);
}


VtgProjectManagerProjectSource* vtg_project_manager_project_target_find_source (VtgProjectManagerProjectTarget* self, const char* uri) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (uri != NULL, NULL);
	{
		GeeIterator* _source_it;
		_source_it = gee_iterable_iterator ((GeeIterable*) self->sources);
		while (gee_iterator_next (_source_it)) {
			VtgProjectManagerProjectSource* source;
			source = (VtgProjectManagerProjectSource*) gee_iterator_get (_source_it);
			if (_vala_strcmp0 (source->uri, uri) == 0) {
				VtgProjectManagerProjectSource* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = source, (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), _tmp0);
			}
			(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
		}
		(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
	}
	return NULL;
}


void vtg_project_manager_project_target_set_type_from_string (VtgProjectManagerProjectTarget* self, const char* data) {
	char* type;
	g_return_if_fail (self != NULL);
	g_return_if_fail (data != NULL);
	type = g_utf8_strdown (data, -1);
	if (_vala_strcmp0 (type, "shared_lib") == 0) {
		self->type = VTG_PROJECT_MANAGER_TARGET_TYPES_SHARED_LIBRARY;
	} else {
		if (_vala_strcmp0 (type, "static_lib") == 0) {
			self->type = VTG_PROJECT_MANAGER_TARGET_TYPES_STATIC_LIBRARY;
		} else {
			if (_vala_strcmp0 (type, "program") == 0) {
				self->type = VTG_PROJECT_MANAGER_TARGET_TYPES_EXECUTABLE;
			} else {
				self->type = VTG_PROJECT_MANAGER_TARGET_TYPES_UNKNOWN;
			}
		}
	}
	type = (g_free (type), NULL);
}


gboolean vtg_project_manager_project_target_get_simple (VtgProjectManagerProjectTarget* self) {
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0 = FALSE;
	if (self->priv->_simple) {
		_tmp0 = !self->generated_sources;
	} else {
		_tmp0 = FALSE;
	}
	return _tmp0;
}


static void vtg_project_manager_project_target_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectManagerProjectTarget * self;
	gpointer boxed;
	self = VTG_PROJECT_MANAGER_PROJECT_TARGET (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_PROJECT_TARGET_SIMPLE:
		g_value_set_boolean (value, vtg_project_manager_project_target_get_simple (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_project_target_class_init (VtgProjectManagerProjectTargetClass * klass) {
	vtg_project_manager_project_target_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerProjectTargetPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_manager_project_target_get_property;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_project_target_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_PROJECT_TARGET_SIMPLE, g_param_spec_boolean ("simple", "simple", "simple", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_project_manager_project_target_instance_init (VtgProjectManagerProjectTarget * self) {
	self->priv = VTG_PROJECT_MANAGER_PROJECT_TARGET_GET_PRIVATE (self);
	self->sources = (GeeList*) gee_array_list_new (VTG_PROJECT_MANAGER_TYPE_PROJECT_SOURCE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->vala_sources = FALSE;
	self->generated_sources = FALSE;
	self->type = VTG_PROJECT_MANAGER_TARGET_TYPES_UNKNOWN;
	self->group = NULL;
	self->priv->_simple = FALSE;
}


static void vtg_project_manager_project_target_finalize (GObject* obj) {
	VtgProjectManagerProjectTarget * self;
	self = VTG_PROJECT_MANAGER_PROJECT_TARGET (obj);
	self->name = (g_free (self->name), NULL);
	self->id = (g_free (self->id), NULL);
	(self->sources == NULL) ? NULL : (self->sources = (gee_collection_object_unref (self->sources), NULL));
	(self->group == NULL) ? NULL : (self->group = (g_object_unref (self->group), NULL));
	G_OBJECT_CLASS (vtg_project_manager_project_target_parent_class)->finalize (obj);
}


static GType vtg_project_manager_project_target_type_id = 0;
GType vtg_project_manager_project_target_get_type (void) {
	return vtg_project_manager_project_target_type_id;
}


GType vtg_project_manager_project_target_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerProjectTargetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_project_target_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerProjectTarget), 0, (GInstanceInitFunc) vtg_project_manager_project_target_instance_init, NULL };
	vtg_project_manager_project_target_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerProjectTarget", &g_define_type_info, 0);
	return vtg_project_manager_project_target_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




