/*
 *  vtgsymbolcompletionhelper.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgsymbolcompletionhelper.h"
#include <gtksourcecompletion/gsc-manager.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gtksourcecompletion/gsc-trigger.h>
#include <gtksourcecompletion/gsc-provider.h>
#include "vtgsymbolcompletionprovider.h"




struct _VtgSymbolCompletionHelperPrivate {
	VtgPlugin* _plugin;
	GeditView* _view;
	VscSymbolCompletion* _completion;
	VtgSymbolCompletionProvider* _provider;
	GscManager* _manager;
	VtgSymbolCompletionTrigger* _trigger;
};

#define VTG_SYMBOL_COMPLETION_HELPER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_SYMBOL_COMPLETION_HELPER, VtgSymbolCompletionHelperPrivate))
enum  {
	VTG_SYMBOL_COMPLETION_HELPER_DUMMY_PROPERTY,
	VTG_SYMBOL_COMPLETION_HELPER_PLUGIN,
	VTG_SYMBOL_COMPLETION_HELPER_VIEW,
	VTG_SYMBOL_COMPLETION_HELPER_COMPLETION,
	VTG_SYMBOL_COMPLETION_HELPER_TRIGGER
};
static void vtg_symbol_completion_helper_setup_gsc_completion (VtgSymbolCompletionHelper* self, GeditView* view);
static void vtg_symbol_completion_helper_set_plugin (VtgSymbolCompletionHelper* self, VtgPlugin* value);
static void vtg_symbol_completion_helper_set_view (VtgSymbolCompletionHelper* self, GeditView* value);
static void vtg_symbol_completion_helper_set_completion (VtgSymbolCompletionHelper* self, VscSymbolCompletion* value);
static GObject * vtg_symbol_completion_helper_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_symbol_completion_helper_parent_class = NULL;
static void vtg_symbol_completion_helper_finalize (GObject* obj);



VtgSymbolCompletionHelper* vtg_symbol_completion_helper_construct (GType object_type, VtgPlugin* plugin, GeditView* view, VscSymbolCompletion* completion) {
	GParameter * __params;
	GParameter * __params_it;
	VtgSymbolCompletionHelper * self;
	g_return_val_if_fail (plugin != NULL, NULL);
	g_return_val_if_fail (view != NULL, NULL);
	g_return_val_if_fail (completion != NULL, NULL);
	__params = g_new0 (GParameter, 3);
	__params_it = __params;
	__params_it->name = "plugin";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN);
	g_value_set_object (&__params_it->value, plugin);
	__params_it++;
	__params_it->name = "view";
	g_value_init (&__params_it->value, GEDIT_TYPE_VIEW);
	g_value_set_object (&__params_it->value, view);
	__params_it++;
	__params_it->name = "completion";
	g_value_init (&__params_it->value, VSC_TYPE_SYMBOL_COMPLETION);
	g_value_set_object (&__params_it->value, completion);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgSymbolCompletionHelper* vtg_symbol_completion_helper_new (VtgPlugin* plugin, GeditView* view, VscSymbolCompletion* completion) {
	return vtg_symbol_completion_helper_construct (VTG_TYPE_SYMBOL_COMPLETION_HELPER, plugin, view, completion);
}


void vtg_symbol_completion_helper_deactivate (VtgSymbolCompletionHelper* self) {
	g_return_if_fail (self != NULL);
}


static void vtg_symbol_completion_helper_setup_gsc_completion (VtgSymbolCompletionHelper* self, GeditView* view) {
	GscManager* _tmp0;
	VtgSymbolCompletionProvider* _tmp1;
	VtgSymbolCompletionTrigger* _tmp2;
	g_return_if_fail (self != NULL);
	g_return_if_fail (view != NULL);
	_tmp0 = NULL;
	self->priv->_manager = (_tmp0 = gsc_manager_new ((GtkTextView*) view), (self->priv->_manager == NULL) ? NULL : (self->priv->_manager = (g_object_unref (self->priv->_manager), NULL)), _tmp0);
	_tmp1 = NULL;
	self->priv->_provider = (_tmp1 = vtg_symbol_completion_provider_new (self->priv->_plugin, view, self->priv->_completion), (self->priv->_provider == NULL) ? NULL : (self->priv->_provider = (g_object_unref (self->priv->_provider), NULL)), _tmp1);
	_tmp2 = NULL;
	self->priv->_trigger = (_tmp2 = vtg_symbol_completion_trigger_new (self->priv->_manager, "SymbolComplete"), (self->priv->_trigger == NULL) ? NULL : (self->priv->_trigger = (g_object_unref (self->priv->_trigger), NULL)), _tmp2);
	gsc_manager_register_trigger (self->priv->_manager, (GscTrigger*) self->priv->_trigger);
	gsc_manager_register_provider (self->priv->_manager, (GscProvider*) self->priv->_provider, "SymbolComplete");
	gsc_manager_activate (self->priv->_manager);
}


VtgPlugin* vtg_symbol_completion_helper_get_plugin (VtgSymbolCompletionHelper* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_plugin;
}


static void vtg_symbol_completion_helper_set_plugin (VtgSymbolCompletionHelper* self, VtgPlugin* value) {
	VtgPlugin* _tmp2;
	VtgPlugin* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_plugin = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "plugin");
}


GeditView* vtg_symbol_completion_helper_get_view (VtgSymbolCompletionHelper* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_view;
}


static void vtg_symbol_completion_helper_set_view (VtgSymbolCompletionHelper* self, GeditView* value) {
	GeditView* _tmp2;
	GeditView* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_view = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_view == NULL) ? NULL : (self->priv->_view = (g_object_unref (self->priv->_view), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "view");
}


VscSymbolCompletion* vtg_symbol_completion_helper_get_completion (VtgSymbolCompletionHelper* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_completion;
}


static void vtg_symbol_completion_helper_set_completion (VtgSymbolCompletionHelper* self, VscSymbolCompletion* value) {
	VscSymbolCompletion* _tmp2;
	VscSymbolCompletion* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_completion = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_completion == NULL) ? NULL : (self->priv->_completion = (g_object_unref (self->priv->_completion), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "completion");
}


VtgSymbolCompletionTrigger* vtg_symbol_completion_helper_get_trigger (VtgSymbolCompletionHelper* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_trigger;
}


static GObject * vtg_symbol_completion_helper_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgSymbolCompletionHelperClass * klass;
	GObjectClass * parent_class;
	VtgSymbolCompletionHelper * self;
	klass = VTG_SYMBOL_COMPLETION_HELPER_CLASS (g_type_class_peek (VTG_TYPE_SYMBOL_COMPLETION_HELPER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_SYMBOL_COMPLETION_HELPER (obj);
	{
		vtg_symbol_completion_helper_setup_gsc_completion (self, vtg_symbol_completion_helper_get_view (self));
	}
	return obj;
}


static void vtg_symbol_completion_helper_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgSymbolCompletionHelper * self;
	gpointer boxed;
	self = VTG_SYMBOL_COMPLETION_HELPER (object);
	switch (property_id) {
		case VTG_SYMBOL_COMPLETION_HELPER_PLUGIN:
		g_value_set_object (value, vtg_symbol_completion_helper_get_plugin (self));
		break;
		case VTG_SYMBOL_COMPLETION_HELPER_VIEW:
		g_value_set_object (value, vtg_symbol_completion_helper_get_view (self));
		break;
		case VTG_SYMBOL_COMPLETION_HELPER_COMPLETION:
		g_value_set_object (value, vtg_symbol_completion_helper_get_completion (self));
		break;
		case VTG_SYMBOL_COMPLETION_HELPER_TRIGGER:
		g_value_set_object (value, vtg_symbol_completion_helper_get_trigger (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_symbol_completion_helper_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgSymbolCompletionHelper * self;
	self = VTG_SYMBOL_COMPLETION_HELPER (object);
	switch (property_id) {
		case VTG_SYMBOL_COMPLETION_HELPER_PLUGIN:
		vtg_symbol_completion_helper_set_plugin (self, g_value_get_object (value));
		break;
		case VTG_SYMBOL_COMPLETION_HELPER_VIEW:
		vtg_symbol_completion_helper_set_view (self, g_value_get_object (value));
		break;
		case VTG_SYMBOL_COMPLETION_HELPER_COMPLETION:
		vtg_symbol_completion_helper_set_completion (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_symbol_completion_helper_class_init (VtgSymbolCompletionHelperClass * klass) {
	vtg_symbol_completion_helper_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgSymbolCompletionHelperPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_symbol_completion_helper_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_symbol_completion_helper_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_symbol_completion_helper_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_symbol_completion_helper_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_HELPER_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VTG_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_HELPER_VIEW, g_param_spec_object ("view", "view", "view", GEDIT_TYPE_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_HELPER_COMPLETION, g_param_spec_object ("completion", "completion", "completion", VSC_TYPE_SYMBOL_COMPLETION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_HELPER_TRIGGER, g_param_spec_object ("trigger", "trigger", "trigger", VTG_TYPE_SYMBOL_COMPLETION_TRIGGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_symbol_completion_helper_instance_init (VtgSymbolCompletionHelper * self) {
	self->priv = VTG_SYMBOL_COMPLETION_HELPER_GET_PRIVATE (self);
}


static void vtg_symbol_completion_helper_finalize (GObject* obj) {
	VtgSymbolCompletionHelper * self;
	self = VTG_SYMBOL_COMPLETION_HELPER (obj);
	{
		GscManager* _tmp0;
		gsc_manager_deactivate (self->priv->_manager);
		gsc_manager_unregister_trigger (self->priv->_manager, (GscTrigger*) self->priv->_trigger);
		gsc_manager_unregister_provider (self->priv->_manager, (GscProvider*) self->priv->_provider, "SymbolComplete");
		_tmp0 = NULL;
		self->priv->_manager = (_tmp0 = NULL, (self->priv->_manager == NULL) ? NULL : (self->priv->_manager = (g_object_unref (self->priv->_manager), NULL)), _tmp0);
	}
	(self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL));
	(self->priv->_view == NULL) ? NULL : (self->priv->_view = (g_object_unref (self->priv->_view), NULL));
	(self->priv->_completion == NULL) ? NULL : (self->priv->_completion = (g_object_unref (self->priv->_completion), NULL));
	(self->priv->_provider == NULL) ? NULL : (self->priv->_provider = (g_object_unref (self->priv->_provider), NULL));
	(self->priv->_manager == NULL) ? NULL : (self->priv->_manager = (g_object_unref (self->priv->_manager), NULL));
	(self->priv->_trigger == NULL) ? NULL : (self->priv->_trigger = (g_object_unref (self->priv->_trigger), NULL));
	G_OBJECT_CLASS (vtg_symbol_completion_helper_parent_class)->finalize (obj);
}


static GType vtg_symbol_completion_helper_type_id = 0;
GType vtg_symbol_completion_helper_get_type (void) {
	return vtg_symbol_completion_helper_type_id;
}


GType vtg_symbol_completion_helper_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgSymbolCompletionHelperClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_symbol_completion_helper_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgSymbolCompletionHelper), 0, (GInstanceInitFunc) vtg_symbol_completion_helper_instance_init, NULL };
	vtg_symbol_completion_helper_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgSymbolCompletionHelper", &g_define_type_info, 0);
	return vtg_symbol_completion_helper_type_id;
}




