/*
 *  vtgsymbolcompletionprovider.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgsymbolcompletionprovider.h"
#include <vscsourcebuffer.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gtksourcecompletion/gsc-info.h>
#include <vscparsermanager.h>
#include <gedit/gedit-window.h>
#include <gedit/gedit-statusbar.h>
#include <gtk/gtk.h>
#include <gedit/gedit-document.h>
#include <gdk/gdk.h>
#include <vscsymbolcompletionitem.h>
#include <vscsymbolcompletionresult.h>
#include <vscsymbolcompletionfilteroptions.h>
#include <gee/collection.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gtksourcecompletion/gsc-utils.h>
#include <gdk/gdkkeysyms.h>
#include <gtksourcecompletion/gsc-proposal.h>
#include <gee/list.h>
#include "vtgsymbolcompletiontrigger.h"
#include "vtgutils.h"




static char* string_substring (const char* self, glong offset, glong len);
static glong string_get_length (const char* self);
struct _VtgSymbolCompletionProviderPrivate {
	VscSourceBuffer* _sb;
	VscSymbolCompletion* _completion;
	GeditView* _view;
	GList* list;
	GdkPixbuf* _icon_generic;
	GdkPixbuf* _icon_field;
	GdkPixbuf* _icon_method;
	GdkPixbuf* _icon_class;
	GdkPixbuf* _icon_struct;
	GdkPixbuf* _icon_property;
	GdkPixbuf* _icon_signal;
	GdkPixbuf* _icon_iface;
	GdkPixbuf* _icon_const;
	GdkPixbuf* _icon_enum;
	GdkPixbuf* _icon_namespace;
	guint timeout_id;
	guint idle_id;
	gboolean all_doc;
	gint prealloc_index;
	gint counter;
	gboolean cache_building;
	gboolean prev_cache_building;
	gboolean tooltip_is_visible;
	VtgSymbolCompletionTrigger* _last_trigger;
	guint sb_msg_id;
	guint sb_context_id;
	VtgPlugin* _plugin;
	GscInfo* _calltip_window;
	gboolean need_parse;
	gint current_edited_line;
};

#define VTG_SYMBOL_COMPLETION_PROVIDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_SYMBOL_COMPLETION_PROVIDER, VtgSymbolCompletionProviderPrivate))
enum  {
	VTG_SYMBOL_COMPLETION_PROVIDER_DUMMY_PROPERTY,
	VTG_SYMBOL_COMPLETION_PROVIDER_COMPLETION,
	VTG_SYMBOL_COMPLETION_PROVIDER_VIEW,
	VTG_SYMBOL_COMPLETION_PROVIDER_PLUGIN
};
static void _g_list_free_g_object_unref (GList* self);
static gboolean _vtg_symbol_completion_provider_on_idle_gsource_func (gpointer self);
static void vtg_symbol_completion_provider_on_cache_building (VtgSymbolCompletionProvider* self, VscParserManager* sender);
static void vtg_symbol_completion_provider_on_cache_builded (VtgSymbolCompletionProvider* self, VscParserManager* sender);
static gboolean vtg_symbol_completion_provider_on_idle (VtgSymbolCompletionProvider* self);
static gboolean vtg_symbol_completion_provider_on_timeout_parse (VtgSymbolCompletionProvider* self);
static gboolean _vtg_symbol_completion_provider_on_timeout_parse_gsource_func (gpointer self);
static gboolean vtg_symbol_completion_provider_on_view_key_press (VtgSymbolCompletionProvider* self, GtkTextView* view, const GdkEventKey* evt);
static void vtg_symbol_completion_provider_show_calltip (VtgSymbolCompletionProvider* self);
static void vtg_symbol_completion_provider_hide_calltip (VtgSymbolCompletionProvider* self);
static void vtg_symbol_completion_provider_initialize_calltip_window (VtgSymbolCompletionProvider* self);
static void vtg_symbol_completion_provider_parse (VtgSymbolCompletionProvider* self, GeditDocument* doc);
static void vtg_symbol_completion_provider_real_finish (GscProvider* base);
static const char* vtg_symbol_completion_provider_real_get_name (GscProvider* base);
static GList* vtg_symbol_completion_provider_real_get_proposals (GscProvider* base, GscTrigger* trigger);
static gint _vtg_symbol_completion_provider_proposal_sort_gcompare_func (void* a, void* b);
static void vtg_symbol_completion_provider_append_symbols (VtgSymbolCompletionProvider* self, GeeList* symbols, GdkPixbuf* icon);
static gint vtg_symbol_completion_provider_proposal_sort (void* a, void* b);
static void vtg_symbol_completion_provider_transform_result (VtgSymbolCompletionProvider* self, VscSymbolCompletionResult* result);
static void vtg_symbol_completion_provider_parse_current_line (VtgSymbolCompletionProvider* self, gboolean skip_leading_spaces, char** symbolname, char** last_symbolname, char** line, gint* lineno, gint* colno);
static VscSymbolCompletionItem* vtg_symbol_completion_provider_get_method_signature (VtgSymbolCompletionProvider* self);
static VscSymbolCompletionResult* vtg_symbol_completion_provider_get_completions (VtgSymbolCompletionProvider* self, VtgSymbolCompletionTrigger* trigger);
static VscSymbolCompletionResult* vtg_symbol_completion_provider_complete (VtgSymbolCompletionProvider* self, VtgSymbolCompletionTrigger* trigger, const char* line, const char* word, gint lineno, gint colno);
static gboolean vtg_symbol_completion_provider_is_vala_keyword (VtgSymbolCompletionProvider* self, const char* keyword);
static char* vtg_symbol_completion_provider_get_document_text (VtgSymbolCompletionProvider* self, GeditDocument* doc, gboolean all_doc);
static void vtg_symbol_completion_provider_set_completion (VtgSymbolCompletionProvider* self, VscSymbolCompletion* value);
static void vtg_symbol_completion_provider_set_view (VtgSymbolCompletionProvider* self, GeditView* value);
static void vtg_symbol_completion_provider_set_plugin (VtgSymbolCompletionProvider* self, VtgPlugin* value);
static gboolean _vtg_symbol_completion_provider_on_view_key_press_gtk_widget_key_press_event (GeditView* _sender, const GdkEventKey* event, gpointer self);
static void _vtg_symbol_completion_provider_on_cache_building_vsc_parser_manager_cache_building (VscParserManager* _sender, gpointer self);
static void _vtg_symbol_completion_provider_on_cache_builded_vsc_parser_manager_cache_builded (VscParserManager* _sender, gpointer self);
static GObject * vtg_symbol_completion_provider_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_symbol_completion_provider_parent_class = NULL;
static GscProviderIface* vtg_symbol_completion_provider_gsc_provider_parent_iface = NULL;
static void vtg_symbol_completion_provider_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


VtgSymbolCompletionProvider* vtg_symbol_completion_provider_construct (GType object_type, VtgPlugin* plugin, GeditView* view, VscSymbolCompletion* completion) {
	GParameter * __params;
	GParameter * __params_it;
	VtgSymbolCompletionProvider * self;
	g_return_val_if_fail (plugin != NULL, NULL);
	g_return_val_if_fail (view != NULL, NULL);
	g_return_val_if_fail (completion != NULL, NULL);
	__params = g_new0 (GParameter, 3);
	__params_it = __params;
	__params_it->name = "plugin";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN);
	g_value_set_object (&__params_it->value, plugin);
	__params_it++;
	__params_it->name = "completion";
	g_value_init (&__params_it->value, VSC_TYPE_SYMBOL_COMPLETION);
	g_value_set_object (&__params_it->value, completion);
	__params_it++;
	__params_it->name = "view";
	g_value_init (&__params_it->value, GEDIT_TYPE_VIEW);
	g_value_set_object (&__params_it->value, view);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgSymbolCompletionProvider* vtg_symbol_completion_provider_new (VtgPlugin* plugin, GeditView* view, VscSymbolCompletion* completion) {
	return vtg_symbol_completion_provider_construct (VTG_TYPE_SYMBOL_COMPLETION_PROVIDER, plugin, view, completion);
}


static gboolean _vtg_symbol_completion_provider_on_idle_gsource_func (gpointer self) {
	return vtg_symbol_completion_provider_on_idle (self);
}


static void vtg_symbol_completion_provider_on_cache_building (VtgSymbolCompletionProvider* self, VscParserManager* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	if (self->priv->cache_building == FALSE) {
		self->priv->cache_building = TRUE;
		self->priv->idle_id = g_idle_add (_vtg_symbol_completion_provider_on_idle_gsource_func, self);
	}
}


static void vtg_symbol_completion_provider_on_cache_builded (VtgSymbolCompletionProvider* self, VscParserManager* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	if (self->priv->cache_building == TRUE) {
		self->priv->cache_building = FALSE;
		if (self->priv->idle_id == 0) {
			self->priv->idle_id = g_idle_add (_vtg_symbol_completion_provider_on_idle_gsource_func, self);
		}
	}
}


static gboolean vtg_symbol_completion_provider_on_idle (VtgSymbolCompletionProvider* self) {
	gboolean _tmp1;
	gboolean _tmp2;
	g_return_val_if_fail (self != NULL, FALSE);
	if (self->priv->_plugin == NULL) {
		return FALSE;
	}
	_tmp1 = FALSE;
	_tmp2 = FALSE;
	if (self->priv->cache_building) {
		_tmp2 = !self->priv->tooltip_is_visible;
	} else {
		_tmp2 = FALSE;
	}
	if (_tmp2) {
		_tmp1 = self->priv->prev_cache_building == FALSE;
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		GeditStatusbar* _tmp3;
		GeditStatusbar* status_bar;
		self->priv->prev_cache_building = self->priv->cache_building;
		_tmp3 = NULL;
		status_bar = (_tmp3 = GEDIT_STATUSBAR (gedit_window_get_statusbar (vtg_plugin_get_gedit_window (self->priv->_plugin))), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3));
		if (self->priv->sb_msg_id != 0) {
			gtk_statusbar_remove ((GtkStatusbar*) status_bar, self->priv->sb_context_id, self->priv->sb_msg_id);
		}
		self->priv->sb_msg_id = gtk_statusbar_push ((GtkStatusbar*) status_bar, self->priv->sb_context_id, "rebuilding symbol cache...");
		(status_bar == NULL) ? NULL : (status_bar = (g_object_unref (status_bar), NULL));
	} else {
		gboolean _tmp4;
		_tmp4 = FALSE;
		if (self->priv->cache_building == FALSE) {
			_tmp4 = self->priv->prev_cache_building == TRUE;
		} else {
			_tmp4 = FALSE;
		}
		if (_tmp4) {
			GeditStatusbar* _tmp5;
			GeditStatusbar* status_bar;
			self->priv->prev_cache_building = FALSE;
			/*hide tip, show proposal list*/
			_tmp5 = NULL;
			status_bar = (_tmp5 = GEDIT_STATUSBAR (gedit_window_get_statusbar (vtg_plugin_get_gedit_window (self->priv->_plugin))), (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5));
			gtk_statusbar_remove ((GtkStatusbar*) status_bar, self->priv->sb_context_id, self->priv->sb_msg_id);
			self->priv->sb_msg_id = (guint) 0;
			if (self->priv->_last_trigger != NULL) {
				VtgSymbolCompletionTrigger* _tmp6;
				VtgSymbolCompletionTrigger* trigger;
				_tmp6 = NULL;
				trigger = (_tmp6 = VTG_SYMBOL_COMPLETION_TRIGGER (self->priv->_last_trigger), (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6));
				vtg_symbol_completion_trigger_trigger_event (trigger, trigger->shortcut_triggered);
				(trigger == NULL) ? NULL : (trigger = (g_object_unref (trigger), NULL));
			}
			(status_bar == NULL) ? NULL : (status_bar = (g_object_unref (status_bar), NULL));
		}
	}
	self->priv->idle_id = (guint) 0;
	return FALSE;
}


static gboolean vtg_symbol_completion_provider_on_timeout_parse (VtgSymbolCompletionProvider* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	if (self->priv->counter == 0) {
		vtg_symbol_completion_provider_parse (self, GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view)));
		self->priv->timeout_id = (guint) 0;
		return FALSE;
	} else {
		self->priv->counter--;
		return TRUE;
	}
}


static gboolean _vtg_symbol_completion_provider_on_timeout_parse_gsource_func (gpointer self) {
	return vtg_symbol_completion_provider_on_timeout_parse (self);
}


static gboolean vtg_symbol_completion_provider_on_view_key_press (VtgSymbolCompletionProvider* self, GtkTextView* view, const GdkEventKey* evt) {
	GeditDocument* doc;
	GtkTextMark* mark;
	GtkTextIter start = {0};
	gint line;
	gunichar ch;
	gboolean _tmp6;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (view != NULL, FALSE);
	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view));
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert ((GtkTextBuffer*) doc));
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) doc, &start, mark);
	line = gtk_text_iter_get_line (&start);
	ch = (gunichar) gdk_keyval_to_unicode ((*evt).keyval);
	if (ch == '(') {
		vtg_symbol_completion_provider_show_calltip (self);
	} else {
		gboolean _tmp0;
		_tmp0 = FALSE;
		if ((*evt).keyval == GDK_Escape) {
			_tmp0 = TRUE;
		} else {
			_tmp0 = ch == ')';
		}
		if (_tmp0) {
			/* || ch == ';' || ch == '.' || evt.keyval == Gdk.Key_Return*/
			vtg_symbol_completion_provider_hide_calltip (self);
		}
	}
	if (self->priv->counter <= 0) {
		gboolean _tmp1;
		_tmp1 = FALSE;
		if ((*evt).keyval == GDK_Return) {
			_tmp1 = TRUE;
		} else {
			_tmp1 = ch == ';';
		}
		if (_tmp1) {
			self->priv->all_doc = TRUE;
			self->priv->counter = 0;
			/*immediatly (0.1sec)*/
			self->priv->current_edited_line = -1;
		} else {
			gboolean _tmp2;
			gboolean _tmp3;
			_tmp2 = FALSE;
			_tmp3 = FALSE;
			if (g_unichar_isprint (ch)) {
				_tmp3 = TRUE;
			} else {
				_tmp3 = (*evt).keyval == GDK_Delete;
			}
			if (_tmp3) {
				_tmp2 = TRUE;
			} else {
				_tmp2 = (*evt).keyval == GDK_BackSpace;
			}
			if (_tmp2) {
				self->priv->need_parse = TRUE;
			} else {
				gboolean _tmp4;
				_tmp4 = FALSE;
				if ((*evt).keyval == GDK_Up) {
					_tmp4 = TRUE;
				} else {
					_tmp4 = (*evt).keyval == GDK_Down;
				}
				if (_tmp4) {
					self->priv->current_edited_line = -1;
				}
			}
		}
	} else {
		gboolean _tmp5;
		_tmp5 = FALSE;
		if ((*evt).keyval == GDK_Return) {
			_tmp5 = TRUE;
		} else {
			_tmp5 = ch == ';';
		}
		/*moved so parse the buffer is its needed*/
		if (_tmp5) {
			self->priv->all_doc = TRUE;
			self->priv->counter = 0;
		} else {
			/*immediatly (0.1sec)*/
			self->priv->all_doc = FALSE;
			self->priv->counter = 5;
		}
	}
	_tmp6 = FALSE;
	if (self->priv->need_parse) {
		_tmp6 = self->priv->current_edited_line != line;
	} else {
		_tmp6 = FALSE;
	}
	if (_tmp6) {
		self->priv->need_parse = FALSE;
		self->priv->current_edited_line = line;
		self->priv->timeout_id = g_timeout_add ((guint) 25, _vtg_symbol_completion_provider_on_timeout_parse_gsource_func, self);
	}
	return FALSE;
}


static void vtg_symbol_completion_provider_show_calltip (VtgSymbolCompletionProvider* self) {
	VscSymbolCompletionItem* completion_result;
	g_return_if_fail (self != NULL);
	completion_result = vtg_symbol_completion_provider_get_method_signature (self);
	if (completion_result != NULL) {
		const char* _tmp0;
		char* calltip_text;
		if (self->priv->_calltip_window == NULL) {
			vtg_symbol_completion_provider_initialize_calltip_window (self);
		}
		_tmp0 = NULL;
		calltip_text = (_tmp0 = completion_result->info, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
		if (calltip_text != NULL) {
			gsc_info_set_markup (self->priv->_calltip_window, calltip_text);
			gsc_info_move_to_cursor (self->priv->_calltip_window, (GtkTextView*) self->priv->_view);
			gtk_widget_show_all ((GtkWidget*) self->priv->_calltip_window);
		}
		calltip_text = (g_free (calltip_text), NULL);
	}
	(completion_result == NULL) ? NULL : (completion_result = (g_object_unref (completion_result), NULL));
}


static void vtg_symbol_completion_provider_hide_calltip (VtgSymbolCompletionProvider* self) {
	g_return_if_fail (self != NULL);
	if (self->priv->_calltip_window == NULL) {
		return;
	}
	gtk_widget_hide ((GtkWidget*) self->priv->_calltip_window);
}


static void vtg_symbol_completion_provider_initialize_calltip_window (VtgSymbolCompletionProvider* self) {
	GscInfo* _tmp0;
	GtkWidget* _tmp1;
	GtkWidget* child;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	self->priv->_calltip_window = (_tmp0 = g_object_ref_sink (gsc_info_new ()), (self->priv->_calltip_window == NULL) ? NULL : (self->priv->_calltip_window = (g_object_unref (self->priv->_calltip_window), NULL)), _tmp0);
	gsc_info_set_info_type (self->priv->_calltip_window, GSC_INFO_VIEW_EXTENDED);
	gtk_window_set_transient_for ((GtkWindow*) self->priv->_calltip_window, (GtkWindow*) vtg_plugin_get_gedit_window (self->priv->_plugin));
	gsc_info_set_adjust_width (self->priv->_calltip_window, TRUE, 800);
	gsc_info_set_adjust_height (self->priv->_calltip_window, TRUE, 600);
	g_object_set ((GtkWindow*) self->priv->_calltip_window, "allow-grow", TRUE, NULL);
	g_object_set ((GtkWindow*) self->priv->_calltip_window, "allow-shrink", TRUE, NULL);
	/*this is an hack*/
	_tmp1 = NULL;
	child = (_tmp1 = gtk_bin_get_child ((GtkBin*) self->priv->_calltip_window), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	while (TRUE) {
		gboolean _tmp2;
		_tmp2 = FALSE;
		if (child != NULL) {
			_tmp2 = !(GTK_IS_LABEL (child));
		} else {
			_tmp2 = FALSE;
		}
		if (!_tmp2) {
			break;
		}
		if (GTK_IS_BIN (child)) {
			GtkWidget* _tmp4;
			GtkWidget* _tmp3;
			_tmp4 = NULL;
			_tmp3 = NULL;
			child = (_tmp4 = (_tmp3 = gtk_bin_get_child ((GTK_BIN (child))), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3)), (child == NULL) ? NULL : (child = (g_object_unref (child), NULL)), _tmp4);
		} else {
			GtkWidget* _tmp5;
			_tmp5 = NULL;
			child = (_tmp5 = NULL, (child == NULL) ? NULL : (child = (g_object_unref (child), NULL)), _tmp5);
		}
	}
	if (GTK_IS_LABEL (child)) {
		GtkLabel* _tmp6;
		GtkLabel* l;
		_tmp6 = NULL;
		l = (_tmp6 = GTK_LABEL (child), (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6));
		g_object_set ((GtkMisc*) l, "xalign", (float) 0, NULL);
		g_object_set ((GtkMisc*) l, "xpad", 8, NULL);
		(l == NULL) ? NULL : (l = (g_object_unref (l), NULL));
	}
	(child == NULL) ? NULL : (child = (g_object_unref (child), NULL));
}


static void vtg_symbol_completion_provider_parse (VtgSymbolCompletionProvider* self, GeditDocument* doc) {
	char* buffer;
	g_return_if_fail (self != NULL);
	g_return_if_fail (doc != NULL);
	buffer = vtg_symbol_completion_provider_get_document_text (self, doc, self->priv->all_doc);
	vsc_source_buffer_set_source (self->priv->_sb, buffer);
	vsc_parser_manager_reparse_source_buffers (vsc_symbol_completion_get_parser (self->priv->_completion));
	buffer = (g_free (buffer), NULL);
}


static void vtg_symbol_completion_provider_real_finish (GscProvider* base) {
	VtgSymbolCompletionProvider * self;
	GList* _tmp0;
	self = (VtgSymbolCompletionProvider*) base;
	_tmp0 = NULL;
	self->priv->list = (_tmp0 = NULL, (self->priv->list == NULL) ? NULL : (self->priv->list = (_g_list_free_g_object_unref (self->priv->list), NULL)), _tmp0);
}


static const char* vtg_symbol_completion_provider_real_get_name (GscProvider* base) {
	VtgSymbolCompletionProvider * self;
	self = (VtgSymbolCompletionProvider*) base;
	return "SymbolCompletionProvider";
}


static GList* vtg_symbol_completion_provider_real_get_proposals (GscProvider* base, GscTrigger* trigger) {
	VtgSymbolCompletionProvider * self;
	GTimer* timer;
	VscSymbolCompletionResult* _tmp0;
	gboolean _tmp1;
	GList* _tmp5;
	GList* _tmp6;
	self = (VtgSymbolCompletionProvider*) base;
	g_return_val_if_fail (trigger != NULL, NULL);
	timer = g_timer_new ();
	_tmp0 = NULL;
	vtg_symbol_completion_provider_transform_result (self, _tmp0 = vtg_symbol_completion_provider_get_completions (self, VTG_SYMBOL_COMPLETION_TRIGGER (trigger)));
	(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	g_timer_stop (timer);
	_tmp1 = FALSE;
	if (g_list_length (self->priv->list) == 0) {
		_tmp1 = self->priv->cache_building;
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		VtgSymbolCompletionTrigger* _tmp3;
		VtgSymbolCompletionTrigger* _tmp2;
		_tmp3 = NULL;
		_tmp2 = NULL;
		self->priv->_last_trigger = (_tmp3 = (_tmp2 = VTG_SYMBOL_COMPLETION_TRIGGER (trigger), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (self->priv->_last_trigger == NULL) ? NULL : (self->priv->_last_trigger = (g_object_unref (self->priv->_last_trigger), NULL)), _tmp3);
	} else {
		VtgSymbolCompletionTrigger* _tmp4;
		_tmp4 = NULL;
		self->priv->_last_trigger = (_tmp4 = NULL, (self->priv->_last_trigger == NULL) ? NULL : (self->priv->_last_trigger = (g_object_unref (self->priv->_last_trigger), NULL)), _tmp4);
	}
	_tmp5 = NULL;
	_tmp6 = NULL;
	return (_tmp6 = (_tmp5 = self->priv->list, self->priv->list = NULL, _tmp5), (timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL)), _tmp6);
}


static gint _vtg_symbol_completion_provider_proposal_sort_gcompare_func (void* a, void* b) {
	return vtg_symbol_completion_provider_proposal_sort (a, b);
}


static void vtg_symbol_completion_provider_append_symbols (VtgSymbolCompletionProvider* self, GeeList* symbols, GdkPixbuf* icon) {
	GscProposal** _tmp1;
	gint proposals_size;
	gint proposals_length1;
	gint _tmp0;
	GscProposal** proposals;
	g_return_if_fail (self != NULL);
	g_return_if_fail (symbols != NULL);
	g_return_if_fail (icon != NULL);
	_tmp1 = NULL;
	proposals = (_tmp1 = vtg_utils_get_proposal_cache (&_tmp0), proposals_length1 = _tmp0, proposals_size = proposals_length1, _tmp1);
	{
		GeeIterator* _symbol_it;
		_symbol_it = gee_iterable_iterator ((GeeIterable*) symbols);
		while (gee_iterator_next (_symbol_it)) {
			VscSymbolCompletionItem* symbol;
			GscProposal* proposal;
			const char* _tmp2;
			const char* _tmp3;
			char* name;
			const char* _tmp4;
			const char* _tmp5;
			char* info;
			GscProposal* _tmp9;
			symbol = (VscSymbolCompletionItem*) gee_iterator_get (_symbol_it);
			proposal = NULL;
			_tmp2 = NULL;
			if (symbol->name != NULL) {
				_tmp2 = symbol->name;
			} else {
				_tmp2 = "<null>";
			}
			_tmp3 = NULL;
			name = (_tmp3 = (_tmp2), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
			_tmp4 = NULL;
			if (symbol->info != NULL) {
				_tmp4 = symbol->info;
			} else {
				_tmp4 = "";
			}
			_tmp5 = NULL;
			info = (_tmp5 = (_tmp4), (_tmp5 == NULL) ? NULL : g_strdup (_tmp5));
			if (self->priv->prealloc_index < VTG_UTILS_prealloc_count) {
				GscProposal* _tmp7;
				GscProposal* _tmp6;
				_tmp7 = NULL;
				_tmp6 = NULL;
				proposal = (_tmp7 = (_tmp6 = proposals[self->priv->prealloc_index], (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6)), (proposal == NULL) ? NULL : (proposal = (g_object_unref (proposal), NULL)), _tmp7);
				self->priv->prealloc_index++;
				g_object_set (proposal, "label", name, NULL);
				g_object_set (proposal, "info", info, NULL);
				g_object_set (proposal, "icon", icon, NULL);
			} else {
				GscProposal* _tmp8;
				_tmp8 = NULL;
				proposal = (_tmp8 = gsc_proposal_new (name, info, icon), (proposal == NULL) ? NULL : (proposal = (g_object_unref (proposal), NULL)), _tmp8);
			}
			_tmp9 = NULL;
			self->priv->list = g_list_append (self->priv->list, (_tmp9 = proposal, (_tmp9 == NULL) ? NULL : g_object_ref (_tmp9)));
			(symbol == NULL) ? NULL : (symbol = (g_object_unref (symbol), NULL));
			(proposal == NULL) ? NULL : (proposal = (g_object_unref (proposal), NULL));
			name = (g_free (name), NULL);
			info = (g_free (info), NULL);
		}
		(_symbol_it == NULL) ? NULL : (_symbol_it = (gee_collection_object_unref (_symbol_it), NULL));
	}
	/*sort list*/
	self->priv->list = g_list_sort (self->priv->list, _vtg_symbol_completion_provider_proposal_sort_gcompare_func);
}


static gint vtg_symbol_completion_provider_proposal_sort (void* a, void* b) {
	GscProposal* _tmp0;
	GscProposal* pa;
	GscProposal* _tmp1;
	GscProposal* pb;
	gint _tmp2;
	_tmp0 = NULL;
	pa = (_tmp0 = GSC_PROPOSAL (a), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	_tmp1 = NULL;
	pb = (_tmp1 = GSC_PROPOSAL (b), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	return (_tmp2 = strcmp (gsc_proposal_get_label (pa), gsc_proposal_get_label (pb)), (pa == NULL) ? NULL : (pa = (g_object_unref (pa), NULL)), (pb == NULL) ? NULL : (pb = (g_object_unref (pb), NULL)), _tmp2);
}


static void vtg_symbol_completion_provider_transform_result (VtgSymbolCompletionProvider* self, VscSymbolCompletionResult* result) {
	GTimer* timer;
	GList* _tmp0;
	gboolean _tmp1;
	g_return_if_fail (self != NULL);
	timer = g_timer_new ();
	self->priv->prealloc_index = 0;
	_tmp0 = NULL;
	self->priv->list = (_tmp0 = NULL, (self->priv->list == NULL) ? NULL : (self->priv->list = (_g_list_free_g_object_unref (self->priv->list), NULL)), _tmp0);
	_tmp1 = FALSE;
	if (result != NULL) {
		_tmp1 = !vsc_symbol_completion_result_get_is_empty (result);
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		if (gee_collection_get_size ((GeeCollection*) result->fields) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->fields, self->priv->_icon_field);
		}
		if (gee_collection_get_size ((GeeCollection*) result->properties) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->properties, self->priv->_icon_property);
		}
		if (gee_collection_get_size ((GeeCollection*) result->methods) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->methods, self->priv->_icon_method);
		}
		if (gee_collection_get_size ((GeeCollection*) result->signals) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->signals, self->priv->_icon_signal);
		}
		if (gee_collection_get_size ((GeeCollection*) result->classes) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->classes, self->priv->_icon_class);
		}
		if (gee_collection_get_size ((GeeCollection*) result->interfaces) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->interfaces, self->priv->_icon_iface);
		}
		if (gee_collection_get_size ((GeeCollection*) result->structs) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->structs, self->priv->_icon_struct);
		}
		if (gee_collection_get_size ((GeeCollection*) result->enums) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->enums, self->priv->_icon_enum);
		}
		if (gee_collection_get_size ((GeeCollection*) result->error_domains) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->error_domains, self->priv->_icon_struct);
		}
		if (gee_collection_get_size ((GeeCollection*) result->constants) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->constants, self->priv->_icon_const);
		}
		if (gee_collection_get_size ((GeeCollection*) result->namespaces) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->namespaces, self->priv->_icon_namespace);
		}
		if (gee_collection_get_size ((GeeCollection*) result->others) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->others, self->priv->_icon_generic);
		}
	}
	g_timer_stop (timer);
	(timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL));
}


static void vtg_symbol_completion_provider_parse_current_line (VtgSymbolCompletionProvider* self, gboolean skip_leading_spaces, char** symbolname, char** last_symbolname, char** line, gint* lineno, gint* colno) {
	GeditDocument* doc;
	GtkTextMark* mark;
	GtkTextIter end = {0};
	GtkTextIter start = {0};
	char* _tmp0;
	char* _tmp1;
	char* _tmp2;
	char* _tmp4;
	const char* _tmp3;
	g_return_if_fail (self != NULL);
	*symbolname = NULL;
	*last_symbolname = NULL;
	*line = NULL;
	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view));
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert ((GtkTextBuffer*) doc));
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) doc, &start, mark);
	(*lineno) = gtk_text_iter_get_line (&start);
	(*colno) = gtk_text_iter_get_line_offset (&start);
	_tmp0 = NULL;
	(*symbolname) = (_tmp0 = g_strdup (""), (*symbolname) = (g_free ((*symbolname)), NULL), _tmp0);
	_tmp1 = NULL;
	(*last_symbolname) = (_tmp1 = g_strdup (""), (*last_symbolname) = (g_free ((*last_symbolname)), NULL), _tmp1);
	_tmp2 = NULL;
	(*line) = (_tmp2 = g_strdup (""), (*line) = (g_free ((*line)), NULL), _tmp2);
	end = start;
	gtk_text_iter_set_line_offset (&start, 0);
	_tmp4 = NULL;
	_tmp3 = NULL;
	(*line) = (_tmp4 = (_tmp3 = gtk_text_iter_get_text (&start, &end), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3)), (*line) = (g_free ((*line)), NULL), _tmp4);
	if ((*colno) > 0) {
		GString* tmp;
		gint bracket_lev_1;
		gint bracket_lev_2;
		gint bracket_lev_3;
		gint string_lev;
		gint _tmp5;
		gint status;
		gunichar prev_char;
		char* _tmp22;
		tmp = g_string_new ("");
		bracket_lev_1 = 0;
		bracket_lev_2 = 0;
		bracket_lev_3 = 0;
		string_lev = 0;
		_tmp5 = 0;
		if (skip_leading_spaces) {
			_tmp5 = 1;
		} else {
			_tmp5 = 0;
		}
		status = (_tmp5);
		prev_char = gtk_text_iter_get_char (&end);
		while (TRUE) {
			gboolean _tmp6;
			gunichar ch;
			gboolean _tmp7;
			gboolean _tmp9;
			if (!gtk_text_iter_backward_char (&end)) {
				break;
			}
			_tmp6 = FALSE;
			if (gtk_text_iter_get_line_offset (&end) == 0) {
				_tmp6 = TRUE;
			} else {
				_tmp6 = gtk_text_iter_get_line (&end) < (*lineno);
			}
			/* end of buffer*/
			if (_tmp6) {
				break;
			}
			ch = gtk_text_iter_get_char (&end);
			_tmp7 = FALSE;
			if (status == 1) {
				gboolean _tmp8;
				_tmp8 = FALSE;
				if (ch != ' ') {
					_tmp8 = ch != '\t';
				} else {
					_tmp8 = FALSE;
				}
				_tmp7 = (_tmp8);
			} else {
				_tmp7 = FALSE;
			}
			if (_tmp7) {
				status = 0;
			}
			_tmp9 = FALSE;
			if (status == 0) {
				gboolean _tmp10;
				_tmp10 = FALSE;
				if (ch == ' ') {
					_tmp10 = TRUE;
				} else {
					_tmp10 = ch == '\t';
				}
				_tmp9 = (_tmp10);
			} else {
				_tmp9 = FALSE;
			}
			/*back to normal*/
			if (_tmp9) {
				break;
			}
			/*word interruption*/
			switch (status) {
				case 0:
				{
					if (ch == ')') {
						status = 2;
						bracket_lev_1++;
					} else {
						if (ch == ']') {
							status = 2;
							bracket_lev_2++;
						} else {
							if (ch == '}') {
								status = 2;
								bracket_lev_3++;
							} else {
								gboolean _tmp11;
								_tmp11 = FALSE;
								if (ch == '.') {
									_tmp11 = TRUE;
								} else {
									gboolean _tmp12;
									_tmp12 = FALSE;
									if (ch == '-') {
										_tmp12 = prev_char == '>';
									} else {
										_tmp12 = FALSE;
									}
									_tmp11 = (_tmp12);
								}
								if (_tmp11) {
									gboolean _tmp14;
									if (_vala_strcmp0 ((*last_symbolname), "") == 0) {
										char* _tmp13;
										_tmp13 = NULL;
										(*last_symbolname) = (_tmp13 = g_utf8_strreverse (tmp->str, -1), (*last_symbolname) = (g_free ((*last_symbolname)), NULL), _tmp13);
									}
									_tmp14 = FALSE;
									if (ch == '-') {
										_tmp14 = prev_char == '>';
									} else {
										_tmp14 = FALSE;
									}
									if (_tmp14) {
										g_string_append_unichar (tmp, (gunichar) '.');
									} else {
										g_string_append_unichar (tmp, ch);
									}
									status = 1;
								} else {
									if (ch == '=') {
										if (_vala_strcmp0 ((*last_symbolname), "") == 0) {
											char* _tmp15;
											_tmp15 = NULL;
											(*last_symbolname) = (_tmp15 = g_utf8_strreverse (tmp->str, -1), (*last_symbolname) = (g_free ((*last_symbolname)), NULL), _tmp15);
										}
										g_string_truncate (tmp, (gsize) 0);
										status = 1;
									} else {
										if (ch == '\"') {
											string_lev++;
											status = 3;
										} else {
											if (ch != '\n') {
												g_string_append_unichar (tmp, ch);
											}
										}
									}
								}
							}
						}
					}
					break;
				}
				case 1:
				{
					break;
				}
				case 2:
				{
					gboolean _tmp16;
					gboolean _tmp19;
					gboolean _tmp20;
					_tmp16 = FALSE;
					if (bracket_lev_1 > 0) {
						_tmp16 = ch == '(';
					} else {
						_tmp16 = FALSE;
					}
					if (_tmp16) {
						bracket_lev_1--;
					} else {
						gboolean _tmp17;
						_tmp17 = FALSE;
						if (bracket_lev_2 > 0) {
							_tmp17 = ch == '[';
						} else {
							_tmp17 = FALSE;
						}
						if (_tmp17) {
							bracket_lev_2--;
						} else {
							gboolean _tmp18;
							_tmp18 = FALSE;
							if (bracket_lev_3 > 0) {
								_tmp18 = ch == '{';
							} else {
								_tmp18 = FALSE;
							}
							if (_tmp18) {
								bracket_lev_3--;
							} else {
								if (ch == ')') {
									bracket_lev_1++;
								} else {
									if (ch == ']') {
										bracket_lev_2++;
									} else {
										if (ch == '{') {
											bracket_lev_3++;
										}
									}
								}
							}
						}
					}
					_tmp19 = FALSE;
					_tmp20 = FALSE;
					if (bracket_lev_1 <= 0) {
						_tmp20 = bracket_lev_2 <= 0;
					} else {
						_tmp20 = FALSE;
					}
					if (_tmp20) {
						_tmp19 = bracket_lev_3 <= 0;
					} else {
						_tmp19 = FALSE;
					}
					if (_tmp19) {
						status = 1;
					}
					break;
				}
				case 3:
				{
					if (ch == '\"') {
						char* _tmp21;
						_tmp21 = NULL;
						g_string_append (tmp, _tmp21 = g_utf8_strreverse ("string", -1));
						_tmp21 = (g_free (_tmp21), NULL);
						string_lev--;
						if (string_lev <= 0) {
							status = 0;
						}
					}
					break;
				}
			}
			prev_char = ch;
		}
		_tmp22 = NULL;
		(*symbolname) = (_tmp22 = g_utf8_strreverse (tmp->str, -1), (*symbolname) = (g_free ((*symbolname)), NULL), _tmp22);
		if (_vala_strcmp0 ((*last_symbolname), "") == 0) {
			char* _tmp24;
			const char* _tmp23;
			_tmp24 = NULL;
			_tmp23 = NULL;
			(*last_symbolname) = (_tmp24 = (_tmp23 = (*symbolname), (_tmp23 == NULL) ? NULL : g_strdup (_tmp23)), (*last_symbolname) = (g_free ((*last_symbolname)), NULL), _tmp24);
		}
		(tmp == NULL) ? NULL : (tmp = (g_string_free (tmp, TRUE), NULL));
	}
}


static VscSymbolCompletionItem* vtg_symbol_completion_provider_get_method_signature (VtgSymbolCompletionProvider* self) {
	char* line;
	char* word;
	char* last_part;
	gint lineno;
	gint colno;
	char* _tmp5;
	char* _tmp4;
	char* _tmp3;
	char* _tmp2;
	char* _tmp1;
	char* _tmp0;
	gboolean _tmp6;
	char* first_part;
	const char* _tmp10;
	char* method;
	VscSymbolCompletionResult* completion_result;
	VscSymbolCompletionResult* _tmp12;
	gboolean _tmp13;
	VscSymbolCompletionItem* _tmp15;
	g_return_val_if_fail (self != NULL, NULL);
	line = NULL;
	word = NULL;
	last_part = NULL;
	lineno = 0;
	colno = 0;
	_tmp5 = NULL;
	_tmp4 = NULL;
	_tmp3 = NULL;
	_tmp2 = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	((vtg_symbol_completion_provider_parse_current_line (self, TRUE, &_tmp0, &_tmp2, &_tmp4, &lineno, &colno), word = (_tmp1 = _tmp0, word = (g_free (word), NULL), _tmp1)), last_part = (_tmp3 = _tmp2, last_part = (g_free (last_part), NULL), _tmp3));
	line = (_tmp5 = _tmp4, line = (g_free (line), NULL), _tmp5);
	_tmp6 = FALSE;
	if (word == NULL) {
		_tmp6 = TRUE;
	} else {
		_tmp6 = _vala_strcmp0 (word, "") == 0;
	}
	if (_tmp6) {
		VscSymbolCompletionItem* _tmp7;
		_tmp7 = NULL;
		return (_tmp7 = NULL, line = (g_free (line), NULL), word = (g_free (word), NULL), last_part = (g_free (last_part), NULL), _tmp7);
	}
	/* 
	  strip last type part. 
	  eg. for demos.demo.demo_method obtains
	  demos.demo + demo_method
	*/
	first_part = NULL;
	if (_vala_strcmp0 (word, last_part) != 0) {
		char* _tmp8;
		_tmp8 = NULL;
		first_part = (_tmp8 = string_substring (word, (glong) 0, (string_get_length (word) - string_get_length (last_part)) - 1), first_part = (g_free (first_part), NULL), _tmp8);
	} else {
		char* _tmp9;
		_tmp9 = NULL;
		first_part = (_tmp9 = g_strdup ("this"), first_part = (g_free (first_part), NULL), _tmp9);
	}
	/*HACK: this won't work for static methods*/
	_tmp10 = NULL;
	method = (_tmp10 = last_part, (_tmp10 == NULL) ? NULL : g_strdup (_tmp10));
	/*don't try to find method signature if is a: for, foreach, if, while etc...*/
	if (vtg_symbol_completion_provider_is_vala_keyword (self, method)) {
		VscSymbolCompletionItem* _tmp11;
		_tmp11 = NULL;
		return (_tmp11 = NULL, line = (g_free (line), NULL), word = (g_free (word), NULL), last_part = (g_free (last_part), NULL), first_part = (g_free (first_part), NULL), method = (g_free (method), NULL), _tmp11);
	}
	completion_result = NULL;
	_tmp12 = NULL;
	completion_result = (_tmp12 = vtg_symbol_completion_provider_complete (self, NULL, line, first_part, lineno, colno), (completion_result == NULL) ? NULL : (completion_result = (g_object_unref (completion_result), NULL)), _tmp12);
	_tmp13 = FALSE;
	if (completion_result != NULL) {
		_tmp13 = gee_collection_get_size ((GeeCollection*) completion_result->methods) > 0;
	} else {
		_tmp13 = FALSE;
	}
	if (_tmp13) {
		{
			GeeIterator* _item_it;
			_item_it = gee_iterable_iterator ((GeeIterable*) completion_result->methods);
			while (gee_iterator_next (_item_it)) {
				VscSymbolCompletionItem* item;
				item = (VscSymbolCompletionItem*) gee_iterator_get (_item_it);
				if (_vala_strcmp0 (item->name, method) == 0) {
					VscSymbolCompletionItem* _tmp14;
					_tmp14 = NULL;
					return (_tmp14 = item, (_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL)), line = (g_free (line), NULL), word = (g_free (word), NULL), last_part = (g_free (last_part), NULL), first_part = (g_free (first_part), NULL), method = (g_free (method), NULL), (completion_result == NULL) ? NULL : (completion_result = (g_object_unref (completion_result), NULL)), _tmp14);
				}
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			}
			(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
		}
	}
	_tmp15 = NULL;
	return (_tmp15 = NULL, line = (g_free (line), NULL), word = (g_free (word), NULL), last_part = (g_free (last_part), NULL), first_part = (g_free (first_part), NULL), method = (g_free (method), NULL), (completion_result == NULL) ? NULL : (completion_result = (g_object_unref (completion_result), NULL)), _tmp15);
}


static VscSymbolCompletionResult* vtg_symbol_completion_provider_get_completions (VtgSymbolCompletionProvider* self, VtgSymbolCompletionTrigger* trigger) {
	char* line;
	char* word;
	char* last_part;
	gint lineno;
	gint colno;
	char* _tmp5;
	char* _tmp4;
	char* _tmp3;
	char* _tmp2;
	char* _tmp1;
	char* _tmp0;
	gboolean _tmp6;
	VscSymbolCompletionResult* _tmp8;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (trigger != NULL, NULL);
	line = NULL;
	word = NULL;
	last_part = NULL;
	lineno = 0;
	colno = 0;
	_tmp5 = NULL;
	_tmp4 = NULL;
	_tmp3 = NULL;
	_tmp2 = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	((vtg_symbol_completion_provider_parse_current_line (self, FALSE, &_tmp0, &_tmp2, &_tmp4, &lineno, &colno), word = (_tmp1 = _tmp0, word = (g_free (word), NULL), _tmp1)), last_part = (_tmp3 = _tmp2, last_part = (g_free (last_part), NULL), _tmp3));
	line = (_tmp5 = _tmp4, line = (g_free (line), NULL), _tmp5);
	_tmp6 = FALSE;
	if (word == NULL) {
		_tmp6 = _vala_strcmp0 (word, "") == 0;
	} else {
		_tmp6 = FALSE;
	}
	if (_tmp6) {
		VscSymbolCompletionResult* _tmp7;
		_tmp7 = NULL;
		return (_tmp7 = NULL, line = (g_free (line), NULL), word = (g_free (word), NULL), last_part = (g_free (last_part), NULL), _tmp7);
	}
	_tmp8 = NULL;
	return (_tmp8 = vtg_symbol_completion_provider_complete (self, trigger, line, word, lineno, colno), line = (g_free (line), NULL), word = (g_free (word), NULL), last_part = (g_free (last_part), NULL), _tmp8);
}


static VscSymbolCompletionResult* vtg_symbol_completion_provider_complete (VtgSymbolCompletionProvider* self, VtgSymbolCompletionTrigger* trigger, const char* line, const char* word, gint lineno, gint colno) {
	GError * inner_error;
	VscSymbolCompletionResult* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (line != NULL, NULL);
	g_return_val_if_fail (word != NULL, NULL);
	inner_error = NULL;
	result = NULL;
	{
		char* typename;
		GTimer* timer;
		char* _tmp0;
		char* _tmp1;
		VscSymbolCompletionFilterOptions* options;
		gboolean _tmp2;
		gboolean search_error_domains;
		gboolean search_error_base;
		typename = NULL;
		timer = g_timer_new ();
		_tmp0 = vsc_symbol_completion_get_datatype_name_for_name (self->priv->_completion, word, vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &inner_error);
		if (inner_error != NULL) {
			typename = (g_free (typename), NULL);
			(timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL));
			goto __catch0_g_error;
			goto __finally0;
		}
		_tmp1 = NULL;
		typename = (_tmp1 = _tmp0, typename = (g_free (typename), NULL), _tmp1);
		options = vsc_symbol_completion_filter_options_new ();
		vsc_symbol_completion_filter_options_public_only (options);
		_tmp2 = FALSE;
		if (strstr (line, "= new ") != NULL) {
			_tmp2 = TRUE;
		} else {
			_tmp2 = strstr (line, "=new ") != NULL;
		}
		if (_tmp2) {
			options->constructors = TRUE;
			options->instance_symbols = FALSE;
			options->static_symbols = FALSE;
		}
		search_error_domains = FALSE;
		search_error_base = FALSE;
		if (strstr (line, "throws ") != NULL) {
			search_error_domains = TRUE;
			search_error_base = TRUE;
		} else {
			if (strstr (line, "throw ") != NULL) {
				search_error_domains = TRUE;
			}
		}
		if (typename != NULL) {
			char* _tmp4;
			const char* _tmp3;
			VscSymbolCompletionResult* _tmp5;
			VscSymbolCompletionResult* _tmp6;
			options->static_symbols = FALSE;
			options->error_domains = search_error_domains;
			options->error_base = search_error_base;
			if (_vala_strcmp0 (word, "this") == 0) {
				options->private_symbols = TRUE;
				options->protected_symbols = TRUE;
			} else {
				if (_vala_strcmp0 (word, "base") == 0) {
					options->protected_symbols = TRUE;
				}
			}
			_tmp4 = NULL;
			_tmp3 = NULL;
			options->exclude_type = (_tmp4 = (_tmp3 = typename, (_tmp3 == NULL) ? NULL : g_strdup (_tmp3)), options->exclude_type = (g_free (options->exclude_type), NULL), _tmp4);
			g_timer_start (timer);
			_tmp5 = vsc_symbol_completion_get_completions_for_name (self->priv->_completion, options, typename, vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &inner_error);
			if (inner_error != NULL) {
				typename = (g_free (typename), NULL);
				(timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL));
				(options == NULL) ? NULL : (options = (vsc_symbol_completion_filter_options_unref (options), NULL));
				goto __catch0_g_error;
				goto __finally0;
			}
			_tmp6 = NULL;
			result = (_tmp6 = _tmp5, (result == NULL) ? NULL : (result = (g_object_unref (result), NULL)), _tmp6);
			g_timer_stop (timer);
		} else {
			gboolean _tmp7;
			_tmp7 = FALSE;
			if (g_str_has_prefix (word, "this.") == FALSE) {
				_tmp7 = g_str_has_prefix (word, "base.") == FALSE;
			} else {
				_tmp7 = FALSE;
			}
			if (_tmp7) {
				VscSymbolCompletionResult* _tmp8;
				VscSymbolCompletionResult* _tmp9;
				gboolean _tmp10;
				options->static_symbols = TRUE;
				options->interface_symbols = FALSE;
				options->error_domains = search_error_domains;
				options->error_base = search_error_base;
				g_timer_start (timer);
				_tmp8 = vsc_symbol_completion_get_completions_for_name (self->priv->_completion, options, word, vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &inner_error);
				if (inner_error != NULL) {
					typename = (g_free (typename), NULL);
					(timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL));
					(options == NULL) ? NULL : (options = (vsc_symbol_completion_filter_options_unref (options), NULL));
					goto __catch0_g_error;
					goto __finally0;
				}
				_tmp9 = NULL;
				result = (_tmp9 = _tmp8, (result == NULL) ? NULL : (result = (g_object_unref (result), NULL)), _tmp9);
				g_timer_stop (timer);
				_tmp10 = FALSE;
				if (vsc_symbol_completion_result_get_is_empty (result)) {
					gboolean _tmp11;
					_tmp11 = FALSE;
					if (trigger == NULL) {
						_tmp11 = TRUE;
					} else {
						_tmp11 = trigger->shortcut_triggered;
					}
					_tmp10 = (_tmp11);
				} else {
					_tmp10 = FALSE;
				}
				if (_tmp10) {
					char* _tmp12;
					char* _tmp13;
					g_timer_start (timer);
					_tmp12 = vsc_symbol_completion_get_datatype_name_for_name (self->priv->_completion, "this", vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &inner_error);
					if (inner_error != NULL) {
						typename = (g_free (typename), NULL);
						(timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL));
						(options == NULL) ? NULL : (options = (vsc_symbol_completion_filter_options_unref (options), NULL));
						goto __catch0_g_error;
						goto __finally0;
					}
					_tmp13 = NULL;
					typename = (_tmp13 = _tmp12, typename = (g_free (typename), NULL), _tmp13);
					if (typename != NULL) {
						VscSymbolCompletionResult* _tmp14;
						VscSymbolCompletionResult* _tmp15;
						vsc_symbol_completion_filter_options_defaults (options);
						vsc_symbol_completion_filter_options_public_only (options);
						options->error_domains = search_error_domains;
						options->error_base = search_error_base;
						options->constructors = TRUE;
						options->static_symbols = TRUE;
						options->private_symbols = TRUE;
						options->protected_symbols = TRUE;
						options->local_variables = TRUE;
						options->imported_namespaces = TRUE;
						_tmp14 = vsc_symbol_completion_get_completions_for_name (self->priv->_completion, options, typename, vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &inner_error);
						if (inner_error != NULL) {
							typename = (g_free (typename), NULL);
							(timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL));
							(options == NULL) ? NULL : (options = (vsc_symbol_completion_filter_options_unref (options), NULL));
							goto __catch0_g_error;
							goto __finally0;
						}
						_tmp15 = NULL;
						result = (_tmp15 = _tmp14, (result == NULL) ? NULL : (result = (g_object_unref (result), NULL)), _tmp15);
					}
					g_timer_stop (timer);
				}
			}
		}
		typename = (g_free (typename), NULL);
		(timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL));
		(options == NULL) ? NULL : (options = (vsc_symbol_completion_filter_options_unref (options), NULL));
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgsymbolcompletionprovider.vala:638: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally0:
	if (inner_error != NULL) {
		(result == NULL) ? NULL : (result = (g_object_unref (result), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return NULL;
	}
	return result;
}


static gboolean vtg_symbol_completion_provider_is_vala_keyword (VtgSymbolCompletionProvider* self, const char* keyword) {
	gboolean _tmp0;
	gboolean _tmp1;
	gboolean _tmp2;
	gboolean _tmp3;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (keyword != NULL, FALSE);
	_tmp0 = FALSE;
	_tmp1 = FALSE;
	_tmp2 = FALSE;
	_tmp3 = FALSE;
	if (_vala_strcmp0 (keyword, "if") == 0) {
		_tmp3 = TRUE;
	} else {
		_tmp3 = _vala_strcmp0 (keyword, "for") == 0;
	}
	if (_tmp3) {
		_tmp2 = TRUE;
	} else {
		_tmp2 = _vala_strcmp0 (keyword, "foreach") == 0;
	}
	if (_tmp2) {
		_tmp1 = TRUE;
	} else {
		_tmp1 = _vala_strcmp0 (keyword, "while") == 0;
	}
	if (_tmp1) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = _vala_strcmp0 (keyword, "switch") == 0;
	}
	return (_tmp0);
}


static char* vtg_symbol_completion_provider_get_document_text (VtgSymbolCompletionProvider* self, GeditDocument* doc, gboolean all_doc) {
	GtkTextMark* mark;
	GtkTextIter end = {0};
	GtkTextIter start = {0};
	char* doc_text;
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (doc != NULL, NULL);
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert ((GtkTextBuffer*) doc));
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) doc, &start, mark);
	doc_text = NULL;
	_tmp0 = FALSE;
	if (all_doc) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = gedit_document_is_untouched (doc);
	}
	if (_tmp0) {
		char* _tmp2;
		const char* _tmp1;
		end = start;
		gtk_text_iter_set_line_offset (&start, 0);
		while (gtk_text_iter_backward_line (&start)) {
			;
		}
		while (gtk_text_iter_forward_line (&end)) {
			;
		}
		_tmp2 = NULL;
		_tmp1 = NULL;
		doc_text = (_tmp2 = (_tmp1 = gtk_text_iter_get_text (&start, &end), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), doc_text = (g_free (doc_text), NULL), _tmp2);
	} else {
		const char* _tmp3;
		char* text1;
		char* text2;
		char* _tmp6;
		end = start;
		gtk_text_iter_set_line_offset (&end, 0);
		while (gtk_text_iter_backward_line (&start)) {
			;
		}
		_tmp3 = NULL;
		text1 = (_tmp3 = gtk_text_iter_get_text (&start, &end), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
		text2 = g_strdup ("");
		/*trick: jump the current edited row (there
		are a lot of probability that this row will
		cause a parser error)*/
		if (gtk_text_iter_forward_line (&end)) {
			char* _tmp5;
			const char* _tmp4;
			start = end;
			while (gtk_text_iter_forward_line (&end)) {
				;
			}
			_tmp5 = NULL;
			_tmp4 = NULL;
			text2 = (_tmp5 = (_tmp4 = gtk_text_iter_get_text (&start, &end), (_tmp4 == NULL) ? NULL : g_strdup (_tmp4)), text2 = (g_free (text2), NULL), _tmp5);
		}
		_tmp6 = NULL;
		doc_text = (_tmp6 = g_strdup_printf ("%s\n%s", text1, text2), doc_text = (g_free (doc_text), NULL), _tmp6);
		text1 = (g_free (text1), NULL);
		text2 = (g_free (text2), NULL);
	}
	return doc_text;
}


static void vtg_symbol_completion_provider_set_completion (VtgSymbolCompletionProvider* self, VscSymbolCompletion* value) {
	VscSymbolCompletion* _tmp1;
	VscSymbolCompletion* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_completion = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_completion == NULL) ? NULL : (self->priv->_completion = (g_object_unref (self->priv->_completion), NULL)), _tmp1);
	g_object_notify ((GObject *) self, "completion");
}


static void vtg_symbol_completion_provider_set_view (VtgSymbolCompletionProvider* self, GeditView* value) {
	GeditView* _tmp1;
	GeditView* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_view = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_view == NULL) ? NULL : (self->priv->_view = (g_object_unref (self->priv->_view), NULL)), _tmp1);
	g_object_notify ((GObject *) self, "view");
}


static void vtg_symbol_completion_provider_set_plugin (VtgSymbolCompletionProvider* self, VtgPlugin* value) {
	VtgPlugin* _tmp1;
	VtgPlugin* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_plugin = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL)), _tmp1);
	g_object_notify ((GObject *) self, "plugin");
}


static gboolean _vtg_symbol_completion_provider_on_view_key_press_gtk_widget_key_press_event (GeditView* _sender, const GdkEventKey* event, gpointer self) {
	return vtg_symbol_completion_provider_on_view_key_press (self, _sender, event);
}


static void _vtg_symbol_completion_provider_on_cache_building_vsc_parser_manager_cache_building (VscParserManager* _sender, gpointer self) {
	vtg_symbol_completion_provider_on_cache_building (self, _sender);
}


static void _vtg_symbol_completion_provider_on_cache_builded_vsc_parser_manager_cache_builded (VscParserManager* _sender, gpointer self) {
	vtg_symbol_completion_provider_on_cache_builded (self, _sender);
}


static GObject * vtg_symbol_completion_provider_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgSymbolCompletionProviderClass * klass;
	GObjectClass * parent_class;
	VtgSymbolCompletionProvider * self;
	GError * inner_error;
	klass = VTG_SYMBOL_COMPLETION_PROVIDER_CLASS (g_type_class_peek (VTG_TYPE_SYMBOL_COMPLETION_PROVIDER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_SYMBOL_COMPLETION_PROVIDER (obj);
	inner_error = NULL;
	{
		{
			GeditDocument* _tmp0;
			GeditDocument* doc;
			const char* _tmp1;
			char* name;
			VscSourceBuffer* _tmp4;
			GdkPixbuf* _tmp5;
			GdkPixbuf* _tmp7;
			GdkPixbuf* _tmp6;
			GdkPixbuf* _tmp9;
			char* _tmp8;
			GdkPixbuf* _tmp11;
			char* _tmp10;
			GdkPixbuf* _tmp13;
			char* _tmp12;
			GdkPixbuf* _tmp15;
			char* _tmp14;
			GdkPixbuf* _tmp17;
			char* _tmp16;
			GdkPixbuf* _tmp19;
			char* _tmp18;
			GdkPixbuf* _tmp21;
			char* _tmp20;
			GdkPixbuf* _tmp23;
			char* _tmp22;
			GdkPixbuf* _tmp25;
			char* _tmp24;
			GdkPixbuf* _tmp27;
			char* _tmp26;
			GeditStatusbar* _tmp28;
			GeditStatusbar* status_bar;
			_tmp0 = NULL;
			doc = (_tmp0 = GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
			_tmp1 = NULL;
			name = (_tmp1 = gedit_document_get_uri (doc), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
			if (name == NULL) {
				char* _tmp3;
				const char* _tmp2;
				_tmp3 = NULL;
				_tmp2 = NULL;
				name = (_tmp3 = (_tmp2 = gedit_document_get_short_name_for_display (doc), (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), name = (g_free (name), NULL), _tmp3);
			}
			_tmp4 = NULL;
			self->priv->_sb = (_tmp4 = vsc_source_buffer_new (name, NULL), (self->priv->_sb == NULL) ? NULL : (self->priv->_sb = (g_object_unref (self->priv->_sb), NULL)), _tmp4);
			g_signal_connect_object ((GtkWidget*) self->priv->_view, "key-press-event", (GCallback) _vtg_symbol_completion_provider_on_view_key_press_gtk_widget_key_press_event, self, 0);
			vsc_parser_manager_add_source_buffer (vsc_symbol_completion_get_parser (self->priv->_completion), self->priv->_sb, &inner_error);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp5 = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), GTK_STOCK_FILE, 16, GTK_ICON_LOOKUP_GENERIC_FALLBACK, &inner_error);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp7 = NULL;
			_tmp6 = NULL;
			self->priv->_icon_generic = (_tmp7 = (_tmp6 = _tmp5, (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6)), (self->priv->_icon_generic == NULL) ? NULL : (self->priv->_icon_generic = (g_object_unref (self->priv->_icon_generic), NULL)), _tmp7);
			_tmp9 = NULL;
			_tmp8 = NULL;
			self->priv->_icon_field = (_tmp9 = gdk_pixbuf_new_from_file (_tmp8 = vtg_utils_get_image_path ("element-field-16.png"), &inner_error), (self->priv->_icon_field == NULL) ? NULL : (self->priv->_icon_field = (g_object_unref (self->priv->_icon_field), NULL)), _tmp9);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp8 = (g_free (_tmp8), NULL);
			_tmp11 = NULL;
			_tmp10 = NULL;
			self->priv->_icon_method = (_tmp11 = gdk_pixbuf_new_from_file (_tmp10 = vtg_utils_get_image_path ("element-method-16.png"), &inner_error), (self->priv->_icon_method == NULL) ? NULL : (self->priv->_icon_method = (g_object_unref (self->priv->_icon_method), NULL)), _tmp11);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp10 = (g_free (_tmp10), NULL);
			_tmp13 = NULL;
			_tmp12 = NULL;
			self->priv->_icon_class = (_tmp13 = gdk_pixbuf_new_from_file (_tmp12 = vtg_utils_get_image_path ("element-class-16.png"), &inner_error), (self->priv->_icon_class == NULL) ? NULL : (self->priv->_icon_class = (g_object_unref (self->priv->_icon_class), NULL)), _tmp13);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp12 = (g_free (_tmp12), NULL);
			_tmp15 = NULL;
			_tmp14 = NULL;
			self->priv->_icon_struct = (_tmp15 = gdk_pixbuf_new_from_file (_tmp14 = vtg_utils_get_image_path ("element-structure-16.png"), &inner_error), (self->priv->_icon_struct == NULL) ? NULL : (self->priv->_icon_struct = (g_object_unref (self->priv->_icon_struct), NULL)), _tmp15);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp14 = (g_free (_tmp14), NULL);
			_tmp17 = NULL;
			_tmp16 = NULL;
			self->priv->_icon_property = (_tmp17 = gdk_pixbuf_new_from_file (_tmp16 = vtg_utils_get_image_path ("element-property-16.png"), &inner_error), (self->priv->_icon_property == NULL) ? NULL : (self->priv->_icon_property = (g_object_unref (self->priv->_icon_property), NULL)), _tmp17);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp16 = (g_free (_tmp16), NULL);
			_tmp19 = NULL;
			_tmp18 = NULL;
			self->priv->_icon_signal = (_tmp19 = gdk_pixbuf_new_from_file (_tmp18 = vtg_utils_get_image_path ("element-event-16.png"), &inner_error), (self->priv->_icon_signal == NULL) ? NULL : (self->priv->_icon_signal = (g_object_unref (self->priv->_icon_signal), NULL)), _tmp19);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp18 = (g_free (_tmp18), NULL);
			_tmp21 = NULL;
			_tmp20 = NULL;
			self->priv->_icon_iface = (_tmp21 = gdk_pixbuf_new_from_file (_tmp20 = vtg_utils_get_image_path ("element-interface-16.png"), &inner_error), (self->priv->_icon_iface == NULL) ? NULL : (self->priv->_icon_iface = (g_object_unref (self->priv->_icon_iface), NULL)), _tmp21);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp20 = (g_free (_tmp20), NULL);
			_tmp23 = NULL;
			_tmp22 = NULL;
			self->priv->_icon_enum = (_tmp23 = gdk_pixbuf_new_from_file (_tmp22 = vtg_utils_get_image_path ("element-enumeration-16.png"), &inner_error), (self->priv->_icon_enum == NULL) ? NULL : (self->priv->_icon_enum = (g_object_unref (self->priv->_icon_enum), NULL)), _tmp23);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp22 = (g_free (_tmp22), NULL);
			_tmp25 = NULL;
			_tmp24 = NULL;
			self->priv->_icon_const = (_tmp25 = gdk_pixbuf_new_from_file (_tmp24 = vtg_utils_get_image_path ("element-literal-16.png"), &inner_error), (self->priv->_icon_const == NULL) ? NULL : (self->priv->_icon_const = (g_object_unref (self->priv->_icon_const), NULL)), _tmp25);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp24 = (g_free (_tmp24), NULL);
			_tmp27 = NULL;
			_tmp26 = NULL;
			self->priv->_icon_namespace = (_tmp27 = gdk_pixbuf_new_from_file (_tmp26 = vtg_utils_get_image_path ("element-namespace-16.png"), &inner_error), (self->priv->_icon_namespace == NULL) ? NULL : (self->priv->_icon_namespace = (g_object_unref (self->priv->_icon_namespace), NULL)), _tmp27);
			if (inner_error != NULL) {
				(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
				name = (g_free (name), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp26 = (g_free (_tmp26), NULL);
			g_signal_connect_object (vsc_symbol_completion_get_parser (self->priv->_completion), "cache-building", (GCallback) _vtg_symbol_completion_provider_on_cache_building_vsc_parser_manager_cache_building, self, 0);
			g_signal_connect_object (vsc_symbol_completion_get_parser (self->priv->_completion), "cache-builded", (GCallback) _vtg_symbol_completion_provider_on_cache_builded_vsc_parser_manager_cache_builded, self, 0);
			_tmp28 = NULL;
			status_bar = (_tmp28 = GEDIT_STATUSBAR (gedit_window_get_statusbar (vtg_plugin_get_gedit_window (self->priv->_plugin))), (_tmp28 == NULL) ? NULL : g_object_ref (_tmp28));
			self->priv->sb_context_id = gtk_statusbar_get_context_id ((GtkStatusbar*) status_bar, "symbol status");
			self->priv->cache_building = TRUE;
			self->priv->all_doc = TRUE;
			vtg_symbol_completion_provider_parse (self, doc);
			(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			name = (g_free (name), NULL);
			(status_bar == NULL) ? NULL : (status_bar = (g_object_unref (status_bar), NULL));
		}
		goto __finally1;
		__catch1_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				g_warning ("vtgsymbolcompletionprovider.vala:135: an error occourred: %s", err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally1:
		if (inner_error != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
	}
	return obj;
}


static void vtg_symbol_completion_provider_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgSymbolCompletionProvider * self;
	self = VTG_SYMBOL_COMPLETION_PROVIDER (object);
	switch (property_id) {
		case VTG_SYMBOL_COMPLETION_PROVIDER_COMPLETION:
		vtg_symbol_completion_provider_set_completion (self, g_value_get_object (value));
		break;
		case VTG_SYMBOL_COMPLETION_PROVIDER_VIEW:
		vtg_symbol_completion_provider_set_view (self, g_value_get_object (value));
		break;
		case VTG_SYMBOL_COMPLETION_PROVIDER_PLUGIN:
		vtg_symbol_completion_provider_set_plugin (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_symbol_completion_provider_class_init (VtgSymbolCompletionProviderClass * klass) {
	vtg_symbol_completion_provider_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgSymbolCompletionProviderPrivate));
	G_OBJECT_CLASS (klass)->set_property = vtg_symbol_completion_provider_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_symbol_completion_provider_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_symbol_completion_provider_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_PROVIDER_COMPLETION, g_param_spec_object ("completion", "completion", "completion", VSC_TYPE_SYMBOL_COMPLETION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_PROVIDER_VIEW, g_param_spec_object ("view", "view", "view", GEDIT_TYPE_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_PROVIDER_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VTG_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vtg_symbol_completion_provider_gsc_provider_interface_init (GscProviderIface * iface) {
	vtg_symbol_completion_provider_gsc_provider_parent_iface = g_type_interface_peek_parent (iface);
	iface->finish = vtg_symbol_completion_provider_real_finish;
	iface->get_name = vtg_symbol_completion_provider_real_get_name;
	iface->get_proposals = vtg_symbol_completion_provider_real_get_proposals;
}


static void vtg_symbol_completion_provider_instance_init (VtgSymbolCompletionProvider * self) {
	self->priv = VTG_SYMBOL_COMPLETION_PROVIDER_GET_PRIVATE (self);
	self->priv->_sb = NULL;
	self->priv->_completion = NULL;
	self->priv->_view = NULL;
	self->priv->timeout_id = (guint) 0;
	self->priv->idle_id = (guint) 0;
	self->priv->all_doc = FALSE;
	self->priv->prealloc_index = 0;
	self->priv->counter = 0;
	self->priv->cache_building = FALSE;
	self->priv->prev_cache_building = FALSE;
	self->priv->tooltip_is_visible = FALSE;
	self->priv->_last_trigger = NULL;
	self->priv->sb_msg_id = (guint) 0;
	self->priv->sb_context_id = (guint) 0;
	self->priv->_calltip_window = NULL;
	self->priv->need_parse = TRUE;
	self->priv->current_edited_line = -1;
}


static void vtg_symbol_completion_provider_finalize (GObject* obj) {
	VtgSymbolCompletionProvider * self;
	self = VTG_SYMBOL_COMPLETION_PROVIDER (obj);
	{
		guint _tmp29;
		guint _tmp30;
		guint _tmp31;
		g_signal_handlers_disconnect_matched ((GtkWidget*) self->priv->_view, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("key-press-event", GTK_TYPE_WIDGET, &_tmp29, NULL, FALSE), _tmp29), 0, NULL, (GCallback) _vtg_symbol_completion_provider_on_view_key_press_gtk_widget_key_press_event, self);
		g_signal_handlers_disconnect_matched (vsc_symbol_completion_get_parser (self->priv->_completion), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("cache-building", VSC_TYPE_PARSER_MANAGER, &_tmp30, NULL, FALSE), _tmp30), 0, NULL, (GCallback) _vtg_symbol_completion_provider_on_cache_building_vsc_parser_manager_cache_building, self);
		g_signal_handlers_disconnect_matched (vsc_symbol_completion_get_parser (self->priv->_completion), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("cache-builded", VSC_TYPE_PARSER_MANAGER, &_tmp31, NULL, FALSE), _tmp31), 0, NULL, (GCallback) _vtg_symbol_completion_provider_on_cache_builded_vsc_parser_manager_cache_builded, self);
		if (self->priv->sb_msg_id != 0) {
			GeditStatusbar* _tmp32;
			GeditStatusbar* status_bar;
			_tmp32 = NULL;
			status_bar = (_tmp32 = GEDIT_STATUSBAR (gedit_window_get_statusbar (vtg_plugin_get_gedit_window (self->priv->_plugin))), (_tmp32 == NULL) ? NULL : g_object_ref (_tmp32));
			gtk_statusbar_remove ((GtkStatusbar*) status_bar, self->priv->sb_context_id, self->priv->sb_msg_id);
			(status_bar == NULL) ? NULL : (status_bar = (g_object_unref (status_bar), NULL));
		}
		if (self->priv->timeout_id != 0) {
			g_source_remove (self->priv->timeout_id);
		}
		if (self->priv->idle_id != 0) {
			g_source_remove (self->priv->idle_id);
		}
		vsc_parser_manager_remove_source_buffer (vsc_symbol_completion_get_parser (self->priv->_completion), self->priv->_sb);
	}
	(self->priv->_sb == NULL) ? NULL : (self->priv->_sb = (g_object_unref (self->priv->_sb), NULL));
	(self->priv->_completion == NULL) ? NULL : (self->priv->_completion = (g_object_unref (self->priv->_completion), NULL));
	(self->priv->_view == NULL) ? NULL : (self->priv->_view = (g_object_unref (self->priv->_view), NULL));
	(self->priv->list == NULL) ? NULL : (self->priv->list = (_g_list_free_g_object_unref (self->priv->list), NULL));
	(self->priv->_icon_generic == NULL) ? NULL : (self->priv->_icon_generic = (g_object_unref (self->priv->_icon_generic), NULL));
	(self->priv->_icon_field == NULL) ? NULL : (self->priv->_icon_field = (g_object_unref (self->priv->_icon_field), NULL));
	(self->priv->_icon_method == NULL) ? NULL : (self->priv->_icon_method = (g_object_unref (self->priv->_icon_method), NULL));
	(self->priv->_icon_class == NULL) ? NULL : (self->priv->_icon_class = (g_object_unref (self->priv->_icon_class), NULL));
	(self->priv->_icon_struct == NULL) ? NULL : (self->priv->_icon_struct = (g_object_unref (self->priv->_icon_struct), NULL));
	(self->priv->_icon_property == NULL) ? NULL : (self->priv->_icon_property = (g_object_unref (self->priv->_icon_property), NULL));
	(self->priv->_icon_signal == NULL) ? NULL : (self->priv->_icon_signal = (g_object_unref (self->priv->_icon_signal), NULL));
	(self->priv->_icon_iface == NULL) ? NULL : (self->priv->_icon_iface = (g_object_unref (self->priv->_icon_iface), NULL));
	(self->priv->_icon_const == NULL) ? NULL : (self->priv->_icon_const = (g_object_unref (self->priv->_icon_const), NULL));
	(self->priv->_icon_enum == NULL) ? NULL : (self->priv->_icon_enum = (g_object_unref (self->priv->_icon_enum), NULL));
	(self->priv->_icon_namespace == NULL) ? NULL : (self->priv->_icon_namespace = (g_object_unref (self->priv->_icon_namespace), NULL));
	(self->priv->_last_trigger == NULL) ? NULL : (self->priv->_last_trigger = (g_object_unref (self->priv->_last_trigger), NULL));
	(self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL));
	(self->priv->_calltip_window == NULL) ? NULL : (self->priv->_calltip_window = (g_object_unref (self->priv->_calltip_window), NULL));
	G_OBJECT_CLASS (vtg_symbol_completion_provider_parent_class)->finalize (obj);
}


static GType vtg_symbol_completion_provider_type_id = 0;
GType vtg_symbol_completion_provider_get_type (void) {
	return vtg_symbol_completion_provider_type_id;
}


GType vtg_symbol_completion_provider_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgSymbolCompletionProviderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_symbol_completion_provider_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgSymbolCompletionProvider), 0, (GInstanceInitFunc) vtg_symbol_completion_provider_instance_init, NULL };
	static const GInterfaceInfo gsc_provider_info = { (GInterfaceInitFunc) vtg_symbol_completion_provider_gsc_provider_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	vtg_symbol_completion_provider_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgSymbolCompletionProvider", &g_define_type_info, 0);
	g_type_add_interface_static (vtg_symbol_completion_provider_type_id, GSC_TYPE_PROVIDER, &gsc_provider_info);
	return vtg_symbol_completion_provider_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




