/*
 *  vtgsymbolcompletiontrigger.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgsymbolcompletiontrigger.h"
#include <gtksourcecompletion/gsc-utils.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "vtgutils.h"




static char* string_replace (const char* self, const char* old, const char* replacement);
struct _VtgSymbolCompletionTriggerPrivate {
	GscManager* _completion;
	char* _trigger_name;
	GscManagerEventOptions _opts;
};

#define VTG_SYMBOL_COMPLETION_TRIGGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_SYMBOL_COMPLETION_TRIGGER, VtgSymbolCompletionTriggerPrivate))
enum  {
	VTG_SYMBOL_COMPLETION_TRIGGER_DUMMY_PROPERTY,
	VTG_SYMBOL_COMPLETION_TRIGGER_COMPLETION,
	VTG_SYMBOL_COMPLETION_TRIGGER_TRIGGER_NAME
};
static gboolean _vtg_symbol_completion_trigger_on_view_key_press_gtk_widget_key_press_event (GtkTextView* _sender, const GdkEventKey* event, gpointer self);
static void _vtg_symbol_completion_trigger_on_buffer_changed_gtk_text_buffer_changed (GtkTextBuffer* _sender, gpointer self);
static gboolean vtg_symbol_completion_trigger_real_activate (GscTrigger* base);
static gboolean vtg_symbol_completion_trigger_real_deactivate (GscTrigger* base);
static const char* vtg_symbol_completion_trigger_real_get_name (GscTrigger* base);
static void vtg_symbol_completion_trigger_on_buffer_changed (VtgSymbolCompletionTrigger* self, GtkTextBuffer* sender);
static gboolean vtg_symbol_completion_trigger_on_view_key_press (VtgSymbolCompletionTrigger* self, GtkTextView* view, const GdkEventKey* event);
static char* vtg_symbol_completion_trigger_get_filter_word (VtgSymbolCompletionTrigger* self, GtkTextBuffer* doc, char** delimiter);
static gboolean vtg_symbol_completion_trigger_is_word_delimiter (VtgSymbolCompletionTrigger* self, gunichar ch);
static void vtg_symbol_completion_trigger_set_completion (VtgSymbolCompletionTrigger* self, GscManager* value);
static void vtg_symbol_completion_trigger_set_trigger_name (VtgSymbolCompletionTrigger* self, const char* value);
static gpointer vtg_symbol_completion_trigger_parent_class = NULL;
static GscTriggerIface* vtg_symbol_completion_trigger_gsc_trigger_parent_iface = NULL;
static void vtg_symbol_completion_trigger_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



static char* string_replace (const char* self, const char* old, const char* replacement) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (old != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	inner_error = NULL;
	{
		char* _tmp0;
		GRegex* _tmp1;
		GRegex* regex;
		char* _tmp2;
		char* _tmp3;
		_tmp0 = NULL;
		_tmp1 = NULL;
		regex = (_tmp1 = g_regex_new (_tmp0 = g_regex_escape_string (old, -1), 0, 0, &inner_error), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
		if (inner_error != NULL) {
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch2_g_regex_error;
			}
			goto __finally2;
		}
		_tmp2 = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &inner_error);
		if (inner_error != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch2_g_regex_error;
			}
			goto __finally2;
		}
		_tmp3 = NULL;
		return (_tmp3 = _tmp2, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp3);
	}
	goto __finally2;
	__catch2_g_regex_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_assert_not_reached ();
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally2:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return NULL;
	}
}


static gboolean _vtg_symbol_completion_trigger_on_view_key_press_gtk_widget_key_press_event (GtkTextView* _sender, const GdkEventKey* event, gpointer self) {
	return vtg_symbol_completion_trigger_on_view_key_press (self, _sender, event);
}


static void _vtg_symbol_completion_trigger_on_buffer_changed_gtk_text_buffer_changed (GtkTextBuffer* _sender, gpointer self) {
	vtg_symbol_completion_trigger_on_buffer_changed (self, _sender);
}


static gboolean vtg_symbol_completion_trigger_real_activate (GscTrigger* base) {
	VtgSymbolCompletionTrigger * self;
	GtkTextView* _tmp0;
	GtkTextView* view;
	gboolean _tmp1;
	self = (VtgSymbolCompletionTrigger*) base;
	_tmp0 = NULL;
	view = (_tmp0 = gsc_manager_get_view (self->priv->_completion), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	g_signal_connect_object ((GtkWidget*) view, "key-press-event", (GCallback) _vtg_symbol_completion_trigger_on_view_key_press_gtk_widget_key_press_event, self, 0);
	g_signal_connect_object (gtk_text_view_get_buffer (view), "changed", (GCallback) _vtg_symbol_completion_trigger_on_buffer_changed_gtk_text_buffer_changed, self, 0);
	return (_tmp1 = TRUE, (view == NULL) ? NULL : (view = (g_object_unref (view), NULL)), _tmp1);
}


static gboolean vtg_symbol_completion_trigger_real_deactivate (GscTrigger* base) {
	VtgSymbolCompletionTrigger * self;
	GtkTextView* _tmp0;
	GtkTextView* view;
	guint _tmp1;
	gboolean _tmp2;
	self = (VtgSymbolCompletionTrigger*) base;
	_tmp0 = NULL;
	view = (_tmp0 = gsc_manager_get_view (self->priv->_completion), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	g_signal_handlers_disconnect_matched ((GtkWidget*) view, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("key-press-event", GTK_TYPE_WIDGET, &_tmp1, NULL, FALSE), _tmp1), 0, NULL, (GCallback) _vtg_symbol_completion_trigger_on_view_key_press_gtk_widget_key_press_event, self);
	return (_tmp2 = TRUE, (view == NULL) ? NULL : (view = (g_object_unref (view), NULL)), _tmp2);
}


static const char* vtg_symbol_completion_trigger_real_get_name (GscTrigger* base) {
	VtgSymbolCompletionTrigger * self;
	self = (VtgSymbolCompletionTrigger*) base;
	return self->priv->_trigger_name;
}


static void vtg_symbol_completion_trigger_on_buffer_changed (VtgSymbolCompletionTrigger* self, GtkTextBuffer* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	if (gsc_manager_is_visible (self->priv->_completion)) {
		char* delimiter;
		char* _tmp2;
		char* _tmp1;
		char* _tmp0;
		char* filter;
		gboolean _tmp3;
		delimiter = NULL;
		_tmp2 = NULL;
		_tmp1 = NULL;
		_tmp0 = NULL;
		filter = (_tmp1 = vtg_symbol_completion_trigger_get_filter_word (self, sender, &_tmp0), delimiter = (_tmp2 = _tmp0, delimiter = (g_free (delimiter), NULL), _tmp2), _tmp1);
		_tmp3 = FALSE;
		if (_vala_strcmp0 (delimiter, ".") != 0) {
			_tmp3 = _vala_strcmp0 (delimiter, "") != 0;
		} else {
			_tmp3 = FALSE;
		}
		if (_tmp3) {
			gsc_manager_finish_completion (self->priv->_completion);
		} else {
			GscManagerEventOptions opts = {0};
			gsc_manager_get_current_event_options (self->priv->_completion, &opts);
			opts.filter_text = filter;
			gsc_manager_update_event_options (self->priv->_completion, &opts);
		}
		delimiter = (g_free (delimiter), NULL);
		filter = (g_free (filter), NULL);
	}
}


void vtg_symbol_completion_trigger_complete_word (VtgSymbolCompletionTrigger* self) {
	g_return_if_fail (self != NULL);
	vtg_symbol_completion_trigger_trigger_event (self, TRUE);
}


static gboolean vtg_symbol_completion_trigger_on_view_key_press (VtgSymbolCompletionTrigger* self, GtkTextView* view, const GdkEventKey* event) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (view != NULL, FALSE);
	if (!gsc_manager_is_visible (self->priv->_completion)) {
		gboolean _tmp0;
		_tmp0 = FALSE;
		if ((*event).keyval == '.') {
			_tmp0 = ((*event).state & ((GDK_SHIFT_MASK | GDK_META_MASK) | GDK_CONTROL_MASK)) == 0;
		} else {
			_tmp0 = FALSE;
		}
		if (_tmp0) {
			vtg_symbol_completion_trigger_trigger_event (self, FALSE);
		}
	}
	return FALSE;
}


static char* vtg_symbol_completion_trigger_get_filter_word (VtgSymbolCompletionTrigger* self, GtkTextBuffer* doc, char** delimiter) {
	GtkTextMark* mark;
	GtkTextIter end = {0};
	GtkTextIter start = {0};
	char* result;
	gint col;
	char* _tmp0;
	char* _tmp8;
	const char* _tmp7;
	const char* _tmp9;
	const char* _tmp10;
	char* _tmp11;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (doc != NULL, NULL);
	*delimiter = NULL;
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert (doc));
	result = NULL;
	gtk_text_buffer_get_iter_at_mark (doc, &start, mark);
	end = start;
	col = gtk_text_iter_get_line_offset (&start);
	_tmp0 = NULL;
	(*delimiter) = (_tmp0 = g_strdup (""), (*delimiter) = (g_free ((*delimiter)), NULL), _tmp0);
	while (col > 0) {
		char* _tmp2;
		const char* _tmp1;
		gunichar ch;
		gtk_text_iter_backward_char (&start);
		_tmp2 = NULL;
		_tmp1 = NULL;
		result = (_tmp2 = (_tmp1 = g_strstrip (gtk_text_iter_get_text (&start, &end)), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), result = (g_free (result), NULL), _tmp2);
		ch = gtk_text_iter_get_char (&start);
		if (vtg_symbol_completion_trigger_is_word_delimiter (self, ch)) {
			GtkTextIter delim;
			char* _tmp4;
			const char* _tmp3;
			delim = start;
			gtk_text_iter_forward_char (&start);
			_tmp4 = NULL;
			_tmp3 = NULL;
			(*delimiter) = (_tmp4 = (_tmp3 = gtk_text_iter_get_text (&delim, &start), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3)), (*delimiter) = (g_free ((*delimiter)), NULL), _tmp4);
			break;
		}
		col--;
	}
	if (!vtg_string_utils_is_null_or_empty ((*delimiter))) {
		char* _tmp5;
		char* _tmp6;
		_tmp5 = NULL;
		(*delimiter) = (_tmp5 = string_replace ((*delimiter), " ", ""), (*delimiter) = (g_free ((*delimiter)), NULL), _tmp5);
		/*TODO: replace with trim!*/
		_tmp6 = NULL;
		(*delimiter) = (_tmp6 = string_replace ((*delimiter), "\t", ""), (*delimiter) = (g_free ((*delimiter)), NULL), _tmp6);
	}
	/*TODO: replace with trim!*/
	_tmp8 = NULL;
	_tmp7 = NULL;
	result = (_tmp8 = (_tmp7 = g_strstrip (gtk_text_iter_get_text (&start, &end)), (_tmp7 == NULL) ? NULL : g_strdup (_tmp7)), result = (g_free (result), NULL), _tmp8);
	_tmp9 = NULL;
	if (result == NULL) {
		_tmp9 = "";
	} else {
		_tmp9 = result;
	}
	_tmp10 = NULL;
	_tmp11 = NULL;
	return (_tmp11 = (_tmp10 = (_tmp9), (_tmp10 == NULL) ? NULL : g_strdup (_tmp10)), result = (g_free (result), NULL), _tmp11);
}


static gboolean vtg_symbol_completion_trigger_is_word_delimiter (VtgSymbolCompletionTrigger* self, gunichar ch) {
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0 = FALSE;
	if (!g_unichar_isalnum (ch)) {
		_tmp0 = ch != '_';
	} else {
		_tmp0 = FALSE;
	}
	return _tmp0;
}


void vtg_symbol_completion_trigger_trigger_event (VtgSymbolCompletionTrigger* self, gboolean trigger_reason) {
	g_return_if_fail (self != NULL);
	self->priv->_opts.position_type = GSC_POPUP_POSITION_CURSOR;
	self->priv->_opts.filter_type = GSC_POPUP_FILTER_TREE_HIDDEN;
	self->priv->_opts.autoselect = trigger_reason;
	self->priv->_opts.show_bottom_bar = TRUE;
	self->shortcut_triggered = trigger_reason;
	gsc_manager_trigger_event_with_opts (self->priv->_completion, self->priv->_trigger_name, &self->priv->_opts, NULL);
	/*if completion was trigger explicitly set the filter*/
	if (trigger_reason) {
		GtkTextView* _tmp0;
		GtkTextView* view;
		GtkTextBuffer* _tmp1;
		GtkTextBuffer* doc;
		char* delimiter;
		char* _tmp4;
		char* _tmp3;
		char* _tmp2;
		char* filter;
		GscManagerEventOptions opts = {0};
		_tmp0 = NULL;
		view = (_tmp0 = gsc_manager_get_view (self->priv->_completion), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
		_tmp1 = NULL;
		doc = (_tmp1 = gtk_text_view_get_buffer (view), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
		delimiter = NULL;
		_tmp4 = NULL;
		_tmp3 = NULL;
		_tmp2 = NULL;
		filter = (_tmp3 = vtg_symbol_completion_trigger_get_filter_word (self, doc, &_tmp2), delimiter = (_tmp4 = _tmp2, delimiter = (g_free (delimiter), NULL), _tmp4), _tmp3);
		gsc_manager_get_current_event_options (self->priv->_completion, &opts);
		opts.filter_text = filter;
		gsc_manager_update_event_options (self->priv->_completion, &opts);
		(view == NULL) ? NULL : (view = (g_object_unref (view), NULL));
		(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
		delimiter = (g_free (delimiter), NULL);
		filter = (g_free (filter), NULL);
	}
}


VtgSymbolCompletionTrigger* vtg_symbol_completion_trigger_construct (GType object_type, GscManager* completion, const char* trigger_name) {
	GParameter * __params;
	GParameter * __params_it;
	VtgSymbolCompletionTrigger * self;
	g_return_val_if_fail (completion != NULL, NULL);
	g_return_val_if_fail (trigger_name != NULL, NULL);
	__params = g_new0 (GParameter, 2);
	__params_it = __params;
	__params_it->name = "completion";
	g_value_init (&__params_it->value, GSC_TYPE_MANAGER);
	g_value_set_object (&__params_it->value, completion);
	__params_it++;
	__params_it->name = "trigger-name";
	g_value_init (&__params_it->value, G_TYPE_STRING);
	g_value_set_string (&__params_it->value, trigger_name);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgSymbolCompletionTrigger* vtg_symbol_completion_trigger_new (GscManager* completion, const char* trigger_name) {
	return vtg_symbol_completion_trigger_construct (VTG_TYPE_SYMBOL_COMPLETION_TRIGGER, completion, trigger_name);
}


GscManager* vtg_symbol_completion_trigger_get_completion (VtgSymbolCompletionTrigger* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_completion;
}


static void vtg_symbol_completion_trigger_set_completion (VtgSymbolCompletionTrigger* self, GscManager* value) {
	GscManager* _tmp2;
	GscManager* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_completion = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_completion == NULL) ? NULL : (self->priv->_completion = (g_object_unref (self->priv->_completion), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "completion");
}


static void vtg_symbol_completion_trigger_set_trigger_name (VtgSymbolCompletionTrigger* self, const char* value) {
	char* _tmp1;
	const char* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_trigger_name = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->priv->_trigger_name = (g_free (self->priv->_trigger_name), NULL), _tmp1);
	g_object_notify ((GObject *) self, "trigger-name");
}


static void vtg_symbol_completion_trigger_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgSymbolCompletionTrigger * self;
	gpointer boxed;
	self = VTG_SYMBOL_COMPLETION_TRIGGER (object);
	switch (property_id) {
		case VTG_SYMBOL_COMPLETION_TRIGGER_COMPLETION:
		g_value_set_object (value, vtg_symbol_completion_trigger_get_completion (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_symbol_completion_trigger_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgSymbolCompletionTrigger * self;
	self = VTG_SYMBOL_COMPLETION_TRIGGER (object);
	switch (property_id) {
		case VTG_SYMBOL_COMPLETION_TRIGGER_COMPLETION:
		vtg_symbol_completion_trigger_set_completion (self, g_value_get_object (value));
		break;
		case VTG_SYMBOL_COMPLETION_TRIGGER_TRIGGER_NAME:
		vtg_symbol_completion_trigger_set_trigger_name (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_symbol_completion_trigger_class_init (VtgSymbolCompletionTriggerClass * klass) {
	vtg_symbol_completion_trigger_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgSymbolCompletionTriggerPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_symbol_completion_trigger_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_symbol_completion_trigger_set_property;
	G_OBJECT_CLASS (klass)->finalize = vtg_symbol_completion_trigger_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_TRIGGER_COMPLETION, g_param_spec_object ("completion", "completion", "completion", GSC_TYPE_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_TRIGGER_TRIGGER_NAME, g_param_spec_string ("trigger-name", "trigger-name", "trigger-name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vtg_symbol_completion_trigger_gsc_trigger_interface_init (GscTriggerIface * iface) {
	vtg_symbol_completion_trigger_gsc_trigger_parent_iface = g_type_interface_peek_parent (iface);
	iface->activate = vtg_symbol_completion_trigger_real_activate;
	iface->deactivate = vtg_symbol_completion_trigger_real_deactivate;
	iface->get_name = vtg_symbol_completion_trigger_real_get_name;
}


static void vtg_symbol_completion_trigger_instance_init (VtgSymbolCompletionTrigger * self) {
	self->priv = VTG_SYMBOL_COMPLETION_TRIGGER_GET_PRIVATE (self);
	self->shortcut_triggered = FALSE;
}


static void vtg_symbol_completion_trigger_finalize (GObject* obj) {
	VtgSymbolCompletionTrigger * self;
	self = VTG_SYMBOL_COMPLETION_TRIGGER (obj);
	(self->priv->_completion == NULL) ? NULL : (self->priv->_completion = (g_object_unref (self->priv->_completion), NULL));
	self->priv->_trigger_name = (g_free (self->priv->_trigger_name), NULL);
	G_OBJECT_CLASS (vtg_symbol_completion_trigger_parent_class)->finalize (obj);
}


static GType vtg_symbol_completion_trigger_type_id = 0;
GType vtg_symbol_completion_trigger_get_type (void) {
	return vtg_symbol_completion_trigger_type_id;
}


GType vtg_symbol_completion_trigger_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgSymbolCompletionTriggerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_symbol_completion_trigger_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgSymbolCompletionTrigger), 0, (GInstanceInitFunc) vtg_symbol_completion_trigger_instance_init, NULL };
	static const GInterfaceInfo gsc_trigger_info = { (GInterfaceInitFunc) vtg_symbol_completion_trigger_gsc_trigger_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	vtg_symbol_completion_trigger_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgSymbolCompletionTrigger", &g_define_type_info, 0);
	g_type_add_interface_static (vtg_symbol_completion_trigger_type_id, GSC_TYPE_TRIGGER, &gsc_trigger_info);
	return vtg_symbol_completion_trigger_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




