/*
 *  vtgutils.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#ifndef __VTGUTILS_H__
#define __VTGUTILS_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gtksourcecompletion/gsc-proposal.h>
#include <gee/list.h>

G_BEGIN_DECLS


#define VTG_TYPE_CACHES (vtg_caches_get_type ())
#define VTG_CACHES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_CACHES, VtgCaches))
#define VTG_CACHES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_CACHES, VtgCachesClass))
#define VTG_IS_CACHES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_CACHES))
#define VTG_IS_CACHES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_CACHES))
#define VTG_CACHES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_CACHES, VtgCachesClass))

typedef struct _VtgCaches VtgCaches;
typedef struct _VtgCachesClass VtgCachesClass;
typedef struct _VtgCachesPrivate VtgCachesPrivate;
typedef struct _VtgParamSpecCaches VtgParamSpecCaches;

#define VTG_TYPE_UTILS (vtg_utils_get_type ())
#define VTG_UTILS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_UTILS, VtgUtils))
#define VTG_UTILS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_UTILS, VtgUtilsClass))
#define VTG_IS_UTILS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_UTILS))
#define VTG_IS_UTILS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_UTILS))
#define VTG_UTILS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_UTILS, VtgUtilsClass))

typedef struct _VtgUtils VtgUtils;
typedef struct _VtgUtilsClass VtgUtilsClass;
typedef struct _VtgUtilsPrivate VtgUtilsPrivate;

struct _VtgCaches {
	GTypeInstance parent_instance;
	volatile int ref_count;
	VtgCachesPrivate * priv;
};

struct _VtgCachesClass {
	GTypeClass parent_class;
	void (*finalize) (VtgCaches *self);
};

struct _VtgParamSpecCaches {
	GParamSpec parent_instance;
};

struct _VtgUtils {
	GObject parent_instance;
	VtgUtilsPrivate * priv;
};

struct _VtgUtilsClass {
	GObjectClass parent_class;
};


void vtg_interaction_error_message (const char* message, GError* err);
GtkListStore* vtg_caches_get_build_cache (void);
GtkListStore* vtg_caches_get_compile_cache (void);
GtkListStore* vtg_caches_get_configure_cache (void);
GtkListStore* vtg_caches_get_executer_cache (void);
gboolean vtg_caches_cache_contains (GtkListStore* cache, const char* data);
void vtg_caches_cache_add (GtkListStore* cache, const char* data);
gint vtg_caches_cache_count (GtkListStore* cache);
VtgCaches* vtg_caches_construct (GType object_type);
VtgCaches* vtg_caches_new (void);
GParamSpec* vtg_param_spec_caches (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer vtg_value_get_caches (const GValue* value);
void vtg_value_set_caches (GValue* value, gpointer v_object);
GType vtg_caches_get_type (void);
GType vtg_caches_register_type (GTypeModule * module);
gpointer vtg_caches_ref (gpointer instance);
void vtg_caches_unref (gpointer instance);
gboolean vtg_string_utils_is_null_or_empty (const char* data);
char* vtg_string_utils_replace (const char* data, const char* search, const char* replace);
char* vtg_path_utils_normalize_path (const char* name);
gint vtg_path_utils_compare_vala_filenames (const char* filea, const char* fileb);
#define VTG_UTILS_prealloc_count 500
GtkBuilder* vtg_utils_get_builder (void);
GscProposal** vtg_utils_get_proposal_cache (int* result_length1);
char* vtg_utils_get_image_path (const char* id);
char* vtg_utils_get_ui_path (const char* id);
GeeList* vtg_utils_get_available_packages (void);
VtgUtils* vtg_utils_construct (GType object_type);
VtgUtils* vtg_utils_new (void);
GType vtg_utils_get_type (void);
GType vtg_utils_register_type (GTypeModule * module);


G_END_DECLS

#endif
