/*
 *  vtgvcsbackendsbzr.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgvcsbackendsbzr.h"
#include <gee/arraylist.h>
#include <sys/wait.h>
#include <gee/collection.h>
#include <glib/gi18n-lib.h>
#include <glib/gstdio.h>




static char* string_substring (const char* self, glong offset, glong len);
static glong string_get_length (const char* self);
enum  {
	VTG_VCS_BACKENDS_BZR_DUMMY_PROPERTY
};
static GeeList* vtg_vcs_backends_bzr_real_get_items (VtgVcsBackendsIVcs* base, const char* path, GError** error);
static gboolean vtg_vcs_backends_bzr_real_test (VtgVcsBackendsIVcs* base, const char* path);
static gpointer vtg_vcs_backends_bzr_parent_class = NULL;
static VtgVcsBackendsIVcsIface* vtg_vcs_backends_bzr_vtg_vcs_backends_ivcs_parent_iface = NULL;
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);



static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


VtgVcsBackendsBzr* vtg_vcs_backends_bzr_construct (GType object_type) {
	VtgVcsBackendsBzr * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgVcsBackendsBzr* vtg_vcs_backends_bzr_new (void) {
	return vtg_vcs_backends_bzr_construct (VTG_VCS_BACKENDS_TYPE_BZR);
}


static GeeList* vtg_vcs_backends_bzr_real_get_items (VtgVcsBackendsIVcs* base, const char* path, GError** error) {
	VtgVcsBackendsBzr * self;
	GError * inner_error;
	GeeList* results;
	char* stdo;
	char* stde;
	gint exit_status;
	char* _tmp7;
	gboolean _tmp6;
	char* _tmp5;
	char* _tmp4;
	gboolean _tmp3;
	char* _tmp2;
	gint _tmp1_length1;
	char** _tmp1;
	char** _tmp0;
	gboolean _tmp8;
	gboolean _tmp9;
	GeeList* _tmp17;
	self = (VtgVcsBackendsBzr*) base;
	g_return_val_if_fail (path != NULL, NULL);
	inner_error = NULL;
	results = (GeeList*) gee_array_list_new (VTG_VCS_BACKENDS_TYPE_ITEM, (GBoxedCopyFunc) vtg_vcs_backends_item_ref, vtg_vcs_backends_item_unref, g_direct_equal);
	stdo = NULL;
	stde = NULL;
	exit_status = 0;
	_tmp7 = NULL;
	_tmp5 = NULL;
	_tmp4 = NULL;
	_tmp2 = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	_tmp9 = (_tmp8 = (_tmp6 = (_tmp3 = g_spawn_sync (path, (_tmp1 = (_tmp0 = g_new0 (char*, 2 + 1), _tmp0[0] = g_strdup ("bzr"), _tmp0[1] = g_strdup ("status"), _tmp0), _tmp1_length1 = 2, _tmp1), NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &_tmp2, &_tmp5, &exit_status, &inner_error), stdo = (_tmp4 = _tmp2, stdo = (g_free (stdo), NULL), _tmp4), _tmp3), stde = (_tmp7 = _tmp5, stde = (g_free (stde), NULL), _tmp7), _tmp6), _tmp1 = (_vala_array_free (_tmp1, _tmp1_length1, (GDestroyNotify) g_free), NULL), _tmp8);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		(results == NULL) ? NULL : (results = (gee_collection_object_unref (results), NULL));
		stdo = (g_free (stdo), NULL);
		stde = (g_free (stde), NULL);
		return NULL;
	}
	if (_tmp9) {
		gint exit;
		exit = WEXITSTATUS (exit_status);
		if (exit == 0) {
			char** _tmp11;
			gint lines_size;
			gint lines_length1;
			char** _tmp10;
			char** lines;
			gint idx;
			gint status;
			_tmp11 = NULL;
			_tmp10 = NULL;
			lines = (_tmp11 = _tmp10 = g_strsplit (stdo, "\n", 0), lines_length1 = _vala_array_length (_tmp10), lines_size = lines_length1, _tmp11);
			idx = 0;
			status = 0;
			while (lines[idx] != NULL) {
				const char* _tmp12;
				char* line;
				_tmp12 = NULL;
				line = (_tmp12 = lines[idx], (_tmp12 == NULL) ? NULL : g_strdup (_tmp12));
				if (g_str_has_prefix (line, "added:")) {
					status = 1;
				} else {
					if (g_str_has_prefix (line, "modified:")) {
						status = 2;
					} else {
						if (!g_str_has_prefix (line, "  ")) {
							status = 0;
						} else {
							if (g_str_has_prefix (line, "  ")) {
								switch (status) {
									case 1:
									case 2:
									{
										VtgVcsBackendsItem* item;
										VtgVcsBackendsStates _tmp13;
										char* _tmp14;
										item = vtg_vcs_backends_item_new ();
										_tmp13 = 0;
										if (status == 1) {
											_tmp13 = VTG_VCS_BACKENDS_STATES_ADDED;
										} else {
											_tmp13 = VTG_VCS_BACKENDS_STATES_MODIFIED;
										}
										item->state = (_tmp13);
										_tmp14 = NULL;
										item->name = (_tmp14 = string_substring (line, (glong) 2, string_get_length (line) - 2), item->name = (g_free (item->name), NULL), _tmp14);
										gee_collection_add ((GeeCollection*) results, item);
										(item == NULL) ? NULL : (item = (vtg_vcs_backends_item_unref (item), NULL));
										break;
									}
								}
							}
						}
					}
				}
				idx++;
				line = (g_free (line), NULL);
			}
			lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
		} else {
			char* _tmp15;
			GError* _tmp16;
			_tmp15 = NULL;
			_tmp16 = NULL;
			inner_error = (_tmp16 = g_error_new_literal (VTG_VCS_BACKENDS_VCS_ERROR, VTG_VCS_BACKENDS_VCS_ERROR_CommandFailed, _tmp15 = g_strdup_printf (_ ("error executing the bzr status command.\n%s"), stde)), _tmp15 = (g_free (_tmp15), NULL), _tmp16);
			if (inner_error != NULL) {
				g_propagate_error (error, inner_error);
				(results == NULL) ? NULL : (results = (gee_collection_object_unref (results), NULL));
				stdo = (g_free (stdo), NULL);
				stde = (g_free (stde), NULL);
				return NULL;
			}
		}
	}
	_tmp17 = NULL;
	return (_tmp17 = results, stdo = (g_free (stdo), NULL), stde = (g_free (stde), NULL), _tmp17);
}


static gboolean vtg_vcs_backends_bzr_real_test (VtgVcsBackendsIVcs* base, const char* path) {
	VtgVcsBackendsBzr * self;
	char* bzr_dir;
	gboolean _tmp1;
	self = (VtgVcsBackendsBzr*) base;
	g_return_val_if_fail (path != NULL, FALSE);
	bzr_dir = g_build_filename (path, ".bzr", NULL);
	if (g_file_test (bzr_dir, G_FILE_TEST_IS_DIR)) {
		gboolean _tmp0;
		return (_tmp0 = TRUE, bzr_dir = (g_free (bzr_dir), NULL), _tmp0);
	}
	return (_tmp1 = FALSE, bzr_dir = (g_free (bzr_dir), NULL), _tmp1);
}


static void vtg_vcs_backends_bzr_class_init (VtgVcsBackendsBzrClass * klass) {
	vtg_vcs_backends_bzr_parent_class = g_type_class_peek_parent (klass);
}


static void vtg_vcs_backends_bzr_vtg_vcs_backends_ivcs_interface_init (VtgVcsBackendsIVcsIface * iface) {
	vtg_vcs_backends_bzr_vtg_vcs_backends_ivcs_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_items = vtg_vcs_backends_bzr_real_get_items;
	iface->test = vtg_vcs_backends_bzr_real_test;
}


static void vtg_vcs_backends_bzr_instance_init (VtgVcsBackendsBzr * self) {
}


static GType vtg_vcs_backends_bzr_type_id = 0;
GType vtg_vcs_backends_bzr_get_type (void) {
	return vtg_vcs_backends_bzr_type_id;
}


GType vtg_vcs_backends_bzr_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgVcsBackendsBzrClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_vcs_backends_bzr_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgVcsBackendsBzr), 0, (GInstanceInitFunc) vtg_vcs_backends_bzr_instance_init, NULL };
	static const GInterfaceInfo vtg_vcs_backends_ivcs_info = { (GInterfaceInitFunc) vtg_vcs_backends_bzr_vtg_vcs_backends_ivcs_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	vtg_vcs_backends_bzr_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgVcsBackendsBzr", &g_define_type_info, 0);
	g_type_add_interface_static (vtg_vcs_backends_bzr_type_id, VTG_VCS_BACKENDS_TYPE_IVCS, &vtg_vcs_backends_ivcs_info);
	return vtg_vcs_backends_bzr_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}




