/*
 *  vtgvcsbackendsgit.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgvcsbackendsgit.h"
#include <gee/arraylist.h>
#include <sys/wait.h>
#include <gee/collection.h>
#include <glib/gi18n-lib.h>
#include <glib/gstdio.h>




static char* string_replace (const char* self, const char* old, const char* replacement);
enum  {
	VTG_VCS_BACKENDS_GIT_DUMMY_PROPERTY
};
static GeeList* vtg_vcs_backends_git_real_get_items (VtgVcsBackendsIVcs* base, const char* path, GError** error);
static gboolean vtg_vcs_backends_git_real_test (VtgVcsBackendsIVcs* base, const char* path);
static gpointer vtg_vcs_backends_git_parent_class = NULL;
static VtgVcsBackendsIVcsIface* vtg_vcs_backends_git_vtg_vcs_backends_ivcs_parent_iface = NULL;
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);



static char* string_replace (const char* self, const char* old, const char* replacement) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (old != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	inner_error = NULL;
	{
		char* _tmp0;
		GRegex* _tmp1;
		GRegex* regex;
		char* _tmp2;
		char* _tmp3;
		_tmp0 = NULL;
		_tmp1 = NULL;
		regex = (_tmp1 = g_regex_new (_tmp0 = g_regex_escape_string (old, -1), 0, 0, &inner_error), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
		if (inner_error != NULL) {
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch26_g_regex_error;
			}
			goto __finally26;
		}
		_tmp2 = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &inner_error);
		if (inner_error != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch26_g_regex_error;
			}
			goto __finally26;
		}
		_tmp3 = NULL;
		return (_tmp3 = _tmp2, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp3);
	}
	goto __finally26;
	__catch26_g_regex_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_assert_not_reached ();
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally26:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return NULL;
	}
}


VtgVcsBackendsGit* vtg_vcs_backends_git_construct (GType object_type) {
	VtgVcsBackendsGit * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgVcsBackendsGit* vtg_vcs_backends_git_new (void) {
	return vtg_vcs_backends_git_construct (VTG_VCS_BACKENDS_TYPE_GIT);
}


static GeeList* vtg_vcs_backends_git_real_get_items (VtgVcsBackendsIVcs* base, const char* path, GError** error) {
	VtgVcsBackendsGit * self;
	GError * inner_error;
	GeeList* results;
	char* stdo;
	char* stde;
	gint exit_status;
	char* _tmp7;
	gboolean _tmp6;
	char* _tmp5;
	char* _tmp4;
	gboolean _tmp3;
	char* _tmp2;
	gint _tmp1_length1;
	char** _tmp1;
	char** _tmp0;
	gboolean _tmp8;
	gboolean _tmp9;
	GeeList* _tmp18;
	self = (VtgVcsBackendsGit*) base;
	g_return_val_if_fail (path != NULL, NULL);
	inner_error = NULL;
	results = (GeeList*) gee_array_list_new (VTG_VCS_BACKENDS_TYPE_ITEM, (GBoxedCopyFunc) vtg_vcs_backends_item_ref, vtg_vcs_backends_item_unref, g_direct_equal);
	stdo = NULL;
	stde = NULL;
	exit_status = 0;
	_tmp7 = NULL;
	_tmp5 = NULL;
	_tmp4 = NULL;
	_tmp2 = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	_tmp9 = (_tmp8 = (_tmp6 = (_tmp3 = g_spawn_sync (path, (_tmp1 = (_tmp0 = g_new0 (char*, 2 + 1), _tmp0[0] = g_strdup ("git"), _tmp0[1] = g_strdup ("status"), _tmp0), _tmp1_length1 = 2, _tmp1), NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &_tmp2, &_tmp5, &exit_status, &inner_error), stdo = (_tmp4 = _tmp2, stdo = (g_free (stdo), NULL), _tmp4), _tmp3), stde = (_tmp7 = _tmp5, stde = (g_free (stde), NULL), _tmp7), _tmp6), _tmp1 = (_vala_array_free (_tmp1, _tmp1_length1, (GDestroyNotify) g_free), NULL), _tmp8);
	if (inner_error != NULL) {
		g_propagate_error (error, inner_error);
		(results == NULL) ? NULL : (results = (gee_collection_object_unref (results), NULL));
		stdo = (g_free (stdo), NULL);
		stde = (g_free (stde), NULL);
		return NULL;
	}
	if (_tmp9) {
		gint exit;
		gboolean _tmp10;
		exit = WEXITSTATUS (exit_status);
		_tmp10 = FALSE;
		if (exit == 0) {
			_tmp10 = TRUE;
		} else {
			_tmp10 = exit == 1;
		}
		if (_tmp10) {
			char** _tmp12;
			gint lines_size;
			gint lines_length1;
			char** _tmp11;
			char** lines;
			gint idx;
			GPatternSpec* modp;
			GPatternSpec* newp;
			_tmp12 = NULL;
			_tmp11 = NULL;
			lines = (_tmp12 = _tmp11 = g_strsplit (stdo, "\n", 0), lines_length1 = _vala_array_length (_tmp11), lines_size = lines_length1, _tmp12);
			idx = 0;
			modp = g_pattern_spec_new ("#\tmodified:   *");
			newp = g_pattern_spec_new ("#\tnew file:   *");
			while (lines[idx] != NULL) {
				const char* _tmp13;
				char* line;
				_tmp13 = NULL;
				line = (_tmp13 = lines[idx], (_tmp13 == NULL) ? NULL : g_strdup (_tmp13));
				if (g_pattern_match_string (modp, line)) {
					VtgVcsBackendsItem* item;
					char* _tmp14;
					item = vtg_vcs_backends_item_new ();
					item->state = VTG_VCS_BACKENDS_STATES_MODIFIED;
					_tmp14 = NULL;
					item->name = (_tmp14 = string_replace (line, "#\tmodified:   ", ""), item->name = (g_free (item->name), NULL), _tmp14);
					gee_collection_add ((GeeCollection*) results, item);
					(item == NULL) ? NULL : (item = (vtg_vcs_backends_item_unref (item), NULL));
				} else {
					if (g_pattern_match_string (newp, line)) {
						VtgVcsBackendsItem* item;
						char* _tmp15;
						item = vtg_vcs_backends_item_new ();
						item->state = VTG_VCS_BACKENDS_STATES_ADDED;
						_tmp15 = NULL;
						item->name = (_tmp15 = string_replace (line, "#\tnew file:   ", ""), item->name = (g_free (item->name), NULL), _tmp15);
						gee_collection_add ((GeeCollection*) results, item);
						(item == NULL) ? NULL : (item = (vtg_vcs_backends_item_unref (item), NULL));
					}
				}
				idx++;
				line = (g_free (line), NULL);
			}
			lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
			(modp == NULL) ? NULL : (modp = (g_pattern_spec_free (modp), NULL));
			(newp == NULL) ? NULL : (newp = (g_pattern_spec_free (newp), NULL));
		} else {
			char* _tmp16;
			GError* _tmp17;
			_tmp16 = NULL;
			_tmp17 = NULL;
			inner_error = (_tmp17 = g_error_new_literal (VTG_VCS_BACKENDS_VCS_ERROR, VTG_VCS_BACKENDS_VCS_ERROR_CommandFailed, _tmp16 = g_strdup_printf (_ ("error executing the git status command.\n%s"), stde)), _tmp16 = (g_free (_tmp16), NULL), _tmp17);
			if (inner_error != NULL) {
				g_propagate_error (error, inner_error);
				(results == NULL) ? NULL : (results = (gee_collection_object_unref (results), NULL));
				stdo = (g_free (stdo), NULL);
				stde = (g_free (stde), NULL);
				return NULL;
			}
		}
	}
	_tmp18 = NULL;
	return (_tmp18 = results, stdo = (g_free (stdo), NULL), stde = (g_free (stde), NULL), _tmp18);
}


static gboolean vtg_vcs_backends_git_real_test (VtgVcsBackendsIVcs* base, const char* path) {
	VtgVcsBackendsGit * self;
	char* git_dir;
	gboolean _tmp1;
	self = (VtgVcsBackendsGit*) base;
	g_return_val_if_fail (path != NULL, FALSE);
	git_dir = g_build_filename (path, ".git", NULL);
	if (g_file_test (git_dir, G_FILE_TEST_IS_DIR)) {
		gboolean _tmp0;
		return (_tmp0 = TRUE, git_dir = (g_free (git_dir), NULL), _tmp0);
	}
	return (_tmp1 = FALSE, git_dir = (g_free (git_dir), NULL), _tmp1);
}


static void vtg_vcs_backends_git_class_init (VtgVcsBackendsGitClass * klass) {
	vtg_vcs_backends_git_parent_class = g_type_class_peek_parent (klass);
}


static void vtg_vcs_backends_git_vtg_vcs_backends_ivcs_interface_init (VtgVcsBackendsIVcsIface * iface) {
	vtg_vcs_backends_git_vtg_vcs_backends_ivcs_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_items = vtg_vcs_backends_git_real_get_items;
	iface->test = vtg_vcs_backends_git_real_test;
}


static void vtg_vcs_backends_git_instance_init (VtgVcsBackendsGit * self) {
}


static GType vtg_vcs_backends_git_type_id = 0;
GType vtg_vcs_backends_git_get_type (void) {
	return vtg_vcs_backends_git_type_id;
}


GType vtg_vcs_backends_git_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgVcsBackendsGitClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_vcs_backends_git_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgVcsBackendsGit), 0, (GInstanceInitFunc) vtg_vcs_backends_git_instance_init, NULL };
	static const GInterfaceInfo vtg_vcs_backends_ivcs_info = { (GInterfaceInitFunc) vtg_vcs_backends_git_vtg_vcs_backends_ivcs_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	vtg_vcs_backends_git_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgVcsBackendsGit", &g_define_type_info, 0);
	g_type_add_interface_static (vtg_vcs_backends_git_type_id, VTG_VCS_BACKENDS_TYPE_IVCS, &vtg_vcs_backends_ivcs_info);
	return vtg_vcs_backends_git_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}




