/*
 *  vtgvcsbackendsivcs.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgvcsbackendsivcs.h"
#include <gobject/gvaluecollector.h>




enum  {
	VTG_VCS_BACKENDS_ITEM_DUMMY_PROPERTY
};
static gpointer vtg_vcs_backends_item_parent_class = NULL;
static void vtg_vcs_backends_item_finalize (VtgVcsBackendsItem* obj);




GType vtg_vcs_backends_states_get_type (void) {
	static GType vtg_vcs_backends_states_type_id = 0;
	if (G_UNLIKELY (vtg_vcs_backends_states_type_id == 0)) {
		static const GEnumValue values[] = {{VTG_VCS_BACKENDS_STATES_UNTRACKED, "VTG_VCS_BACKENDS_STATES_UNTRACKED", "untracked"}, {VTG_VCS_BACKENDS_STATES_ADDED, "VTG_VCS_BACKENDS_STATES_ADDED", "added"}, {VTG_VCS_BACKENDS_STATES_MODIFIED, "VTG_VCS_BACKENDS_STATES_MODIFIED", "modified"}, {0, NULL, NULL}};
		vtg_vcs_backends_states_type_id = g_enum_register_static ("VtgVcsBackendsStates", values);
	}
	return vtg_vcs_backends_states_type_id;
}


VtgVcsBackendsItem* vtg_vcs_backends_item_construct (GType object_type) {
	VtgVcsBackendsItem* self;
	self = (VtgVcsBackendsItem*) g_type_create_instance (object_type);
	return self;
}


VtgVcsBackendsItem* vtg_vcs_backends_item_new (void) {
	return vtg_vcs_backends_item_construct (VTG_VCS_BACKENDS_TYPE_ITEM);
}


static void vtg_vcs_backends_value_item_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vtg_vcs_backends_value_item_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vtg_vcs_backends_item_unref (value->data[0].v_pointer);
	}
}


static void vtg_vcs_backends_value_item_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vtg_vcs_backends_item_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vtg_vcs_backends_value_item_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vtg_vcs_backends_value_item_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		VtgVcsBackendsItem* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vtg_vcs_backends_item_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vtg_vcs_backends_value_item_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	VtgVcsBackendsItem** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vtg_vcs_backends_item_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vtg_vcs_backends_param_spec_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	VtgVcsBackendsParamSpecItem* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VTG_VCS_BACKENDS_TYPE_ITEM), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vtg_vcs_backends_value_get_item (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_VCS_BACKENDS_TYPE_ITEM), NULL);
	return value->data[0].v_pointer;
}


void vtg_vcs_backends_value_set_item (GValue* value, gpointer v_object) {
	VtgVcsBackendsItem* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_VCS_BACKENDS_TYPE_ITEM));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VTG_VCS_BACKENDS_TYPE_ITEM));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vtg_vcs_backends_item_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vtg_vcs_backends_item_unref (old);
	}
}


static void vtg_vcs_backends_item_class_init (VtgVcsBackendsItemClass * klass) {
	vtg_vcs_backends_item_parent_class = g_type_class_peek_parent (klass);
	VTG_VCS_BACKENDS_ITEM_CLASS (klass)->finalize = vtg_vcs_backends_item_finalize;
}


static void vtg_vcs_backends_item_instance_init (VtgVcsBackendsItem * self) {
	self->name = g_strdup ("");
	self->state = VTG_VCS_BACKENDS_STATES_UNTRACKED;
	self->ref_count = 1;
}


static void vtg_vcs_backends_item_finalize (VtgVcsBackendsItem* obj) {
	VtgVcsBackendsItem * self;
	self = VTG_VCS_BACKENDS_ITEM (obj);
	self->name = (g_free (self->name), NULL);
}


static GType vtg_vcs_backends_item_type_id = 0;
GType vtg_vcs_backends_item_get_type (void) {
	return vtg_vcs_backends_item_type_id;
}


GType vtg_vcs_backends_item_register_type (GTypeModule * module) {
	static const GTypeValueTable g_define_type_value_table = { vtg_vcs_backends_value_item_init, vtg_vcs_backends_value_item_free_value, vtg_vcs_backends_value_item_copy_value, vtg_vcs_backends_value_item_peek_pointer, "p", vtg_vcs_backends_value_item_collect_value, "p", vtg_vcs_backends_value_item_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (VtgVcsBackendsItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_vcs_backends_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgVcsBackendsItem), 0, (GInstanceInitFunc) vtg_vcs_backends_item_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	vtg_vcs_backends_item_type_id = g_type_register_fundamental (g_type_fundamental_next (), "VtgVcsBackendsItem", &g_define_type_info, &g_define_type_fundamental_info, 0);
	return vtg_vcs_backends_item_type_id;
}


gpointer vtg_vcs_backends_item_ref (gpointer instance) {
	VtgVcsBackendsItem* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vtg_vcs_backends_item_unref (gpointer instance) {
	VtgVcsBackendsItem* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VTG_VCS_BACKENDS_ITEM_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


GQuark vtg_vcs_backends_vcs_error_quark (void) {
	return g_quark_from_static_string ("vtg_vcs_backends_vcs_error-quark");
}


GeeList* vtg_vcs_backends_ivcs_get_items (VtgVcsBackendsIVcs* self, const char* path, GError** error) {
	return VTG_VCS_BACKENDS_IVCS_GET_INTERFACE (self)->get_items (self, path, error);
}


gboolean vtg_vcs_backends_ivcs_test (VtgVcsBackendsIVcs* self, const char* path) {
	return VTG_VCS_BACKENDS_IVCS_GET_INTERFACE (self)->test (self, path);
}


static void vtg_vcs_backends_ivcs_base_init (VtgVcsBackendsIVcsIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
	}
}


GType vtg_vcs_backends_ivcs_get_type (void) {
	static GType vtg_vcs_backends_ivcs_type_id = 0;
	if (vtg_vcs_backends_ivcs_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VtgVcsBackendsIVcsIface), (GBaseInitFunc) vtg_vcs_backends_ivcs_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		vtg_vcs_backends_ivcs_type_id = g_type_register_static (G_TYPE_INTERFACE, "VtgVcsBackendsIVcs", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (vtg_vcs_backends_ivcs_type_id, G_TYPE_OBJECT);
	}
	return vtg_vcs_backends_ivcs_type_id;
}




